@@ -1,6 +1,6 @@
 /***************************************************************************
  * full_test.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -49,11 +49,11 @@ int type = 0;
 int dotest  = 0;
 int width  = 800;
 int height = 600;
-int mini = 0;
 int big  = 0;
 int srnd = 0;
 int use_mgl = 0;
 int verbose = 0;
+int quality  = MGL_DRAW_NORM;
 //-----------------------------------------------------------------------------
 void mgls_prepare1d(mglData *y, mglData *y1=0, mglData *y2=0, mglData *x1=0, mglData *x2=0);
 void mgls_prepare2d(mglData *a, mglData *b=0, mglData *v=0);
@@ -64,23 +64,59 @@ void mgls_prepare3v(mglData *ex, mglData *ey, mglData *ez);
 void save(mglGraph *gr,const char *name,const char *suf);
 void smgl_stfa(mglGraph *gr);	// STFA sample
 void smgl_text(mglGraph *gr);	// text drawing
+void smgl_surf(mglGraph *gr);
+#include <mgl2/base.h>
+#include <mgl2/font.h>
 void test(mglGraph *gr)
 {
-	smgl_text(gr);	return;
+	union {unsigned long b;double d;float f;} t;	t.b=0;
+	t.d = NAN;	printf("NANd: %g --> %lx\t",t.d,t.b);	t.b=0;
+	t.f = NAN;	printf("NANf: %g --> %lx\n",t.f,t.b);	t.b=0;
 	
+	t.d = INFINITY;	printf("INFd: %g --> %lx\t",t.d,t.b);	t.b=0;
+	t.f = INFINITY;	printf("INFf: %g --> %lx\n",t.f,t.b);	t.b=0;
+
+const unsigned long mgl_nan[2] = {0x7fffffffffffffff, 0x7fffffff};
+#define NANd    (*(double*)mgl_nan)
+#define NANf    (*(float*)(mgl_nan+1))
+	t.d = NANd;	printf("NANd: %g --> %lx\t",t.d,t.b);	t.b=0;
+	t.f = NANf;	printf("NANf: %g --> %lx\n",t.f,t.b);	t.b=0;
+
+	
+const unsigned long mgl_inf[2] = {0x7ff0000000000000, 0x7f800000};
+#define INFd    (*(double*)mgl_inf)
+#define INFf    (*(float*)(mgl_inf+1))
+	t.d = INFd;	printf("INFd: %g --> %lx\t",t.d,t.b);	t.b=0;
+	t.f = INFf;	printf("INFf: %g --> %lx\n",t.f,t.b);	t.b=0;
+
+	return;
+
+	mglData y(50);
+	y.Modify("sin(10*x) + 10");
+	gr->SetRange('y', y);
+	gr->Box();
+	gr->Axis();
+	gr->Plot(y);
+	y.Save("test.dat");
+	return;
+
+
 	mglParse par;
-	par.AllowSetSize(true);
 	setlocale(LC_CTYPE, "");
-	par.Execute(gr,"xrange 1362565462 1365935062:ticktime 'x' 7*86400 '%d/%m/%y':axis");
-//	FILE *fp=fopen("/home/balakin/progr/mathgl-code/mathgl-2x/build/test.mgl","r");
-//	par.Execute(gr,fp,true);
-//	fclose(fp);
+	par.Execute(gr,"new x 50 40 '0.8*sin(pi*x)*sin(pi*(y+1)/2)'\n\
+					new y 50 40 '0.8*cos(pi*x)*sin(pi*(y+1)/2)'\n\
+					new z 50 40 '0.8*cos(pi*(y+1)/2)'\nlight on\n\
+					title 'parametric form':rotate 50 60:box\n\
+					surf x y z ''\nwrite '1.tex'");
+//	par.Execute(gr,"light on:addlegend 'r' 'r':legend");
 }
 //-----------------------------------------------------------------------------
 #if !defined(_MSC_VER) && !defined(__BORLANDC__)
 static struct option longopts[] =
 {
+	{ "mini",	no_argument,	&big,	3 },
 	{ "big",	no_argument,	&big,		1 },
+	{ "web",	no_argument,	&big,		2 },
 	{ "bps",	no_argument,	&type,		8 },
 	{ "help",	no_argument,	NULL,		'?' },
 	{ "height",	required_argument,	NULL,	'h' },
@@ -91,7 +127,6 @@ static struct option longopts[] =
 	{ "kind",	required_argument,	NULL,	'k' },
 	{ "list",	no_argument,	NULL,		'l' },
 	{ "mgl",	no_argument,	&use_mgl,	1 },
-	{ "mini",	no_argument,	&mini,		1 },
 	{ "none",	no_argument,	&type,		7 },
 	{ "obj",	no_argument,	&type,		11 },
 	{ "obj_old",no_argument,	&type,		10 },
@@ -107,9 +142,11 @@ static struct option longopts[] =
 	{ "jsonz",	no_argument,	&type,		16 },
 	{ "test",	no_argument,	&dotest,	1 },
 	{ "font",	no_argument,	&dotest,	2 },
+	{ "time",	no_argument,	&dotest,	3 },
 	{ "thread",	required_argument,	NULL,	't' },
 	{ "verbose",no_argument,	&verbose,	1 },
 	{ "width",	required_argument,	NULL,	'w' },
+	{ "quality",required_argument,	NULL,	'q' },
 	{ NULL,		0,				NULL,		0 }
 };
 //-----------------------------------------------------------------------------
@@ -121,10 +158,11 @@ void usage()
 		"--height=num	- png picture height\n"
 		"--mini		- png picture is 200x150\n"
 		"--big		- png picture is 1920x1440\n"
+		"--web		- png picture is 640x480\n"
 		"--prc		- output prc\n"
 		"--pdf		- output pdf\n"
 		"--eps		- output EPS\n"
-		"--eps		- output LaTeX\n"
+		"--tex		- output LaTeX\n"
 		"--jpeg		- output JPEG\n"
 		"--json		- output JSON\n"
 		"--jsonz		- output JSONz\n"
@@ -140,8 +178,10 @@ void usage()
 		"--kind=name	- produce only this sample\n"
 		"--thread=num	- number of threads used\n"
 		"--mgl		- use MGL scripts for samples\n"
-		"--test		- perform test\n"
+		"--test		- run in test mode\n"
+		"--time		- measure execution time for all samples\n"
 		"--font		- write current font as C++ file\n"
+		"--quality=val	- use specified quality for plot(s)\n"
 	);
 }
 #endif
@@ -150,7 +190,6 @@ void save(mglGraph *gr,const char *name,const char *suf="")
 {
 	//	return;
 	char buf[128];
-	printf("%s ",name);	fflush(stdout);
 	switch(type)
 	{
 		case 1:	// EPS
@@ -172,7 +211,7 @@ void save(mglGraph *gr,const char *name,const char *suf="")
 		case 6:	// GIF
 			snprintf(buf,128,"%s%s.gif",name,suf);
 			gr->WriteGIF(buf);	break;
-		case 7:	// none
+		case 7:	gr->Finish();	// none
 			break;
 		case 8:	// EPS to PNG
 			snprintf(buf,128,"%s%s.png",name,suf);
@@ -201,7 +240,7 @@ void save(mglGraph *gr,const char *name,const char *suf="")
 			gr->WriteJSON(buf);	break;
 		case 16:	// JSON
 			snprintf(buf,128,"%s%s.jsonz",name,suf);
-			gr->WriteJSON(buf);	break;
+			gr->WriteJSON(buf,"",true);	break;
 		default:// PNG (no alpha)
 #if MGL_HAVE_PNG
 			snprintf(buf,128,"%s%s.png",name,suf);
@@ -228,6 +267,7 @@ int main(int argc,char **argv)
 			case 0:		break;
 			case 'w':	width =atoi(optarg);	break;
 			case 'h':	height=atoi(optarg);	break;
+			case 'q':	quality =atoi(optarg);	break;
 			case 'k':	strncpy(name, optarg,256);
 						tmp=strchr(name,'.');	if(tmp)	*tmp=0;
 						tmp=strchr(name,'-');	if(tmp)	*tmp=0;
@@ -242,12 +282,16 @@ int main(int argc,char **argv)
 #endif
 
 	if(dotest==1)	printf("Global (before):%s\n",mglGlobalMess.c_str());
-	gr = new mglGraph;	//gr->SetQuality(0);
+	gr = new mglGraph;
 	if(	type==11|| type==12|| type==5 || type==9)	width=height;
-	if(mini)		{	gr->SetSize(190,145);	suf = "-sm";	}
-	else if(big)
-	{	gr->SetSize(1920,1440);	suf = "-lg";	}
-	else	gr->SetSize(width,height);
+	switch(big)
+	{
+	case 1:	gr->SetSize(1920,1440);	suf = "-lg";	break;
+	case 2:	gr->SetSize(640,480);	break;
+	case 3:	gr->SetSize(192,144);	suf = "-sm";	break;
+	default:	gr->SetSize(width,height);
+	}
+	gr->SetQuality(quality);
 
 	if(dotest==1)
 	{
@@ -262,29 +306,68 @@ int main(int argc,char **argv)
 	else if(dotest==2)
 	{	mgl_create_cpp_font(gr->Self(), L"!-~,¡-ÿ,̀-̏,Α-ω,ϑ,ϕ,ϖ,ϰ,ϱ,ϵ,А-я,ℏ,ℑ,ℓ,ℜ,←-↙,∀-∯,≠-≯,⟂");
 		delete gr;	return 0;	}
+	else if(dotest==3)
+	{
+		int qual[7]={0,1,2,4,5,6,8};
+		size_t ll=strlen(mmgl_dat_prepare)+1;
+		mglParse par;
+		par.AllowSetSize(true);	setlocale(LC_CTYPE, "");
+		FILE *fp = fopen(big?"time_big.texi":"time.texi","w");
+		fprintf(fp,"@multitable @columnfractions .16 .12 .12 .12 .12 .12 .12 .12\n");
+		fprintf(fp,"@headitem Name");
+		for(int i=0;i<7;i++)	fprintf(fp," @tab q=%d",qual[i]);
+		clock_t beg,end;
+		while(s->name[0])	// all samples
+		{
+			char *buf = new char[strlen(s->mgl)+ll];
+			strcpy(buf,s->mgl);	strcat(buf,mmgl_dat_prepare);
+			fprintf(fp,"\n@item %s",s->name);
+
+			printf("%s",s->name);
+			for(int i=0;i<7;i++)
+			{
+				gr->DefaultPlotParam();
+				gr->SetQuality(qual[i]);	gr->Clf();
+				beg = clock();
+				if(!use_mgl)	s->func(gr);
+				else 	par.Execute(gr,buf);
+				gr->Finish();
+				end = clock();
+				fprintf(fp," @tab %.3g",double(end-beg)/CLOCKS_PER_SEC);
+				printf("\t%d->%g",qual[i],double(end-beg)/CLOCKS_PER_SEC);
+				fflush(fp);	fflush(stdout);
+			}
+			printf("\n");	delete []buf;	s++;
+		}
+		fprintf(fp,"\n@end multitable\n");	fclose(fp);
+	}
+
+	if(type==15 || type==16)	big=3;	// save mini version for json
 
-	if(type==15 || type==16)	mini=1;	// save mini version for json
-	
 	if(srnd)	mgl_srnd(1);
 	gr->VertexColor(false);	gr->Compression(false);
-	if(name[0]==0)	while(s->name[0])	// all samples
+	if(name[0]==0)
 	{
-		gr->DefaultPlotParam();	gr->Clf();
-		if(use_mgl)
+		while(s->name[0])	// all samples
 		{
-			mglParse par;
-			par.AllowSetSize(true);
-			setlocale(LC_CTYPE, "");
-			char *buf = new char[strlen(s->mgl)+strlen(mmgl_dat_prepare)+1];
-			strcpy(buf,s->mgl);		strcat(buf,mmgl_dat_prepare);
-			printf("\n-------\n%s\n-------\n",verbose?buf:s->mgl);
-			par.Execute(gr,buf);	delete []buf;
-			const char *mess = gr->Message();
-			if(*mess)	printf("Warnings: %s\n-------\n",mess);
+			gr->DefaultPlotParam();	gr->Clf();
+			if(use_mgl)
+			{
+				mglParse par;
+				par.AllowSetSize(true);
+				setlocale(LC_CTYPE, "");
+				char *buf = new char[strlen(s->mgl)+strlen(mmgl_dat_prepare)+1];
+				strcpy(buf,s->mgl);		strcat(buf,mmgl_dat_prepare);
+				printf("\n-------\n%s\n-------\n",verbose?buf:s->mgl);
+				par.Execute(gr,buf);	delete []buf;
+				const char *mess = gr->Message();
+				if(*mess)	printf("Warnings: %s\n-------\n",mess);
+			}
+			else	s->func(gr);
+			save(gr, s->name, suf);
+			printf("%s ",s->name);	fflush(stdout);	s++;
 		}
-		else	s->func(gr);
-		save(gr, s->name, suf);
-		fflush(stdout);	s++;
+		printf("\n");
 	}
 	else	// manual sample
 	{
@@ -312,7 +395,6 @@ int main(int argc,char **argv)
 		}
 		else	printf("no sample %s\n",name);
 	}
-	printf("\n");
 	delete gr;	return 0;
 }
 //-----------------------------------------------------------------------------
