@@ -1,6 +1,6 @@
 /***************************************************************************
  * base.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -19,62 +19,21 @@
  ***************************************************************************/
 #ifndef _MGL_BASE_H_
 #define _MGL_BASE_H_
-#if !defined(_MSC_VER) && !defined(__BORLANDC__)
-#include <stdint.h>
-#endif
-#if defined(__BORLANDC__)
-typedef unsigned uintptr_t;
-#endif
-#include "mgl2/define.h"
+//#if !defined(_MSC_VER) && !defined(__BORLANDC__)
+#include "mgl2/abstract.h"
 
 #ifdef __cplusplus
 #include <vector>
 #include <string>
-#include "mgl2/type.h"
-//-----------------------------------------------------------------------------
-class mglBase;
-class mglData;
-class mglParser;
-class mglFormula;
-class mglFormulaC;
-class mglFont;
-typedef mglBase* HMGL;
-typedef mglData* HMDT;
-typedef mglParser* HMPR;
-typedef mglFormula* HMEX;
-typedef mglFormulaC* HAEX;
-//-----------------------------------------------------------------------------
-#if MGL_NO_DATA_A
-#define mglDataA mglData
-typedef const mglData* HCDT;
-#include "mgl2/data.h"
+
+#if MGL_HAVE_PTHREAD
+#include <pthread.h>
+#define MGL_PUSH(a,v,m)		{pthread_mutex_lock(&m);	a.push_back(v);	pthread_mutex_unlock(&m);}
 #else
-//-----------------------------------------------------------------------------
-/// Callback function for asking user a question. Result shouldn't exceed 1024.
-extern MGL_EXPORT void (*mgl_ask_func)(const wchar_t *quest, wchar_t *res);
-//-----------------------------------------------------------------------------
-/// Abstract class for data array
-class mglDataA
-{
-public:
-	virtual ~mglDataA()	{}
-	virtual mreal v(long i,long j=0,long k=0) const = 0;
-	virtual mreal vthr(long i) const = 0;
-	virtual long GetNx() const = 0;
-	virtual long GetNy() const = 0;
-	virtual long GetNz() const = 0;
-	inline long GetNN() const {	return GetNx()*GetNy()*GetNz();	}
-	virtual mreal Maximal() const = 0;
-	virtual mreal Minimal() const = 0;
-	virtual mreal dvx(long i,long j=0,long k=0) const = 0;
-//	{	return i>0 ? (i<GetNx()-1 ? (v(i+1,j,k)-v(i-1,j,k))/2 : v(i,j,k)-v(i-1,j,k)) : v(1,j,k)-v(0,j,k);	}
-	virtual mreal dvy(long i,long j=0,long k=0) const = 0;
-//	{	return j>0 ? (j<GetNy()-1 ? (v(i,j+1,k)-v(i,j-1,k))/2 : v(i,j,k)-v(i,j-1,k)) : v(i,1,k)-v(i,0,k);	}
-	virtual mreal dvz(long i,long j=0,long k=0) const = 0;
-//	{	return k>0 ? (k<GetNz()-1 ? (v(i,j,k+1)-v(i,j,k-1))/2 : v(i,j,k)-v(i,j,k-1)) : v(i,j,1)-v(i,j,0);	}
-};
+#define MGL_PUSH(a,v,m)		a.push_back(v);
 #endif
-typedef const mglDataA* HCDT;
+
+#define MGL_TO_WCS(str,code)	if(str){size_t s=mbstowcs(0,str,0); wchar_t *wcs=new wchar_t[s+1]; mbstowcs(wcs,str,s); wcs[s]=0; code; delete []wcs;}
 //-----------------------------------------------------------------------------
 inline mreal mgl_d(mreal v,mreal v1,mreal v2) { return v2!=v1?(v-v1)/(v2-v1):NAN; }
 //-----------------------------------------------------------------------------
@@ -82,35 +41,60 @@ mglPoint GetX(HCDT x, int i, int j, int k=0);
 mglPoint GetY(HCDT y, int i, int j, int k=0);
 mglPoint GetZ(HCDT z, int i, int j, int k=0);
 //-----------------------------------------------------------------------------
+/// Structure for transformation matrix
+struct MGL_EXPORT mglMatrix
+{
+	mreal b[9];
+	mreal x,y,z,pf;
+	bool norot;	// flag to disable pnts rotation
+	mglMatrix()	{	memset(this,0,sizeof(mglMatrix));	clear();	}
+	void Rotate(mreal tetz,mreal tetx,mreal tety);
+	void RotateN(mreal Tet,mreal x,mreal y,mreal z);
+	inline void clear()	{	x=y=z=pf=0;	memset(b,0,9*sizeof(mreal));	b[0]=b[4]=b[8]=1;	norot=false;	}
+	inline mglMatrix &operator=(const mglMatrix &a)
+	{	x=a.x;	y=a.y;	z=a.z;	pf=a.pf;	memcpy(b,a.b,9*sizeof(mreal));	norot=false;	return *this;	}
+};
+inline bool operator==(const mglMatrix &a, const mglMatrix &b)
+{	return b.x==a.x&&b.y==a.y&&b.z==a.z&&b.pf==a.pf&&!memcmp(b.b,a.b,9*sizeof(mreal));}
+inline bool operator!=(const mglMatrix &a, const mglMatrix &b)
+{	return b.x!=a.x||b.y!=a.y||b.z!=a.z||b.pf!=a.pf||memcmp(b.b,a.b,9*sizeof(mreal));	}
+//-----------------------------------------------------------------------------
 /// Structure for simplest primitives
-struct mglPrim	// NOTE: use float for reducing memory size
+struct MGL_EXPORT mglPrim	// NOTE: use float for reducing memory size
 {
 	// NOTE: n4 is used as mark; n3 -- as pen style for type=0,1,4
 	// NOTE: n3 is used as position of txt,font in Ptxt for type=6
 	long n1,n2,n3,n4;	///< coordinates of corners
-	int type;	///< primitive type (0-point, 1-line, 2-trig, 3-quad, 4-glyph, 6-text)
+	short type;	///< primitive type (0-point, 1-line, 2-trig, 3-quad, 4-glyph, 6-text)
+	short angl;	///< rotation angle for mask
 	int id;		///< object id
-	float z;		///< z-position
-	float s;		///< size (if applicable) or fscl
-	float w;		///< width (if applicable) or ftet
-	float p;
-
-	mglPrim(int t=0)	{	n1=n2=n3=n4=id=0;	z=s=w=p=0;	type = t;	}
+	float z;	///< z-position
+	float w;	///< width (if applicable) or ftet
+	union
+	{
+		struct
+		{
+			float s;		///< size (if applicable) or fscl
+			float p;
+		};
+		uint64_t m;
+	};
+	mglPrim(int t=0)	{	n1=n2=n3=n4=id=0;	z=s=w=p=0;	type=t;	}
 };
 bool operator<(const mglPrim &a,const mglPrim &b);
 bool operator>(const mglPrim &a,const mglPrim &b);
 //-----------------------------------------------------------------------------
 /// Structure for group of primitives
-struct mglGroup
+struct MGL_EXPORT mglGroup
 {
 	std::vector<long> p;	///< list of primitives (not filled!!!)
-	int Id;				///< Current list of primitives
+	int Id;					///< Current list of primitives
 	std::string Lbl;		///< Group label
 	mglGroup(const char *lbl="", int id=0)	{	Lbl=lbl;	Id=id;	}
 };
 //-----------------------------------------------------------------------------
 /// Structure for text label
-struct mglText
+struct MGL_EXPORT mglText
 {
 	std::wstring text;
 	std::string stl;
@@ -120,14 +104,15 @@ struct mglText
 };
 //-----------------------------------------------------------------------------
 /// Structure for internal point representation
-struct mglPnt	// NOTE: use float for reducing memory size
+struct MGL_EXPORT mglPnt	// NOTE: use float for reducing memory size
 {
 	float xx,yy,zz;	// original coordinates
 	float x,y,z;	// coordinates
 	float c,t,ta;	// index in color scheme
 	float u,v,w;	// normales
 	float r,g,b,a;	// RGBA color
-	mglPnt()	{	xx=yy=zz=x=y=z=c=t=ta=u=v=w=r=g=b=a=0;	}
+	short sub;		// subplot id and rotation information (later will be in subplot)
+	mglPnt()	{	xx=yy=zz=x=y=z=c=t=ta=u=v=w=r=g=b=a=sub=0;	}
 };
 inline mglPnt operator+(const mglPnt &a, const mglPnt &b)
 {	mglPnt c=a;
@@ -153,8 +138,8 @@ struct MGL_EXPORT mglGlyph
 	short *trig, *line;	///< vertexes of triangles and lines
 
 	mglGlyph()	{	nl=nt=0;	trig=line=0;	}
-	mglGlyph(const mglGlyph &a)	{	trig=line=0;	*this=a;	}
-	mglGlyph(long Nt, long Nl)		{	trig=line=0;	Create(Nt,Nl);	}
+	mglGlyph(const mglGlyph &a)	{	nl=nt=0;	trig=line=0;	*this=a;	}
+	mglGlyph(long Nt, long Nl)	{	nl=nt=0;	trig=line=0;	Create(Nt,Nl);	}
 	~mglGlyph()	{	if(trig)	delete []trig;	if(line)	delete []line;	}
 
 	void Create(long Nt, long Nl);
@@ -174,7 +159,7 @@ struct MGL_EXPORT mglTexture
 	char Sch[260];		///< Color scheme used
 	int Smooth;			///< Type of texture (smoothing and so on)
 	mreal Alpha;			///< Transparency
-	
+
 	mglTexture()	{	n=Smooth=0;	Alpha=1;	}
 	mglTexture(const char *cols, int smooth=0,mreal alpha=1)
 	{	n=0;	Set(cols,smooth,alpha);	}
@@ -195,19 +180,10 @@ const mglColor BC( 0, 0, 0);
 const mglColor WC( 1, 1, 1);
 const mglColor RC( 1, 0, 0);
 //-----------------------------------------------------------------------------
-/// Structure for color ID
-struct mglColorID
-{
-	char id;
-	mglColor col;
-};
-MGL_EXPORT extern mglColorID mglColorIds[31];
-MGL_EXPORT extern std::string mglGlobalMess;	///< Buffer for receiving global messages
-//-----------------------------------------------------------------------------
 /// Structure active points
-struct mglActivePos
+struct MGL_EXPORT mglActivePos
 {
-	int x,y;		///< coordinates of active point
+	int x,y;	///< coordinates of active point
 	int id;		///< object id for active point
 	int n;		///< position of active point in command (object id)
 };
@@ -233,16 +209,16 @@ public:
 	mreal CDef;			///< Default (current) color in texture
 	mreal AlphaDef;		///< Default value of alpha channel (transparency)
 	mreal BarWidth;		///< Relative width of rectangles in Bars().
-	int MeshNum;			///< Set approximate number of lines in Mesh and Grid. By default (=0) it draw all lines.
-	int FaceNum;			///< Set approximate number of visible faces and lines. By default (=0) it draw everything.
+	int MeshNum;		///< Set approximate number of lines in Mesh and Grid. By default (=0) it draw all lines.
+	int FaceNum;		///< Set approximate number of visible faces and lines. By default (=0) it draw everything.
 	char Arrow1, Arrow2;///< Style of arrows at end and at start of curve
 	long InUse;			///< Smart pointer (number of users)
-	long Flag;			///< Flags for controlling drawing
+	uint32_t Flag;			///< Flags for controlling drawing
 
-	inline bool get(long fl) const	{	return Flag&fl;	}
-	inline void set(long fl)	{	Flag |= fl;	}
-	inline void clr(long fl)	{	Flag &=~fl;	}
-	inline void set(bool v,long fl)	{	Flag = v ? Flag|fl : Flag&(~fl);	}
+	inline bool get(uint32_t fl) const	{	return Flag&fl;	}
+	inline void set(uint32_t fl)	{	Flag |= fl;	}
+	inline void clr(uint32_t fl)	{	Flag &=~fl;	}
+	inline void set(bool v,uint32_t fl)	{	Flag = v ? Flag|fl : Flag&(~fl);	}
 
 	/// Set axis range scaling -- simplified way to shift/zoom axis range -- need to replot whole image!
 	inline void ZoomAxis(mglPoint p1=mglPoint(0,0,0,0), mglPoint p2=mglPoint(1,1,1,1))	{	AMin = p1;	AMax = p2;	}
@@ -252,16 +228,16 @@ public:
 	void SetRanges(mglPoint v1, mglPoint v2);
 	/// Set values of mglGraph::Cmin and mglGraph::Cmax as minimal and maximal values of data a
 	void CRange(HCDT a, bool add = false, mreal fact=0);
-	inline void CRange(mreal v1,mreal v2)	{	if(v1!=v2)	{OMin.c=Min.c=v1;	OMax.c=Max.c=v2;	RecalcCRange();}	}
+	void CRange(mreal v1,mreal v2,bool add=false);
 	/// Set values of mglGraph::Min.x and mglGraph::Max.x as minimal and maximal values of data a
 	void XRange(HCDT a, bool add = false, mreal fact=0);
-	inline void XRange(mreal v1,mreal v2)	{	if(v1!=v2)	{OMin.x=Min.x=v1;	OMax.x=Max.x=v2;	RecalcBorder();}	}
+	void XRange(mreal v1,mreal v2,bool add=false);
 	/// Set values of mglGraph::Min.x and mglGraph::Max.x as minimal and maximal values of data a
 	void YRange(HCDT a, bool add = false, mreal fact=0);
-	inline void YRange(mreal v1,mreal v2)	{	if(v1!=v2)	{OMin.y=Min.y=v1;	OMax.y=Max.y=v2;	RecalcBorder();}	}
+	void YRange(mreal v1,mreal v2,bool add=false);
 	/// Set values of mglGraph::Min.x and mglGraph::Max.x as minimal and maximal values of data a
 	void ZRange(HCDT a, bool add = false, mreal fact=0);
-	inline void ZRange(mreal v1,mreal v2)	{	if(v1!=v2)	{OMin.z=Min.z=v1;	OMax.z=Max.z=v2;	RecalcBorder();}	}
+	void ZRange(mreal v1,mreal v2,bool add=false);
 	/// Set ranges for automatic variables
 	void SetAutoRanges(mreal x1, mreal x2, mreal y1=0, mreal y2=0, mreal z1=0, mreal z2=0, mreal c1=0, mreal c2=0);
 	/// Set axis origin
@@ -288,14 +264,20 @@ public:
 	inline void SetCutBox(mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2)
 	{	CutMin=mglPoint(x1,y1,z1);	CutMax=mglPoint(x2,y2,z2);	}
 	inline void SetCutBox(mglPoint v1, mglPoint v2)	{	CutMin=v1;	CutMax=v2;	}
+	/// Reset mask to solid state
+	inline void ResetMask()	{	mask = MGL_SOLID_MASK;	MaskAn = DefMaskAn;	}
+	/// Set default mask rotation angle
+	inline void SetMaskAngle(int angle)	{	DefMaskAn = angle;	}
 
 	/// Set the using of light on/off.
 	virtual bool Light(bool enable)
 	{	bool t=get(MGL_ENABLE_LIGHT);	set(enable,MGL_ENABLE_LIGHT);	return t;	}
 	/// Set ambient light brightness
 	virtual void SetAmbient(mreal bright=0.5);
+	/// Set diffusive light brightness
+	virtual void SetDiffuse(mreal bright=0.5);
 	/// Use diffusive light (only for local light sources)
-	inline void SetDifLight(bool dif)	{	set(dif,MGL_DIFFUSIVE);	}
+	inline void SetDifLight(bool dif)	{	SetDiffuse(dif?0.5:0);	}
 	/// Set the transparency on/off.
 	virtual bool Alpha(bool enable)
 	{	bool t=get(MGL_ENABLE_ALPHA);	set(enable,MGL_ENABLE_ALPHA);	return t;	}
@@ -354,7 +336,7 @@ public:
 	virtual mreal GetRatio() const;
 	virtual int GetWidth() const;
 	virtual int GetHeight() const;
-	
+
 	/// Set to use or not text rotation
 	inline void SetRotatedText(bool val)	{	set(val,MGL_ENABLE_RTEXT);	}
 	/// Set default font style and color
@@ -373,10 +355,13 @@ public:
 	/// Set plot quality
 	virtual void SetQuality(int qual=MGL_DRAW_NORM)	{	Quality=qual;	}
 	inline int GetQuality() const	{	return Quality;	}
+	inline void SetDrawReg(long nx=1, long ny=1, long m=0)	{	dr_x=nx;	dr_y=ny;	dr_p=m;	}
 
 	// ~~~~~~~~~~~~~~~~~~~~~~ Developer functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 	/// Add point to the Pnt and return its position
-	long AddPnt(mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
+	inline long AddPnt(mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1)
+	{	return AddPnt(&B,p,c,n,a,scl);	}
+	long AddPnt(const mglMatrix *M, mglPoint p, mreal c=-1, mglPoint n=mglPoint(NAN), mreal a=-1, int scl=1);
 	long CopyNtoC(long k, mreal c);
 	long CopyProj(long from, mglPoint p, mglPoint n);
 	virtual void Reserve(long n);		///< Allocate n-cells for Pnt and return current position
@@ -388,7 +373,10 @@ public:
 	void AddActive(long k,int n=0);
 	/// Clear unused points and primitives
 	void ClearUnused();
-	
+
+	inline mreal GetPenWidth()	{	return PenWidth;	}
+
+	inline const mglMatrix *GetB()	const	{	return &B;	}
 	inline mglPoint GetPntP(long i) const
 	{	const mglPnt &p=Pnt[i];	return mglPoint(p.x,p.y,p.z);	}
 	inline mglPoint GetPntN(long i) const
@@ -407,11 +395,11 @@ public:
 	inline const mglTexture &GetTxt(long i) const	{	return Txt[i];	}
 	inline long GetTxtNum() const		{	return Txt.size();	}
 	/// Scale coordinates and cut off some points
-	virtual bool ScalePoint(mglPoint &p, mglPoint &n, bool use_nan=true) const;
+	virtual bool ScalePoint(const mglMatrix *M, mglPoint &p, mglPoint &n, bool use_nan=true) const;
 
-	virtual mreal GetOrgX(char dir) const=0;	///< Get Org.x (parse NAN value)
-	virtual mreal GetOrgY(char dir) const=0;	///< Get Org.y (parse NAN value)
-	virtual mreal GetOrgZ(char dir) const=0;	///< Get Org.z (parse NAN value)
+	virtual mreal GetOrgX(char dir, bool inv=false) const=0;	///< Get Org.x (parse NAN value)
+	virtual mreal GetOrgY(char dir, bool inv=false) const=0;	///< Get Org.y (parse NAN value)
+	virtual mreal GetOrgZ(char dir, bool inv=false) const=0;	///< Get Org.z (parse NAN value)
 
 	/// Get color depending on single variable z, which should be scaled if scale=true
 	inline mreal GetC(long s,mreal z,bool scale = true) const
@@ -419,14 +407,17 @@ public:
 	/// Get alpha value depending on single variable a
 	mreal GetA(mreal a) const;
 	/// Set pen/palette
-	char SetPenPal(const char *stl, long *id=0);
+	char SetPenPal(const char *stl, long *id=0, bool pal=true);
 	/// Add texture (like color scheme) and return the position of first color
 	long AddTexture(const char *cols, int smooth=0);
 //	inline mreal AddTexture(char col)	{	return AddTexture(mglColor(col));	}
 	mreal AddTexture(mglColor col);
 	inline void DefColor(mglColor col)	{	CDef = AddTexture(col);	}
+	/// Set mask for face coloring
+	void SetMask(const char *mask);
 	/// Set next color from palette
 	mreal NextColor(long &id);
+	mreal NextColor(long id, long sh);
 
 	virtual void mark_plot(long p, char type, mreal size=1)=0;
 	virtual void arrow_plot(long p1, long p2, char st)=0;
@@ -456,7 +447,7 @@ protected:
 	std::vector<mglPrim> Sub;	///< InPlot regions {n1=x1,n2=x2,n3=y1,n4=y2,id}
 	std::vector<mglText> Ptx;	///< Text labels for mglPrim
 	std::vector<mglText> Leg;	///< Text labels for legend
-	std::vector<mglGlyph> Glf;	///< Glyphs data 
+	std::vector<mglGlyph> Glf;	///< Glyphs data
 	std::vector<mglTexture> Txt;	///< Pointer to textures
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_t mutexPnt, mutexTxt, mutexLeg, mutexGlf, mutexAct, mutexDrw;
@@ -469,6 +460,11 @@ protected:
 	mreal PenWidth;		///< Pen width for further line plotting (must be >0 !!!)
 //	long numT;			///< Number of textures
 	mreal AmbBr;		///< Default ambient light brightness
+	mreal DifBr;		///< Default diffusive light brightness
+
+	mglMatrix Bp;		///< Transformation matrix for View() and Zoom()
+	mglMatrix B;		///< Transformation matrix
+	mglMatrix B1;		///< Transformation matrix for colorbar
 
 	mglFont *fnt;		///< Class for printing vector text
 	mreal FontSize;		///< The size of font for tick and axis labels
@@ -486,8 +482,10 @@ protected:
 	mreal ArrowSize;	///< The size of arrows.
 	char last_style[64];///< Last pen style
 	mreal font_factor;	///< Font scaling factor
+	
+	long dr_x, dr_y, dr_p;	///< default drawing region for quality&4 mode
 
-	virtual void LightScale()=0;			///< Scale positions of light sources
+	virtual void LightScale(const mglMatrix *M)=0;			///< Scale positions of light sources
 
 	// block for SaveState()
 	mglPoint MinS;		///< Saved lower edge of bounding box for graphics.
@@ -498,14 +496,22 @@ protected:
 	bool saved;			///< State is saved
 	std::string leg_str;///< text to be save in legend
 
+	union
+	{
+		uint64_t mask;	///< Mask to be used for coloring
+		unsigned char mask_ch[8];
+	};
+	int MaskAn;		///< Mask rotation angle in degrees
+	int DefMaskAn;	///< Default mask rotation angle in degrees
+
 private:
 
 	mglPoint CutMin;	///< Lower edge of bounding box for cut off.
 	mglPoint CutMax;	///< Upper edge of bounding box for cut off.
 
-	void RecalcCRange();	///< Recalculate internal parameter for correct coloring.
+	bool RecalcCRange();	///< Recalculate internal parameter for correct coloring.
 	void RecalcBorder();	///< Recalculate internal parameter for correct transformation rules.
-	void SetFBord(mreal x,mreal y,mreal z);	///< Set internal boundng box depending on transformation formula
+	bool SetFBord(mreal x,mreal y,mreal z);	///< Set internal boundng box depending on transformation formula
 	void ClearEq();			///< Clear the used variables for axis transformation
 };
 //-----------------------------------------------------------------------------
@@ -517,20 +523,5 @@ bool MGL_EXPORT mgl_check_vec3(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT ax, HCDT ay
 bool MGL_EXPORT mgl_check_trig(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCDT a, const char *name, int d=3);
 bool MGL_EXPORT mgl_isboth(HCDT x, HCDT y, HCDT z, HCDT a);
 //-----------------------------------------------------------------------------
-#define _Da_(d)	(*((const mglDataA *)(d)))
-#define _DA_(a)	((const mglDataA *)*(a))
-#define _GR_	((mglBase *)(*gr))
-//-----------------------------------------------------------------------------
-//#define _D_(d)	*((mglData *)*(d))
-#define _DM_(a)	((mglData *)*(a))
-#define _DT_	((mglData *)*d)
-//-----------------------------------------------------------------------------
-#else
-typedef void *HMGL;
-typedef void *HMDT;
-typedef void *HMEX;
-typedef void *HAEX;
-typedef void *HMPR;
-typedef const void *HCDT;
 #endif
 #endif
