@@ -1,6 +1,6 @@
 /***************************************************************************
  * canvas.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -23,26 +23,15 @@
 //-----------------------------------------------------------------------------
 struct GifFileType;
 //-----------------------------------------------------------------------------
-/// Structure for transformation matrix
-struct mglMatrix
-{
-	mreal b[9];
-	mreal x,y,z,pf;
-	mglMatrix()	{	clear();	}
-	inline void clear()	{	x=y=z=0;	memset(b,0,9*sizeof(mreal));	b[0]=b[4]=b[8]=1;	}
-	inline mglMatrix &operator=(const mglMatrix &a)
-	{	x=a.x;	y=a.y;	z=a.z;	pf=a.pf;	memcpy(b,a.b,9*sizeof(mreal));	return *this;	}
-};
-//-----------------------------------------------------------------------------
 /// Structure for drawing axis and ticks
-struct mglAxis
+struct MGL_EXPORT mglAxis
 {
-	mglAxis()	{	dv=ds=d=v0=v1=v2=o=sh=0;	ns=f=ch=*t=0;	pos = 't';	}
+	mglAxis()	{	dv=ds=d=v0=v1=v2=o=sh=0;	ns=f=ch=0;	pos = 't';	inv=false;	}
 	mglAxis(const mglAxis &aa)
 	{	dv=aa.dv;	ds=aa.ds;	d=aa.d;		dir=aa.dir;	sh=aa.sh;
 		v0=aa.v0;	v1=aa.v1;	v2=aa.v2;	o=aa.o;		pos=aa.pos;
-		a = aa.a;	b = aa.b;	org=aa.org;	txt=aa.txt;
-		ns=aa.ns;	f=aa.f;		ch=aa.ch;	wcscpy(t,aa.t);	}
+		a = aa.a;	b = aa.b;	org=aa.org;	txt=aa.txt;	inv=aa.inv;
+		ns=aa.ns;	f=aa.f;		ch=aa.ch;	t=aa.t;	}
 	inline void AddLabel(const wchar_t *lbl, mreal v)
 	{	txt.push_back(mglText(lbl,"",v));	}
 	inline void AddLabel(const std::wstring &lbl, mreal v)
@@ -51,7 +40,7 @@ struct mglAxis
 	mreal dv,ds;		///< Actual step for ticks and subticks.
 	mreal d;			///< Step for axis ticks (if positive) or its number (if negative).
 	int ns;			///< Number of axis subticks.
-	wchar_t t[256];	///< Tick template (set "" to use default one ("%.2g" in simplest case))
+	std::wstring t;	///< Tick template (set "" to use default one ("%.2g" in simplest case))
 	mglPoint dir;	///< Axis direction
 	mglPoint a,b;	///< Directions of over axis
 	mglPoint org;
@@ -61,13 +50,14 @@ struct mglAxis
 	mreal o;			///< Point of starting ticks numbering (if NAN then Org is used).
 	int f;			///< Flag 0x1 - time, 0x2 - manual, 0x4 - fixed dv
 	std::vector<mglText> txt;	///< Axis labels
-	char ch;			///< Character of axis (like 'x','y','z','c')
+	char ch;		///< Character of axis (like 'x','y','z','c')
 	char pos;		///< Text position ('t' by default, or 'T' for opposite)
 	mreal sh;		///< Extra shift of ticks and axis labels
+	bool inv;		///< Inverse automatic origin position
 };
 //-----------------------------------------------------------------------------
 /// Structure for light source
-struct mglLight
+struct MGL_EXPORT mglLight
 {
 	mglLight()	{	n=false;	a=b=0;	}
 	bool n;			///< Availability of light sources
@@ -82,9 +72,14 @@ struct mglLight
 //-----------------------------------------------------------------------------
 class mglCanvas;
 /// Structure for light source
-struct mglDrawReg
+struct MGL_EXPORT mglDrawReg
 {
-	unsigned long PDef;
+	union
+	{
+		uint64_t PDef;
+		unsigned char m[8];
+	};
+	int angle;	///< mask rotation values in degrees
 	int ObjId;
 	mreal PenWidth, pPos;
 	int x1,x2,y1,y2;
@@ -92,7 +87,7 @@ struct mglDrawReg
 };
 //-----------------------------------------------------------------------------
 /// Structure contains everything for drawing
-struct mglDrawDat
+struct MGL_EXPORT mglDrawDat
 {
 	std::vector<mglPnt>  Pnt;	///< Internal points
 	std::vector<mglPrim> Prm;	///< Primitives (lines, triangles and so on) -- need for export
@@ -101,10 +96,13 @@ struct mglDrawDat
 	std::vector<mglTexture> Txt;	///< Pointer to textures
 };
 //-----------------------------------------------------------------------------
+union mglRGBA	{	uint32_t c; unsigned char r[4];	};
+//-----------------------------------------------------------------------------
 /// Class contains all functionality for creating different mathematical plots
 class MGL_EXPORT mglCanvas : public mglBase
 {
 friend struct mglPrim;
+friend struct mglDrawReg;
 public:
 using mglBase::Light;
 
@@ -123,8 +121,9 @@ using mglBase::Light;
 
 	/// Clear transformation matrix.
 	inline void Identity(bool rel=false)	{	InPlot(0,1,0,1,rel);	}
+	inline void Identity(mglMatrix &M, bool rel=false)	{	InPlot(M,0,1,0,1,rel);	}
 	/// Push transformation matrix into stack
-	inline void Push()	{	MGL_PUSH(stack,B,mutexStk);	}
+	void Push();
 	/// Set PlotFactor
 	inline void SetPlotFactor(mreal val)
 	{	if(val<=0)	{B.pf=1.55;	set(MGL_AUTO_FACTOR);}	else {B.pf=val;	clr(MGL_AUTO_FACTOR);}	}
@@ -133,13 +132,15 @@ using mglBase::Light;
 	/// Pop transformation matrix from stack
 	inline void Pop()	{	B = stack.back(); stack.pop_back();	}
 	/// Clear up the frame
-	virtual void Clf(mglColor back=WC);
+	virtual void Clf(mglColor back=NC);
 
 	/// Put further plotting in cell of stick rotated on angles tet, phi
 	void StickPlot(int num, int i, mreal tet, mreal phi);
 	/// Put further plotting in some region of whole frame surface.
-	void InPlot(mreal x1,mreal x2,mreal y1,mreal y2,bool rel=true);
+	inline void InPlot(mreal x1,mreal x2,mreal y1,mreal y2,bool rel=true)
+	{	InPlot(B,x1,x2,y1,y2,rel);	}
 	void InPlot(mreal x1,mreal x2,mreal y1,mreal y2, const char *style);
+	void InPlot(mglMatrix &M,mreal x1,mreal x2,mreal y1,mreal y2,bool rel=true);
 	/// Add title for current subplot/inplot
 	void Title(const char *title,const char *stl="#",mreal size=-2);
 	void Title(const wchar_t *title,const char *stl="#",mreal size=-2);
@@ -148,7 +149,7 @@ using mglBase::Light;
 	/// Rotate a further plotting.
 	void Rotate(mreal TetX,mreal TetZ,mreal TetY=0);
 	/// Rotate a further plotting around vector {x,y,z}.
-	void RotateN(mreal Tet,mreal x,mreal y,mreal z);
+	void RotateN(mreal Tet,mreal x,mreal y,mreal z)	;
 	/// Set perspective (in range [0,1)) for plot. Set to zero for switching off.
 	void Perspective(mreal a)	{	Bp.pf = fabs(a);	}
 
@@ -185,7 +186,7 @@ using mglBase::Light;
 	int GetSplId(long xs,long ys) const;
 	/// Check if there is active point or primitive (n=-1)
 	int IsActive(int xs, int ys,int &n);
-	
+
 	/// Create new frame.
 	virtual int NewFrame();
 	/// Finish frame drawing
@@ -202,19 +203,20 @@ using mglBase::Light;
 	virtual void SetFrame(long i);
 	/// Add drawing data from i-th frame to the current drawing
 	void ShowFrame(long i);
-	
+
 	/// Start write frames to cinema using GIF format
 	void StartGIF(const char *fname, int ms=100);
 	/// Stop writing cinema using GIF format
 	void CloseGIF();
 	/// Finish plotting. Normally this function is called internaly.
-	virtual void Finish(bool fast=true);
+	virtual void Finish();
 	/// Export points and primitives in file using MGLD format
 	bool ExportMGLD(const char *fname, const char *descr=0);
 	/// Import points and primitives from file using MGLD format
 	bool ImportMGLD(const char *fname, bool add=false);
 	/// Export in JSON format suitable for later drawing by JavaScript
-	bool WriteJSON(const char *fname);
+	bool WriteJSON(const char *fname, bool force_zlib=false);
+	std::string GetJSON();
 
 	/// Set the transparency type
 	inline void SetTranspType(int val)
@@ -288,8 +290,6 @@ using mglBase::Light;
 
 	void StartAutoGroup (const char *);
 	void EndGroup();
-	/// Retur color for primitive depending lighting
-	mglColor GetColor(const mglPrim &p);
 	/// Set extra shift for tick and axis labels
 	inline void SetTickShift(mglPoint p)
 	{	ax.sh = p.x;	ay.sh = p.y;	az.sh = p.z;	ac.sh = p.c;	}
@@ -302,7 +302,12 @@ using mglBase::Light;
 	int *OI;			///< ObjId arrays (size width*height)
 	/// Plot point p with color c
 	void pnt_plot(long x,long y,mreal z,const unsigned char c[4], int obj_id);
-	
+	void pnt_fast(long x,long y,mreal z,const unsigned char c[4], int obj_id);
+	/// preparing primitives for 2d export or bitmap drawing (0 default, 1 for 2d vector, 2 for 3d vector)
+	void PreparePrim(int fast);
+	inline uint32_t GetPntCol(long i)	{	return pnt_col[i];	}
+	inline uint32_t GetPrmCol(long i)	{	return prm_col[i];	}
+
 protected:
 	mreal Delay;		///< Delay for animation in seconds
 	// NOTE: Z should be float for reducing space and for compatibility reasons
@@ -326,9 +331,6 @@ protected:
 	int Width;			///< Width of the image
 	int Height;			///< Height of the image
 	int Depth;			///< Depth of the image
-	mglMatrix Bp;		///< Transformation matrix for View() and Zoom()
-	mglMatrix B;		///< Transformation matrix
-	mglMatrix B1;		///< Transformation matrix for colorbar
 	mreal inW, inH;		///< Width and height of last InPlot
 	mreal inX, inY;		///< Coordinates of last InPlot
 	mglLight light[10];	///< Light sources
@@ -352,14 +354,16 @@ protected:
 	/// Clear ZBuffer only
 	void ClfZB(bool force=false);
 	/// Scale coordinates and cut off some points
-	bool ScalePoint(mglPoint &p, mglPoint &n, bool use_nan=true) const;
-	void LightScale();	///< Additionally scale positions of light sources
+	bool ScalePoint(const mglMatrix *M, mglPoint &p, mglPoint &n, bool use_nan=true) const;
+	void LightScale(const mglMatrix *M);	///< Additionally scale positions of light sources
 	/// Push drawing data (for frames only). NOTE: can be VERY large
 	long PushDrwDat();
-	
-	mreal GetOrgX(char dir) const;	///< Get Org.x (parse NAN value)
-	mreal GetOrgY(char dir) const;	///< Get Org.y (parse NAN value)
-	mreal GetOrgZ(char dir) const;	///< Get Org.z (parse NAN value)
+	/// Retur color for primitive depending lighting
+	uint32_t GetColor(const mglPrim &p);
+
+	mreal GetOrgX(char dir, bool inv=false) const;	///< Get Org.x (parse NAN value)
+	mreal GetOrgY(char dir, bool inv=false) const;	///< Get Org.y (parse NAN value)
+	mreal GetOrgZ(char dir, bool inv=false) const;	///< Get Org.z (parse NAN value)
 
 	void mark_plot(long p, char type, mreal size=1);
 	void arrow_plot(long p1, long p2, char st);
@@ -370,12 +374,12 @@ protected:
 	mreal text_plot(long p,const wchar_t *text,const char *fnt,mreal size=-1,mreal sh=0,mreal  col=-('k'), bool rot=true);
 
 	void add_prim(mglPrim &a);	///< add primitive to list
-	void mark_draw(const mglPnt &p, char type, mreal size, mglDrawReg *d);
-	void arrow_draw(const mglPnt &p1, const mglPnt &p2, char st, mreal size, mglDrawReg *d);
-	virtual void line_draw(const mglPnt &p1, const mglPnt &p2, mglDrawReg *d);
-	virtual void trig_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, bool anorm, mglDrawReg *d);
-	virtual void quad_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4, mglDrawReg *d);
-	virtual void pnt_draw(const mglPnt &p, mglDrawReg *d);
+	void arrow_draw(const mglPnt &p1, const mglPnt &p2, char st, mreal size, const mglDrawReg *d);
+	virtual void mark_draw(const mglPnt &p, char type, mreal size, mglDrawReg *d);
+	virtual void line_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *d);
+	virtual void trig_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, bool anorm, const mglDrawReg *d);
+	virtual void quad_draw(const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4, const mglDrawReg *d);
+	virtual void pnt_draw(const mglPnt &p, const mglDrawReg *d);
 	void arrow_draw(long n1, long n2, char st, float ll);
 	void arrow_plot_3d(long n1, long n2, char st, float ll);
 	void glyph_draw(const mglPrim &P, mglDrawReg *d);
@@ -385,66 +389,83 @@ protected:
 	mglPoint RestorePnt(mglPoint ps, bool norm=false) const;
 
 	// functions for multi-threading
-	void PreparePrim(bool fast);
-	void pxl_combine(size_t id, size_t n, const void *);
-	void pxl_memcpy(size_t id, size_t n, const void *);
-	void pxl_backgr(size_t id, size_t n, const void *);
-	void pxl_primdr(size_t id, size_t n, const void *);
-	void pxl_transform(size_t id, size_t n, const void *);
-	void pxl_setz(size_t id, size_t n, const void *);
-	void pxl_setz_adv(size_t id, size_t n, const void *);
-	void pxl_other(size_t id, size_t n, const void *p);
+	void pxl_pntcol(long id, long n, const void *);
+	void pxl_prmcol(long id, long n, const void *);
+	void pxl_combine(long id, long n, const void *);
+	void pxl_memcpy(long id, long n, const void *);
+	void pxl_backgr(long id, long n, const void *);
+	void pxl_primdr(long id, long n, const void *);
+	void pxl_dotsdr(long id, long n, const void *);
+	void pxl_primpx(long id, long n, const void *);
+	void pxl_transform(long id, long n, const void *);
+	void pxl_setz(long id, long n, const void *);
+	void pxl_setz_adv(long id, long n, const void *);
+	void pxl_other(long id, long n, const void *p);
 	/// Put drawing from other mglCanvas (for multithreading, like subplots)
 	void PutDrawReg(mglDrawReg *d, const mglCanvas *gr);
-	
+
 private:
+	std::vector<uint32_t> pnt_col, prm_col;
 //	mreal _tetx,_tety,_tetz;		// extra angles
 	std::vector<mglMatrix> stack;	///< stack for transformation matrices
-	int dr_nx1, dr_nx2, dr_ny1, dr_ny2;	// Allowed drawing region
 	GifFileType *gif;
 	mreal fscl,ftet;	///< last scale and rotation for glyphs
 	long forg;		///< original point (for directions)
 	size_t grp_counter;	///< Counter for StartGroup(); EndGroup();
+	mglMatrix Bt;	///< temporary matrix for text
 
 	/// Draw generic colorbar
-	void colorbar(HCDT v, const mreal *s, int where, mreal x, mreal y, mreal w, mreal h);
+	void colorbar(HCDT v, const mreal *s, int where, mreal x, mreal y, mreal w, mreal h, bool text);
 	/// Draw labels for ticks
-	void DrawLabels(mglAxis &aa);
+	void DrawLabels(mglAxis &aa, bool inv=false, const mglMatrix *M=0);
 	/// Get label style
 	char GetLabelPos(mreal c, long kk, mglAxis &aa);
 	/// Draw tick
-	void tick_draw(mglPoint o, mglPoint d1, mglPoint d2, int f, const char *stl);
+	void tick_draw(mglPoint o, mglPoint d1, mglPoint d2, int f);
 	mreal FindOptOrg(char dir, int ind) const;
 	/// Transform mreal color and alpha to bits format
 	unsigned char* col2int(const mglPnt &p, unsigned char *r, int obj_id);
 	/// Combine colors in 2 plane.
 	void combine(unsigned char *c1, const unsigned char *c2);
 	/// Fast drawing of line between 2 points
-	void fast_draw(const mglPnt &p1, const mglPnt &p2, mglDrawReg *d);
+	void fast_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *d);
 
 	/// Additionally scale points p for positioning in image
-	void PostScale(mglPoint &p) const;
+	void PostScale(const mglMatrix *M, mglPoint &p) const;
 	/// Scale points p for projection to the face number nface in image
 	long ProjScale(int nface, long p, bool text=false);
-	inline void PostScale(mglPoint *p,long n) const	{	for(long i=0;i<n;i++)	PostScale(p[i]);	}
 	/// Set coordinate and add the point, return its id
 	long setPp(mglPnt &q, const mglPoint &p);
-	
+
 	// functions for glyph drawing
-	void glyph_fill(const mglPnt &p, mreal f, const mglGlyph &g, mglDrawReg *d);
-	void glyph_wire(const mglPnt &p, mreal f, const mglGlyph &g, mglDrawReg *d);
-	void glyph_line(const mglPnt &p, mreal f, bool solid, mglDrawReg *d);
+	void glyph_fill(const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
+	void glyph_wire(const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
+	void glyph_line(const mglMatrix *M, const mglPnt &p, mreal f, bool solid, const mglDrawReg *d);
+
+	// fill pixel for given primitive
+	void mark_pix(long i,long j,const mglPnt &p, char type, mreal size, mglDrawReg *d);
+	void arrow_pix(long i,long j,const mglPnt &p1, const mglPnt &p2, char st, mreal size, const mglDrawReg *d);
+	void line_pix(long i,long j,const mglPnt &p1, const mglPnt &p2, const mglDrawReg *d);
+	void trig_pix(long i,long j,const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, bool anorm, const mglDrawReg *d);
+	void quad_pix(long i,long j,const mglPnt &p1, const mglPnt &p2, const mglPnt &p3, const mglPnt &p4, const mglDrawReg *d);
+	void glyph_pix(long i,long j,const mglPrim &P, mglDrawReg *d);
+	void pnt_pix(long i,long j,const mglPnt &p, const mglDrawReg *d);
+	void glyph_fpix(long i,long j,const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
+	void glyph_wpix(long i,long j,const mglMatrix *M, const mglPnt &p, mreal f, const mglGlyph &g, const mglDrawReg *d);
+	void glyph_lpix(long i,long j,const mglMatrix *M, const mglPnt &p, mreal f, bool solid, const mglDrawReg *d);
+
+
 };
 //-----------------------------------------------------------------------------
 struct mglThreadG
 {
 	mglCanvas *gr;		// grapher
-	void (mglCanvas::*f)(size_t i, size_t n, const void *);
+	void (mglCanvas::*f)(long i, long n, const void *);
 	unsigned id;		// thread id
-	size_t n;	// total number of iteration
+	long n;	// total number of iteration
 	const void *p;		// external parameter
 };
 /// Start several thread for the task
-void mglStartThread(void (mglCanvas::*func)(size_t i, size_t n), mglCanvas *gr, size_t n);
+void mglStartThread(void (mglCanvas::*func)(long i, long n), mglCanvas *gr, long n);
 //-----------------------------------------------------------------------------
 #endif
