@@ -1,6 +1,6 @@
 /***************************************************************************
  * canvas_cf.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -19,7 +19,7 @@
  ***************************************************************************/
 #ifndef MGL_CANVAS_CF_H
 #define MGL_CANVAS_CF_H
-#include "mgl2/base.h"
+#include "mgl2/abstract.h"
 //-----------------------------------------------------------------------------
 #ifdef __cplusplus
 extern "C" {
@@ -38,8 +38,8 @@ void MGL_EXPORT mgl_set_size_(uintptr_t *gr, int *width, int *height);
 void MGL_EXPORT mgl_set_def_param(HMGL gr);
 void MGL_EXPORT mgl_set_def_param_(uintptr_t *gr);
 /// Combine plots from 2 canvases. Result will be saved into gr
-void MGL_EXPORT mgl_combine_gr(HMGL gr, HMGL in);
-void MGL_EXPORT mgl_combine_gr_(uintptr_t *gr, uintptr_t *in);
+void MGL_EXPORT mgl_combine_gr(HMGL gr, HMGL gr2);
+void MGL_EXPORT mgl_combine_gr_(uintptr_t *gr, uintptr_t *gr2);
 /// Force preparing the image. It can be useful for OpenGL mode mostly.
 void MGL_EXPORT mgl_finish(HMGL gr);
 void MGL_EXPORT mgl_finish_(uintptr_t *gr);
@@ -93,7 +93,7 @@ void MGL_EXPORT mgl_axis_grid(HMGL gr, const char *dir,const char *pen, const ch
 void MGL_EXPORT mgl_axis_grid_(uintptr_t *gr, const char *dir,const char *pen, const char *opt,int,int,int);
 /// Print the label text for axis dir.
 void MGL_EXPORT mgl_label(HMGL gr, char dir, const char *text, double pos, const char *opt);
-void MGL_EXPORT mgl_label_(uintptr_t *gr, const char *dir, const char *text, const char *opt,int,int,int);
+void MGL_EXPORT mgl_label_(uintptr_t *gr, const char *dir, const char *text, mreal *pos, const char *opt,int,int,int);
 void MGL_EXPORT mgl_labelw(HMGL gr, char dir, const wchar_t *text, double pos, const char *opt);
 
 /// Draw colorbar at edge of axis
@@ -203,6 +203,9 @@ void MGL_EXPORT mgl_import_mgld_(uintptr_t *gr, const char *fname, int *add, int
 /// Export in JSON format suitable for later drawing by JavaScript
 void MGL_EXPORT mgl_write_json(HMGL gr, const char *fname,const char *descr);
 void MGL_EXPORT mgl_write_json_(uintptr_t *gr, const char *fname,const char *descr,int,int);
+void MGL_EXPORT mgl_write_json_z(HMGL gr, const char *fname,const char *descr);
+void MGL_EXPORT mgl_write_json_z_(uintptr_t *gr, const char *fname,const char *descr,int,int);
+MGL_EXPORT const char *mgl_get_json(HMGL gr);
 
 /// Get RGB values of current bitmap
 MGL_EXPORT const unsigned char *mgl_get_rgb(HMGL gr);
@@ -261,8 +264,8 @@ void MGL_EXPORT mgl_del_frame(HMGL gr, int i);
 void MGL_EXPORT mgl_del_frame_(uintptr_t *gr, int *i);
 
 /// Set the transparency type (0 - usual, 1 - glass, 2 - lamp)
-void MGL_EXPORT mgl_set_transp_type(HMGL gr, int type);
-void MGL_EXPORT mgl_set_transp_type_(uintptr_t *gr, int *type);
+void MGL_EXPORT mgl_set_transp_type(HMGL gr, int kind);
+void MGL_EXPORT mgl_set_transp_type_(uintptr_t *gr, int *kind);
 /// Set the transparency on/off.
 void MGL_EXPORT mgl_set_alpha(HMGL gr, int enable);
 void MGL_EXPORT mgl_set_alpha_(uintptr_t *gr, int *enable);
@@ -299,6 +302,9 @@ void MGL_EXPORT mgl_clf_(uintptr_t *gr);
 /// Clear up the frame and fill background by specified color
 void MGL_EXPORT mgl_clf_rgb(HMGL gr, double r, double g, double b);
 void MGL_EXPORT mgl_clf_rgb_(uintptr_t *gr, mreal *r, mreal *g, mreal *b);
+/// Clear up the frame and fill background by specified color
+void MGL_EXPORT mgl_clf_chr(HMGL gr, char col);
+void MGL_EXPORT mgl_clf_chr_(uintptr_t *gr, const char *col, int);
 
 /// Put further plotting in some region of whole frame.
 void MGL_EXPORT mgl_subplot(HMGL gr, int nx,int ny,int m,const char *style);
@@ -320,7 +326,7 @@ void MGL_EXPORT mgl_columnplot(HMGL gr, int num, int ind, double d);
 void MGL_EXPORT mgl_columnplot_(uintptr_t *gr, int *num, int *i, mreal *d);
 /// Put further plotting in matrix cell of previous subplot/inplot.
 void MGL_EXPORT mgl_gridplot(HMGL gr, int nx, int ny, int m, double d);
-void MGL_EXPORT mgl_gridplot_(uintptr_t *gr, int *nx, int *ny, int *i, mreal *d);
+void MGL_EXPORT mgl_gridplot_(uintptr_t *gr, int *nx, int *ny, int *m, mreal *d);
 /// Put further plotting in cell of stick rotated on angles tet, phi.
 void MGL_EXPORT mgl_stickplot(HMGL gr, int num, int ind, double tet, double phi);
 void MGL_EXPORT mgl_stickplot_(uintptr_t *gr, int *num, int *i, mreal *tet, mreal *phi);
@@ -479,31 +485,8 @@ HMDT MGL_EXPORT mgl_parser_calc(HMPR pr, const char *formula);
 uintptr_t MGL_EXPORT mgl_parser_calc_(uintptr_t *pr, const char *formula,int);
 HMDT MGL_EXPORT mgl_parser_calcw(HMPR pr, const wchar_t *formula);
 
-//-----------------------------------------------------------------------------
-/// Create HMEX object for expression evaluating
-HMEX MGL_EXPORT mgl_create_expr(const char *expr);
-uintptr_t MGL_EXPORT mgl_create_expr_(const char *expr, int);
-HAEX MGL_EXPORT mgl_create_cexpr(const char *expr);
-/// Delete HMEX object
-void MGL_EXPORT mgl_delete_expr(HMEX ex);
-void MGL_EXPORT mgl_delete_expr_(uintptr_t *ex);
-void MGL_EXPORT mgl_delete_cexpr(HAEX ex);
-/// Return value of expression for given x,y,z variables
-double MGL_EXPORT mgl_expr_eval(HMEX ex, double x, double y,double z);
-double MGL_EXPORT mgl_eval_expr_(uintptr_t *ex, mreal *x, mreal *y, mreal *z);
-dual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z);
-/// Return value of expression for given variables
-double MGL_EXPORT mgl_expr_eval_v(HMEX ex, mreal *var);
-dual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *var);
-/// Return value of expression differentiation over variable dir for given x,y,z variables
-double MGL_EXPORT mgl_expr_diff(HMEX ex, char dir, double x, double y,double z);
-double MGL_EXPORT mgl_diff_expr_(uintptr_t *ex, const char *dir, mreal *x, mreal *y, mreal *z, int);
-/// Return value of expression differentiation over variable dir for given variables
-double MGL_EXPORT mgl_expr_diff_v(HMEX ex, char dir, mreal *var);
-
 #ifdef __cplusplus
 }
 #endif
 //-----------------------------------------------------------------------------
 #endif
-
