@@ -1,6 +1,6 @@
 /***************************************************************************
  * data.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -21,6 +21,7 @@
 #define _MGL_DATA_H_
 
 #include "mgl2/data_cf.h"
+#include "mgl2/pde.h"
 #ifdef __cplusplus
 //-----------------------------------------------------------------------------
 #include <vector>
@@ -28,9 +29,9 @@
 //-----------------------------------------------------------------------------
 /// Class for working with data array
 #if MGL_NO_DATA_A
-class mglData
+class MGL_EXPORT mglData
 #else
-class mglData : public mglDataA
+class MGL_EXPORT mglData : public mglDataA
 #endif
 {
 public:
@@ -88,16 +89,16 @@ public:
 	inline void Set(const double *A,long NX,long NY=1,long NZ=1)
 	{	mgl_data_set_double(this,A,NX,NY,NZ);	}
 	/// Allocate memory and copy the data from the (float **) array
-	inline void Set(const float **A,long N1,long N2)
+	inline void Set(float const * const *A,long N1,long N2)
 	{	mgl_data_set_float2(this,A,N1,N2);	}
 	/// Allocate memory and copy the data from the (double **) array
-	inline void Set(const double **A,long N1,long N2)
+	inline void Set(double const * const *A,long N1,long N2)
 	{	mgl_data_set_double2(this,A,N1,N2);	}
 	/// Allocate memory and copy the data from the (float ***) array
-	inline void Set(const float ***A,long N1,long N2,long N3)
+	inline void Set(float const * const * const *A,long N1,long N2,long N3)
 	{	mgl_data_set_float3(this,A,N1,N2,N3);	}
 	/// Allocate memory and copy the data from the (double ***) array
-	inline void Set(const double ***A,long N1,long N2,long N3)
+	inline void Set(double const * const * const *A,long N1,long N2,long N3)
 	{	mgl_data_set_double3(this,A,N1,N2,N3);	}
 	/// Allocate memory and scanf the data from the string
 	inline void Set(const char *str,long NX,long NY=1,long NZ=1)
@@ -146,7 +147,7 @@ public:
 	/// Join with another data array
 	inline void Join(const mglDataA &d)
 	{	mgl_data_join(this,&d);	}
-	
+
 	/// Modify the data by specified formula
 	inline void Modify(const char *eq,long dim=0)
 	{	mgl_data_modify(this, eq, dim);	}
@@ -166,9 +167,28 @@ public:
 	/// Equidistantly fill the data to range [x1,x2] in direction dir
 	inline void Fill(mreal x1,mreal x2=NaN,char dir='x')
 	{	return mgl_data_fill(this,x1,x2,dir);	}
-	/// Set the data by triangulated surface values assuming x,y,z in range [r1,r2]
+	/// Fill the data by interpolated values of vdat parametrically depended on xdat,ydat,zdat for x,y,z in range [p1,p2]
+	inline void Refill(const mglDataA &xdat, const mglDataA &vdat, mreal x1, mreal x2,long sl=-1)
+	{	mgl_data_refill_x(this,&xdat,&vdat,x1,x2,sl);	}
+	inline void Refill(const mglDataA &xdat, const mglDataA &vdat, mglPoint p1, mglPoint p2,long sl=-1)
+	{	mgl_data_refill_x(this,&xdat,&vdat,p1.x,p2.x,sl);	}
+	inline void Refill(const mglDataA &xdat, const mglDataA &ydat, const mglDataA &vdat, mglPoint p1, mglPoint p2,long sl=-1)
+	{	mgl_data_refill_xy(this,&xdat,&ydat,&vdat,p1.x,p2.x,p1.y,p2.y,sl);	}
+	inline void Refill(const mglDataA &xdat, const mglDataA &ydat, const mglDataA &zdat, const mglDataA &vdat, mglPoint p1, mglPoint p2)
+	{	mgl_data_refill_xyz(this,&xdat,&ydat,&zdat,&vdat,p1.x,p2.x,p1.y,p2.y,p1.z,p2.z);	}
+	/// Fill the data by interpolated values of vdat parametrically depended on xdat,ydat,zdat for x,y,z in axis range of gr
+	inline void Refill(mglBase *gr, const mglDataA &xdat, const mglDataA &vdat, long sl=-1, const char *opt="")
+	{	mgl_data_refill_gr(gr,this,&xdat,0,0,&vdat,sl,opt);	}
+	inline void Refill(mglBase *gr, const mglDataA &xdat, const mglDataA &ydat, const mglDataA &vdat, long sl=-1, const char *opt="")
+	{	mgl_data_refill_gr(gr,this,&xdat,&ydat,0,&vdat,sl,opt);	}
+	inline void Refill(mglBase *gr, const mglDataA &xdat, const mglDataA &ydat, const mglDataA &zdat, const mglDataA &vdat, const char *opt="")
+	{	mgl_data_refill_gr(gr,this,&xdat,&ydat,&zdat,&vdat,-1,opt);	}
+	/// Set the data by triangulated surface values assuming x,y,z in axis range of gr
 	inline void Grid(mglBase *gr, const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *opt="")
 	{	mgl_data_grid(gr,this,&x,&y,&z,opt);	}
+	/// Set the data by triangulated surface values assuming x,y,z in range [p1, p2]
+	inline void Grid(const mglDataA &xdat, const mglDataA &ydat, const mglDataA &vdat, mglPoint p1, mglPoint p2)
+	{	mgl_data_grid_xy(this,&xdat,&ydat,&vdat,p1.x,p2.x,p1.y,p2.y);	}
 	/// Put value to data element(s)
 	inline void Put(mreal val, long i=-1, long j=-1, long k=-1)
 	{	mgl_data_put_val(this,val,i,j,k);	}
@@ -248,7 +268,7 @@ public:
 	inline mglData Combine(const mglDataA &dat) const
 	{	return mglData(true,mgl_data_combine(this,&dat));	}
 	/// Resize the data to new size of box [x1,x2]*[y1,y2]*[z1,z2]
-	inline mglData Resize(long mx,long my=1,long mz=1, mreal x1=0,mreal x2=1, mreal y1=0,mreal y2=1, mreal z1=0,mreal z2=1) const
+	inline mglData Resize(long mx,long my=0,long mz=0, mreal x1=0,mreal x2=1, mreal y1=0,mreal y2=1, mreal z1=0,mreal z2=1) const
 	{	return mglData(true,mgl_data_resize_box(this,mx,my,mz,x1,x2,y1,y2,z1,z2));	}
 	/// Get array which values is result of interpolation this for coordinates from other arrays
 	inline mglData Evaluate(const mglData &idat, bool norm=true) const
@@ -260,7 +280,13 @@ public:
 	/// Find roots for set of nonlinear equations defined by textual formula
 	inline mglData Roots(const char *func, char var='x') const
 	{	return mglData(true,mgl_data_roots(func, this, var));	}
-	
+	/// Find correlation with another data arrays
+	inline mglData Correl(const mglDataA &dat, const char *dir) const
+	{	return mglData(true,mgl_data_correl(this,&dat,dir));	}
+	/// Find auto correlation function
+	inline mglData AutoCorrel(const char *dir) const
+	{	return mglData(true,mgl_data_correl(this,this,dir));	}
+
 	/// Cumulative summation the data in given direction or directions
 	inline void CumSum(const char *dir)	{	mgl_data_cumsum(this,dir);	}
 	/// Integrate (cumulative summation) the data in given direction or directions
@@ -331,7 +357,7 @@ public:
 	{	return mglData(true,mgl_data_solve(this, val, dir, 0, norm));	}
 	inline mglData Solve(mreal val, char dir, const mglData &i0, bool norm=true) const
 	{	return mglData(true,mgl_data_solve(this, val, dir, &i0, norm));	}
-	
+
 	/// Interpolate by cubic spline the data and return its derivatives at given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 	inline mreal Spline(mglPoint &dif, mreal x,mreal y=0,mreal z=0) const
 	{	return mgl_data_spline_ext(this, x,y,z, &(dif.x),&(dif.y), &(dif.z));	}
@@ -346,7 +372,7 @@ public:
 	inline mreal Linear1(mglPoint &dif, mreal x,mreal y=0,mreal z=0) const
 	{	mreal res=mgl_data_linear_ext(this,x*(nx-1),y*(ny-1),z*(nz-1), &(dif.x),&(dif.y), &(dif.z));
 		dif.x*=nx>1?nx-1:1;	dif.y*=ny>1?ny-1:1;	dif.z*=nz>1?nz-1:1;	return res;	}
-	
+
 	/// Get information about the data (sizes and momentum) to string
 	inline const char *PrintInfo() const	{	return mgl_data_info(this);	}
 	/// Print information about the data (sizes and momentum) to FILE (for example, stdout)
@@ -356,6 +382,10 @@ public:
 	inline mreal Maximal() const	{	return mgl_data_max(this);	}
 	/// Get minimal value of the data
 	inline mreal Minimal() const	{	return mgl_data_min(this);	}
+	/// Get maximal value of the data which is less than 0
+	inline mreal MaximalNeg() const	{	return mgl_data_neg_max(this);	}
+	/// Get minimal value of the data which is larger than 0
+	inline mreal MinimalPos() const	{	return mgl_data_pos_min(this);	}
 	/// Get maximal value of the data and its position
 	inline mreal Maximal(long &i,long &j,long &k) const
 	{	return mgl_data_max_int(this,&i,&j,&k);	}
@@ -415,16 +445,14 @@ public:
 #endif
 #if MGL_NO_DATA_A
 	inline long GetNN() const {	return nx*ny*nz;	}
-#else
-protected:
 #endif
 	/// Get the value in given cell of the data without border checking
 	inline mreal v(long i,long j=0,long k=0) const
-#ifdef DEBUG
+#ifndef DEBUG
+	{	return a[i+nx*(j+ny*k)];	}
+#else
 	{	if(i<0 || j<0 || k<0 || i>=nx || j>=ny || k>=nz)	printf("Wrong index in mglData");
 		return a[i+nx*(j+ny*k)];	}
-#else
-	{	return a[i+nx*(j+ny*k)];	}
 #endif
 	inline mreal vthr(long i) const {	return a[i];	}
 	// add for speeding up !!!
@@ -517,5 +545,80 @@ inline mglData mglTriangulation(const mglDataA &x, const mglDataA &y, const mglD
 inline mglData mglTriangulation(const mglDataA &x, const mglDataA &y)
 {	return mglData(true,mgl_triangulation_2d(&x,&y));	}
 //-----------------------------------------------------------------------------
+/// Wrapper class expression evaluating
+class MGL_EXPORT mglExpr
+{
+	HMEX ex;
+public:
+	mglExpr(const char *expr)		{	ex = mgl_create_expr(expr);	}
+	~mglExpr()	{	mgl_delete_expr(ex);	}
+	/// Return value of expression for given x,y,z variables
+	inline double Eval(double x, double y=0, double z=0)
+	{	return mgl_expr_eval(ex,x,y,z);	}
+	/// Return value of expression differentiation over variable dir for given x,y,z variables
+	inline double Diff(char dir, double x, double y=0, double z=0)
+	{	return mgl_expr_diff(ex,dir, x,y,z);	}
+#ifndef SWIG
+	/// Return value of expression for given variables
+	inline double Eval(mreal var[26])
+	{	return mgl_expr_eval_v(ex,var);	}
+	/// Return value of expression differentiation over variable dir for given variables
+	inline double Diff(char dir, mreal var[26])
+	{	return mgl_expr_diff_v(ex,dir, var);	}
+#endif
+};
+//-----------------------------------------------------------------------------
+#ifndef SWIG
+/// Structure for handling named mglData (used by mglParse class).
+class MGL_EXPORT mglVar : public mglData
+{
+public:
+	std::wstring s;	///< Data name
+	void *o; 		///< Pointer to external object
+	mglVar *next;	///< Pointer to next instance in list
+	mglVar *prev;	///< Pointer to previous instance in list
+	bool temp;		///< This is temporary variable
+	void (*func)(void *);	///< Callback function for destroying
+
+	mglVar(std::wstring name=L""):mglData()
+	{	o=0;	next=prev=0;	func=0;	temp=false;	s=name;	}
+	mglVar(mglVar **head, std::wstring name=L""):mglData()
+	{	o=0;	next=*head;	prev=0;	*head=this;	func=0;	temp=false;	s=name;	}
+	mglVar(mglVar **head, const mglData &dat, std::wstring name):mglData(dat)
+	{	o=0;	next=*head;	prev=0;	*head=this;	func=0;	temp=false;	s=name;	}
+	mglVar(mglVar **head, HCDT dat, std::wstring name):mglData(dat)
+	{	o=0;	next=*head;	prev=0;	*head=this;	func=0;	temp=false;	s=name;	}
+	mglVar(mglVar *v, std::wstring name, bool link=true):mglData()	// NOTE: use carefully due to Link()!
+	{	if(!v)	throw mglWarnZero;
+		if(link)	Link(*v);	else	Set(*v);
+		o=0;	temp=false;	s=name;	func = v->func;
+		prev = v;	next = v->next;	v->next = this;
+		if(next)	next->prev = this;	}
+	virtual ~mglVar()
+	{
+		if(func)	func(o);
+		if(prev)	prev->next = next;
+		if(next)	next->prev = prev;
+	}
+	/// Make copy which link on the same data but have different name. NOTE: use carefully due to Link()!
+	inline void Duplicate(std::wstring name)
+	{	mglVar *v=new mglVar(name);	v->Link(*this);	v->MoveAfter(this);	}
+	/// Move variable after var and copy func from var (if func is 0)
+	inline void MoveAfter(mglVar *var)
+	{
+		if(prev)	prev->next = next;
+		if(next)	next->prev = prev;
+		prev = next = 0;
+		if(var)
+		{
+			prev = var;	next = var->next;
+			var->next = this;
+			if(func==0)	func = var->func;
+		}
+		if(next)	next->prev = this;
+	}
+};
+#endif
+//-----------------------------------------------------------------------------
 #endif
 #endif
