@@ -1,6 +1,6 @@
 /***************************************************************************
  * data.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -31,7 +31,7 @@
 #define mgl4 	mreal(4)
 //-----------------------------------------------------------------------------
 /// Class for working with data array
-class mglDataC : public mglDataA
+class MGL_EXPORT mglDataC : public mglDataA
 {
 public:
 
@@ -162,6 +162,13 @@ public:
 	inline void Fill(dual x1,dual x2=NaN,char dir='x')
 	{	return mgl_datac_fill(this,x1,x2,dir);	}
 
+		/// Put value to data element(s)
+	inline void Put(dual val, long i=-1, long j=-1, long k=-1)
+	{	mgl_datac_put_val(this,val,i,j,k);	}
+	/// Put array to data element(s)
+	inline void Put(const mglDataA &dat, long i=-1, long j=-1, long k=-1)
+	{	mgl_datac_put_dat(this,&dat,i,j,k);	}
+
 	/// Set names for columns (slices)
 	inline void SetColumnId(const char *ids)
 	{	mgl_datac_set_id(this,ids);	}
@@ -256,6 +263,13 @@ public:
 	inline mglData Evaluate(const mglData &idat, const mglData &jdat, const mglData &kdat, bool norm=true) const
 	{	return mglData(true,mgl_data_evaluate(this,&idat,&jdat,&kdat,norm));	}
 
+	/// Find correlation with another data arrays
+	inline mglDataC Correl(const mglData &dat, const char *dir) const
+	{	return mglDataC(true,mgl_datac_correl(this,&dat,dir));	}
+	/// Find auto correlation function
+	inline mglDataC AutoCorrel(const char *dir) const
+	{	return mglDataC(true,mgl_datac_correl(this,this,dir));	}
+
 	/// Cumulative summation the data in given direction or directions
 	inline void CumSum(const char *dir)	{	mgl_datac_cumsum(this,dir);	}
 	/// Integrate (cumulative summation) the data in given direction or directions
@@ -279,7 +293,15 @@ public:
 	inline void Hankel(const char *dir)	{	mgl_datac_hankel(this,dir);	}
 	/// Fourier transform
 	inline void FFT(const char *dir)	{	mgl_datac_fft(this,dir);	}
+	/// Calculate one step of diffraction by finite-difference method with parameter q
+	inline void Diffraction(const char *how, mreal q)	{	mgl_datac_diffr(this,how,q);	}
 
+	/// Interpolate by cubic spline the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
+	inline dual Spline(mreal x,mreal y=0,mreal z=0) const
+	{	return mgl_datac_spline(this, x,y,z);	}
+	/// Interpolate by cubic spline the data to given point x,\a y,\a z which normalized in range [0, 1]
+	inline dual Spline1(mreal x,mreal y=0,mreal z=0) const
+	{	return mgl_datac_spline(this, x*(nx-1),y*(ny-1),z*(nz-1));	}
 	/// Interpolate by linear function the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 	inline dual Linear(mreal x,mreal y=0,mreal z=0)	const
 	{	return mgl_datac_linear(this,x,y,z);	}
@@ -345,11 +367,7 @@ public:
 	/// Direct access to the data cell
 	inline dual &operator[](long i)	{	return a[i];	}
 #endif
-#if MGL_NO_DATA_A
-	inline long GetNN() const {	return nx*ny*nz;	}
-#else
-protected:
-#endif
+
 	/// Get the value in given cell of the data without border checking
 	inline mreal v(long i,long j=0,long k=0) const
 #ifdef DEBUG
@@ -371,8 +389,51 @@ protected:
 	return k>0? abs(k<nz-1? (a[i0+n]-a[i0-n])/mgl2:a[i0]-a[i0-n]) : abs(a[i0+n]-a[i0]);	}
 };
 //-----------------------------------------------------------------------------
+#ifndef SWIG
+dual mglLinearC(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z);
+dual mglSpline3C(const dual *a, long nx, long ny, long nz, mreal x, mreal y, mreal z,dual *dx=0, dual *dy=0, dual *dz=0);
+#endif
+//-----------------------------------------------------------------------------
+/// Saves result of PDE solving (|u|^2) for "Hamiltonian" ham with initial conditions ini
+inline mglDataC mglPDEc(mglBase *gr, const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, mreal dz=0.1, mreal k0=100,const char *opt="")
+{	return mglDataC(true, mgl_pde_solve_c(gr,ham, &ini_re, &ini_im, dz, k0,opt));	}
+/// Saves result of PDE solving for "Hamiltonian" ham with initial conditions ini along a curve ray (must have nx>=7 - x,y,z,px,py,pz,tau or nx=5 - x,y,px,py,tau)
+inline mglDataC mglQO2dc(const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, const mglDataA &ray, mreal r=1, mreal k0=100)
+{	return mglDataC(true, mgl_qo2d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, 0, 0));	}
+inline mglDataC mglQO2dc(const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, const mglDataA &ray, mglData &xx, mglData &yy, mreal r=1, mreal k0=100)
+{	return mglDataC(true, mgl_qo2d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, &xx, &yy));	}
+/// Saves result of PDE solving for "Hamiltonian" ham with initial conditions ini along a curve ray (must have nx>=7 - x,y,z,px,py,pz,tau or nx=5 - x,y,px,py,tau)
+inline mglDataC mglQO3dc(const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, const mglDataA &ray, mreal r=1, mreal k0=100)
+{	return mglDataC(true, mgl_qo3d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, 0, 0, 0));	}
+inline mglDataC mglQO3dc(const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, const mglDataA &ray, mglData &xx, mglData &yy, mglData &zz, mreal r=1, mreal k0=100)
+{	return mglDataC(true, mgl_qo3d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, &xx, &yy, &zz));	}
+//-----------------------------------------------------------------------------
 #define _DN_(a)	((mglDataC *)*(a))
 #define _DC_		((mglDataC *)*d)
 //-----------------------------------------------------------------------------
+#ifndef SWIG
+/// Wrapper class expression evaluating
+class MGL_EXPORT mglExprC
+{
+	HAEX ex;
+public:
+	mglExprC(const char *expr)		{	ex = mgl_create_cexpr(expr);	}
+	~mglExprC()	{	mgl_delete_cexpr(ex);	}
+	/// Return value of expression for given x,y,z variables
+	inline dual Eval(dual x, dual y=0, dual z=0)
+	{	return mgl_cexpr_eval(ex,x,y,z);	}
+	/// Return value of expression for given x,y,z,u,v,w variables
+	inline dual Eval(dual x, dual y, dual z, dual u, dual v, dual w)
+	{
+		dual var[26];
+		var['x'-'a']=x;	var['y'-'a']=y;	var['z'-'a']=z;
+		var['u'-'a']=u;	var['v'-'a']=v;	var['w'-'a']=w;
+		return mgl_cexpr_eval_v(ex,var);	}
+	/// Return value of expression for given variables
+	inline dual Eval(dual var[26])
+	{	return mgl_cexpr_eval_v(ex,var);	}
+};
+#endif
+//-----------------------------------------------------------------------------
 #endif
 #endif
