@@ -1,6 +1,6 @@
 /***************************************************************************
  * define.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -21,9 +21,18 @@
 #define _MGL_DEFINE_H_
 //-----------------------------------------------------------------------------
 #include "mgl2/config.h"
-#include "mgl2/dllexport.h"
+#ifndef SWIG
 
-#define MGL_VER2 	1.3
+#include "mgl2/dllexport.h"
+#ifdef MGL_SRC
+#if MGL_HAVE_OMP
+#include <omp.h>
+#endif
+#endif
+
+#endif
+
+#define MGL_VER2 	2.2	// minor version of MathGL 2.* (like 1.3 for v.2.1.3)
 //-----------------------------------------------------------------------------
 #ifdef WIN32 //_MSC_VER needs this before math.h
 #define	_USE_MATH_DEFINES
@@ -46,6 +55,22 @@
 #endif
 #endif
 
+#if (defined(_MSC_VER) && (_MSC_VER<1600)) || defined(__BORLANDC__)
+typedef signed char int8_t;
+typedef signed short int16_t;
+typedef signed long int32_t;
+typedef signed long long int64_t;
+typedef unsigned char uint8_t;
+typedef unsigned short uint16_t;
+typedef unsigned long uint32_t;
+typedef unsigned long long uint64_t;
+#else
+#include <stdint.h>
+#endif
+#if defined(__BORLANDC__)
+typedef unsigned long uintptr_t;
+#endif
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -58,6 +83,8 @@
 #endif
 
 #if defined(_MSC_VER)
+#define collapse(a)	// MSVS don't support OpenMP 3.*
+#define strtoull _strtoui64
 #define hypot	_hypot
 #define getcwd	_getcwd
 #define chdir	_chdir // BORLAND has chdir
@@ -66,17 +93,39 @@
 
 #if defined(_MSC_VER) || defined(__BORLANDC__)
 #include <float.h>
+#include <math.h>
 
-const unsigned long mgl_nan[2] = {0xffffffff, 0x7fffffff};
-#define NANd	(*(double*)mgl_nan)
-#define NANf	(*(float*)&(mgl_nan[1]))
+#ifdef WIN32
+const unsigned long long mgl_nan[2] = {0x7fffffffffffffff, 0x7fffffff};
+const unsigned long long mgl_inf[2] = {0x7ff0000000000000, 0x7f800000};
+#else
+const unsigned long mgl_nan[2] = {0x7fffffffffffffff, 0x7fffffff};
+const unsigned long mgl_inf[2] = {0x7ff0000000000000, 0x7f800000};
+#endif
+
+#define NANd    (*(double*)mgl_nan)
+#define NANf    (*(float*)(mgl_nan+1))
+#define INFd    (*(double*)mgl_inf)
+#define INFf    (*(float*)(mgl_inf+1))
+
+#if !defined(NAN)
 #if MGL_USE_DOUBLE
 #define NAN		NANd
 #else
-#define NAN		NANd
+#define NAN		NANf
 #endif
 #endif
 
+#if !defined(INFINITY)
+#if MGL_USE_DOUBLE
+#define INFINITY	INFd
+#else
+#define INFINITY	INFf
+#endif
+#endif
+
+#endif
+
 #ifndef M_PI
 #define M_PI	3.14159265358979323846  /* pi */
 #endif
@@ -108,6 +157,10 @@ typedef float mreal;
 #define mgl_min(a,b)	(((a)>(b)) ? (b) : (a))
 #define mgl_max(a,b)	(((a)>(b)) ? (a) : (b))
 #define mgl_isnan(a)	((a)!=(a))
+//#define mgl_isnum(a)	((a)==(a) && 2*(a)!=(a))
+#define mgl_isnum(a)	((a)==(a))
+#define mgl_isfin(a)	((a)-(a)==0.)
+#define mgl_isbad(a)	((a)-(a)!=0.)
 //-----------------------------------------------------------------------------
 #define SMOOTH_NONE		0
 #define SMOOTH_LINE_3	1
@@ -141,6 +194,8 @@ enum{	// types of predefined curvelinear coordinate systems
 #define MGL_DRAW_FAST	1	// fast, no color interpolation
 #define MGL_DRAW_NORM	2	// high quality, slower
 #define MGL_DRAW_LMEM	4	// low memory usage (direct to pixel)
+#define MGL_DRAW_DOTS	8	// draw dots instead of primitives
+#define MGL_DRAW_NONE	9	// no ouput (for testing only)
 //-----------------------------------------------------------------------------
 enum{	// Codes for warnings/messages
 	mglWarnNone = 0,// Everything OK
@@ -160,6 +215,10 @@ enum{	// Codes for warnings/messages
 	mglWarnTern, 	// Axis ranges are incompatible
 	mglWarnNull, 	// Pointer is NULL
 	mglWarnSpc,		// Not enough space for plot
+	mglScrArg,		// Wrong argument(s) in MGL script
+	mglScrCmd,		// Wrong command in MGL script
+	mglScrLong,		// Too long line in MGL script
+	mglScrStr,		// Unbalanced ' in MGL script
 	mglWarnEnd		// Maximal number of warnings (must be last)
 };
 //-----------------------------------------------------------------------------
@@ -184,7 +243,7 @@ enum{	// Codes for warnings/messages
 #define MGL_SHOW_POS		0x001000 	///< Switch to show or not mouse click position
 #define MGL_CLF_ON_UPD		0x002000 	///< Clear plot before Update()
 #define MGL_NOSUBTICKS		0x004000 	///< Disable subticks drawing (for bounding box)
-#define MGL_DIFFUSIVE		0x008000 	///< Use diffusive light instead of specular
+//#define MGL_DIFFUSIVE		0x008000 	///< Use diffusive light instead of specular
 #define MGL_VECT_FRAME		0x010000 	///< Use DrwDat to remember all data of frames
 #define MGL_REDUCEACC		0x020000 	///< Reduce accuracy of points (to reduc size of output files)
 #define MGL_PREFERVC 		0x040000 	///< Prefer vertex color instead of texture if output format supports
@@ -193,66 +252,11 @@ enum{	// Codes for warnings/messages
 //-----------------------------------------------------------------------------
 #ifdef __cplusplus
 //-----------------------------------------------------------------------------
+extern float mgl_cos[360];	///< contain cosine with step 1 degree
+//-----------------------------------------------------------------------------
 #include <complex>
 typedef std::complex<mreal> dual;
 //-----------------------------------------------------------------------------
-struct mglThreadD
-{
-	mreal *a;		// float* array with parameters or results
-	const mreal *b,*c,*d,*e;	// float* arrays with parameters
-	const long *p;	// long* array with parameters
-	const void *v;	// pointer to data/grapher
-	int id;			// thread id
-	long n;			// total number of iteration
-	const char *s;
-};
-struct mglThreadC
-{
-	dual *a;		// dual* array with parameters or results
-	const dual *b,*c,*d,*e;	// dual* arrays with parameters
-	const long *p;	// long* array with parameters
-	const void *v;	// pointer to data/grapher
-	int id;			// thread id
-	long n;			// total number of iteration
-	const char *s;
-};
-struct mglThreadV
-{
-	mreal *a;		// float* array with parameters or results
-	dual *aa;		// dual* array with parameters or results
-	const void *b,*c;	// float* arrays with parameters
-	const mreal *d;	// float* arrays with parameters
-	const long *p;	// long* array with parameters
-	const void *v;	// pointer to data/grapher
-	int id;			// thread id
-	long n;			// total number of iteration
-};
-struct mglThreadT
-{
-	void *a; 		// dual* or mreal* array with input or results
-	double *b; 		// dual* array with input or results
-	const long *p;	// long* array with parameters
-	const void *v;	// pointer to table/parameter
-	void **w; 		// pointer to workspace
-	int id;			// thread id
-	long n;			// total number of iteration
-	const void *re,*im;
-};
-/// Start several thread for the task
-void MGL_EXPORT mglStartThread(void *(*func)(void *), void (*post)(mglThreadD *,mreal *), long n,
-					mreal *a=0, const mreal *b=0, const mreal *c=0, const long *p=0,
-					const void *v=0, const mreal *d=0, const mreal *e=0, const char *s=0);
-void MGL_EXPORT mglStartThreadV(void *(*func)(void *), long n, mreal *a, const void *b=0,
-					const void *c=0, const long *p=0, const void *v=0, const mreal *d=0);
-void MGL_EXPORT mglStartThreadV(void *(*func)(void *), long n, dual *a, const void *b=0,
-					const void *c=0, const long *p=0, const void *v=0, const mreal *d=0);
-void MGL_EXPORT mglStartThreadC(void *(*func)(void *), void (*post)(mglThreadC *,dual *), long n,
-					dual *a=0, const dual *b=0, const dual *c=0, const long *p=0,
-					const void *v=0, const dual *d=0, const dual *e=0, const char *s=0);
-void MGL_EXPORT mglStartThreadT(void *(*func)(void *), long n, void *a, double *b, const void *v=0,
-					void **w=0, const long *p=0, const void *re=0, const void *im=0);
-MGL_EXPORT extern int mglNumThr;		///< Number of thread for plotting and data handling
-//-----------------------------------------------------------------------------
 extern "C" {
 #else
 #include <complex.h>
@@ -262,14 +266,21 @@ typedef double _Complex dual;
 typedef float _Complex dual;
 #endif
 #endif
+/// Find length of wchar_t string (bypass standard wcslen bug)
+double MGL_EXPORT mgl_hypot(double x, double y);
+/// Find length of wchar_t string (bypass standard wcslen bug)
+size_t MGL_EXPORT mgl_wcslen(const wchar_t *str);
 /// Get RGB values for given color id or fill by -1 if no one found
 void MGL_EXPORT mgl_chrrgb(char id, float rgb[3]);
 /// Check if string contain color id and return its number
 long MGL_EXPORT mgl_have_color(const char *stl);
 /// Find symbol in string excluding {} and return its position or NULL
 const char *mglchr(const char *str, char ch);
-/// Set number of thread for plotting and data handling
+/// Find any symbol from chr in string excluding {} and return its position or NULL
+const char *mglchrs(const char *str, const char *chr);
+/// Set number of thread for plotting and data handling (for pthread version only)
 void MGL_EXPORT mgl_set_num_thr(int n);
+void MGL_EXPORT mgl_set_num_thr_(int *n);
 void MGL_EXPORT mgl_test_txt(const char *str, ...);
 void MGL_EXPORT mgl_set_test_mode(int enable);
 /// Remove spaces at begining and at the end of the string
@@ -280,16 +291,11 @@ void MGL_EXPORT mgl_strlwr(char *str);
 void MGL_EXPORT mgl_wcslwr(wchar_t *str);
 /// Convert wchar_t* string into char* one
 void MGL_EXPORT mgl_wcstombs(char *dst, const wchar_t *src, int size);
+/// Clear internal data for speeding up FFT and Hankel transforms
+void MGL_EXPORT mgl_clear_fft();
 #ifdef __cplusplus
 }
 #endif
-//#if MGL_HAVE_PTHREAD && defined(MGL_SRC)
-#if MGL_HAVE_PTHREAD
-#include <pthread.h>
-#define MGL_PUSH(a,v,m)		{pthread_mutex_lock(&m);	a.push_back(v);	pthread_mutex_unlock(&m);}
-#else
-#define MGL_PUSH(a,v,m)		a.push_back(v);
-#endif
 //-----------------------------------------------------------------------------
 #endif
 //-----------------------------------------------------------------------------
