@@ -1,6 +1,6 @@
 /***************************************************************************
  * mgl.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,11 +21,12 @@
 #define _MGL_H_
 
 #include "mgl2/mgl_cf.h"
+#ifdef __cplusplus
 #include "mgl2/data.h"
 #include "mgl2/datac.h"
 //-----------------------------------------------------------------------------
 /// Wrapper class for all graphics
-class mglGraph
+class MGL_EXPORT mglGraph
 {
 protected:
 	HMGL gr;
@@ -56,7 +57,7 @@ public:
 	inline void SetPlotId(const char *id)	{	mgl_set_plotid(gr,id);	}
 	/// Get name of plot for saving filename
 	inline const char *GetPlotId()	{	return mgl_get_plotid(gr);	}
-	
+
 	/// Set the transparency on/off.
 	inline void Alpha(bool enable)			{	mgl_set_alpha(gr, enable);	}
 	/// Set default value of alpha-channel
@@ -68,7 +69,7 @@ public:
 	inline void Light(bool enable)			{	mgl_set_light(gr, enable);	}
 	/// Switch on/off the specified light source.
 	inline void Light(int n,bool enable)	{	mgl_set_light_n(gr, n, enable);	}
-	/// Use diffusive light (only for local light sources)
+	/// Use diffusive light (only for local light sources) -- OBSOLETE
 	inline void SetDifLight(bool dif)		{	mgl_set_light_dif(gr, dif);	}
 	/// Add a light source.
 	inline void AddLight(int n, mglPoint p, char col='w', double bright=0.5, double ap=0)
@@ -77,6 +78,8 @@ public:
 	{	mgl_add_light_loc(gr, n, r.x, r.y, r.z, p.x, p.y, p.z, col, bright, ap);	}
 	/// Set ambient light brightness
 	inline void SetAmbient(double i)			{	mgl_set_ambbr(gr, i);	}
+	/// Set diffusive light brightness
+	inline void SetDiffuse(double i)			{	mgl_set_difbr(gr, i);	}
 	/// Set the fog distance or switch it off (if d=0).
 	inline void Fog(double d, double dz=0.25)	{	mgl_set_fog(gr, d, dz);		}
 
@@ -90,7 +93,7 @@ public:
 	inline void SetMeshNum(int num)			{	mgl_set_meshnum(gr, num);	}
 	/// Set number of visible faces (use 0 to draw all of them)
 	inline void SetFaceNum(int num)			{	mgl_set_facenum(gr, num);	}
-	
+
 	/// Set cutting for points outside of bounding box
 	inline void SetCut(bool cut)				{	mgl_set_cut(gr, cut);	}
 	/// Set additional cutting box
@@ -124,6 +127,15 @@ public:
 	/// Set default color scheme
 	inline void SetDefScheme(const char *sch)	{	mgl_set_def_sch(gr, sch);	}
 
+	/// Sets RGB values for color with given id
+	static inline void SetColor(char id, double r, double g, double b)	{	mgl_set_color(id, r, g, b);	}
+	/// Set mask for face coloring as array of type 'unsigned char[8]'
+	static inline void SetMask(char id, const char *mask)	{	mgl_set_mask(id, mask);	}
+	/// Set mask for face coloring as uint64_t number
+	static inline void SetMask(char id, uint64_t mask)	{	mgl_set_mask_val(id, mask);	}
+	/// Set default mask rotation angle
+	inline void SetMaskAngle(int angle)	{	mgl_set_mask_angle(gr, angle);	}
+
 	/// Get last warning code
 	inline int  GetWarn()	{	return mgl_get_warn(gr);}
 	/// Set warning code ant fill message
@@ -134,6 +146,9 @@ public:
 	/// Set axis range scaling -- simplified way to shift/zoom axis range -- need to replot whole image!
 	inline void ZoomAxis(mglPoint p1=mglPoint(0,0,0,0), mglPoint p2=mglPoint(1,1,1,1))
 	{	mgl_zoom_axis(gr, p1.x,p1.y,p1.z,p1.c, p2.x,p2.y,p2.z,p2.c);	}
+	/// Add [v1, v2] to the current range in direction dir
+	inline void AddRange(char dir, double v1, double v2)
+	{	mgl_add_range_val(gr, dir, v1, v2);	}
 	/// Set range in direction dir as [v1, v2]
 	inline void SetRange(char dir, double v1, double v2)
 	{	mgl_set_range_val(gr, dir, v1, v2);	}
@@ -224,7 +239,7 @@ public:
 	/// Set to draw tick labels at axis origin
 	inline void SetOriginTick(bool enable=true)
 	{	mgl_set_flag(gr,!enable, MGL_NO_ORIGIN);	}
-	
+
 	/// Put further plotting in some region of whole frame.
 	inline void SubPlot(int nx,int ny,int m,const char *style="<>_^", double dx=0, double dy=0)
 	{	mgl_subplot_d(gr, nx, ny, m, style, dx, dy);	}
@@ -252,7 +267,7 @@ public:
 	inline void Push()	{	mgl_mat_push(gr);	}
 	/// Pop transformation matrix from stack
 	inline void Pop()	{	mgl_mat_pop(gr);	}
-	
+
 	/// Add title for current subplot/inplot
 	inline 	void Title(const char *title,const char *stl="",double size=-2)
 	{	mgl_title(gr,title,stl,size);	}
@@ -283,6 +298,8 @@ public:
 	inline void SetQuality(int qual=MGL_DRAW_NORM)	{	mgl_set_quality(gr, qual);	}
 	/// Get plot quality
 	inline int GetQuality()	{	return mgl_get_quality(gr);	}
+	/// Set drawing region for Quality&4
+	inline void SetDrawReg(long nx=1, long ny=1, long m=0)	{	mgl_set_draw_reg(gr,nx,ny,m);	}
 	/// Start group of objects
 	inline void StartGroup(const char *name)		{	mgl_start_group(gr, name);	}
 	/// End group of objects
@@ -347,8 +364,11 @@ public:
 	inline void WritePRC(const char *fname,const char *descr="",bool make_pdf=true)
 	{	mgl_write_prc(gr, fname, descr, make_pdf);	}
 	/// Export in JSON format suitable for later drawing by JavaScript
-	inline void WriteJSON(const char *fname,const char *descr="")
-	{	mgl_write_json(gr, fname, descr);	}
+	inline void WriteJSON(const char *fname,const char *descr="",bool force_z=false)
+	{	if(force_z)	mgl_write_json_z(gr, fname, descr);
+		else 	mgl_write_json(gr, fname, descr);	}
+	/// Return string of JSON data suitable for later drawing by JavaScript
+	inline const char *GetJSON()	{	return mgl_get_json(gr);	}
 
 	/// Force preparing the image. It can be useful for OpenGL mode mostly.
 	inline void Finish()			{	mgl_finish(gr);	}
@@ -368,7 +388,7 @@ public:
 	inline void SetFrame(int i)	{	mgl_set_frame(gr, i);	}
 	/// Append drawing data from i-th frame (work if MGL_VECT_FRAME is set on)
 	inline void ShowFrame(int i){	mgl_show_frame(gr, i);	}
-	
+
 	/// Start write frames to cinema using GIF format
 	inline void StartGIF(const char *fname, int ms=100)
 	{	mgl_start_gif(gr, fname,ms);	}
@@ -440,6 +460,7 @@ public:
 
 	/// Clear up the frame
 	inline void Clf(double r, double g, double b)	{	mgl_clf_rgb(gr, r, g, b);	}
+	inline void Clf(char col)	{	mgl_clf_chr(gr, col);	}
 	inline void Clf()	{	mgl_clf(gr);	}
 	/// Clear unused points and primitives. Useful only in combination with SetFaceNum().
 	inline void ClearUnused()	{	mgl_clear_unused(gr);	}
@@ -605,6 +626,11 @@ public:
 	{	mgl_region(gr, &y1, &y2, pen, opt);	}
 	inline void Region(const mglDataA &x, const mglDataA &y1, const mglDataA &y2, const char *pen="", const char *opt="")
 	{	mgl_region_xy(gr, &x, &y1, &y2, pen, opt);	}
+	/// Fill area (draw ribbon) between curves {x1,y1,z1} and {x2,y2,z2}
+	inline void Region(const mglDataA &x1, const mglDataA &y1, const mglDataA &z1, const mglDataA &x2, const mglDataA &y2, const mglDataA &z2, const char *pen="", const char *opt="")
+	{	mgl_region_3d(gr, &x1, &y1, &z1, &x2, &y2, &z2, pen, opt);	}
+	inline void Region(const mglDataA &x1, const mglDataA &y1, const mglDataA &x2, const mglDataA &y2, const char *pen="", const char *opt="")
+	{	mgl_region_3d(gr, &x1, &y1, NULL, &x2, &y2, NULL, pen, opt);	}
 	/// Draw vertical lines from points {x,y,z} to axis plane
 	inline void Stem(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *pen="", const char *opt="")
 	{	mgl_stem_xyz(gr, &x, &y, &z, pen, opt);	}
@@ -628,6 +654,13 @@ public:
 	/// Draw chart for data a
 	inline void Chart(const mglDataA &a, const char *colors="", const char *opt="")
 	{	mgl_chart(gr, &a, colors,opt);	}
+
+	/// Draw Open-High-Low-Close (OHLC) diagram
+	inline void OHLC(const mglDataA &x, const mglDataA &open, const mglDataA &high, const mglDataA &low, const mglDataA &close, const char *pen="", const char *opt="")
+	{	mgl_ohlc_x(gr, &x, &open,&high,&low,&close,pen,opt);	}
+	inline void OHLC(const mglDataA &open, const mglDataA &high, const mglDataA &low, const mglDataA &close, const char *pen="", const char *opt="")
+	{	mgl_ohlc(gr, &open,&high,&low,&close,pen,opt);	}
+
 	/// Draw box-plot (special 5-value plot used in statistic)
 	inline void BoxPlot(const mglDataA &x, const mglDataA &y, const char *pen="", const char *opt="")
 	{	mgl_boxplot_xy(gr, &x, &y, pen,opt);	}
@@ -708,7 +741,7 @@ public:
 	{	mgl_table(gr, x, y, &val, text, fnt, opt);	}
 	inline void Table(double x, double y, const mglDataA &val, const wchar_t *text, const char *fnt="#|", const char *opt="")
 	{	mgl_tablew(gr, x, y, &val, text, fnt, opt);	}
-	
+
 	/// Draw tube with radius r around curve {x,y,z}
 	inline void Tube(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &r, const char *pen="", const char *opt="")
 	{	mgl_tube_xyzr(gr, &x, &y, &z, &r, pen, opt);	}
@@ -1060,62 +1093,65 @@ public:
 	/// Draw dots in points {x,y,z}.
 	inline void Dots(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *sch="", const char *opt="")
 	{	mgl_dots(gr, &x, &y, &z, sch, opt);	}
-	/// Draw semitransparent dots in points {x,y,z} with alpha a.
+	/// Draw semitransparent dots in points {x,y,z} with specified alpha a.
 	inline void Dots(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const char *sch="", const char *opt="")
 	{	mgl_dots_a(gr, &x, &y, &z, &a, sch, opt);	}
+	/// Draw semitransparent dots in points {x,y,z} with specified color c and alpha a.
+	inline void Dots(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &c, const mglDataA &a, const char *sch="", const char *opt="")
+	{	mgl_dots_ca(gr, &x, &y, &z, &c, &a, sch, opt);	}
 	/// Draw surface reconstructed for points in arrays {x,y,z}.
 	inline void Crust(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *sch="", const char *opt="")
 	{	mgl_crust(gr, &x, &y, &z, sch, opt);	}
 
 	/// Fit data along x-direction for each data row. Return array with values for found formula.
-	inline mglData Fit(const mglDataA &y, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_1(gr, &y, eq,var,0, opt));	}
-	inline mglData Fit(const mglDataA &y, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_1(gr, &y, eq, var, &ini, opt));	}
+	inline mglData Fit(const mglDataA &y, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_1(gr, &y, eq,vars,0, opt));	}
+	inline mglData Fit(const mglDataA &y, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_1(gr, &y, eq, vars, &ini, opt));	}
 	/// Fit data along x-, y-directions for each data slice. Return array with values for found formula.
-	inline mglData Fit2(const mglDataA &z, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_2(gr, &z, eq, var,0, opt));	}
-	inline mglData Fit2(const mglDataA &z, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_2(gr, &z, eq, var, &ini, opt));	}
+	inline mglData Fit2(const mglDataA &z, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_2(gr, &z, eq, vars,0, opt));	}
+	inline mglData Fit2(const mglDataA &z, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_2(gr, &z, eq, vars, &ini, opt));	}
 	/// Fit data along along all directions. Return array with values for found formula.
-	inline mglData Fit3(const mglDataA &a, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_3(gr, &a, eq, var,0, opt));	}
-	inline mglData Fit3(const mglDataA &a, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_3(gr, &a, eq, var, &ini, opt));	}
+	inline mglData Fit3(const mglDataA &a, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_3(gr, &a, eq, vars,0, opt));	}
+	inline mglData Fit3(const mglDataA &a, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_3(gr, &a, eq, vars, &ini, opt));	}
 	/// Fit data along x-direction for each data row. Return array with values for found formula.
-	inline mglData Fit(const mglDataA &x, const mglDataA &y, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_xy(gr, &x, &y, eq, var,0, opt));	}
-	inline mglData Fit(const mglDataA &x, const mglDataA &y, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_xy(gr, &x, &y, eq, var, &ini, opt));	}
+	inline mglData Fit(const mglDataA &x, const mglDataA &y, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_xy(gr, &x, &y, eq, vars,0, opt));	}
+	inline mglData Fit(const mglDataA &x, const mglDataA &y, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_xy(gr, &x, &y, eq, vars, &ini, opt));	}
 	/// Fit data along x-, y-directions for each data slice. Return array with values for found formula.
-	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_xyz(gr, &x, &y, &z, eq, var,0, opt));	}
-	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_xyz(gr, &x, &y, &z, eq, var, &ini, opt));	}
+	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_xyz(gr, &x, &y, &z, eq, vars,0, opt));	}
+	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_xyz(gr, &x, &y, &z, eq, vars, &ini, opt));	}
 	/// Fit data along along all directions. Return array with values for found formula.
-	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_xyza(gr, &x, &y, &z, &a, eq, var,0, opt));	}
-	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_xyza(gr, &x, &y, &z, &a, eq,var, &ini, opt));	}
+	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_xyza(gr, &x, &y, &z, &a, eq, vars,0, opt));	}
+	inline mglData Fit(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_xyza(gr, &x, &y, &z, &a, eq,vars, &ini, opt));	}
 	/// Fit data with dispersion s along x-direction for each data row. Return array with values for found formula.
-	inline mglData FitS(const mglDataA &y, const mglDataA &s, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_ys(gr, &y, &s, eq, var,0, opt));	}
-	inline mglData FitS(const mglDataA &y, const mglDataA &s, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_ys(gr, &y, &s, eq, var, &ini, opt));	}
-	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &s, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_xys(gr, &x, &y, &s, eq, var,0, opt));	}
-	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &s, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_xys(gr, &x, &y, &s, eq, var, &ini, opt));	}
+	inline mglData FitS(const mglDataA &y, const mglDataA &s, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_ys(gr, &y, &s, eq, vars,0, opt));	}
+	inline mglData FitS(const mglDataA &y, const mglDataA &s, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_ys(gr, &y, &s, eq, vars, &ini, opt));	}
+	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &s, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_xys(gr, &x, &y, &s, eq, vars,0, opt));	}
+	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &s, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_xys(gr, &x, &y, &s, eq, vars, &ini, opt));	}
 	/// Fit data with dispersion s along x-, y-directions for each data slice. Return array with values for found formula.
-	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &s, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_xyzs(gr, &x, &y, &z, &s, eq, var,0, opt));	}
-	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &s, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_xyzs(gr, &x, &y, &z, &s, eq, var, &ini, opt));	}
+	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &s, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_xyzs(gr, &x, &y, &z, &s, eq, vars,0, opt));	}
+	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &s, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_xyzs(gr, &x, &y, &z, &s, eq, vars, &ini, opt));	}
 	/// Fit data with dispersion s along all directions. Return array with values for found formula.
-	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const mglDataA &s, const char *eq, const char *var, const char *opt="")
-	{	return mglData(true,mgl_fit_xyzas(gr, &x, &y, &z, &a, &s, eq, var,0, opt));	}
-	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const mglDataA &s, const char *eq, const char *var, mglData &ini, const char *opt="")
-	{	return mglData(true,mgl_fit_xyzas(gr, &x, &y, &z, &a, &s, eq, var, &ini, opt));	}
+	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const mglDataA &s, const char *eq, const char *vars, const char *opt="")
+	{	return mglData(true,mgl_fit_xyzas(gr, &x, &y, &z, &a, &s, eq, vars,0, opt));	}
+	inline mglData FitS(const mglDataA &x, const mglDataA &y, const mglDataA &z, const mglDataA &a, const mglDataA &s, const char *eq, const char *vars, mglData &ini, const char *opt="")
+	{	return mglData(true,mgl_fit_xyzas(gr, &x, &y, &z, &a, &s, eq, vars, &ini, opt));	}
 	/// Print fitted last formula (with coefficients)
 	inline void PutsFit(mglPoint p, const char *prefix=0, const char *font="", double size=-1)
 	{	mgl_puts_fit(gr, p.x, p.y, p.z, prefix, font, size);	}
@@ -1140,11 +1176,19 @@ public:
 	{	mgl_datac_fill_eq(gr, &u, eq, &v, 0, opt);	}
 	inline void Fill(mglDataC &u, const char *eq, const mglDataA &v, const mglDataA &w, const char *opt="")
 	{	mgl_datac_fill_eq(gr, &u, eq, &v, &w, opt);	}
-	
+
+	/// Fill dat by interpolated values of vdat parametrically depended on xdat,ydat,zdat for x,y,z in axis range
+	inline void Refill(mglData &dat, const mglDataA &xdat, const mglDataA &vdat, long sl=-1, const char *opt="")
+	{	mgl_data_refill_gr(gr,&dat,&xdat,0,0,&vdat,sl,opt);	}
+	inline void Refill(mglData &dat, const mglDataA &xdat, const mglDataA &ydat, const mglDataA &vdat, long sl=-1, const char *opt="")
+	{	mgl_data_refill_gr(gr,&dat,&xdat,&ydat,0,&vdat,sl,opt);	}
+	inline void Refill(mglData &dat, const mglDataA &xdat, const mglDataA &ydat, const mglDataA &zdat, const mglDataA &vdat, const char *opt="")
+	{	mgl_data_refill_gr(gr,&dat,&xdat,&ydat,&zdat,&vdat,-1,opt);	}
+
 	/// Set the data by triangulated surface values assuming x,y,z in range [Min, Max]
 	inline void DataGrid(mglData &d, const mglDataA &x, const mglDataA &y, const mglDataA &z, const char *opt="")
 	{	mgl_data_grid(gr,&d,&x,&y,&z,opt);	}
-	
+
 	/// Make histogram (distribution) of data. This function do not plot data.
 	inline mglData Hist(const mglDataA &x, const mglDataA &a, const char *opt="")
 	{	return mglData(true, mgl_hist_x(gr, &x, &a, opt));	}
@@ -1161,42 +1205,8 @@ public:
 //	inline void TextureColor(bool){}	// NOTE: Add later -- IDTF
 };
 //-----------------------------------------------------------------------------
-/// Structure for handling named mglData (used by mglParse class).
-class mglVar : public mglData
-{
-public:
-	std::wstring s;	///< Data name
-	void *o; 		///< Pointer to external object
-	mglVar *next;	///< Pointer to next instance in list
-	mglVar *prev;	///< Pointer to previous instance in list
-	bool temp;		///< This is temporary variable
-	void (*func)(void *);	///< Callback function for destroying
-	
-	mglVar():mglData()	{	o=0;	next=prev=0;	func=0;	temp=false;	}
-	virtual ~mglVar()
-	{
-		if(func)	func(o);
-		if(prev)	prev->next = next;
-		if(next)	next->prev = prev;
-	}
-	/// Move variable after var and copy func from var (if func is 0)
-	void MoveAfter(mglVar *var)
-	{
-		if(prev)	prev->next = next;
-		if(next)	next->prev = prev;
-		prev = next = 0;
-		if(var)
-		{
-			prev = var;	next = var->next;
-			var->next = this;
-			if(func==0)	func = var->func;
-		}
-		if(next)	next->prev = this;
-	}
-};
-//-----------------------------------------------------------------------------
 /// Wrapper class for MGL parsing
-class mglParse
+class MGL_EXPORT mglParse
 {
 	HMPR pr;
 public:
@@ -1259,7 +1269,7 @@ public:
 	{	return mglData(true,mgl_parser_calc(pr,formula)); 	}
 	inline mglData Calc(const wchar_t *formula)
 	{	return mglData(true,mgl_parser_calcw(pr,formula));	}
-	
+
 	/// Find variable with given name or add a new one
 	/// NOTE !!! You must not delete obtained data arrays !!!
 	inline mglVar *AddVar(const char *name)
@@ -1279,51 +1289,5 @@ public:
 	void DeleteAll()	{	mgl_parser_del_all(pr);	}
 };
 //-----------------------------------------------------------------------------
-/// Wrapper class expression evaluating
-class mglExpr
-{
-	HMEX ex;
-public:
-	mglExpr(const char *expr)		{	ex = mgl_create_expr(expr);	}
-	~mglExpr()	{	mgl_delete_expr(ex);	}
-	/// Return value of expression for given x,y,z variables
-	inline double Eval(double x, double y=0, double z=0)
-	{	return mgl_expr_eval(ex,x,y,z);	}
-	/// Return value of expression differentiation over variable dir for given x,y,z variables
-	inline double Diff(char dir, double x, double y=0, double z=0)
-	{	return mgl_expr_diff(ex,dir, x,y,z);	}
-#ifndef SWIG
-	/// Return value of expression for given variables
-	inline double Eval(mreal var[26])
-	{	return mgl_expr_eval_v(ex,var);	}
-	/// Return value of expression differentiation over variable dir for given variables
-	inline double Diff(char dir, mreal var[26])
-	{	return mgl_expr_diff_v(ex,dir, var);	}
-#endif
-};
-//-----------------------------------------------------------------------------
-/// Wrapper class expression evaluating
-class mglExprC
-{
-	HAEX ex;
-public:
-	mglExprC(const char *expr)		{	ex = mgl_create_cexpr(expr);	}
-	~mglExprC()	{	mgl_delete_cexpr(ex);	}
-	/// Return value of expression for given x,y,z variables
-	inline dual Eval(dual x, dual y=0, dual z=0)
-	{	return mgl_cexpr_eval(ex,x,y,z);	}
-	/// Return value of expression for given x,y,z,u,v,w variables
-	inline dual Eval(dual x, dual y, dual z, dual u, dual v, dual w)
-	{
-		dual var[26];
-		var['x'-'a']=x;	var['y'-'a']=y;	var['z'-'a']=z;
-		var['u'-'a']=u;	var['v'-'a']=v;	var['w'-'a']=w;
-		return mgl_cexpr_eval_v(ex,var);	}
-#ifndef SWIG
-	/// Return value of expression for given variables
-	inline dual Eval(dual var[26])
-	{	return mgl_cexpr_eval_v(ex,var);	}
-#endif
-};
-//-----------------------------------------------------------------------------
 #endif
+#endif
\ No newline at end of file
