@@ -1,6 +1,6 @@
 /***************************************************************************
  * prim.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -19,7 +19,7 @@
  ***************************************************************************/
 #ifndef _MGL_PRIM_H_
 #define _MGL_PRIM_H_
-#include "mgl2/base.h"
+#include "mgl2/abstract.h"
 //-----------------------------------------------------------------------------
 #ifdef __cplusplus
 extern "C" {
@@ -77,7 +77,7 @@ void MGL_EXPORT mgl_cones_xyz(HMGL graph, HCDT x, HCDT y, HCDT z, const char *pe
 void MGL_EXPORT mgl_cones_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen, const char *opt,int,int);
 /// Draw cones from points {x,z} to axis plane
 void MGL_EXPORT mgl_cones_xz(HMGL graph, HCDT x, HCDT z, const char *pen, const char *opt);
-void MGL_EXPORT mgl_coners_xz_(uintptr_t *graph, uintptr_t *x, uintptr_t *z, const char *pen, const char *opt,int,int);
+void MGL_EXPORT mgl_cones_xz_(uintptr_t *graph, uintptr_t *x, uintptr_t *z, const char *pen, const char *opt,int,int);
 /// Draw cones from points {x,z} with x in x-axis range to axis plane
 void MGL_EXPORT mgl_cones(HMGL graph, HCDT z,	const char *pen, const char *opt);
 void MGL_EXPORT mgl_cones_(uintptr_t *graph, uintptr_t *z,	const char *pen, const char *opt,int,int);
