@@ -1,6 +1,6 @@
 /***************************************************************************
  * window.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -21,8 +21,8 @@
 #define _MGL_QMATHGL_H_
 //-----------------------------------------------------------------------------
 #include <string>
-#include <QtGui/QWidget>
-#include <QtGui/QPixmap>
+#include <QWidget>
+#include <QPixmap>
 #include <mgl2/wnd.h>
 //-----------------------------------------------------------------------------
 class QTextEdit;
@@ -55,12 +55,10 @@ public:
 	{	setGraph(GR->Self());	}
 	inline HMGL getGraph()	{	return (HMGL)gr;	}
 	/// Set drawing functions and its parameter
-	inline void setDraw(int (*func)(mglBase *gr, void *par), void *par=0)
-	{	draw_func = func;	draw_par = par;	emit usePrimChanged(draw_func || draw);	}
-	inline void setDraw(mglDraw *dr)
-	{	draw = dr;	emit usePrimChanged(draw_func || draw);	}
-	inline void setDraw(int (*draw)(mglGraph *gr))
-	{	setDraw(draw?mgl_draw_graph:0,(void*)draw);	}
+	void setDraw(int (*func)(mglBase *gr, void *par), void *par);
+	void setDraw(mglDraw *dr);
+	inline void setDraw(int (*func)(mglGraph *gr))
+	{	setDraw(func?mgl_draw_graph:0,(void*)func);	}
 	inline void zoomRegion(mreal xx1,mreal xx2,mreal yy1, mreal yy2)
 	{	x1=xx1;	y1=yy1;	x2=xx2;	y2=yy2;	}
 
@@ -90,6 +88,8 @@ public slots:
 	void imgSize(int w, int h);	///< Set image size
 	void setViewYZ(bool v);	///< Switch on/off rotation around Y and Z axis
 
+	void setCustZoom(bool a);	///< Switch on/off using custom zoom
+	void setCustDraw(bool a);	///< Switch on/off using custom draw
 	void setZoom(bool z);	///< Switch on/off mouse zooming
 	void setRotate(bool r);	///< Switch on/off mouse rotation
 	void zoomIn();			///< Zoom in graphics
@@ -155,7 +155,11 @@ signals:
 	void objChanged(int objId);	///< User double-click to select object/line
 	void refreshData();
 	void doubleClick(int id);	///< Double mouse click by object with id
-	void askStyle(int id);	///< Update style
+	void askStyle(int id);		///< Update style
+	/// user can define its own zooming function
+	void customZoom(double x1, double y1, double x2, double y2, double tet, double phi, double per);
+	/// user can define its own drawing/setting function which will be called before main drawing
+	void customDraw(double x1, double y1, double x2, double y2, bool draw);
 
 protected:
 	void paintEvent(QPaintEvent *);
@@ -177,6 +181,8 @@ protected:
 	double per;			///< Value of perspective ( must be in [0,1) )
 	bool alpha;			///< Transparency state
 	bool light;			///< Lightning state
+	bool custZoom;		///< Use custom zoom instead of built in
+	bool custDraw;		///< Use custom draw before main drawing
 	bool zoom;			///< Mouse zoom state
 	bool grid;			///< Grid drawing state
 	bool rotate;			///< Mouse rotation state
@@ -193,8 +199,9 @@ private:
 };
 //-----------------------------------------------------------------------------
 /// Class for drawing the MGL script
-struct mglDrawScript : public mglDraw
+class MGL_EXPORT mglDrawScript : public mglDraw
 {
+public:
 	HMPR par;		///< Parser to be used
 	QString text;	///< Script to be drawn
 	long line;		///< Line which will be highlited
