@@ -1,6 +1,6 @@
 /***************************************************************************
  * type.h is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -19,9 +19,7 @@
  ***************************************************************************/
 #ifndef _MGL_TYPE_H_
 #define _MGL_TYPE_H_
-#if !defined(_MSC_VER) && !defined(__BORLANDC__)
-#include <stdint.h>
-#endif
+
 #include "mgl2/define.h"
 //-----------------------------------------------------------------------------
 const mreal Pi = M_PI;
@@ -34,7 +32,7 @@ const mreal mgl_min_a = 1./256;
 #define MGL_SET_RGB(p,rr,gg,bb)		{p.r=(rr);p.g=(gg);p.b=(bb);}
 //-----------------------------------------------------------------------------
 /// Class for point in 3D space
-struct mglPoint
+struct MGL_EXPORT mglPoint
 {
 	mreal x,y,z,c;
 	mglPoint(mreal X=0,mreal Y=0,mreal Z=0,mreal C=0){x=X;y=Y;z=Z;c=C;}
@@ -74,7 +72,7 @@ inline mglPoint operator|(const mglPoint &a, const mglPoint &b)
 inline mglPoint operator^(const mglPoint &a, const mglPoint &b)
 {	return mglPoint(a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x);	}
 inline mglPoint operator!(const mglPoint &a)
-{	mreal f=hypot(a.x,a.y);	return f==0?mglPoint(0.,1.,0.):mglPoint(-a.y/f, a.x/f, 0);	}
+{	mreal f=mgl_hypot(a.x,a.y);	return f==0?mglPoint(0.,1.,0.):mglPoint(-a.y/f, a.x/f, 0);	}
 inline bool operator==(const mglPoint &a, const mglPoint &b)
 {	return !memcmp(&a, &b, sizeof(mglPoint));	}
 inline bool operator!=(const mglPoint &a, const mglPoint &b)
@@ -88,7 +86,7 @@ inline mreal mgl_norm(const mglPoint &p)
 #endif
 //-----------------------------------------------------------------------------
 /// Class for RGBA color
-struct mglColor
+struct MGL_EXPORT mglColor
 {
 	float r;	///< Red component of color
 	float g;	///< Green component of color
