@@ -1,5 +1,5 @@
 /* editor.cpp is part of UDAV
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License
@@ -290,7 +290,8 @@ void load_file(char *newfile, int ipos)
 
 		char *t = textbuf->text();
 #ifndef WIN32
-		for(size_t i=0;i<strlen(t);i++)	if(t[i]=='\r')	t[i]=' ';
+		register size_t i,l=strlen(t);
+		for(i=0;i<l;i++)	if(t[i]=='\r')	t[i]=' ';
 		textbuf->text(t);
 #endif
 		fill_animate(t);	free(t);
