@@ -1,6 +1,6 @@
 /***************************************************************************
  * axis.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -26,6 +26,7 @@
 #define islog(a, b) (((a)>0 && (b)>10*(a)) || ((b)<0 && (a)<10*(b)))
 // NOTE: I use <=0 for proper tick labels rotation. But this mirror labels for central origin!
 #define sign(a)	((a)<0 ? -1:1)
+//#define sign(a)	((a)<0 ? -1:((a)>0 ? 1:0))
 //-----------------------------------------------------------------------------
 MGL_NO_EXPORT inline struct tm *mgl_localtime (const time_t *clock, tm *result, bool use_utc)
 {	if (!clock || !result) return NULL;
@@ -35,19 +36,31 @@ MGL_NO_EXPORT inline struct tm *mgl_localtime (const time_t *clock, tm *result,
 long MGL_EXPORT mgl_have_color(const char *stl)
 {
 	register long i,j=0;
-	if(stl)	for(i=0;stl[i];i++)	if(strchr(MGL_COLORS,stl[i]))	j++;
+	if(stl)	for(i=0;stl[i];i++)
+	{
+		if(strchr(MGL_COLORS,stl[i]))	j++;
+		if(stl[i]=='{' && stl[i+1]=='x')	j++;
+	}
 	return j;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_wcstrim(wchar_t *str)
 {
-	size_t n=wcslen(str), k, i;
+	if(!str || *str==0)	return;
+	size_t n=mgl_wcslen(str), k, i;
 	for(k=0;k<n;k++)	if(str[k]>' ')	break;
-	for(i=n;i>0;i--)	if(str[i-1]>' ')	break;
+	for(i=n;i>k;i--)	if(str[i-1]>' ')	break;
 	memmove(str, str+k, (i-k)*sizeof(wchar_t));
 	str[i-k]=0;
 }
 //-----------------------------------------------------------------------------
+size_t MGL_EXPORT mgl_wcslen(const wchar_t *str)
+{
+	long i=0;
+	if(str)	while(str[i])	i++;
+	return i;
+}
+//-----------------------------------------------------------------------------
 //		Ticks setup
 //-----------------------------------------------------------------------------
 void mglCanvas::SetAxisStl(const char *stl, const char *tck, const char *sub)
@@ -68,7 +81,7 @@ void mglCanvas::SetTicks(char dir, mreal d, int ns, mreal org)
 	if(!strchr("xyzca",dir))	return;
 	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
 
-	if(aa.f==1)	aa.t[0]=0;
+	if(aa.f==1)	aa.t.clear();
 	aa.d=d;	aa.f=0;	aa.ns=ns;	aa.o=org;
 	aa.txt.clear();
 }
@@ -102,16 +115,12 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const wchar_t *lbl, bool add)
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTicksVal(char dir, HCDT v, const char *lbl, bool add)
 {
-	long ll=mbstowcs(0,lbl,0)+1;
-	wchar_t *wcs = new wchar_t[ll];
-	mbstowcs(wcs,lbl,ll);
-	SetTicksVal(dir,v,wcs,add);
-	delete []wcs;
+	MGL_TO_WCS(lbl,SetTicksVal(dir,v,wcs,add));
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTicksVal(char dir, const wchar_t *lbl, bool add)
 {
-	register long i,j,len=wcslen(lbl);
+	register long i,j,len=mgl_wcslen(lbl);
 	for(i=0,j=1;j<len;j++)
 		if(lbl[j]=='\n' || (lbl[j]=='n' && lbl[j-1]=='\\'))	i++;
 	if(i>63)	i=63;
@@ -153,15 +162,7 @@ void mglCanvas::SetTicksVal(char dir, HCDT v, const char **lbl, bool add)
 	if(add)	{	UpdateAxis();	AdjustTicks(aa,ff);	}
 	if(!v || !lbl)	{	aa.f = 0;	return;	}
 	aa.f = 2;	aa.ns=0;	aa.ds=0;
-	register size_t i,n=v->GetNx(),l=0;
-	for(i=0;i<n;i++)	if(strlen(lbl[i])>l)	l=mbstowcs(0,lbl[i],0)+1;
-	wchar_t *str=new wchar_t[l+1];
-	for(i=0;i<n;i++)
-	{
-		mbstowcs(str,lbl[i],strlen(lbl[i])+1);
-		aa.AddLabel(str,v->v(i));
-	}
-	delete []str;
+	for(long i=0;i<v->GetNx();i++)	MGL_TO_WCS(lbl[i],aa.AddLabel(wcs,v->v(i)));
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTickTempl(char dir, const wchar_t *t)
@@ -170,8 +171,7 @@ void mglCanvas::SetTickTempl(char dir, const wchar_t *t)
 	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
 
 	if(aa.f==1)	aa.f = 0;	// remove time ticks
-	if(!t || !t[0])	aa.t[0]=0;
-	else if(wcslen(t)<255) wcscpy(aa.t,t);
+	if(!t)	aa.t.clear();	else aa.t=t;
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::SetTickTempl(char dir, const char *t)
@@ -180,8 +180,8 @@ void mglCanvas::SetTickTempl(char dir, const char *t)
 	mglAxis &aa = (dir=='x' ? ax : (dir=='y' ? ay : (dir=='z' ? az : ac)));
 
 	if(aa.f==1)	aa.f = 0;	// remove time ticks
-	if(!t || !t[0])	aa.t[0]=0;
-	else if(mbstowcs(0,t,0)<256) mbstowcs(aa.t,t,256);
+	if(!t || !t[0])	aa.t.clear();
+	else	MGL_TO_WCS(t,aa.t=wcs);
 }
 //-----------------------------------------------------------------------------
 MGL_NO_EXPORT double mgl_adj_val(double v,mreal *ds=0)
@@ -245,10 +245,10 @@ void mglCanvas::SetTickTime(char dir, mreal d, const char *t)
 	}
 
 	aa.ds = 0;	aa.dv = d;	aa.f = 1;	aa.txt.clear();
-	if(mbstowcs(0,t,0)<256) mbstowcs(aa.t,t,256);
+	MGL_TO_WCS(t,aa.t=wcs);
 
 	if(strchr("xyztuvw",aa.ch))
-		aa.org = mglPoint(GetOrgX(aa.ch), GetOrgY(aa.ch), GetOrgZ(aa.ch));
+		aa.org = mglPoint(GetOrgX(aa.ch,aa.inv), GetOrgY(aa.ch,aa.inv), GetOrgZ(aa.ch,aa.inv));
 	if(aa.ch=='x')	aa.v0 = aa.org.x;
 	if(aa.ch=='y')	aa.v0 = aa.org.y;
 	if(aa.ch=='z')	aa.v0 = aa.org.z;
@@ -262,13 +262,13 @@ void mglCanvas::SetTickTime(char dir, mreal d, const char *t)
 	if(v0+aa.dv!=v0 && v1+aa.dv!=v1)	for(v=v0;v<=v1;v+=aa.dv)
 	{
 		tt = v;	tm tp;		mgl_localtime(&tt, &tp, get(MGL_USE_GMTIME));
-		wcsftime(buf,64,aa.t,&tp);	aa.AddLabel(buf,v);
+		wcsftime(buf,64,aa.t.c_str(),&tp);	aa.AddLabel(buf,v);
 	}
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::AdjustTicks(const char *dir, bool force)
 {
-	if(force)	SetTuneTicks(-1);
+	if(force)	SetTuneTicks(3);
 	UpdateAxis();
 	if(strchr(dir,'x') || strchr(dir,'X'))	// NOTE dir have to be non-NULL here !!!
 	{	if(force)	ax.d=0;	AdjustTicks(ax,fx!=0);	}
@@ -303,7 +303,7 @@ void mglCanvas::AdjustTicks(mglAxis &aa, bool ff)
 int MGL_NO_EXPORT mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 {
 	int kind = 0;
-	if(fabs(a-b)<=0.001*fabs(a))
+	if(fabs(a-b)<=0.01*fabs(a))
 	{
 		kind = 1;
 		v = fabs(a-b);
@@ -313,7 +313,7 @@ int MGL_NO_EXPORT mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 			kind=3;		v=mgl_ipow(10.,k);
 			mglprintf(s, 32, L" (@{\\times{}10^{%d}})", k);
 		}
-		if(v<0.02f)
+		else if(v<0.02f)
 		{
 			int k=int(log10(v)-0.01)-1;
 			kind=3;		v=mgl_ipow(10.,k);
@@ -330,7 +330,7 @@ int MGL_NO_EXPORT mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 			v=mgl_ipow(10.,k);
 			mglprintf(s, 32, L" \\times 10^{%d}", k);
 		}
-		if(v<=1e-3f)
+		else if(v<=1e-3f)
 		{
 			kind = 2;
 			int k=int(log10(v)-0.01)-1;
@@ -341,33 +341,50 @@ int MGL_NO_EXPORT mgl_tick_ext(mreal a, mreal b, wchar_t s[32], mreal &v)
 	return kind;
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_tick_text(mreal z, mreal z0, mreal d, mreal v, int kind, wchar_t str[64], bool tune)
+std::wstring MGL_NO_EXPORT mgl_format(mreal v1, mreal v2, bool zero)
+{
+	v1=fabs(v1);	v2=fabs(v2);	if(v1>v2)	v2=v1;
+	wchar_t str[5]=L"%.3g";
+	int prec=int(2-log10(v2));	if(prec<0)	prec=0;
+	if(v2<=0.001 || v2>=10000)
+	{	str[2] = '2';	str[3]='e';	}
+	else if(zero)
+	{	str[2] = '0'+prec;	str[3]='f';	}
+	return str;
+}
+//-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_tick_text(mreal z, mreal z0, mreal d, std::wstring fmt, mreal v, int kind, wchar_t str[64])
 {
 	mreal u = fabs(z)<d ? 0:z;
+	if((kind&1) && z>z0)	u = fabs(z-z0)<d ? 0:(z-z0);
+	if(kind==2 || (kind==3 && z>z0))	u /= v;
+	if(kind&1)	fmt = z>z0?L"@{(+"+fmt+L")}":L"%g";
+	mglprintf(str,64,fmt.c_str(), u);
+	
+/*	mreal u = fabs(z)<d ? 0:z;
 	if((kind&1) && z>z0)	u = fabs(z-z0)<d ? 0:(z-z0);
 	if(kind==2 || (kind==3 && z>z0))	u /= v;
 	if((kind&1) && z>z0)
 	{
 		size_t n1,n2;
 		mglprintf(str, 64, L"@{(+%.2g)}",u);
-//		mglprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);
-		n1=wcslen(str);	mglprintf(str, 64, L"@{(+%g)}",u);	n2=wcslen(str);
-//		if(n1<n2)	mglprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);
+		n1=mgl_wcslen(str);	mglprintf(str, 64, L"@{(+%g)}",u);	n2=mgl_wcslen(str);
 		if(n1<n2)	mglprintf(str, 64, L"@{(+%.2g)}",u);
 	}
 	else
 	{
 		size_t n1,n2;
-		mglprintf(str, 64, fabs(u)<1 ? L"%.2g" :  L"%.3g",u);
-		n1=wcslen(str);	mglprintf(str, 64, L"%g",u);	n2=wcslen(str);
-		if(n1<n2 && tune)	mglprintf(str, 64, fabs(u)<1 ? L"%.2g" :  L"%.3g",u);
-	}
+		mglprintf(str, 64, fabs(u)<1 ? L"%.2g" : L"%.3g",u);
+		n1=mgl_wcslen(str);	mglprintf(str, 64, L"%g",u);	n2=mgl_wcslen(str);
+		if(n1<n2 && tune)	mglprintf(str, 64, fabs(u)<1 ? L"%.2g" : L"%.3g",u);
+	} 
+*/
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::LabelTicks(mglAxis &aa)
 {
 	if(strchr("xyztuvw",aa.ch))
-		aa.org = mglPoint(GetOrgX(aa.ch), GetOrgY(aa.ch), GetOrgZ(aa.ch));
+		aa.org = mglPoint(GetOrgX(aa.ch,aa.inv), GetOrgY(aa.ch,aa.inv), GetOrgZ(aa.ch,aa.inv));
 	if(aa.ch=='x')	aa.v0 = aa.org.x;
 	if(aa.ch=='y')	aa.v0 = aa.org.y;
 	if(aa.ch=='z')	aa.v0 = aa.org.z;
@@ -409,7 +426,7 @@ void mglCanvas::LabelTicks(mglAxis &aa)
 	{
 		int kind=0;
 		wchar_t s[32]=L"";
-		if(aa.t[0]==0 && TuneTicks) kind = mgl_tick_ext(aa.v2, aa.v1, s, w);
+		if(aa.t.empty() && TuneTicks) kind = mgl_tick_ext(aa.v2, aa.v1, s, w);
 		if((TuneTicks&1)==0 && kind==2)	kind=0;
 		if((TuneTicks&2)==0 && kind!=2)	kind=0;
 
@@ -419,12 +436,21 @@ void mglCanvas::LabelTicks(mglAxis &aa)
 		else
 		{	v1 = aa.v1;		v0 = v0 - aa.dv*floor((v0-aa.v2)/aa.dv+1e-3);	}
 
+		std::wstring fmt;
+		switch(kind)
+		{
+			case 1:	fmt=mgl_format(0,v1-v0,TuneTicks&4);	break;
+			case 2:	fmt=mgl_format(v0/w,v1/w,TuneTicks&4);	break;
+			case 3:	fmt=mgl_format(0,(v1-v0)/w,TuneTicks&4);break;
+			default:fmt=mgl_format(v0,v1,TuneTicks&4);	break;
+		}
+
 		if(v0+aa.dv!=v0 && v1+aa.dv!=v1)	for(v=v0;v<=v1;v+=aa.dv)
 		{
-			if(aa.t[0])
-				mglprintf(buf, 64, aa.t, fabs(v)<aa.dv/100 ? 0 : v);
+			if(aa.t.empty())
+				mgl_tick_text(v,v0,aa.dv/100,fmt,w,kind,buf);
 			else
-				mgl_tick_text(v,v0,aa.dv/100,w,kind,buf,TuneTicks);
+				mglprintf(buf, 64, aa.t.c_str(), fabs(v)<aa.dv/100 ? 0 : v);
 			mgl_wcstrim(buf);	aa.AddLabel(buf,v);
 		}
 		if(kind&2)	aa.AddLabel(s,FactorPos*(aa.v2-aa.v1)+aa.v1);
@@ -433,16 +459,18 @@ void mglCanvas::LabelTicks(mglAxis &aa)
 //-----------------------------------------------------------------------------
 void mglCanvas::Axis(const char *dir, const char *stl, const char *opt)
 {
-	if(!dir || !dir[0])	dir="xyz";
-	bool text = !strchr(dir,'_');
+	bool text = !(mglchr(dir,'_') || mglchr(dir,'~'));
+	bool inv = mglchr(dir,'^');
+	bool adjust = mglchr(stl,'a');
+	bool ret = get(MGL_ENABLE_RTEXT);
+	if(mglchr(dir,'U'))	clr(MGL_ENABLE_RTEXT);
+
 	const char *ar = "AKDTVISO";
 	char arr=0;
 	for(size_t i=0;i<strlen(ar);i++)
 		if(strchr(dir,ar[i]))	{	arr=ar[i];	break;	}
-	bool adjust = mglchr(stl,'a');
+	if(!mglchrs(dir,"xXyYzZ"))	dir="xyz";
 
-	bool ret = get(MGL_ENABLE_RTEXT);
-	if(strchr(dir,'U'))	clr(MGL_ENABLE_RTEXT);
 	SaveState(opt);
 	AdjustTicks(dir,adjust);
 	LoadState();
@@ -450,8 +478,12 @@ void mglCanvas::Axis(const char *dir, const char *stl, const char *opt)
 	ax.pos = strchr(dir,'X') ? 'T':'t';
 	ay.pos = strchr(dir,'Y') ? 'T':'t';
 	az.pos = strchr(dir,'Z') ? 'T':'t';
-	if(strchr(dir,'X') || strchr(dir,'x'))	DrawAxis(ax, text, arr, stl, opt);
-	if(strchr(dir,'Z') || strchr(dir,'z'))	DrawAxis(az, text, arr, stl, opt);
+	ax.inv = ay.inv = az.inv = false;
+
+	if(strchr(dir,'X') || strchr(dir,'x'))
+	{	ax.inv = inv;	DrawAxis(ax, text, arr, stl, opt);	}
+	if(strchr(dir,'Z') || strchr(dir,'z'))
+	{	az.inv = inv;	DrawAxis(az, text, arr, stl, opt);	}
 	if((TernAxis&3))
 	{
 		mglAxis ty(ay);		ty.pos='t';	ty.ch='T';
@@ -460,7 +492,8 @@ void mglCanvas::Axis(const char *dir, const char *stl, const char *opt)
 		ty.dir = mglPoint(0,-1);		ty.org = mglPoint(0,1,ay.org.z);
 		DrawAxis(ty, text, arr, stl, opt);
 	}
-	else if(strchr(dir,'Y') || strchr(dir,'y')) 	DrawAxis(ay, text, arr, stl, opt);
+	else if(strchr(dir,'Y') || strchr(dir,'y'))
+	{	ay.inv = inv;	DrawAxis(ay, text, arr, stl, opt);	}
 	set(ret, MGL_ENABLE_RTEXT);
 }
 //-----------------------------------------------------------------------------
@@ -468,7 +501,7 @@ void mglCanvas::DrawAxis(mglAxis &aa, bool text, char arr,const char *stl,const
 {
 	SaveState(opt);
 	if(strchr("xyz",aa.ch))
-		aa.org = mglPoint(GetOrgX(aa.ch), GetOrgY(aa.ch), GetOrgZ(aa.ch));
+		aa.org = mglPoint(GetOrgX(aa.ch,aa.inv), GetOrgY(aa.ch,aa.inv), GetOrgZ(aa.ch,aa.inv));
 	if(aa.ch=='x')	aa.v0 = aa.org.x;
 	if(aa.ch=='y')	aa.v0 = aa.org.y;
 	if(aa.ch=='z')	aa.v0 = aa.org.z;
@@ -480,56 +513,65 @@ void mglCanvas::DrawAxis(mglAxis &aa, bool text, char arr,const char *stl,const
 	da = aa.a*(dv*aa.a);	db = aa.b*(dv*aa.b);
 	if(aa.v2<aa.v1)	{	da *= -1;	db *= -1;	}
 
-	register long i,j,k1,k2;
-	SetPenPal(mgl_have_color(stl) ? stl:AxisStl);
+	long k1,k2;
+	bool have_color=mgl_have_color(stl);
+	bool dif_color = !have_color && aa.dv==0 && strcmp(TickStl,SubTStl);
+	SetPenPal(have_color ? stl:AxisStl);
 	static int cgid=1;	StartGroup("Axis",cgid++);
-	
-	p = o + d*aa.v1;	k1 = AddPnt(p,CDef,q,-1,3);
-	for(i=1;i<31;i++)	// axis itself
+
+	p = o + d*aa.v1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
+	for(long i=1;i<31;i++)	// axis itself
 	{
 		p = o + d*(aa.v1+(aa.v2-aa.v1)*i/30.);
-		k2 = k1;	k1 = AddPnt(p,CDef,q,-1,3);
+		k2 = k1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
 		line_plot(k2,k1);
 	}
 	if(arr)
 	{
 		p = o + d*(aa.v1+(aa.v2-aa.v1)*1.05);
-		k2 = k1;	k1 = AddPnt(p,CDef,q,-1,3);
-		arrow_plot(k1,k2,arr);
+		k2 = k1;	k1 = AddPnt(&B, p,CDef,q,-1,3);
+		line_plot(k1,k2);	arrow_plot(k1,k2,arr);
 	}
 
 	k2 = aa.txt.size();
 	mreal v, u, v0 = mgl_isnan(aa.o) ? aa.v0 : aa.o;
-	if(k2>0)	for(i=0;i<k2;i++)
+	if(*TickStl && !have_color)	SetPenPal(TickStl);
+	if(k2>0)	for(long i=0;i<k2;i++)
 	{
 		v = aa.txt[i].val;	u = fabs(v);
-		if((v-aa.v2)*(v-aa.v1)<=0)	tick_draw(o+d*v, da, db, 0, stl);
+		if((v-aa.v2)*(v-aa.v1)<=0)	tick_draw(o+d*v, da, db, 0);
+		if(dif_color)	SetPenPal(SubTStl);
 		if(aa.dv==0 && aa.v2>aa.v1 && fabs(u-exp(M_LN10*floor(0.1+log10(u))))<0.01*u)
-			for(j=2;j<10 && v*j<aa.v2;j++)	tick_draw(o+d*(v*j),da,db,1,stl);
+			for(long j=2;j<10 && v*j<aa.v2;j++)	tick_draw(o+d*(v*j),da,db,1);
 		if(aa.dv==0 && aa.v2<aa.v1 && fabs(u-exp(M_LN10*floor(0.1+log10(u))))<0.01*u)
-			for(j=2;j<10 && v*j<aa.v1;j++)	tick_draw(o+d*(v*j),da,db,1,stl);
+			for(long j=2;j<10 && v*j<aa.v1;j++)	tick_draw(o+d*(v*j),da,db,1);
+		if(dif_color)	SetPenPal(TickStl);
 	}
 	if(aa.ds>0 && !get(MGL_NOSUBTICKS))
 	{
 		if(aa.v2>aa.v1)	v0 = v0 - aa.ds*floor((v0-aa.v1)/aa.ds+1e-3);
 		else			v0 = v0 - aa.ds*floor((v0-aa.v2)/aa.ds+1e-3);
 		if(v0+aa.ds!=v0 && aa.v2+aa.ds!=aa.v2)
+		{
+			if(*SubTStl && !have_color)	SetPenPal(SubTStl);
 			for(v=v0;(v-aa.v2)*(v-aa.v1)<=0;v+=aa.ds)
-				tick_draw(o+d*v,da,db,1,stl);
+				tick_draw(o+d*v,da,db,1);
+		}
 	}
-	SetPenPal(mgl_have_color(stl) ? stl:AxisStl);
+	if(!have_color)	SetPenPal(AxisStl);
 	if(text)	DrawLabels(aa);
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
-void mglCanvas::DrawLabels(mglAxis &aa)
+void mglCanvas::DrawLabels(mglAxis &aa, bool inv, const mglMatrix *M)
 {
 	if(strchr("xyz",aa.ch))
-		aa.org = mglPoint(GetOrgX(aa.ch), GetOrgY(aa.ch), GetOrgZ(aa.ch));
+		aa.org = mglPoint(GetOrgX(aa.ch,aa.inv), GetOrgY(aa.ch,aa.inv), GetOrgZ(aa.ch,aa.inv));
 	mglPoint d = aa.dir, o = aa.org;	// "transverse" org
 	if(strchr("xyz",aa.ch))	o -= d*(o*d);
 	mglPoint p,q, s=(Min+Max)/2, nn;
 	s = s - d*(s*d);
+	if(M==0)	M=&B;
 
 	register long i,n = aa.txt.size();
 	char pos[4]="t:C";
@@ -540,41 +582,44 @@ void mglCanvas::DrawLabels(mglAxis &aa)
 	for(i=0;i<n;i++)
 	{
 		w[i] = TextWidth(aa.txt[i].text.c_str(),FontDef,-1);
-		kk[i] = AddPnt(o+d*aa.txt[i].val,-1,d,0,7);
+		kk[i] = AddPnt(M, o+d*aa.txt[i].val,-1,d,0,7);
 	}
 
-	for(l=0,c=1e7,i=0;i<n-1;i++)
+	for(l=0,c=INFINITY,i=0;i<n-1;i++)
 	{
 		// exclude factors
 		if(aa.ch!='c' && (aa.txt[i].val<aa.v1 || aa.txt[i+1].val<aa.v1 || aa.txt[i].val>aa.v2 || aa.txt[i+1].val>aa.v2))
 			continue;
+		if(kk[i]<0 || kk[i+1]<0)	continue;
 		v = (GetPntP(kk[i+1])-GetPntP(kk[i])).norm();	// distance between ticks
 		vv = (w[i]+w[i+1])/2;	// length of labels
 		if(v>0 && l < vv/v)	l = vv/v;
 		if(c>v)	c = v;
 	}
 	if(get(MGL_ENABLE_RTEXT) && get(MGL_TICKS_ROTATE) && l>1 && c>0)	// try rotate first
-	{	tet = c>1.1*h ? asin(1.1*h/c) : M_PI/2;	pos[2]=aa.ch=='c'?'R':'L';
+	{	tet = c>1.1*h ? asin(1.1*h/c) : M_PI/2;	pos[2]=(aa.ch=='c' && !inv)?'R':'L';
 		l=0.99*h/sin(tet)/c;	for(i=0;i<n;i++)	w[i]=l*c;	}
 	// TODO: do clever points exclusion (i.e. longest and so on)
 	long k = get(MGL_TICKS_SKIP) ? 1+l : 1;
 	if(n>0)	for(i=0;i<n;i++)
 	{
-		if(kk[i]<0)	continue;	// should be never here?!
+		if(kk[i]<0)	continue;
 		c = aa.txt[i].val;
 		if(get(MGL_NO_ORIGIN) && c==aa.v0)	continue;
 		if(c>aa.v1 && c<aa.v2 && i%k!=0)	continue;
-		p = o+d*c;	nn = (s-o)/(Max-Min);	ScalePoint(p,nn);
+		p = o+d*c;	nn = (s-o)/(Max-Min);	ScalePoint(&B,p,nn);
 		mglPnt &qq = Pnt[kk[i]];
 		mreal ux=qq.u*cos(tet) + qq.v*sin(tet), uy=qq.v*cos(tet) - qq.u*sin(tet);
+		if(ux==0)	uy = fabs(uy);
 		qq.u = ux;	qq.v = uy;
 
-		if((!get(MGL_ENABLE_RTEXT) || tet) && nn.x!=0)	pos[2] = nn.x<0 ? 'L':'R';
+		if((!get(MGL_ENABLE_RTEXT) || tet) && nn.x!=0 && aa.ch!='c')	pos[2] = nn.x<0 ? 'L':'R';
 //		if(tet && nn.x==0)	pos[2] = 'R';
 		if(aa.ch=='c' && aa.txt[i].text[0]==' ')	qq.u = qq.v = NAN;
 		int ts = 1;
 		if(!get(MGL_DISABLE_SCALE))	ts = sign(qq.v*nn.x-qq.u*nn.y)*sign(aa.v2-aa.v1);
-		if(aa.ch=='c')	ts=(aa.ns==0 || aa.ns==3)?1:-1;
+		if(aa.ch=='c')	ts=inv?-1:1;	// use manual settings by inv argument
+//		else if(ux==0 && uy<0)	ts *= -1;
 		if(aa.ch=='T')	ts *= -1;
 		if(aa.pos=='T')	ts *= -1;
 		pos[0] = ts>0 ? 't':'T';
@@ -587,7 +632,7 @@ void mglCanvas::DrawLabels(mglAxis &aa)
 char mglCanvas::GetLabelPos(mreal c, long kk, mglAxis &aa)
 {
 	if(strchr("xyz",aa.ch))
-		aa.org = mglPoint(GetOrgX(aa.ch), GetOrgY(aa.ch), GetOrgZ(aa.ch));
+		aa.org = mglPoint(GetOrgX(aa.ch,aa.inv), GetOrgY(aa.ch,aa.inv), GetOrgZ(aa.ch,aa.inv));
 	mglPoint d = aa.dir, o = aa.org;	// "transverse" org
 	if(strchr("xyz",aa.ch))	o -= d*(o*d);
 	mglPoint p,q, s=(Min+Max)/2, nn;
@@ -596,10 +641,10 @@ char mglCanvas::GetLabelPos(mreal c, long kk, mglAxis &aa)
 	int ts = 1;
 	if(aa.ch=='c')	ts=(aa.ns==0 || aa.ns==3)?1:-1;
 	if(aa.ch=='T')	ts=-1;
-	
-	p = o+d*c;	nn = (s-o)/(Max-Min);	ScalePoint(p,nn);
+
+	p = o+d*c;	nn = (s-o)/(Max-Min);	ScalePoint(&B,p,nn);
 	mglPnt &qq = Pnt[kk];
-	
+
 	if(aa.ch=='c')	qq.u = qq.v = NAN;
 	if(!get(MGL_DISABLE_SCALE))	ts = sign(qq.v*nn.x-qq.u*nn.y)*sign(aa.v2-aa.v1);
 	if(aa.ch=='T')	ts *= -1;
@@ -607,7 +652,7 @@ char mglCanvas::GetLabelPos(mreal c, long kk, mglAxis &aa)
 	return ts>0 ? 't':'T';
 }
 //-----------------------------------------------------------------------------
-void mglCanvas::tick_draw(mglPoint o, mglPoint d1, mglPoint d2, int f, const char *stl)
+void mglCanvas::tick_draw(mglPoint o, mglPoint d1, mglPoint d2, int f)
 {
 	if(TickLen==0)	return;
 	// try to exclude ticks out of axis range
@@ -615,16 +660,12 @@ void mglCanvas::tick_draw(mglPoint o, mglPoint d1, mglPoint d2, int f, const cha
 		return;
 	mreal v = font_factor*TickLen/sqrt(1.f+f*st_t);
 	mglPoint p=o;
-	long k1,k2,k3=mgl_have_color(stl);
 
-	if(*TickStl && !f)	SetPenPal(k3 ? stl:TickStl);
-	if(*SubTStl && f)	SetPenPal(k3 ? stl:SubTStl);
-
-	ScalePoint(o, d1, false);	d1.Normalize();
-	ScalePoint(p, d2, false);	d2.Normalize();
-	k2 = AddPnt(p, CDef, mglPoint(NAN), 0, 0);
-	p += d1*v;	k1 = AddPnt(p, CDef, mglPoint(NAN), 0, 0);
-	p = o+d2*v;	k3 = AddPnt(p, CDef, mglPoint(NAN), 0, 0);
+	ScalePoint(&B,o, d1, false);	d1.Normalize();
+	ScalePoint(&B,p, d2, false);	d2.Normalize();
+	long k2 = AddPnt(&B, p, CDef, mglPoint(NAN), 0, 0);
+	long k1 = AddPnt(&B, p+d1*v, CDef, mglPoint(NAN), 0, 0);
+	long k3 = AddPnt(&B, p+d2*v, CDef, mglPoint(NAN), 0, 0);
 	line_plot(k1,k2);	line_plot(k2,k3);
 }
 //-----------------------------------------------------------------------------
@@ -647,11 +688,11 @@ void mglCanvas::DrawGrid(mglAxis &aa)
 	mglPoint pp[8]={Min,Min,Min,Min,Max,Max,Max,Max},nan=mglPoint(NAN), oo[8], org=Min;
 	pp[1].x=Max.x;	pp[2].y=Max.y;	pp[3].z=Max.z;
 	pp[4].x=Min.x;	pp[5].y=Min.y;	pp[6].z=Min.z;
-	mreal zm=1e5;
+	mreal zm=INFINITY;
 	memcpy(oo,pp,8*sizeof(mglPoint));
 	for(int i=0;i<8;i++)	// find deepest point
 	{
-		ScalePoint(pp[i],nan,false);
+		ScalePoint(&B,pp[i],nan,false);
 		if(pp[i].z<zm)	{	zm=pp[i].z;	org=oo[i];	}
 	}
 	if(Org.x==Org.x) 	org.x = Org.x;
@@ -669,21 +710,21 @@ void mglCanvas::DrawGrid(mglAxis &aa)
 	if(n>0)	for(i=0;i<n;i++)
 	{
 		q = oa+d*aa.txt[i].val;	p = q+da1;	// lines along 'a'
-		k1 = AddPnt(p,CDef);
+		k1 = AddPnt(&B, p,CDef);
 		for(j=1;j<31;j++)
 		{
 			v = j/30.;
 			p = q+da1*(1-v)+da2*v;
-			k2 = k1;	k1 = AddPnt(p,CDef);
+			k2 = k1;	k1 = AddPnt(&B, p,CDef);
 			line_plot(k2,k1);
 		}
 		q = ob+d*aa.txt[i].val;	p = q+db1;	// lines along 'b'
-		k1 = AddPnt(p,CDef);
+		k1 = AddPnt(&B, p,CDef);
 		for(j=1;j<31;j++)
 		{
 			v = j/30.;
 			p = q+db1*(1-v)+db2*v;
-			k2 = k1;	k1 = AddPnt(p,CDef);
+			k2 = k1;	k1 = AddPnt(&B, p,CDef);
 			line_plot(k2,k1);
 		}
 	}
@@ -691,18 +732,12 @@ void mglCanvas::DrawGrid(mglAxis &aa)
 //-----------------------------------------------------------------------------
 void mglCanvas::Label(char dir, const char *str, mreal pos, const char *opt)
 {
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	Labelw(dir, wcs, pos, opt);
-	delete []wcs;
+	MGL_TO_WCS(str,Labelw(dir, wcs, pos, opt));
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt)
 {
-	mreal shift =  SaveState(opt);	if(mgl_isnan(shift))	shift=0;
-	mreal t=0, x0, y0, z0;
-	x0 = GetOrgX(dir);	y0 = GetOrgY(dir);	z0 = GetOrgZ(dir);
+	mreal shift =  SaveState(opt), t=0;	if(mgl_isnan(shift))	shift=0;
 	mglPoint p,q;
 	mglAxis *aa=0;
 
@@ -713,14 +748,16 @@ void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt
 		AdjustTicks(ax,fx!=0);	aa = &ax;
 		if(ax.dv)	t = (Min.x+Max.x+pos*(Max.x-Min.x))/2;
 		else	t = Min.x*pow(Max.x/Min.x, (pos+1)/2);
-		p = mglPoint(t,y0,z0);	q = mglPoint(1,0,0);	shift += ax.sh;
+		p = mglPoint(t, GetOrgY(ax.ch,ax.inv), GetOrgZ(ax.ch,ax.inv));
+		q = mglPoint(1,0,0);	shift += ax.sh;
 	}
 	if(dir=='y' && !(TernAxis&3))
 	{
 		AdjustTicks(ay,fy!=0);	aa = &ay;
 		if(ay.dv)	t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
 		else	t = Min.y*pow(Max.y/Min.y, (pos+1)/2);
-		p = mglPoint(x0,t,z0);	q = mglPoint(0,1,0);	shift += ay.sh;
+		p = mglPoint(GetOrgX(ay.ch,ay.inv), t, GetOrgZ(ay.ch,ay.inv));
+		q = mglPoint(0,1,0);	shift += ay.sh;
 		if(TernAxis&3)
 		{
 			q = mglPoint(-1,1,0);	pos=-pos;
@@ -732,7 +769,8 @@ void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt
 		AdjustTicks(ty,fy!=0);	aa = &ty;
 		if(ty.dv)	t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
 		else	t = Min.y*pow(Max.y/Min.y, (pos+1)/2);
-		p = mglPoint(x0,t,z0);	q = mglPoint(0,1,0);	shift += ty.sh;
+		p = mglPoint(GetOrgX(ty.ch,ty.inv), t, GetOrgZ(ty.ch,ty.inv));
+		q = mglPoint(0,1,0);	shift += ty.sh;
 		if(TernAxis&3)
 		{
 			q = mglPoint(-1,1,0);	pos=-pos;
@@ -744,21 +782,26 @@ void mglCanvas::Labelw(char dir, const wchar_t *text, mreal pos, const char *opt
 		AdjustTicks(ty,fy!=0);	pos = -pos;	aa = &ty;
 		if(ty.dv)	t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
 		else	t = Min.y*pow(Max.y/Min.y, (pos+1)/2);
-		p = mglPoint(x0,t,z0);	q = mglPoint(0,1,0);	shift += ty.sh;
+		p = mglPoint(GetOrgX(ty.ch,ty.inv), t, GetOrgZ(ty.ch,ty.inv));
+		q = mglPoint(0,1,0);	shift += ty.sh;
 	}
 	if(dir=='z')
 	{
 		AdjustTicks(az,fz!=0);	aa = &az;
 		if(az.dv)	t = (Min.z+Max.z+pos*(Max.z-Min.z))/2;
 		else	t = Min.z*pow(Max.z/Min.z, (pos+1)/2);
-		p = mglPoint(x0,y0,t);	q = mglPoint(0,0,1);	shift += az.sh;
+		p = mglPoint(GetOrgX(az.ch,az.inv), GetOrgY(az.ch,az.inv), t);
+		q = mglPoint(0,0,1);	shift += az.sh;
+	}
+	if(aa)
+	{
+		char font[64],ff[3]=":C";	memset(font,0,64);
+		if(pos<-0.2)	ff[1]='L';	if(pos>0.2)	ff[1]='R';
+		strncpy(font,FontDef,63);	strcat(font,ff);
+		long kk = AddPnt(&B, p,-1,q,0,7);	ff[1]=0;
+		ff[0] = GetLabelPos(t, kk, *aa);	strcat(font,ff);
+		text_plot(kk,text,font,-1.4,0.35+shift);
 	}
-	char font[64],ff[3]=":C";	memset(font,0,64);
-	if(pos<-0.2)	ff[1]='L';	if(pos>0.2)	ff[1]='R';
-	strncpy(font,FontDef,63);	strcat(font,ff);
-	long kk = AddPnt(p,-1,q,0,7);	ff[1]=0;
-	ff[0] = GetLabelPos(t, kk, *aa);	strcat(font,ff);
-	text_plot(kk,text,font,-1.4,0.35+shift);
 	LoadState();
 }
 //-----------------------------------------------------------------------------
@@ -772,8 +815,10 @@ void mglCanvas::Box(const char *col, bool ticks)
 	Axis("xyz_",col);
 	if(TernAxis&1)
 	{
-		Org.z=Max.z;	Org.x=Max.x;	Axis("xz_",col);
-		Org.x=Min.x;	Org.y=Max.y;	Axis("z_",col);
+		Org.x=Max.x;	Org.y=Min.y;	Org.z=Max.z;
+		DrawAxis(ax, false, 0,col);	DrawAxis(az, false, 0,col);
+		Org.x=Min.x;	Org.y=Max.y;	Org.z=Max.z;
+		DrawAxis(az, false, 0,col);
 
 		mglAxis ty(ay);				ty.ch='T';
 		ty.dir = mglPoint(-1,1);	ty.org = mglPoint(1,0,Max.z);
@@ -798,30 +843,52 @@ void mglCanvas::Box(const char *col, bool ticks)
 		Org.z=Min.z;	Axis("xy_",col);
 		Org.x=Min.x;	DrawAxis(az,0,0,col);
 		Org.x=Max.x;	Org.y=Min.y;	DrawAxis(az,0,0,col);
-		if(col && strchr(col,'@'))
+		if(mglchr(col,'@'))
 		{
 			// edge points
 			mglPoint p[8]={Min,Min,Min,Min,Max,Max,Max,Max},nan=mglPoint(NAN),oo[8];
 			p[1].x=Max.x;	p[2].y=Max.y;	p[3].z=Max.z;
 			p[4].x=Min.x;	p[5].y=Min.y;	p[6].z=Min.z;
-			mreal zm=1e5;	int im=0;
+			mreal zm=INFINITY;	int im=0;
 			memcpy(oo,p,8*sizeof(mglPoint));
 			for(int i=0;i<8;i++)	// find deepest point
 			{
-				ScalePoint(p[i],nan,false);
+				ScalePoint(&B,p[i],nan,false);
 				if(p[i].z<zm)	{	zm=p[i].z;	im=i;	}
 			}
 			// now draw faces
-			char color[5]="{y9}";
-			register int i;	// first color used for faces, last one for edges
-			for(i=0;col[i];i++)	if(strchr(MGL_COLORS,col[i]))
+			char color[10]="{y9}";
+			for(int i=0;col[i];i++)
 			{
-				if(i>1 && col[i-1]=='{')	{	color[1]=col[i];	color[2]=col[i+1];	break;	}
-				else	{	color[0]=col[i];	color[1]=0;	break;	}
+				if(col[i]=='{' && col[i+1]=='x')
+				{	memcpy(color,col+i,9);	color[9]=0;	break;	}
+				if(strchr(MGL_COLORS,col[i]))
+				{
+					if(i>1 && col[i-1]=='{')	{	color[1]=col[i];	color[2]=col[i+1];	break;	}
+					else	{	color[0]=col[i];	color[1]=0;	break;	}
+				}
 			}
-			mgl_facex(this, oo[im].x, Min.y, Min.z, Max.y-Min.y, Max.z-Min.z, color,0,0);
-			mgl_facey(this, Min.x, oo[im].y, Min.z, Max.x-Min.x, Max.z-Min.z, color,0,0);
-			mgl_facez(this, Min.x, Min.y, oo[im].z, Max.x-Min.x, Max.y-Min.y, color,0,0);
+			long *pos = new long[3*31*31];
+			SetPenPal(color);
+			mreal dx = (Max.x-Min.x)/30, dy = (Max.y-Min.y)/30, dz = (Max.z-Min.z)/30;
+			for(long i=0;i<31;i++)	for(long j=0;j<31;j++)
+			{
+				register long i0=3*(i+31*j);
+				pos[i0]   = AddPnt(mglPoint(oo[im].x,Min.y+dy*i,Min.z+dz*j));
+				pos[i0+1] = AddPnt(mglPoint(Min.x+dx*i,oo[im].y,Min.z+dz*j));
+				pos[i0+2] = AddPnt(mglPoint(Min.x+dx*i,Min.y+dy*j,oo[im].z));
+			}
+			for(long i=0;i<30;i++)	for(long j=0;j<30;j++)
+			{
+				register long i0=3*(i+31*j);
+				quad_plot(pos[i0],pos[i0+3],pos[i0+93],pos[i0+96]);
+				quad_plot(pos[i0+1],pos[i0+4],pos[i0+94],pos[i0+97]);
+				quad_plot(pos[i0+2],pos[i0+5],pos[i0+95],pos[i0+98]);
+			}
+			delete []pos;
+// 			mgl_facex(this, oo[im].x, Min.y, Min.z, Max.y-Min.y, Max.z-Min.z, color,0,0);
+// 			mgl_facey(this, Min.x, oo[im].y, Min.z, Max.x-Min.x, Max.z-Min.z, color,0,0);
+// 			mgl_facez(this, Min.x, Min.y, oo[im].z, Max.x-Min.x, Max.y-Min.y, color,0,0);
 		}
 	}
 	EndGroup();
@@ -831,17 +898,19 @@ void mglCanvas::Box(const char *col, bool ticks)
 void mglCanvas::Colorbar(const char *sch)
 {
 	bool in = mglchr(sch,'I');
-	mreal s=1/B.pf, x=1, y=0;
-	if(mglchr(sch,'>'))	{	x=in?(1+s)/2:1;	y=0;	}
-	if(mglchr(sch,'<'))	{	x=in?(1-s)/2:0;	y=0;	}
-	if(mglchr(sch,'^'))	{	x=0;	y=in?(1+s)/2:1;	}
-	if(mglchr(sch,'_'))	{	x=0;	y=in?(1-s)/2:0;	}
+	mreal sx = (fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B.pf/B1.b[0], x=1;
+	mreal sy = (fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B.pf/B1.b[4], y=0;
+	if(mglchr(sch,'>'))	{	x=in?(1+sx)/2:1;	y=0;	}
+	if(mglchr(sch,'<'))	{	x=in?(1-sx)/2:0;	y=0;	}
+	if(mglchr(sch,'^'))	{	x=0;	y=in?(1+sy)/2:1;	}
+	if(mglchr(sch,'_'))	{	x=0;	y=in?(1-sy)/2:0;	}
 	Colorbar(sch, x, y, 1, 1);
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Colorbar(const char *sch, mreal x, mreal y, mreal w, mreal h)
 {
 	bool in = mglchr(sch,'I');
+	bool text = !mglchr(sch,'~');
 	int where = 0;		// ‘0’ - right, ‘1’ - left, ‘2’ - above, ‘3’ - under
 	if(mglchr(sch,'>'))	where = in?1:0;
 	if(mglchr(sch,'<'))	where = in?0:1;
@@ -858,7 +927,7 @@ void mglCanvas::Colorbar(const char *sch, mreal x, mreal y, mreal w, mreal h)
 	{	v.Fill(log(-Min.c), log(-Max.c));	v.Modify("-exp(u)");	}
 	mreal *c=new mreal[n];
 	for(long i=0;i<n;i++)	c[i] = GetC(s,v.a[i]);
-	colorbar(&v, c, where, x, y, w, h);
+	colorbar(&v, c, where, x, y, w, h, text);
 	delete []c;
 	if(mglchr(sch,'A'))	Pop();
 }
@@ -866,17 +935,19 @@ void mglCanvas::Colorbar(const char *sch, mreal x, mreal y, mreal w, mreal h)
 void mglCanvas::Colorbar(HCDT v, const char *sch)
 {
 	bool in = mglchr(sch,'I');
-	mreal s=1/B.pf, x=1, y=0;
-	if(mglchr(sch,'>'))	{	x=in?(1+s)/2:1;	y=0;	}
-	if(mglchr(sch,'<'))	{	x=in?(1-s)/2:0;	y=0;	}
-	if(mglchr(sch,'^'))	{	x=0;	y=in?(1+s)/2:1;	}
-	if(mglchr(sch,'_'))	{	x=0;	y=in?(1-s)/2:0;	}
+	mreal sx = (fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B.pf/B1.b[0], x=1;
+	mreal sy = (fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B.pf/B1.b[4], y=0;
+	if(mglchr(sch,'>'))	{	x=in?(1+sx)/2:1;	y=0;	}
+	if(mglchr(sch,'<'))	{	x=in?(1-sx)/2:0;	y=0;	}
+	if(mglchr(sch,'^'))	{	x=0;	y=in?(1+sy)/2:1;	}
+	if(mglchr(sch,'_'))	{	x=0;	y=in?(1-sy)/2:0;	}
 	Colorbar(v, sch, x, y, 1, 1);
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Colorbar(HCDT v, const char *sch, mreal x, mreal y, mreal w, mreal h)
 {
 	bool in = mglchr(sch,'I');
+	bool text = !mglchr(sch,'~');
 	int where = 0;
 	if(mglchr(sch,'>'))	where = in?1:0;
 	if(mglchr(sch,'<'))	where = in?0:1;
@@ -890,22 +961,24 @@ void mglCanvas::Colorbar(HCDT v, const char *sch, mreal x, mreal y, mreal w, mre
 	int nc = GetNumPal(s*256);
 	mreal dc = nc>1 ? 1/(MGL_EPSILON*(nc-1)):0;
 	for(long i=0;i<v->GetNx();i++)	c[i] = s+i*dc;
-	colorbar(v, c, where, x, y, w, h);
+	colorbar(v, c, where, x, y, w, h, text);
 	delete []c;
 	if(mglchr(sch,'A'))	Pop();
 }
 //-----------------------------------------------------------------------------
-void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, mreal w, mreal h)
+void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, mreal w, mreal h, bool text)
 {
 	static int cgid=1;	StartGroup("Colorbar",cgid++);
-	register size_t i,n=vv->GetNx();
+	long n=vv->GetNx();
 	long n1,n2,n3,n4;
 	mreal d,s3=B.pf,ss=1/s3;		// NOTE: colorbar was wider ss=0.9;
 	mglPoint p1,p2;
+	mglMatrix M=B1;	M.pf=s3;	M.norot=true;
 
-	Push();	set(MGL_DISABLE_SCALE);	B=B1;	B.pf=s3;
+	set(MGL_DISABLE_SCALE);		// NOTE this make colorbar non-thread-safe!!!
 	x = s3*(2*x-1);	y = s3*(2*y-1);	w *= s3;	h *= s3;
-	for(i=0;i<n-1;i++)
+	mask = MGL_SOLID_MASK;
+	for(long i=0;i<n-1;i++)
 	{
 		d = GetA(vv->v(i))*2-1;
 		p1 = p2 = mglPoint((ss*d+1)*w+x, (ss*d+1)*h+y, s3);
@@ -916,7 +989,7 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 			case 3:	p1.y = y;	p2.y = y+0.1*h;	break;
 			default:p1.x = x-0.1*w;	p2.x = x;	break;
 		}
-		n1 = AddPnt(p1,c[i]);	n2 = AddPnt(p2,c[i]);
+		n1 = AddPnt(&M, p1,c[i]);	n2 = AddPnt(&M, p2,c[i]);
 		d = GetA(vv->v(i+1))*2-1;
 		p1 = p2 = mglPoint((ss*d+1)*w+x, (ss*d+1)*h+y, s3);
 		switch(where)
@@ -926,23 +999,23 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 			case 3:	p1.y = y;	p2.y = y+0.1*h;	break;
 			default:p1.x = x-0.1*w;	p2.x = x;	break;
 		}
-		n3 = AddPnt(p1,c[i]);	n4 = AddPnt(p2,c[i]);
+		n3 = AddPnt(&M, p1,c[i]);	n4 = AddPnt(&M, p2,c[i]);
 		quad_plot(n1,n2,n3,n4);
 	}
 	if(n<64)
 	{
-		wchar_t buf[64];
-		for(i=0;i<n;i++)
+		wchar_t buf[64];	ac.txt.clear();
+		for(long i=0;i<n;i++)
 		{
 			d = vv->v(i);
-			mglprintf(buf,64,ac.t[0]?ac.t:(fabs(d)<1 ? L"%.2g" :  L"%.3g"),d);
+			mglprintf(buf,64,ac.t.empty()?(fabs(d)<1 ? L"%.2g" :  L"%.3g"):ac.t.c_str(),d);
 			ac.AddLabel(buf,d);
 		}
 	}
 	else	{	UpdateAxis();	AdjustTicks(ac,fa!=0);	}
 	// hint for using standard label drawing function
 	SetPenPal(TickStl);
-	for(i=0;i<ac.txt.size();i++)
+	for(size_t i=0;i<ac.txt.size();i++)
 	{
 		d = ac.txt[i].val = GetA(ac.txt[i].val)*2-1;
 		p1 = p2 = mglPoint((ss*d+1)*w+x, (ss*d+1)*h+y, s3);
@@ -953,7 +1026,7 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 			case 3:	p1.y = y;	p2.y = y+0.1*h;	break;
 			default:p1.x = x-0.1*w;	p2.x = x;	break;
 		}
-		n1 = AddPnt(p1);	n2 = AddPnt(p2);
+		n1 = AddPnt(&M, p1);	n2 = AddPnt(&M, p2);
 		line_plot(n1,n2);
 	}
 	ac.dir = mglPoint(ss*w,ss*h,0);
@@ -966,7 +1039,10 @@ void mglCanvas::colorbar(HCDT vv, const mreal *c, int where, mreal x, mreal y, m
 		default:ac.dir.x = 0;	ac.org.x = x-0.1*w;	break;
 	}
 	SetPenPal(AxisStl);
-	ac.ns = where;	DrawLabels(ac);	// NOTE ns isn't used for colorbar
-	Pop();	clr(MGL_DISABLE_SCALE);	EndGroup();
+//	bool out = fabs(x)>1 && fabs(y)>1;
+	bool inv = where!=3 && where!=0;
+	ac.ns = where;
+	if(text)	DrawLabels(ac,inv,&M);	// NOTE ns isn't used for colorbar
+	clr(MGL_DISABLE_SCALE);	EndGroup();
 }
 //-----------------------------------------------------------------------------
