@@ -1,6 +1,6 @@
 /***************************************************************************
  * base.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -30,7 +30,7 @@ char *mgl_strdup(const char *s)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_create_cpp_font(HMGL gr, const wchar_t *how)
 {
-	unsigned long l=wcslen(how), i, n=0, m;
+	unsigned long l=mgl_wcslen(how), i, n=0, m;
 	wchar_t ch=*how;
 	const mglFont *f = gr->GetFont();
 	std::vector<wchar_t> s;	s.push_back(ch);
@@ -72,19 +72,18 @@ void MGL_EXPORT mgl_create_cpp_font(HMGL gr, const wchar_t *how)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_strtrim(char *str)
 {
-	char *c = mgl_strdup(str);
-	long k,n=strlen(str);
+	if(!str || *str==0)	return;
+	size_t n=strlen(str), k, i;
 	for(k=0;k<n;k++)	if(str[k]>' ')	break;
-	strcpy(c,&(str[k]));
-	n = strlen(c);
-	for(k=n-1;k>=0;k--)	if(c[k]>' ')	break;
-	c[k+1] = 0;
-	strcpy(str,c);	free(c);
+	for(i=n;i>k;i--)	if(str[i-1]>' ')	break;
+	memmove(str, str+k, (i-k));
+	str[i-k]=0;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_strlwr(char *str)
 {
-	for(long k=0;k<(long)strlen(str);k++)
+	register size_t k,l=strlen(str);
+	for(k=0;k<l;k++)
 		str[k] = (str[k]>='A' && str[k]<='Z') ? str[k]+'a'-'A' : str[k];
 }
 //-----------------------------------------------------------------------------
@@ -109,9 +108,13 @@ mglBase::mglBase()
 
 	InUse = 1;	SetQuality();	FaceNum = 0;
 	// Always create default palette txt[0] and default scheme txt[1]
-	Txt.reserve(3);
-	MGL_PUSH(Txt,mglTexture(MGL_DEF_PAL,-1),mutexTxt);
-	MGL_PUSH(Txt,mglTexture(MGL_DEF_SCH,1),mutexTxt);
+#pragma omp critical(txt)
+	{
+		mglTexture t1(MGL_DEF_PAL,-1), t2(MGL_DEF_SCH,1);
+		Txt.reserve(3);
+		MGL_PUSH(Txt,t1,mutexTxt);
+		MGL_PUSH(Txt,t2,mutexTxt);
+	}
 	memcpy(last_style,"{k5}-1\0",8);
 	MinS=mglPoint(-1,-1,-1);	MaxS=mglPoint(1,1,1);
 	fnt = new mglFont;	fnt->gr = this;	PrevState=NAN;
@@ -137,6 +140,7 @@ void mglBase::AddActive(long k,int n)
 	int h=GetHeight();
 	p.x = int(q.x);	p.y = h>1?h-1-int(q.y):int(q.y);
 	p.id = ObjId;	p.n = n;
+#pragma omp critical(act)
 	MGL_PUSH(Act,p,mutexAct);
 }
 //-----------------------------------------------------------------------------
@@ -146,29 +150,32 @@ int mglBase::GetHeight() const	{	return 1;	}
 //-----------------------------------------------------------------------------
 void mglBase::StartGroup(const char *name, int id)
 {
-	LightScale();
+	LightScale(&B);
 	char buf[128];
 	snprintf(buf,128,"%s_%d",name,id);
 	StartAutoGroup(buf);
 }
 //-----------------------------------------------------------------------------
-const char *mglWarn[mglWarnEnd] = {"data dimension(s) is incompatible",
-								"data dimension(s) is too small",
-								"minimal data value is negative",
-								"no file or wrong data dimensions",
-								"not enough memory",
-								"data values are zero",
-								"too many legend entries",
-								"no legend entries",
-								"slice value is out of range",
-								"number of contours is zero or negative",
-								"couldn't open file",
-								"light: ID is out of range",
-								"size(s) is zero or negative",
-								"format is not supported for that build",
-								"axis ranges are incompatible",
-								"pointer is NULL",
-								"not enough space for plot"};
+const char *mglWarn[mglWarnEnd] = {"data dimension(s) is incompatible",	//mglWarnDim
+								"data dimension(s) is too small",		//mglWarnLow
+								"minimal data value is negative",		//mglWarnNeg
+								"no file or wrong data dimensions",		//mglWarnFile
+								"not enough memory", 					//mglWarnMem
+								"data values are zero",					//mglWarnZero
+								"no legend entries",					//mglWarnLeg
+								"slice value is out of range",			//mglWarnSlc
+								"number of contours is zero or negative",//mglWarnCnt
+								"couldn't open file",					//mglWarnOpen
+								"light: ID is out of range",			//mglWarnLId
+								"size(s) is zero or negative",			//mglWarnSize
+								"format is not supported for that build",//mglWarnFmt
+								"axis ranges are incompatible",			//mglWarnTern
+								"pointer is NULL",						//mglWarnNull
+								"not enough space for plot",			//mglWarnSpc
+								"There are wrong argument(s) in script",//mglScrArg
+								"There are wrong command in script",	//mglScrCmd
+								"There are too long string in script",	//mglScrLong
+								"There are unbalanced ' in script"};	//mglScrStr
 //-----------------------------------------------------------------------------
 void mglBase::SetWarn(int code, const char *who)
 {
@@ -189,7 +196,7 @@ void mglBase::SetWarn(int code, const char *who)
 void mglGlyph::Create(long Nt, long Nl)
 {
 	if(Nt<0 || Nl<0)	return;
-	nt=Nt;	nl=Nl;	
+	nt=Nt;	nl=Nl;
 	if(trig)	delete []trig;
 	trig = nt>0?new short[6*nt]:0;
 	if(line)	delete []line;
@@ -208,22 +215,29 @@ long mglBase::AddGlyph(int s, long j)
 {
 	// first create glyph for current typeface
 	s = s&3;
-	mglGlyph g(fnt->GetNt(s,j), fnt->GetNl(s,j));	
+	mglGlyph g(fnt->GetNt(s,j), fnt->GetNl(s,j));
 	memcpy(g.trig, fnt->GetTr(s,j), 6*g.nt*sizeof(short));
 	memcpy(g.line, fnt->GetLn(s,j), 2*g.nl*sizeof(short));
 	// now let find the similar glyph
-	register size_t i;
-	for(i=0;i<Glf.size();i++)	if(g==Glf[i])	return i;
+	for(size_t i=0;i<Glf.size();i++)	if(g==Glf[i])	return i;
 	// if no one then let add it
-	MGL_PUSH(Glf,g,mutexGlf);	return Glf.size()-1;
+	long k;
+#pragma omp critical(glf)
+	{k=Glf.size();	MGL_PUSH(Glf,g,mutexGlf);}	return k;
 }
 //-----------------------------------------------------------------------------
 //		Add points to the buffer
 //-----------------------------------------------------------------------------
-long mglBase::AddPnt(mglPoint p, mreal c, mglPoint n, mreal a, int scl)
+long mglBase::AddPnt(const mglMatrix *mat, mglPoint p, mreal c, mglPoint n, mreal a, int scl)
 {
+	// scl=0 -- no scaling
+	// scl&1 -- usual scaling
+	// scl&2 -- disable NAN at scaling
+	// scl&4 -- ???
+	// scl&8 -- bypass palette for enabling alpha
 	if(mgl_isnan(c) || mgl_isnan(a))	return -1;
-	if(scl>0)	ScalePoint(p,n,!(scl&2));
+	bool norefr = mgl_isnan(n.x) && mgl_isnan(n.y) && !mgl_isnan(n.z);
+	if(scl>0)	ScalePoint(mat,p,n,!(scl&2));
 	if(mgl_isnan(p.x))	return -1;
 	a = (a>=0 && a<=1) ? a : AlphaDef;
 	c = (c>=0) ? c:CDef;
@@ -233,7 +247,9 @@ long mglBase::AddPnt(mglPoint p, mreal c, mglPoint n, mreal a, int scl)
 	{
 		q.x=q.xx=int(p.x*10)*0.1;	q.y=q.yy=int(p.y*10)*0.1;	q.z=q.zz=int(p.z*10)*0.1;
 		q.c=int(c*100)*0.01;	q.t=q.ta=int(a*100)*0.01;
-		q.u=int(n.x*100)*0.01;	q.v=int(n.y*100)*0.01;	q.w=int(n.z*100)*0.01;
+		q.u=mgl_isnum(n.x)?int(n.x*100)*0.01:NAN;
+		q.v=mgl_isnum(n.y)?int(n.y*100)*0.01:NAN;
+		q.w=mgl_isnum(n.z)?int(n.z*100)*0.01:NAN;
 	}
 	else
 	{
@@ -250,20 +266,25 @@ long mglBase::AddPnt(mglPoint p, mreal c, mglPoint n, mreal a, int scl)
 	q.c = ci+(q.c-ci)*(1-2*gap)+gap;
 	q.t = q.t*(1-2*gap)+gap;
 	q.ta = q.t;
-	
+
 	if(scl&8 && scl>0)	q.a=a;	// bypass palette for enabling alpha in Error()
 	if(!get(MGL_ENABLE_ALPHA))	{	q.a=1;	if(txt.Smooth!=2)	q.ta=1-gap;	}
-//	if(q.ta<0.005)	q.ta = 0.005;	// bypass OpenGL/OBJ/PRC bug
+	if(norefr)	q.v=0;
 	if(!get(MGL_ENABLE_LIGHT) && !(scl&4))	q.u=q.v=NAN;
-	MGL_PUSH(Pnt,q,mutexPnt);	return Pnt.size()-1;
+	if(mat->norot)	q.sub=-1;	// NOTE: temporary -- later should be mglInPlot here
+	long k;
+#pragma omp critical(pnt)
+	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
 }
 //-----------------------------------------------------------------------------
 long mglBase::CopyNtoC(long from, mreal c)
 {
 	if(from<0)	return -1;
 	mglPnt p=Pnt[from];
-	if(!mgl_isnan(c))	{	p.c=c;	p.t=0;	Txt[long(c)].GetC(c,0,p);	}
-	MGL_PUSH(Pnt,p,mutexPnt);	return Pnt.size()-1;
+	if(mgl_isnum(c))	{	p.c=c;	p.t=0;	Txt[long(c)].GetC(c,0,p);	}
+	long k;
+#pragma omp critical(pnt)
+	{k=Pnt.size();	MGL_PUSH(Pnt,p,mutexPnt);}	return k;
 }
 //-----------------------------------------------------------------------------
 long mglBase::CopyProj(long from, mglPoint p, mglPoint n)
@@ -272,61 +293,70 @@ long mglBase::CopyProj(long from, mglPoint p, mglPoint n)
 	mglPnt q=Pnt[from];
 	q.x=q.xx=p.x;	q.y=q.yy=p.y;	q.z=q.zz=p.z;
 	q.u = n.x;		q.v = n.y;		q.w = n.z;
-	MGL_PUSH(Pnt,q,mutexPnt);	return Pnt.size()-1;
+	long k;
+#pragma omp critical(pnt)
+	{k=Pnt.size();	MGL_PUSH(Pnt,q,mutexPnt);}	return k;
 }
 //-----------------------------------------------------------------------------
 void mglBase::Reserve(long n)
 {
 	if(TernAxis&4)	n*=4;
+#pragma omp critical(pnt)
 	Pnt.reserve(n);
+#pragma omp critical(prm)
+	Prm.reserve(n);
 }
 //-----------------------------------------------------------------------------
 //		Boundaries and scaling
-//---------------------------------------------------------------------------
-void mglBase::RecalcCRange()
+//-----------------------------------------------------------------------------
+bool mglBase::RecalcCRange()
 {
+	bool wrong=false;
 	if(!fa)
 	{	FMin.c = Min.c;	FMax.c = Max.c;	}
 	else
 	{
-		FMin.c = 1e30;	FMax.c = -1e30;
+		FMin.c = INFINITY;	FMax.c = -INFINITY;
 		register int i;
 		mreal a;
 		int n=30;
 		for(i=0;i<=n;i++)
 		{
 			a = fa->Calc(0,0,0,Min.c+i*(Max.c-Min.c)/n);
+			if(mgl_isbad(a))	wrong=true;
 			if(a<FMin.c)	FMin.c=a;
 			if(a>FMax.c)	FMax.c=a;
 		}
 	}
+	return wrong;
 }
 //-----------------------------------------------------------------------------
 void mglBase::RecalcBorder()
 {
 	ZMin = 1.;
+	bool wrong=false;
 	if(!fx && !fy && !fz)
 	{	FMin = Min;	FMax = Max;	}
 	else
 	{
-		FMin = mglPoint( 1e30, 1e30, 1e30);
-		FMax = mglPoint(-1e30,-1e30,-1e30);
+		FMin = mglPoint( INFINITY, INFINITY, INFINITY);
+		FMax = mglPoint(-INFINITY,-INFINITY,-INFINITY);
 		register int i,j;
 		int n=30;
 		for(i=0;i<=n;i++)	for(j=0;j<=n;j++)	// x range
 		{
-			SetFBord(Min.x, Min.y+i*(Max.y-Min.y)/n, Min.z+j*(Max.z-Min.z)/n);
-			SetFBord(Max.x, Min.y+i*(Max.y-Min.y)/n, Min.z+j*(Max.z-Min.z)/n);
+			if(SetFBord(Min.x, Min.y+i*(Max.y-Min.y)/n, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
+			if(SetFBord(Max.x, Min.y+i*(Max.y-Min.y)/n, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
 		}
 		for(i=0;i<=n;i++)	for(j=0;j<=n;j++)	// y range
 		{
-			SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y, Min.z+j*(Max.z-Min.z)/n);
-			SetFBord(Min.x+i*(Max.x-Min.x)/n, Max.y, Min.z+j*(Max.z-Min.z)/n);
+			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
+			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Max.y, Min.z+j*(Max.z-Min.z)/n))	wrong=true;
 		}
 		for(i=0;i<=n;i++)	for(j=0;j<=n;j++)	// x range
 		{
-			SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y+j*(Max.y-Min.y)/n, Min.x);
-			SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y+j*(Max.y-Min.y)/n, Max.z);
+			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y+j*(Max.y-Min.y)/n, Min.z))	wrong=true;
+			if(SetFBord(Min.x+i*(Max.x-Min.x)/n, Min.y+j*(Max.y-Min.y)/n, Max.z))	wrong=true;
 		}
 		mreal d;
 		if(!fx)	{	FMin.x = Min.x;	FMax.x = Max.x;	}
@@ -336,32 +366,38 @@ void mglBase::RecalcBorder()
 		if(!fz)	{	FMin.z = Min.z;	FMax.z = Max.z;	}
 		else	{	d=0.01*(FMax.z-FMin.z);	FMin.z-=d;	FMax.z+=d;	}
 	}
-	RecalcCRange();
+	if(RecalcCRange())	wrong=true;
+	if(wrong)	SetWarn(mglWarnTern, "Curved coordinates");
 }
 //-----------------------------------------------------------------------------
-void mglBase::SetFBord(mreal x,mreal y,mreal z)
+bool mglBase::SetFBord(mreal x,mreal y,mreal z)
 {
+	bool wrong=false;
 	if(fx)
 	{
 		mreal v = fx->Calc(x,y,z);
+		if(mgl_isbad(v))	wrong = true;
 		if(FMax.x < v)	FMax.x = v;
 		if(FMin.x > v)	FMin.x = v;
 	}
 	if(fy)
 	{
 		mreal v = fy->Calc(x,y,z);
+		if(mgl_isbad(v))	wrong = true;
 		if(FMax.y < v)	FMax.y = v;
 		if(FMin.y > v)	FMin.y = v;
 	}
 	if(fz)
 	{
 		mreal v = fz->Calc(x,y,z);
+		if(mgl_isbad(v))	wrong = true;
 		if(FMax.z < v)	FMax.z = v;
 		if(FMin.z > v)	FMin.z = v;
 	}
+	return wrong;
 }
 //-----------------------------------------------------------------------------
-bool mglBase::ScalePoint(mglPoint &p, mglPoint &n, bool use_nan) const
+bool mglBase::ScalePoint(const mglMatrix *, mglPoint &p, mglPoint &n, bool use_nan) const
 {
 	mreal &x=p.x, &y=p.y, &z=p.z;
 	if(mgl_isnan(x) || mgl_isnan(y) || mgl_isnan(z))	{	x=NAN;	return false;	}
@@ -466,12 +502,12 @@ void mglBase::SetRanges(mglPoint m1, mglPoint m2)
 	if(m1.c!=m2.c)	{	Min.c=m1.c;	Max.c=m2.c;	}
 	else			{	Min.c=Min.z;Max.c=Max.z;}
 
-	if(Org.x<Min.x && !mgl_isnan(Org.x))	Org.x = Min.x;
-	if(Org.x>Max.x && !mgl_isnan(Org.x))	Org.x = Max.x;
-	if(Org.y<Min.y && !mgl_isnan(Org.y))	Org.y = Min.y;
-	if(Org.y>Max.y && !mgl_isnan(Org.y))	Org.y = Max.y;
-	if(Org.z<Min.z && !mgl_isnan(Org.z))	Org.z = Min.z;
-	if(Org.z>Max.z && !mgl_isnan(Org.z))	Org.z = Max.z;
+	if(Org.x<Min.x && mgl_isnum(Org.x))	Org.x = Min.x;
+	if(Org.x>Max.x && mgl_isnum(Org.x))	Org.x = Max.x;
+	if(Org.y<Min.y && mgl_isnum(Org.y))	Org.y = Min.y;
+	if(Org.y>Max.y && mgl_isnum(Org.y))	Org.y = Max.y;
+	if(Org.z<Min.z && mgl_isnum(Org.z))	Org.z = Min.z;
+	if(Org.z>Max.z && mgl_isnum(Org.z))	Org.z = Max.z;
 
 	if((TernAxis&3)==0)
 	{
@@ -480,7 +516,7 @@ void mglBase::SetRanges(mglPoint m1, mglPoint m2)
 		mglScaleAxis(Min.y, Max.y, Org.y, AMin.y, AMax.y);
 		mglScaleAxis(Min.z, Max.z, Org.z, AMin.z, AMax.z);
 		mglScaleAxis(Min.c, Max.c, Org.c, AMin.c, AMax.c);
-	}	
+	}
 
 	CutMin = mglPoint(0,0,0);	CutMax = mglPoint(0,0,0);
 	RecalcBorder();
@@ -490,8 +526,16 @@ void mglBase::CRange(HCDT a,bool add, mreal fact)
 {
 	mreal v1=a->Minimal(), v2=a->Maximal(), dv;
 	dv=(v2-v1)*fact;	v1 -= dv;	v2 += dv;
-	if(v1==v2)	return;
-	if(!add)	{	Min.c = v1;	Max.c = v2;	}
+	CRange(v1,v2,add);
+}
+void mglBase::CRange(mreal v1,mreal v2,bool add)
+{
+	if(v1==v2 && !add)	return;
+	if(!add)
+	{
+		if(v1==v1)	Min.c = v1;	
+		if(v2==v2)	Max.c = v2;		
+	}
 	else if(Min.c<Max.c)
 	{
 		if(Min.c>v1)	Min.c=v1;
@@ -499,12 +543,12 @@ void mglBase::CRange(HCDT a,bool add, mreal fact)
 	}
 	else
 	{
-		dv = Min.c;
+		mreal dv = Min.c;
 		Min.c = v1<Max.c ? v1:Max.c;
 		Max.c = v2>dv ? v2:dv;
 	}
-	if(Org.c<Min.c && !mgl_isnan(Org.c))	Org.c = Min.c;
-	if(Org.c>Max.c && !mgl_isnan(Org.c))	Org.c = Max.c;
+	if(Org.c<Min.c && mgl_isnum(Org.c))	Org.c = Min.c;
+	if(Org.c>Max.c && mgl_isnum(Org.c))	Org.c = Max.c;
 	if((TernAxis&3)==0)
 	{
 		OMax.c = Max.c;	OMin.c = Min.c;
@@ -517,8 +561,16 @@ void mglBase::XRange(HCDT a,bool add,mreal fact)
 {
 	mreal v1=a->Minimal(), v2=a->Maximal(), dv;
 	dv=(v2-v1)*fact;	v1 -= dv;	v2 += dv;
-	if(v1==v2)	return;
-	if(!add)	{	Min.x = v1;	Max.x = v2;	}
+	XRange(v1,v2,add);
+}
+void mglBase::XRange(mreal v1,mreal v2,bool add)
+{
+	if(v1==v2 && !add)	return;
+	if(!add)
+	{
+		if(v1==v1)	Min.x = v1;	
+		if(v2==v2)	Max.x = v2;		
+	}
 	else if(Min.x<Max.x)
 	{
 		if(Min.x>v1)	Min.x=v1;
@@ -526,12 +578,12 @@ void mglBase::XRange(HCDT a,bool add,mreal fact)
 	}
 	else
 	{
-		dv = Min.x;
+		mreal dv = Min.x;
 		Min.x = v1<Max.x ? v1:Max.x;
 		Max.x = v2>dv ? v2:dv;
 	}
-	if(Org.x<Min.x && !mgl_isnan(Org.x))	Org.x = Min.x;
-	if(Org.x>Max.x && !mgl_isnan(Org.x))	Org.x = Max.x;
+	if(Org.x<Min.x && mgl_isnum(Org.x))	Org.x = Min.x;
+	if(Org.x>Max.x && mgl_isnum(Org.x))	Org.x = Max.x;
 	if((TernAxis&3)==0)
 	{
 		OMax.x = Max.x;	OMin.x = Min.x;
@@ -544,8 +596,16 @@ void mglBase::YRange(HCDT a,bool add,mreal fact)
 {
 	mreal v1=a->Minimal(), v2=a->Maximal(), dv;
 	dv=(v2-v1)*fact;	v1 -= dv;	v2 += dv;
-	if(v1==v2)	return;
-	if(!add)	{	Min.y = v1;	Max.y = v2;	}
+	YRange(v1,v2,add);
+}
+void mglBase::YRange(mreal v1,mreal v2,bool add)
+{
+	if(v1==v2 && !add)	return;
+	if(!add)
+	{
+		if(v1==v1)	Min.y = v1;	
+		if(v2==v2)	Max.y = v2;		
+	}
 	else if(Min.y<Max.y)
 	{
 		if(Min.y>v1)	Min.y=v1;
@@ -553,17 +613,17 @@ void mglBase::YRange(HCDT a,bool add,mreal fact)
 	}
 	else
 	{
-		dv = Min.y;
+		mreal dv = Min.y;
 		Min.y = v1<Max.y ? v1:Max.y;
 		Max.y = v2>dv ? v2:dv;
 	}
-	if(Org.y<Min.y && !mgl_isnan(Org.y))	Org.y = Min.y;
-	if(Org.y>Max.y && !mgl_isnan(Org.y))	Org.y = Max.y;
+	if(Org.y<Min.y && mgl_isnum(Org.y))	Org.y = Min.y;
+	if(Org.y>Max.y && mgl_isnum(Org.y))	Org.y = Max.y;
 	if((TernAxis&3)==0)
 	{
 		OMax.y = Max.y;	OMin.y = Min.y;
 		mglScaleAxis(Min.y, Max.y, Org.y, AMin.y, AMax.y);
-	
+
 	}
 	RecalcBorder();
 }
@@ -572,8 +632,16 @@ void mglBase::ZRange(HCDT a,bool add,mreal fact)
 {
 	mreal v1=a->Minimal(), v2=a->Maximal(), dv;
 	dv=(v2-v1)*fact;	v1 -= dv;	v2 += dv;
-	if(v1==v2)	return;
-	if(!add)	{	Min.z = v1;	Max.z = v2;	}
+	ZRange(v1,v2,add);
+}
+void mglBase::ZRange(mreal v1,mreal v2,bool add)
+{
+	if(v1==v2 && !add)	return;
+	if(!add)
+	{
+		if(v1==v1)	Min.z = v1;	
+		if(v2==v2)	Max.z = v2;		
+	}
 	else if(Min.z<Max.z)
 	{
 		if(Min.z>v1)	Min.z=v1;
@@ -581,12 +649,12 @@ void mglBase::ZRange(HCDT a,bool add,mreal fact)
 	}
 	else
 	{
-		dv = Min.z;
+		mreal dv = Min.z;
 		Min.z = v1<Max.z ? v1:Max.z;
 		Max.z = v2>dv ? v2:dv;
 	}
-	if(Org.z<Min.z && !mgl_isnan(Org.z))	Org.z = Min.z;
-	if(Org.z>Max.z && !mgl_isnan(Org.z))	Org.z = Max.z;
+	if(Org.z<Min.z && mgl_isnum(Org.z))	Org.z = Min.z;
+	if(Org.z>Max.z && mgl_isnum(Org.z))	Org.z = Max.z;
 	if((TernAxis&3)==0)
 	{
 		OMax.z = Max.z;	OMin.z = Min.z;
@@ -611,9 +679,8 @@ void mglBase::Ternary(int t)
 	if(t&3)
 	{
 		if(c)	{	x1 = Min;	x2 = Max;	o = Org;	}
-//		c = get(MGL_ENABLE_CUT);	clr(MGL_ENABLE_CUT);
-		SetRanges(mglPoint(0,0),mglPoint(1,1,t==1?0:1));
-		Org=mglPoint(0,0,0);	c = false;
+		SetRanges(mglPoint(0,0,0),mglPoint(1,1,(t&3)==1?0:1));
+		Org=mglPoint(0,0,(t&3)==1?NAN:0);	c = false;
 	}
 	else if(!c)	{	SetRanges(x1,x2);	Org=o;	c=true;	}
 }
@@ -780,7 +847,7 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 		if(s[i]=='A' && j<1 && m>0 && s[i+1]>'0' && s[i+1]<='9')
 		{	man=false;	alpha = 0.1*(s[i+1]-'0');	i++;	}
 	}
-	for(i=0;i<n;i++)	// default texture
+	for(long i=0;i<n;i++)	// default texture
 	{	c[2*i+1]=c[2*i];	c[2*i].a=man?0:alpha;	c[2*i+1].a=alpha;	}
 	if(map && sm)		// map texture
 	{
@@ -790,7 +857,7 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 		{	c[1]=c[2];	c[2]=c[0];	c[0]=BC;	c[3]=c[4];	n=2;}
 		else
 		{	c[1]=c[4];	c[3]=c[6];	n=2;	}
-		for(i=0;i<4;i++)	c[i].a=alpha;
+		c[0].a = c[1].a = c[2].a = c[3].a = alpha;
 		val[0]=val[1]=-1;
 	}
 	// TODO if(!sm && n==1)	then try to find color in palette ???
@@ -799,7 +866,7 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 	float  v1=0,v2=1;
 	std::vector <long>  def;
 	val[0]=0;	val[n-1]=1;	// boundary have to be [0,1]
-	for(i=0;i<n;i++) if(val[i]>0 && val[i]<1) 	def.push_back(i);
+	for(long i=0;i<n;i++) if(val[i]>0 && val[i]<1) 	def.push_back(i);
 	def.push_back(n-1);
 	long i1=0,i2;
 	for(size_t j=0;j<def.size();j++)	for(i=i1+1;i<def[j];i++)
@@ -810,24 +877,27 @@ void mglTexture::Set(const char *s, int smooth, mreal alpha)
 		val[i]=v1+v2*(i-i1);
 	}
 	// fill texture itself
-	register mreal u,v=sm?(n-1)/255.:n/256.;
-	for(i=0,i1=0;i<256;i++)
+	mreal v=sm?(n-1)/255.:n/256.;
+	if(!sm)
+//#pragma omp parallel for	// remove parallel here due to possible race conditions for v<1
+		for(long i=0;i<256;i++)
+		{
+			register long j = 2*long(v*i);	//u-=j;
+			col[2*i] = c[j];	col[2*i+1] = c[j+1];
+		}
+	else	for(i=i1=0;i<256;i++)
 	{
-		u = v*i;	j = long(u);	//u-=j;
-		if(!sm || j==n-1)
-		{	col[2*i] = c[2*j];	col[2*i+1] = c[2*j+1];	}
-		else if(j>n-1)	// NOTE: never should be here!
-		{	col[2*i] = c[2*n-2];col[2*i+1] = c[2*n-1];	/*printf("AddTexture -- out of bounds");*/	}
-		else
+		register mreal u = v*i;	j = long(u);	//u-=j;
+		if(j<n-1)	// advanced scheme using val
 		{
-			// advanced scheme using val
 			for(;i1<n-1 && i>=255*val[i1];i1++);
 			v2 = i1<n?1/(val[i1]-val[i1-1]):0;
 			j=i1-1;	u=(i/255.-val[j])*v2;
-
 			col[2*i] = c[2*j]*(1-u)+c[2*j+2]*u;
 			col[2*i+1]=c[2*j+1]*(1-u)+c[2*j+3]*u;
 		}
+		else
+		{	col[2*i] = c[2*n-2];col[2*i+1] = c[2*n-1];	}
 	}
 	delete []c;	delete []val;
 }
@@ -836,13 +906,13 @@ mglColor mglTexture::GetC(mreal u,mreal v) const
 {
 	u -= long(u);
 	register long i=long(255*u);	u = u*255-i;
-	const mglColor *s=col+2*i;	mglColor p;
-	p.r = (s[0].r*(1-u)+s[2].r*u)*(1-v) + (s[1].r*(1-u)+s[3].r*u)*v;
-	p.g = (s[0].g*(1-u)+s[2].g*u)*(1-v) + (s[1].g*(1-u)+s[3].g*u)*v;
-	p.b = (s[0].b*(1-u)+s[2].b*u)*(1-v) + (s[1].b*(1-u)+s[3].b*u)*v;
-	p.a = (s[0].a*(1-u)+s[2].a*u)*(1-v) + (s[1].a*(1-u)+s[3].a*u)*v;
-	//	p.a = (s[0].a*(1-u)+s[2].a*u)*v + (s[1].a*(1-u)+s[3].a*u)*(1-v);	// for alpha use inverted
-	return p;
+	const mglColor *s=col+2*i;	//mglColor p;
+	return (s[0]*(1-u)+s[2]*u)*(1-v) + (s[1]*(1-u)+s[3]*u)*v;
+// 	p.r = (s[0].r*(1-u)+s[2].r*u)*(1-v) + (s[1].r*(1-u)+s[3].r*u)*v;
+// 	p.g = (s[0].g*(1-u)+s[2].g*u)*(1-v) + (s[1].g*(1-u)+s[3].g*u)*v;
+// 	p.b = (s[0].b*(1-u)+s[2].b*u)*(1-v) + (s[1].b*(1-u)+s[3].b*u)*v;
+// 	p.a = (s[0].a*(1-u)+s[2].a*u)*(1-v) + (s[1].a*(1-u)+s[3].a*u)*v;
+// 	return p;
 }
 //-----------------------------------------------------------------------------
 void mglTexture::GetC(mreal u,mreal v,mglPnt &p) const
@@ -859,27 +929,32 @@ void mglTexture::GetC(mreal u,mreal v,mglPnt &p) const
 //-----------------------------------------------------------------------------
 long mglBase::AddTexture(const char *cols, int smooth)
 {
+	if(smooth>=0)	SetMask(cols);
 	mglTexture t(cols,smooth,smooth==2?AlphaDef:1);
 	if(t.n==0)	return smooth<0 ? 0:1;
 	if(smooth<0)	CurrPal=0;
 	// check if already exist
 	for(size_t i=0;i<Txt.size();i++)	if(t.IsSame(Txt[i]))	return i;
 	// create new one
-	MGL_PUSH(Txt,t,mutexTxt);	return Txt.size()-1;
+	long k;
+#pragma omp critical(txt)
+	{k=Txt.size();	MGL_PUSH(Txt,t,mutexTxt);}	return k;
 }
 //-----------------------------------------------------------------------------
 mreal mglBase::AddTexture(mglColor c)
 {
-	register size_t i,j;
 	if(!c.Valid())	return -1;
 	// first lets try an existed one
-	for(i=0;i<Txt.size();i++)	for(j=0;j<255;j++)
+	for(size_t i=0;i<Txt.size();i++)	for(int j=0;j<255;j++)
 		if(c==Txt[i].col[2*j])
 			return i+j/255.;
 	// add new texture
 	mglTexture t;
-	for(i=0;i<MGL_TEXTURE_COLOURS;i++)	t.col[i]=c;
-	MGL_PUSH(Txt,t,mutexTxt);	return Txt.size()-1;
+#pragma omp parallel for
+	for(long i=0;i<MGL_TEXTURE_COLOURS;i++)	t.col[i]=c;
+	long k;
+#pragma omp critical(txt)
+	{k=Txt.size();	MGL_PUSH(Txt,t,mutexTxt);}	return k;
 }
 //-----------------------------------------------------------------------------
 //		Coloring and palette
@@ -908,14 +983,32 @@ mreal mglBase::NextColor(long &id)
 	return CDef;
 }
 //-----------------------------------------------------------------------------
+mreal mglBase::NextColor(long id, long sh)
+{
+	long i=abs(id)/256, n=Txt[i].n, p=abs(id)&0xff;
+	if(id>=0)	p=(p+sh)%n;
+	return i + (n>0 ? (p+0.5)/n : 0);
+}
+//-----------------------------------------------------------------------------
+const char *mglchrs(const char *str, const char *chr)
+{
+	if(!str || !str[0] || !chr || !chr[0])	return NULL;
+	size_t l=strlen(chr);
+	for(size_t i=0;i<l;i++)
+	{
+		const char *res = mglchr(str,chr[i]);
+		if(res)	return res;
+	}
+	return NULL;
+}
+//-----------------------------------------------------------------------------
 const char *mglchr(const char *str, char ch)
 {
 	if(!str || !str[0])	return NULL;
-	register char c;
-	register size_t l=strlen(str),i,k=0;
-	for(i=0;i<l;i++)
+	size_t l=strlen(str),k=0;
+	for(size_t i=0;i<l;i++)
 	{
-		c = str[i];
+		register char c = str[i];
 		if(c=='{')	k++;
 		if(c=='}')	k--;
 		if(c==ch && k==0)	return str+i;
@@ -923,7 +1016,7 @@ const char *mglchr(const char *str, char ch)
 	return NULL;
 }
 //-----------------------------------------------------------------------------
-char mglBase::SetPenPal(const char *p, long *Id)
+char mglBase::SetPenPal(const char *p, long *Id, bool pal)
 {
 	char mk=0;
 	PDef = 0xffff;	// reset to solid line
@@ -933,30 +1026,20 @@ char mglBase::SetPenPal(const char *p, long *Id)
 	if(p && *p)
 	{
 //		const char *col = "wkrgbcymhRGBCYMHWlenuqpLENUQP";
-		const char *stl = " -|;:ji=";
+		unsigned val[8] = {0x0000, 0xffff, 0x00ff, 0x0f0f, 0x1111, 0x087f, 0x2727, 0x3333};
+		const char *stl = " -|;:ji=", *s;
 		const char *mrk = "*o+xsd.^v<>";
+		const char *MRK = "YOPXSDCTVLR";
 		const char *wdh = "123456789";
 		const char *arr = "AKDTVISO_";
 		long m=0;
-		for(size_t i=0;i<strlen(p);i++)
+		size_t l=strlen(p);
+		for(size_t i=0;i<l;i++)
 		{
 			if(p[i]=='{')	m++;	if(p[i]=='}')	m--;
 			if(m>0)	continue;
-			if(mglchr(stl,p[i]))
-			{
-				switch(p[i])
-				{
-				case '|': PDef = 0x00ff;	break;
-				case ';': PDef = 0x0f0f;	break;
-				case '=': PDef = 0x3333;	break;
-				case ':': PDef = 0x1111;	break;
-				case 'j': PDef = 0x087f;	break;
-				case 'i': PDef = 0x2727;	break;
-				case ' ': PDef = 0x0000;	break;
-				default:  PDef = 0xffff;	break;	// '-'
-				}
-				last_style[4]=p[i];
-			}
+			s = mglchr(stl,p[i]);
+			if(s)	{	PDef = val[s-stl];	last_style[4]=p[i];	}
 			else if(mglchr(mrk,p[i]))	mk = p[i];
 			else if(mglchr(wdh,p[i]))
 			{	last_style[5] = p[i];	PenWidth = p[i]-'0';	}
@@ -969,27 +1052,56 @@ char mglBase::SetPenPal(const char *p, long *Id)
 		if(Arrow1=='_')	Arrow1=0;	if(Arrow2=='_')	Arrow2=0;
 		if(mglchr(p,'#'))
 		{
-			if(mk=='.')	mk = 'C';
-			if(mk=='+')	mk = 'P';
-			if(mk=='x')	mk = 'X';
-			if(mk=='o')	mk = 'O';
-			if(mk=='d')	mk = 'D';
-			if(mk=='s')	mk = 'S';
-			if(mk=='^')	mk = 'T';
-			if(mk=='v')	mk = 'V';
-			if(mk=='<')	mk = 'L';
-			if(mk=='>')	mk = 'R';
-			if(mk=='*')	mk = 'Y';
+			s = mglchr(mrk,mk);
+			if(s)	mk = MRK[s-mrk];
 		}
 	}
-	last_style[6]=mk;
-	long tt, n;
-	tt = AddTexture(p,-1);	n=Txt[tt].n;
-	CDef = tt+((n+CurrPal-1)%n+0.5)/n;
-	if(Id)	*Id=long(tt)*256+(n+CurrPal-1)%n;
+	if(pal)
+	{
+		last_style[6]=mk;
+		long tt, n;
+		tt = AddTexture(p,-1);	n=Txt[tt].n;
+		CDef = tt+((n+CurrPal-1)%n+0.5)/n;
+		if(Id)	*Id=long(tt)*256+(n+CurrPal-1)%n;
+	}
 	return mk;
 }
 //-----------------------------------------------------------------------------
+// keep this for restore default mask
+uint64_t mgl_mask_def[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
+							0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
+							0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
+							0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
+uint64_t mgl_mask_val[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
+							0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
+							0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
+							0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
+void mglBase::SetMask(const char *p)
+{
+	mask = MGL_SOLID_MASK;	// reset to solid face
+	PenWidth = 1;	MaskAn=DefMaskAn;
+	if(p && *p)
+	{
+		const char *msk = MGL_MASK_ID, *s;
+		const char *wdh = "123456789";
+		long m=0, l=strlen(p);
+		for(long i=0;i<l;i++)
+		{
+			if(p[i]=='{')	m++;	if(p[i]=='}')	m--;
+			if(m>0)	continue;
+			if(p[i]==':')	break;
+			s = mglchr(msk, p[i]);
+			if(s)	mask = mgl_mask_val[s-msk];
+			else if(mglchr(wdh,p[i]))	PenWidth = p[i]-'0';
+			else if(p[i]=='I')	MaskAn=90;
+			else if(p[i]=='/')	MaskAn=315;	// =360-45
+			else if(p[i]=='\\')	MaskAn=45;
+		}
+		// use line if rotation only specified
+		if(mask==MGL_SOLID_MASK && MaskAn!=0)	mask = mgl_mask_val[0];
+	}
+}
+//-----------------------------------------------------------------------------
 mreal mglBase::GetA(mreal a) const
 {
 	if(fa)	a = fa->Calc(0,0,0,a);
@@ -1036,15 +1148,18 @@ void mglBase::vect_plot(long p1, long p2, mreal s)
 	s2.y=s2.yy = q2.y - 3*s*(q2.y-q1.y) + s*(q2.x-q1.x);
 	s1.z=s1.zz=s2.z=s2.zz = q2.z - 3*s*(q2.z-q1.z);
 	long n1,n2;
-	n1=Pnt.size();	MGL_PUSH(Pnt,s1,mutexPnt);
-	n2=Pnt.size();	MGL_PUSH(Pnt,s2,mutexPnt);
+#pragma omp critical(pnt)
+	{
+		n1=Pnt.size();	MGL_PUSH(Pnt,s1,mutexPnt);
+		n2=Pnt.size();	MGL_PUSH(Pnt,s2,mutexPnt);
+	}
 	line_plot(p1,p2);	line_plot(n1,p2);	line_plot(p2,n2);
 }
 //-----------------------------------------------------------------------------
 int mglFindArg(const char *str)
 {
-	register long l=0,k=0,i;//,j,len=strlen(lst);
-	for(i=0;i<long(strlen(str));i++)
+	long l=0,k=0,len=strlen(str);
+	for(long i=0;i<len;i++)
 	{
 		if(str[i]=='\'') l++;
 		if(str[i]=='{') k++;
@@ -1059,6 +1174,7 @@ int mglFindArg(const char *str)
 }
 //-----------------------------------------------------------------------------
 void mglBase::SetAmbient(mreal bright)	{	AmbBr = bright;	}
+void mglBase::SetDiffuse(mreal bright)	{	DifBr = bright;	}
 //-----------------------------------------------------------------------------
 mreal mglBase::SaveState(const char *opt)
 {
@@ -1122,16 +1238,15 @@ void mglBase::LoadState()
 }
 //-----------------------------------------------------------------------------
 void mglBase::AddLegend(const wchar_t *text,const char *style)
-{	if(text)	MGL_PUSH(Leg,mglText(text,style),mutexLeg);	}
+{
+	if(text)
+#pragma omp critical(leg)
+		MGL_PUSH(Leg,mglText(text,style),mutexLeg);
+}
 //-----------------------------------------------------------------------------
 void mglBase::AddLegend(const char *str,const char *style)
 {
-	if(!str || *str==0)	return;
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	AddLegend(wcs, style);
-	delete []wcs;
+	MGL_TO_WCS(str,AddLegend(wcs, style));
 }
 //-----------------------------------------------------------------------------
 bool MGL_EXPORT mgl_check_dim2(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char *name, bool less)
@@ -1246,39 +1361,42 @@ void mglBase::ClearUnused()
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_lock(&mutexPnt);	pthread_mutex_lock(&mutexPrm);
 #endif
-	register size_t i, l=Prm.size();
-	// find points which are actually used
-	long *used = new long[Pnt.size()];	memset(used,0,Pnt.size()*sizeof(long));
-	for(i=0;i<l;i++)
+#pragma omp critical
 	{
-		const mglPrim &p=Prm[i];
-		if(p.n1<0)	continue;
-		used[p.n1] = 1;
-		switch(p.type)
+		size_t l=Prm.size();
+		// find points which are actually used
+		long *used = new long[Pnt.size()];	memset(used,0,Pnt.size()*sizeof(long));
+		for(size_t i=0;i<l;i++)
 		{
-		case 1:	if(p.n2>=0)	used[p.n2] = 1;	break;
-		case 2:	if(p.n2>=0 && p.n3>=0)
-			used[p.n2] = used[p.n3] = 1;	break;
-		case 3:	if(p.n2>=0 && p.n3>=0 && p.n4>=0)
-			used[p.n2] = used[p.n3] = used[p.n4] = 1;	break;
+			const mglPrim &p=Prm[i];
+			if(p.n1<0)	continue;
+			used[p.n1] = 1;
+			switch(p.type)
+			{
+			case 1:	case 4:	if(p.n2>=0)	used[p.n2] = 1;	break;
+			case 2:	if(p.n2>=0 && p.n3>=0)
+				used[p.n2] = used[p.n3] = 1;	break;
+			case 3:	if(p.n2>=0 && p.n3>=0 && p.n4>=0)
+				used[p.n2] = used[p.n3] = used[p.n4] = 1;	break;
+			}
 		}
+		// now add proper indexes
+		l=Pnt.size();
+		std::vector<mglPnt> pnt;
+		for(size_t i=0;i<l;i++)	if(used[i])
+		{	pnt.push_back(Pnt[i]);	used[i]=pnt.size();	}
+		Pnt = pnt;	pnt.clear();
+		// now replace point id
+		l=Prm.size();
+		for(size_t i=0;i<l;i++)
+		{
+			mglPrim &p=Prm[i];	p.n1=used[p.n1]-1;
+			if(p.type==1 || p.type==4)	p.n2=used[p.n2]-1;
+			if(p.type==2)	{	p.n2=used[p.n2]-1;	p.n3=used[p.n3]-1;	}
+			if(p.type==3)	{	p.n2=used[p.n2]-1;	p.n3=used[p.n3]-1;	p.n4=used[p.n4]-1;	}
+		}
+		delete []used;
 	}
-	// now add proper indexes
-	l=Pnt.size();
-	std::vector<mglPnt> pnt;
-	for(i=0;i<l;i++)	if(used[i])
-	{	pnt.push_back(Pnt[i]);	used[i]=pnt.size();	}
-	Pnt = pnt;	pnt.clear();
-	// now replace point id
-	l=Prm.size();
-	for(i=0;i<l;i++)
-	{
-		mglPrim &p=Prm[i];	p.n1=used[p.n1]-1;
-		if(p.type==1)	p.n2=used[p.n2]-1;
-		if(p.type==2)	{	p.n2=used[p.n2]-1;	p.n3=used[p.n3]-1;	}
-		if(p.type==3)	{	p.n2=used[p.n2]-1;	p.n3=used[p.n3]-1;	p.n4=used[p.n4]-1;	}
-	}
-	delete []used;
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexPnt);	pthread_mutex_unlock(&mutexPrm);
 #endif
