@@ -1,6 +1,6 @@
 /***************************************************************************
  * base.cpp is part of Math gric Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -19,6 +19,7 @@
  ***************************************************************************/
 #include "mgl2/font.h"
 #include "mgl2/base_cf.h"
+#include "mgl2/base.h"
 //-----------------------------------------------------------------------------
 //
 //		C interfaces
@@ -29,15 +30,19 @@ void MGL_EXPORT mgl_set_quality_(uintptr_t *gr, int *qual)	{	_GR_->SetQuality(*q
 int MGL_EXPORT mgl_get_quality(HMGL gr)	{	return gr->GetQuality();	}
 int MGL_EXPORT mgl_get_quality_(uintptr_t *gr)	{	return _GR_->GetQuality();	}
 int MGL_EXPORT mgl_is_frames(HMGL gr)
-{	return gr->get(MGL_VECT_FRAME) && !(gr->GetQuality()&4);	}
+{	return gr->get(MGL_VECT_FRAME) && !(gr->GetQuality()&MGL_DRAW_LMEM);	}
+void MGL_EXPORT mgl_set_draw_reg(HMGL gr, long nx, long ny, long m)	{	gr->SetDrawReg(nx,ny,m);	}
+void MGL_EXPORT mgl_set_draw_reg_(uintptr_t *gr, int *nx, int *ny, int *m)	{	_GR_->SetDrawReg(*nx,*ny,*m);	}
 //-----------------------------------------------------------------------------
-int MGL_EXPORT mgl_get_flag(HMGL gr, long flag)			{	return gr->get(flag);	}
-void MGL_EXPORT mgl_set_flag(HMGL gr, int val, long flag)	{	gr->set(val,flag);	}
+int MGL_EXPORT mgl_get_flag(HMGL gr, uint32_t flag)	{	return gr->get(flag);	}
+int MGL_EXPORT mgl_get_flag_(uintptr_t *gr, unsigned long *flag)	{	return _GR_->get(*flag);	}
+void MGL_EXPORT mgl_set_flag(HMGL gr, int val, uint32_t flag)		{	gr->set(val,flag);	}
+void MGL_EXPORT mgl_set_flag_(uintptr_t *gr, int *val, unsigned long *flag)	{	_GR_->set(*val,*flag);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_set_color(char id, double r, double g, double b)
 {
-	register size_t i;
-	for(i=0;mglColorIds[i].id;i++)	if(mglColorIds[i].id==id)	mglColorIds[i].col = mglColor(r,g,b);
+	for(long i=0;mglColorIds[i].id;i++)
+		if(mglColorIds[i].id==id)	mglColorIds[i].col = mglColor(r,g,b);
 }
 void MGL_EXPORT mgl_set_color_(char *id, mreal *r, mreal *g, mreal *b, int)	{	mgl_set_color(*id,*r,*g,*b);	}
 //-----------------------------------------------------------------------------
@@ -76,16 +81,25 @@ void MGL_EXPORT mgl_set_cutoff(HMGL gr, const char *EqC)	{	gr->CutOff(EqC);	}
 void MGL_EXPORT mgl_set_ternary(HMGL gr, int enable)			{	gr->Ternary(enable);	}
 void MGL_EXPORT mgl_set_range_val(HMGL gr, char dir, double v1,double v2)
 {
-	if(dir=='c')		gr->CRange(v1,v2);
+	if(dir=='c' || dir=='a')	gr->CRange(v1,v2);
 	else if(dir=='x')	gr->XRange(v1,v2);
 	else if(dir=='y')	gr->YRange(v1,v2);
-	else if(dir=='z')	gr->ZRange(v1,v2);	}
+	else if(dir=='z')	gr->ZRange(v1,v2);
+}
+void MGL_EXPORT mgl_add_range_val(HMGL gr, char dir, double v1,double v2)
+{
+	if(dir=='c' || dir=='a')	gr->CRange(v1,v2,true);
+	else if(dir=='x')	gr->XRange(v1,v2,true);
+	else if(dir=='y')	gr->YRange(v1,v2,true);
+	else if(dir=='z')	gr->ZRange(v1,v2,true);
+}
 void MGL_EXPORT mgl_set_range_dat(HMGL gr, char dir, HCDT a, int add)
 {
-	if(dir=='c')		gr->CRange(a,add);
+	if(dir=='c' || dir=='a')	gr->CRange(a,add);
 	else if(dir=='x')	gr->XRange(a,add);
 	else if(dir=='y')	gr->YRange(a,add);
-	else if(dir=='z')	gr->ZRange(a,add);	}
+	else if(dir=='z')	gr->ZRange(a,add);
+}
 void MGL_EXPORT mgl_set_ranges(HMGL gr, double x1, double x2, double y1, double y2, double z1, double z2)
 {	gr->SetRanges(x1,x2,y1,y2,z1,z2);	}
 void MGL_EXPORT mgl_set_auto_ranges(HMGL gr, double x1, double x2, double y1, double y2, double z1, double z2, double c1, double c2)
@@ -128,6 +142,8 @@ void MGL_EXPORT mgl_set_cutoff_(uintptr_t *gr, const char *EqC, int l)
 void MGL_EXPORT mgl_set_ternary_(uintptr_t *gr, int *enable)	{	_GR_->Ternary(*enable);	}
 void MGL_EXPORT mgl_set_range_val_(uintptr_t *gr, const char *dir, mreal *v1, mreal *v2,int)
 {	mgl_set_range_val(_GR_,*dir,*v1,*v2);	}
+void MGL_EXPORT mgl_add_range_val_(uintptr_t *gr, const char *dir, mreal *v1, mreal *v2,int)
+{	mgl_add_range_val(_GR_,*dir,*v1,*v2);	}
 void MGL_EXPORT mgl_set_range_dat_(uintptr_t *gr, const char *dir, uintptr_t *a, int *add,int)
 {	mgl_set_range_dat(_GR_,*dir,_DA_(a),*add);	}
 void MGL_EXPORT mgl_set_ranges_(uintptr_t *gr, mreal *x1, mreal *x2, mreal *y1, mreal *y2, mreal *z1, mreal *z2)
@@ -166,7 +182,7 @@ void MGL_EXPORT mgl_set_rotated_text_(uintptr_t *gr, int *rotated)	{	_GR_->SetRo
 void MGL_EXPORT mgl_set_mark_size_(uintptr_t *gr, mreal *size)		{	_GR_->SetMarkSize(*size);	}
 void MGL_EXPORT mgl_set_arrow_size_(uintptr_t *gr, mreal *size)	{	_GR_->SetArrowSize(*size);	}
 void MGL_EXPORT mgl_set_font_size_(uintptr_t *gr, mreal *size)		{	_GR_->SetFontSize(*size);	}
-void MGL_EXPORT mgl_set_font_def_(uintptr_t *gr, char *name, int l)
+void MGL_EXPORT mgl_set_font_def_(uintptr_t *gr, const char *name, int l)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
 	_GR_->SetFontDef(s);	delete []s;	}
 void MGL_EXPORT mgl_load_font_(uintptr_t *gr, char *name, char *path, int l,int n)
@@ -217,8 +233,35 @@ long MGL_EXPORT mgl_use_graph_(uintptr_t *gr, int *inc)
 void MGL_EXPORT mgl_set_ambbr(HMGL gr, double i)		{	gr->SetAmbient(i);	}
 void MGL_EXPORT mgl_set_ambbr_(uintptr_t *gr, mreal *i){	_GR_->SetAmbient(*i);	}
 //---------------------------------------------------------------------------
+void MGL_EXPORT mgl_set_difbr(HMGL gr, double i)		{	gr->SetDiffuse(i);	}
+void MGL_EXPORT mgl_set_difbr_(uintptr_t *gr, mreal *i){	_GR_->SetDiffuse(*i);	}
+//---------------------------------------------------------------------------
 void MGL_EXPORT mgl_zoom_axis(HMGL gr, double x1,double y1,double z1,double c1,double x2,double y2,double z2,double c2)
 {	gr->ZoomAxis(mglPoint(x1,y1,z1,c1), mglPoint(x2,y2,z2,c2));	}
 void MGL_EXPORT mgl_zoom_axis_(uintptr_t *gr, mreal *x1, mreal *y1, mreal *z1, mreal *c1, mreal *x2, mreal *y2, mreal *z2, mreal *c2)
 {	_GR_->ZoomAxis(mglPoint(*x1,*y1,*z1,*c1), mglPoint(*x2,*y2,*z2,*c2));	}
 //---------------------------------------------------------------------------
+extern uint64_t mgl_mask_def[16];
+void MGL_EXPORT mgl_set_mask(char id, const char *mask)
+{
+	const char *msk = MGL_MASK_ID, *s = mglchr(msk, id);
+	if(s)
+	{
+		uint64_t val = (mask && *mask) ? strtoull(mask,NULL,16) : mgl_mask_def[s-msk];
+		mgl_mask_val[s-msk] = val;
+	}
+}
+void MGL_EXPORT mgl_set_mask_(const char *id, const char *mask,int,int l)
+{	char *s=new char[l+1];	memcpy(s,mask,l);	s[l]=0;	mgl_set_mask(*id,s);	delete []s;	}
+//---------------------------------------------------------------------------
+void MGL_EXPORT mgl_set_mask_val(char id, uint64_t mask)
+{
+	const char *msk = MGL_MASK_ID, *s = mglchr(msk, id);
+	if(s)	mgl_mask_val[s-msk]=mask;
+}
+void MGL_EXPORT mgl_set_mask_val_(const char *id, uint64_t *mask,int)
+{	mgl_set_mask_val(*id,*mask);	}
+//---------------------------------------------------------------------------
+void MGL_EXPORT mgl_set_mask_angle(HMGL gr, int angle)	{	gr->SetMaskAngle(angle);	}
+void MGL_EXPORT mgl_set_mask_angle_(uintptr_t *gr, int *angle)	{	_GR_->SetMaskAngle(*angle);	}
+//---------------------------------------------------------------------------
