@@ -1,6 +1,6 @@
 /***************************************************************************
  * canvas.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -31,7 +31,6 @@ mglCanvas::mglCanvas(int w, int h) : mglBase()
 	CurFrameId=0;	Delay=0.5;
 	Width=Height=Depth=0;	ObjId=-1;
 	fscl=ftet=0;		PlotId = "frame";
-	dr_nx1=dr_nx2=dr_ny1=dr_ny2=0;	// Allowed drawing region
 
 	ac.ch='c';
 	ax.dir = mglPoint(1,0,0);	ax.a = mglPoint(0,1,0);	ax.b = mglPoint(0,0,1);	ax.ch='x';
@@ -50,6 +49,7 @@ long mglCanvas::PushDrwDat()
 {
 	mglDrawDat d;
 	d.Pnt=Pnt;	d.Prm=Prm;	d.Glf=Glf;	d.Ptx=Ptx;	d.Txt=Txt;
+#pragma omp critical(drw)
 	MGL_PUSH(DrwDat,d,mutexDrw);
 	return DrwDat.size();
 }
@@ -58,16 +58,18 @@ void mglCanvas::ResetFrames()	{	CurFrameId=0;	DrwDat.clear();	}
 //-----------------------------------------------------------------------------
 void mglCanvas::SetFrame(long i)
 {
-	if(get(MGL_VECT_FRAME) && i>=0 && i<DrwDat.size())
+	if(get(MGL_VECT_FRAME) && i>=0 && i<long(DrwDat.size()))
 	{
 		Finish();	CurFrameId--;
 		mglDrawDat d;
 		d.Pnt=Pnt;	d.Prm=Prm;	d.Glf=Glf;	d.Ptx=Ptx;	d.Txt=Txt;
 #if MGL_HAVE_PTHREAD
 		pthread_mutex_lock(&mutexDrw);
+#pragma omp critical(drw)
 		DrwDat[i] = d;
 		pthread_mutex_unlock(&mutexDrw);
 #else
+#pragma omp critical(drw)
 		DrwDat[i] = d;
 #endif
 	}
@@ -85,7 +87,8 @@ void mglCanvas::GetFrame(long k)
 	pthread_mutex_lock(&mutexPtx);
 	pthread_mutex_lock(&mutexTxt);
 #endif
-	Pnt=d.Pnt;	Prm=d.Prm;	Glf=d.Glf;	Ptx=d.Ptx;	Txt=d.Txt;
+#pragma omp critical
+	{	Pnt=d.Pnt;	Prm=d.Prm;	Glf=d.Glf;	Ptx=d.Ptx;	Txt=d.Txt;	}
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexTxt);
 	pthread_mutex_unlock(&mutexPtx);
@@ -107,33 +110,35 @@ void mglCanvas::ShowFrame(long k)
 	pthread_mutex_lock(&mutexPtx);
 	pthread_mutex_lock(&mutexTxt);
 #endif
-	const mglDrawDat &d=DrwDat[k];
-	register size_t i;
-	Glf.reserve(d.Glf.size());	for(i=0;i<d.Glf.size();i++)	Glf.push_back(d.Glf[i]);
-	Ptx.reserve(d.Ptx.size());	for(i=0;i<d.Ptx.size();i++)	Ptx.push_back(d.Ptx[i]);
-	Txt.reserve(d.Pnt.size());	for(i=0;i<d.Txt.size();i++)	Txt.push_back(d.Txt[i]);
-	Pnt.reserve(d.Pnt.size());
-	for(i=0;i<d.Pnt.size();i++)
+#pragma omp critical
 	{
-		mglPnt p = d.Pnt[i]; 	p.c += ntxt;
-		Pnt.push_back(p);
-	}
-	Prm.reserve(d.Prm.size());
-	for(i=0;i<d.Prm.size();i++)
-	{
-		mglPrim p = d.Prm[i];
-		p.n1 += npnt;
-
-		switch(p.type)
+		const mglDrawDat &d=DrwDat[k];
+		Glf.reserve(d.Glf.size());	for(size_t i=0;i<d.Glf.size();i++)	Glf.push_back(d.Glf[i]);
+		Ptx.reserve(d.Ptx.size());	for(size_t i=0;i<d.Ptx.size();i++)	Ptx.push_back(d.Ptx[i]);
+		Txt.reserve(d.Pnt.size());	for(size_t i=0;i<d.Txt.size();i++)	Txt.push_back(d.Txt[i]);
+		Pnt.reserve(d.Pnt.size());
+		for(size_t i=0;i<d.Pnt.size();i++)
 		{
-		case 1:	p.n2 += npnt;	break;
-		case 2:	p.n2 += npnt;	p.n3 += npnt;	break;
-		case 3:	p.n2 += npnt;	p.n3 += npnt;	p.n4 += npnt;	break;
-		case 4: p.n4 += nglf;	break;
-		case 5:	p.n2 += npnt;	break;
-		case 6: p.n3 += nptx;	break;
+			mglPnt p = d.Pnt[i]; 	p.c += ntxt;
+			Pnt.push_back(p);
+		}
+		Prm.reserve(d.Prm.size());
+		for(size_t i=0;i<d.Prm.size();i++)
+		{
+			mglPrim p = d.Prm[i];
+			p.n1 += npnt;
+
+			switch(p.type)
+			{
+			case 1:	p.n2 += npnt;	break;
+			case 2:	p.n2 += npnt;	p.n3 += npnt;	break;
+			case 3:	p.n2 += npnt;	p.n3 += npnt;	p.n4 += npnt;	break;
+			case 4: p.n4 += nglf;	break;
+			case 5:	p.n2 += npnt;	break;
+			case 6: p.n3 += nptx;	break;
+			}
+			Prm.push_back(p);
 		}
-		Prm.push_back(p);
 	}
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexPnt);
@@ -153,11 +158,14 @@ void mglCanvas::add_prim(mglPrim &a)
 	if(a.n1>=0)
 	{
 		a.z = Pnt[a.n1].z;	// this is a bit less accurate but simpler for transformation
-		a.id = ObjId;	MGL_PUSH(Prm,a,mutexPrm);
+		a.id = ObjId;
+#pragma omp critical(prm)
+		MGL_PUSH(Prm,a,mutexPrm);
 		clr(MGL_FINISHED);
 	}
 }
 //-----------------------------------------------------------------------------
+extern uint64_t mgl_mask_def[16];
 void mglCanvas::DefaultPlotParam()
 {
 /* NOTE: following variables and mutex will not be changed by DefaultPlotParam()
@@ -169,6 +177,9 @@ int Height;			///< Height of the image
 int Depth;			///< Depth of the image
 int CurFrameId;		///< Number of automaticle created frames
 GifFileType *gif;*/
+	SetDrawReg(1,1,0);
+	memcpy(mgl_mask_val, mgl_mask_def, 16*sizeof(uint64_t));	// should be > 16*8
+	mgl_clear_fft();		DefMaskAn=0;	ResetMask();
 	SetTickRotate(true);	SetTickSkip(true);
 	SetWarn(mglWarnNone,"");	mglGlobalMess = "";
 	ObjId = -1;	HighId = INT_MIN;
@@ -180,7 +191,7 @@ GifFileType *gif;*/
 	SetTranspType(0);		SetMeshNum(0);	// NOTE: default MeshNum=0
 	SetRotatedText(true);	CurrPal = 0;
 	SetLegendMarks();		SetFontSize(4);
-	SetTuneTicks(-1);		SetAmbient();
+	SetTuneTicks(3);		SetAmbient();	SetDiffuse();
 	clr(MGL_DISABLE_SCALE);
 	clr(MGL_USE_GMTIME);	clr(MGL_NOSUBTICKS);
 	SetDifLight(false);		SetReduceAcc(false);
@@ -190,7 +201,7 @@ GifFileType *gif;*/
 	stack.clear();	Restore();	DefColor('k');
 	SetPlotFactor(0);	InPlot(0,1,0,1,false);
 	SetTickLen(0);	SetCut(true);
-	AdjustTicks("xyzc",true);	Clf();
+	AdjustTicks("xyzc",true);	Clf('w');
 
 	for(int i=0;i<10;i++)	{	AddLight(i, mglPoint(0,0,1));	Light(i,false);	}
 	Light(0,true);	Light(false);	SetDifLight(true);
@@ -200,70 +211,54 @@ GifFileType *gif;*/
 //-----------------------------------------------------------------------------
 mreal mglCanvas::FindOptOrg(char dir, int ind) const
 {
-	static mglPoint px, py, pz, m1, m2;
-	static mglMatrix bb;	bb.b[0]=1e30;
+	static mglPoint px, py, pz;
+	static mglMatrix bb;
 	mglPoint nn[8]={mglPoint(0,0,0), mglPoint(0,0,1), mglPoint(0,1,0,0), mglPoint(0,1,1),
-	mglPoint(1,0,0), mglPoint(1,0,1), mglPoint(1,1,0), mglPoint(1,1,1)}, pp[8];
+					mglPoint(1,0,0), mglPoint(1,0,1), mglPoint(1,1,0), mglPoint(1,1,1)}, pp[8];
 	memcpy(pp, nn, 8*sizeof(mglPoint));
-	// do nothing if transformation matrix the same
-	if(memcmp(B.b,bb.b,9*sizeof(mreal)) || m1!=Min || m2!=Max)
+	// do nothing if transformation matrix is the same
+	if(B!=bb)
 	{
-		m1 = Min;	m2 = Max;	memcpy(&bb,&B,sizeof(mglMatrix));
-		PostScale(pp,8);
+		bb = B;
+#pragma omp parallel for
+		for(long i=0;i<8;i++)	PostScale(&B,pp[i]);
 		// find point with minimal y
-		register long i,j=0;
-		for(i=1;i<8;i++)	if(pp[i].y<pp[j].y)	j=i;
+		long j=0;
+		for(long i=1;i<8;i++)	if(pp[i].y<pp[j].y)	j=i;
 		pp[0]=pp[j];
-		// find max angle and left point
 		// first select 3 closest points
-		pp[1]=nn[j];	pp[1].x=1-pp[1].x;
-		pp[2]=nn[j];	pp[2].y=1-pp[2].y;
-		pp[3]=nn[j];	pp[3].z=1-pp[3].z;
-		PostScale(pp+1,3);
-		pp[1]-=pp[0];	pp[2]-=pp[0];	pp[3]-=pp[0];
+		pp[1].x=1-nn[j].x;	pp[1].y=nn[j].y;	pp[1].z=nn[j].z;	PostScale(&B,pp[1]);	pp[1]-=pp[0];
+		pp[2].x=nn[j].x;	pp[2].y=1-nn[j].y;	pp[2].z=nn[j].z;	PostScale(&B,pp[2]);	pp[2]-=pp[0];
+		pp[3].x=nn[j].x;	pp[3].y=nn[j].y;	pp[3].z=1-nn[j].z;	PostScale(&B,pp[3]);	pp[3]-=pp[0];
 		// find cosine of axis projection
-		mreal cxy, cxz, cyz, dx, dy, dz;
-		dx = pp[1].x*pp[1].x + pp[1].y*pp[1].y;
-		dy = pp[2].x*pp[2].x + pp[2].y*pp[2].y;
-		dz = pp[3].x*pp[3].x + pp[3].y*pp[3].y;
-		cxy= pp[1].x*pp[2].x + pp[1].y*pp[1].y;
-		cxz= pp[1].x*pp[3].x + pp[1].y*pp[3].y;
-		cyz= pp[3].x*pp[2].x + pp[3].y*pp[2].y;
-		if(dx==0)		cxy = cxz = 1;
-		else if(dy==0)	cxy = cyz = 1;
-		else if(dz==0)	cyz = cxz = 1;
-		else
-		{	cxy /= sqrt(dx*dy);	cxz /= sqrt(dx*dz);	cyz /= sqrt(dz*dy);	}
-		// find points for axis
-		px = py = pz = nn[j];
-		if(cxy<cxz && cxy<cyz)	// xy is lowest
-		{	// px, py the same as pp
-			if(pp[1].x<pp[2].x)	pz.x = 1-pz.x;
-			else	pz.y = 1-pz.y;
+		register mreal tx=fabs(pp[1].x/pp[1].y), ty=fabs(pp[2].x/pp[2].y), tz=fabs(pp[3].x/pp[3].y);
+		px=py=pz=nn[j];
+		if(tz==0 && (ty==0 || tx==0))	// (x- & z-) or (y- & z-) axis are vertical
+		{	if(pp[1].x>pp[2].x)	pz.y=1-pz.y;	else	pz.x=1-pz.x;	}
+		else if(tx==0 && ty==0)	// x- && y-axis is vertical
+		{
+			py.x=1-py.x;
+			if(pp[1].x>pp[3].x)
+			{	px.z=1-px.z;	py.z=1-py.z;	}
 		}
-		if(cxz<cxy && cxz<cyz)	// xz is lowest
-		{	// px, pz the same as pp
-			if(pp[1].x<pp[3].x)	py.x = 1-py.x;
-			else	py.z = 1-py.z;
-		}
-		if(cyz<cxz && cyz<cxy)	// yz is lowest
-		{	// py, pz the same as pp
-			if(pp[3].x<pp[2].x)	px.z = 1-px.z;
-			else	px.y = 1-px.y;
-		}
-		// return to normal variables
-		// NOTE: this may not work in "inverse" curvilinear coordinates like x->1-x
-		px = Min+(Max-Min)/px;
-		py = Min+(Max-Min)/py;
-		pz = Min+(Max-Min)/pz;
+		else if(tz<tx && tz<ty)	// z-axis is vertical
+		{	if(pp[1].x>pp[2].x)	pz.y=1-pz.y;	else	pz.x=1-pz.x;	}
+		else if(ty<tx && ty<tz)	// y-axis is vertical
+		{	if(pp[1].x>pp[3].x)	py.z=1-py.z;	else	py.x=1-py.x;	}
+		else if(tx<tz && tx<tz)	// x-axis is vertical
+		{	if(pp[3].x>pp[2].x)	px.y=1-px.y;	else	px.z=1-px.z;	}
 	}
-	mreal res = px.val(ind);
-	if(dir=='y')	res = py.val(ind);
-	if(dir=='z')	res = pz.val(ind);
+	// return to normal variables
+	mglPoint rx = Min+(Max-Min)/px;
+	mglPoint ry = Min+(Max-Min)/py;
+	mglPoint rz = Min+(Max-Min)/pz;
+	mreal res = rx.val(ind);
+	if(dir=='y')	res = ry.val(ind);
+	if(dir=='z')	res = rz.val(ind);
 	return res;
 }
 //-----------------------------------------------------------------------------
-mreal mglCanvas::GetOrgX(char dir) const
+mreal mglCanvas::GetOrgX(char dir, bool inv) const
 {
 	mreal res = Org.x;
 	if(mgl_isnan(res))
@@ -271,11 +266,12 @@ mreal mglCanvas::GetOrgX(char dir) const
 		if(strchr("xyz",dir))	res = FindOptOrg(dir,0);
 		else if(dir=='t')		res = Min.x;
 		else res = B.b[6]>0 ? Max.x:Min.x;
+		if(inv)	res = Min.x+Max.x-res;
 	}
 	return res;
 }
 //-----------------------------------------------------------------------------
-mreal mglCanvas::GetOrgY(char dir) const
+mreal mglCanvas::GetOrgY(char dir, bool inv) const
 {
 	mreal res = Org.y;
 	if(mgl_isnan(res))
@@ -283,11 +279,12 @@ mreal mglCanvas::GetOrgY(char dir) const
 		if(strchr("xyz",dir))	res = FindOptOrg(dir,1);
 		else if(dir=='t')	res = Min.y;
 		else res = B.b[7]>0 ? Max.y:Min.y;
+		if(inv)	res = Min.y+Max.y-res;
 	}
 	return res;
 }
 //-----------------------------------------------------------------------------
-mreal mglCanvas::GetOrgZ(char dir) const
+mreal mglCanvas::GetOrgZ(char dir, bool inv) const
 {
 	mreal res = Org.z;
 	if(mgl_isnan(res))
@@ -295,36 +292,37 @@ mreal mglCanvas::GetOrgZ(char dir) const
 		if(strchr("xyz",dir))	res = FindOptOrg(dir,2);
 		else if(dir=='t')	res = Min.z;
 		else res = B.b[8]>0 ? Max.z:Min.z;
+		if(inv)	res = Min.z+Max.z-res;
 	}
 	return res;
 }
 //-----------------------------------------------------------------------------
 //	Put primitives
 //-----------------------------------------------------------------------------
-#define MGL_MARK_PLOT	if(Quality&4)	mark_draw(Pnt[p],type,size,&d);else	\
+#define MGL_MARK_PLOT	if(Quality&MGL_DRAW_LMEM)	mark_draw(Pnt[p],type,size,&d);else	\
 						{	mglPrim a;	a.w = pw;	a.s = size;	\
-							a.n1 = p;	a.n4 = type;	add_prim(a);	}
+							a.n1 = p;	a.n4 = type;	a.angl=0;	add_prim(a);	}
 void mglCanvas::mark_plot(long p, char type, mreal size)
 {
 	if(p<0 || mgl_isnan(Pnt[p].x) || mgl_isnan(size))	return;
 	long pp=p;
 //	mreal pw = fabs(PenWidth)*0.15/sqrt(font_factor);
 	mreal pw = 0.15/sqrt(font_factor);
-	mglDrawReg d;	d.set(this,1,1,0);
-	d.PDef = PDef;	d.pPos = pPos;
-	d.ObjId = ObjId;	d.PenWidth=pw;
-	if(size>=0)	size *= MarkSize;
-	if(size==0)	size = MarkSize;
-	size *= 0.35*font_factor;
+	mglDrawReg d;	d.set(this,dr_x,dr_y,dr_p);
+	d.PDef = PDef;	d.pPos = pPos;	d.PenWidth=pw;
+//	if(size>=0)	size *= MarkSize;
+//	if(size==0)	size = MarkSize;
+	size = size?fabs(size):1;
+	size *= MarkSize*0.35*font_factor;
 	if(type=='.')	size = fabs(PenWidth)*sqrt(font_factor/400);
 	if(TernAxis&4) for(int i=0;i<4;i++)
 	{	p = ProjScale(i, pp);	MGL_MARK_PLOT	}
 	else	{	MGL_MARK_PLOT	}
 }
 //-----------------------------------------------------------------------------
-#define MGL_LINE_PLOT	if(Quality&4)	line_draw(Pnt[p1],Pnt[p2],&dd);else	\
+#define MGL_LINE_PLOT	if(Quality&MGL_DRAW_LMEM)	line_draw(Pnt[p1],Pnt[p2],&dd);else	\
 						{	mglPrim a(1);	a.n3=PDef;	a.s = pPos;	\
-							a.n1 = p1;	a.n2 = p2;	a.w = pw;	add_prim(a);	}
+							a.n1 = p1;	a.n2 = p2;	a.w = pw;	a.angl=0;	add_prim(a);	}
 void mglCanvas::line_plot(long p1, long p2)
 {
 	if(PDef==0)	return;
@@ -334,9 +332,8 @@ void mglCanvas::line_plot(long p1, long p2)
 	mreal pw = fabs(PenWidth)*sqrt(font_factor/400), d;
 	d = hypot(Pnt[p1].x-Pnt[p2].x, Pnt[p1].y-Pnt[p2].y);
 
-	mglDrawReg dd;		dd.set(this,1,1,0);
-	dd.PDef = PDef;		dd.pPos = pPos;
-	dd.ObjId = ObjId;	dd.PenWidth=pw;
+	mglDrawReg dd;	dd.set(this,dr_x,dr_y,dr_p);
+	dd.PDef = PDef;	dd.pPos = pPos;	dd.PenWidth=pw;
 
 	if(TernAxis&4) for(int i=0;i<4;i++)
 	{	p1 = ProjScale(i, pp1);	p2 = ProjScale(i, pp2);
@@ -345,23 +342,24 @@ void mglCanvas::line_plot(long p1, long p2)
 	pPos = fmod(pPos+d/pw/1.5, 16);
 }
 //-----------------------------------------------------------------------------
-#define MGL_TRIG_PLOT	if(Quality&4)	trig_draw(Pnt[p1],Pnt[p2],Pnt[p3],true,&d);else	\
-						{	mglPrim a(2);	a.n1 = p1;	a.n2 = p2;	\
-							a.n3 = p3;	add_prim(a);}
+#define MGL_TRIG_PLOT	if(Quality&MGL_DRAW_LMEM)	trig_draw(Pnt[p1],Pnt[p2],Pnt[p3],true,&d);else	\
+						{	mglPrim a(2);	a.n1 = p1;	a.n2 = p2;	a.n3 = p3;	\
+							a.m=mask;	a.angl=MaskAn;	a.w = pw;	add_prim(a);}
 void mglCanvas::trig_plot(long p1, long p2, long p3)
 {
 	if(p1<0 || p2<0 || p3<0 || mgl_isnan(Pnt[p1].x) || mgl_isnan(Pnt[p2].x) || mgl_isnan(Pnt[p3].x))	return;
 	long pp1=p1,pp2=p2,pp3=p3;
-	mglDrawReg d;	d.set(this,1,1,0);	d.ObjId = ObjId;
+	mreal pw = fabs(PenWidth)*sqrt(font_factor/400);
+	mglDrawReg d;	d.set(this,dr_x,dr_y,dr_p);	d.PenWidth=pw;
 	if(TernAxis&4) for(int i=0;i<4;i++)
 	{	p1 = ProjScale(i, pp1);	p2 = ProjScale(i, pp2);
 		p3 = ProjScale(i, pp3);	MGL_TRIG_PLOT	}
 	else	{	MGL_TRIG_PLOT	}
 }
 //-----------------------------------------------------------------------------
-#define MGL_QUAD_PLOT	if(Quality&4)	quad_draw(Pnt[p1],Pnt[p2],Pnt[p3],Pnt[p4],&d);else	\
-						{	mglPrim a(3);	a.n1 = p1;	a.n2 = p2;	\
-							a.n3 = p3;	a.n4 = p4;	add_prim(a);	}
+#define MGL_QUAD_PLOT	if(Quality&MGL_DRAW_LMEM)	quad_draw(Pnt[p1],Pnt[p2],Pnt[p3],Pnt[p4],&d);else	\
+						{	mglPrim a(3);	a.n1 = p1;	a.n2 = p2;	a.n3 = p3;	a.n4 = p4;	\
+							a.m=mask;	a.angl=MaskAn;	a.w = pw;	add_prim(a);	}
 void mglCanvas::quad_plot(long p1, long p2, long p3, long p4)
 {
 	if(p1<0 || mgl_isnan(Pnt[p1].x))	{	trig_plot(p4,p2,p3);	return;	}
@@ -369,7 +367,8 @@ void mglCanvas::quad_plot(long p1, long p2, long p3, long p4)
 	if(p3<0 || mgl_isnan(Pnt[p3].x))	{	trig_plot(p1,p2,p4);	return;	}
 	if(p4<0 || mgl_isnan(Pnt[p4].x))	{	trig_plot(p1,p2,p3);	return;	}
 	long pp1=p1,pp2=p2,pp3=p3,pp4=p4;
-	mglDrawReg d;	d.set(this,1,1,0);	d.ObjId = ObjId;
+	mreal pw = fabs(PenWidth)*sqrt(font_factor/400);
+	mglDrawReg d;	d.set(this,dr_x,dr_y,dr_p);	d.PenWidth=pw;
 	if(TernAxis&4) for(int i=0;i<4;i++)
 	{	p1 = ProjScale(i, pp1);	p2 = ProjScale(i, pp2);
 		p3 = ProjScale(i, pp3);	p4 = ProjScale(i, pp4);
@@ -401,99 +400,107 @@ mreal mglCanvas::text_plot(long p,const wchar_t *text,const char *font,mreal siz
 
 	mreal fsize=size/6.5*font_factor, h = fnt->Height(font)*fsize, w, shift = -(sh+0.02)*h;
 	// text drawing itself
-	Push();
+
 #if MGL_HAVE_PTHREAD
 pthread_mutex_lock(&mutexPtx);
 #endif
-	inv = inv ^ (strchr(font,'T')!=0);
-	if(inv)	shift = 0.2*h-shift;
-	shift += 0.015*h;	// Correction for glyph rotation around proper point
-//	shift *= h;
-
-	int align;	mglGetStyle(font,0,&align);	align = align&3;
-	B.x = q.x;	B.y = q.y - shift;	B.z = q.z;
-	if(ll>0)
+#pragma omp critical(ptx)
 	{
-		B.x += shift*q.v/sqrt(ll);	B.y += shift*(1-q.u/sqrt(ll));
-		if(q.u==0 && !get(MGL_ENABLE_RTEXT))	B.y -= 0.1*h;
-	}
-	fscl = fsize;	forg = p;
+		Bt = B;	Bt.norot=(q.sub<0);	// NOTE check this later for mglInPlot
+		inv = inv ^ (strchr(font,'T')!=0);
+		if(inv)	shift = 0.2*h-shift;
+		shift += 0.015*h;	// Correction for glyph rotation around proper point
+	//		shift *= h;
 
-	if(mgl_isnan(ll) || !get(MGL_ENABLE_RTEXT))	ftet = 0;
-	else if(ll)	ftet = -180*atan2(q.v,q.u)/M_PI;
-	else 	ftet = NAN;
-
-	if(!(Quality&4))	// add text itself
-	{
-		char ch = mglGetStyle(font,0,0);
-		mglColor mc(ch);
-		if(!ch)	mc = col<0 ? mglColor(char(0.5-col)):Txt[long(col)].GetC(col);
-
-/*		for(long i=0;i<Prm.size();i++)	// try don't draw text if one present at this point
+		int align;	
+		char cc = mglGetStyle(font,0,&align);	align = align&3;
+		if(cc)	col = -cc;
+		Bt.x = q.x;	Bt.y = q.y - shift;	Bt.z = q.z;
+		if(ll>0)
 		{
-			const mglPnt &t=Pnt[Prm[i].n1];
-			if(Prm[i].type==6 && t.x==q.x && t.y==q.y)
-			{
-#if MGL_HAVE_PTHREAD
-				pthread_mutex_unlock(&mutexPtx);
-#endif
-				Pop();
-				return 0;
-			}
-		}*/
+			Bt.x += shift*q.v/sqrt(ll);	Bt.y += shift*(1-q.u/sqrt(ll));
+			if(q.u==0 && !get(MGL_ENABLE_RTEXT))	Bt.y -= 0.1*h;
+		}
+		fscl = fsize;	forg = p;
 
-		mglPrim a(6);	a.n1 = p;
-		a.n2 = int(255*mc.r) + 256*(int(255*mc.g) + 256*int(255*mc.b));
-		mglText txt(text,font);
-		Ptx.push_back(txt);	a.n3 = Ptx.size()-1;
-		a.s = size;	a.w = shift;	a.p=ftet;
-		add_prim(a);
-	}
+		if(mgl_isnan(ll) || !get(MGL_ENABLE_RTEXT))	ftet = 0;
+		else if(ll)	ftet = -180*atan2(q.v,q.u)/M_PI;
+		else 	ftet = NAN;
 
-	memset(B.b,0,9*sizeof(mreal));
-	B.b[0] = B.b[4] = B.b[8] = fscl;
-	register mreal opf = B.pf;
-	RotateN(ftet,0,0,1);	B.pf = opf;
-	if(strchr(font,'@'))	// draw box around text
-	{
-		long k1,k2,k3,k4;	mglPnt pt;	mglPoint pp;
-		w = fnt->Width(text,font);	h = fnt->Height(font);
-//		int align;	mglGetStyle(font,0,&align);	align = align&3;
-		mreal d=-w*align/2.-h*0.2;	w+=h*0.4;
-		pt = q;	pp = mglPoint(d,-h*0.4);		PostScale(pp);
-		pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;	MGL_PUSH(Pnt,pt,mutexPnt);	k1=Pnt.size()-1;
-		pt = q;	pp = mglPoint(w+d,-h*0.4);		PostScale(pp);
-		pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;	MGL_PUSH(Pnt,pt,mutexPnt);	k2=Pnt.size()-1;
-		pt = q;	pp = mglPoint(d,h*1.2);			PostScale(pp);
-		pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;	MGL_PUSH(Pnt,pt,mutexPnt);	k3=Pnt.size()-1;
-		pt = q;	pp = mglPoint(w+d,h*1.2);		PostScale(pp);
-		pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;	MGL_PUSH(Pnt,pt,mutexPnt);	k4=Pnt.size()-1;
-		line_plot(k1,k2);	line_plot(k1,k3);	line_plot(k4,k2);	line_plot(k4,k3);
+		if(!(Quality&MGL_DRAW_LMEM))	// add text itself
+		{
+			char ch = mglGetStyle(font,0,0);
+			mglColor mc(ch);
+			if(!ch)	mc = col<0 ? mglColor(char(0.5-col)):Txt[long(col)].GetC(col);
+
+			mglPrim a(6);	a.n1 = p;
+			a.n2 = int(255*mc.r) + 256*(int(255*mc.g) + 256*int(255*mc.b));
+			mglText txt(text,font);
+			a.n3 = Ptx.size();	Ptx.push_back(txt);
+			a.s = size;	a.w = shift;	a.p=ftet;
+			add_prim(a);
+		}
+
+		col = col<0 ? AddTexture(char(0.5-col)):col;
+		q.c=col;	q.t=0;	Txt[long(col)].GetC(col,0,q);
+		q.u = q.v = NAN;
+		memset(Bt.b,0,9*sizeof(mreal));
+		Bt.b[0] = Bt.b[4] = Bt.b[8] = fscl;
+		register mreal opf = Bt.pf;
+		Bt.RotateN(ftet,0,0,1);	Bt.pf = Bt.norot?1.55:opf;
+		if(strchr(font,'@'))	// draw box around text
+		{
+			long k1,k2,k3,k4;	mglPnt pt;	mglPoint pp;
+			w = fnt->Width(text,font);	h = fnt->Height(font);
+//			int align;	mglGetStyle(font,0,&align);	align = align&3;
+			mreal d=-w*align/2.-h*0.2;	w+=h*0.4;
+			pt = q;	pp = mglPoint(d,-h*0.4);		PostScale(&Bt,pp);
+			pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;
+#pragma omp critical(pnt)
+			{k1=Pnt.size();	MGL_PUSH(Pnt,pt,mutexPnt);}
+			pt = q;	pp = mglPoint(w+d,-h*0.4);		PostScale(&Bt,pp);
+			pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;
+#pragma omp critical(pnt)
+			{k2=Pnt.size();	MGL_PUSH(Pnt,pt,mutexPnt);}
+			pt = q;	pp = mglPoint(d,h*1.2);			PostScale(&Bt,pp);
+			pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;
+#pragma omp critical(pnt)
+			{k3=Pnt.size();	MGL_PUSH(Pnt,pt,mutexPnt);}
+			pt = q;	pp = mglPoint(w+d,h*1.2);		PostScale(&Bt,pp);
+			pt.x=pt.xx=pp.x;	pt.y=pt.yy=pp.y;
+#pragma omp critical(pnt)
+			{k4=Pnt.size();	MGL_PUSH(Pnt,pt,mutexPnt);}
+			line_plot(k1,k2);	line_plot(k1,k3);
+			line_plot(k4,k2);	line_plot(k4,k3);
+			mreal bl = AddTexture('w');
+			k1 = CopyNtoC(k1,bl);	k2 = CopyNtoC(k2,bl);
+			k3 = CopyNtoC(k3,bl);	k4 = CopyNtoC(k4,bl);
+			quad_plot(k1,k2,k3,k4);
+		}
+		fsize *= fnt->Puts(text,font,col)/2;
 	}
-	fsize *= fnt->Puts(text,font,col)/2;
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexPtx);
 #endif
-	Pop();	return fsize;
+	return fsize;
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Glyph(mreal x, mreal y, mreal f, int s, long j, mreal col)
 {
 	mglPrim a(4);	// NOTE: no projection since text_plot() did it
-	a.s = fscl/B.pf;
+	a.s = fscl/Bt.pf;
 	a.w = get(MGL_ENABLE_RTEXT)?ftet:1e5;
 	a.p = f/fnt->GetFact(s&3);
 	mreal cc = col<0 ? AddTexture(char(0.5-col)):col;
 	if(cc<0)	cc = CDef;
-	a.n1 = AddPnt(mglPoint(B.x,B.y,B.z), cc, mglPoint(x,y,NAN), -1, -1);
+	a.n1 = AddPnt(&Bt, mglPoint(Bt.x,Bt.y,Bt.z), cc, mglPoint(x,y,NAN), -1, -1);
 	a.n2 = forg; 	a.n3 = s;	a.n4 = AddGlyph(s,j);
 	if(a.n1<0)	return;
 
-	mglDrawReg d;	d.set(this,1,1,0);
-	d.PDef = s;		d.pPos = a.s;
-	d.ObjId=ObjId;	d.PenWidth=a.w;
-	
-	if(Quality&4)	glyph_draw(a,&d);
+	mglDrawReg d;	d.set(this,dr_x,dr_y,dr_p);
+	d.PDef = s;		d.pPos = a.s;	d.PenWidth=a.w;
+
+	if(Quality&MGL_DRAW_LMEM)	glyph_draw(a,&d);
 	else	add_prim(a);
 }
 //-----------------------------------------------------------------------------
@@ -507,6 +514,7 @@ void mglCanvas::InPlot(mreal x1,mreal x2,mreal y1,mreal y2, const char *st)
 	inX=Width*x1;	inY=Height*y1;	ZMin=1;
 	mglPrim p;	p.id = ObjId;
 	p.n1=x1*Width;	p.n2=x2*Width;	p.n3=y1*Height;	p.n4=y2*Height;
+#pragma omp critical(sub)
 	MGL_PUSH(Sub,p,mutexSub);
 
 	if(strchr(st,'T'))	{	y1*=0.9;	y2*=0.9;	}	// general title
@@ -534,33 +542,34 @@ void mglCanvas::InPlot(mreal x1,mreal x2,mreal y1,mreal y2, const char *st)
 	B1=B;	font_factor = B.b[0] < B.b[4] ? B.b[0] : B.b[4];
 }
 //-----------------------------------------------------------------------------
-void mglCanvas::InPlot(mreal x1,mreal x2,mreal y1,mreal y2, bool rel)
+void mglCanvas::InPlot(mglMatrix &M,mreal x1,mreal x2,mreal y1,mreal y2, bool rel)
 {
 	if(Width<=0 || Height<=0 || Depth<=0)	return;
-	B.clear();
-	if(get(MGL_AUTO_FACTOR)) B.pf = 1.55;	// Automatically change plot factor !!!
+	M.clear();
+	if(get(MGL_AUTO_FACTOR)) M.pf = 1.55;	// Automatically change plot factor !!!
 	if(rel)
 	{
-		B.x = B1.x + (x1+x2-1)/2*B1.b[0]/1.55;
-		B.y = B1.y + (y1+y2-1)/2*B1.b[4]/1.55;
-		B.b[0] = B1.b[0]*(x2-x1);	B.b[4] = B1.b[4]*(y2-y1);
-		B.b[8] = sqrt(B.b[0]*B.b[4]);
-		B.z = B1.z + (1.f-B.b[8]/(2*Depth))*B1.b[8];
+		M.x = B1.x + (x1+x2-1)/2*B1.b[0]/1.55;
+		M.y = B1.y + (y1+y2-1)/2*B1.b[4]/1.55;
+		M.b[0] = B1.b[0]*(x2-x1);	M.b[4] = B1.b[4]*(y2-y1);
+		M.b[8] = sqrt(M.b[0]*M.b[4]);
+		M.z = B1.z + (1.f-M.b[8]/(2*Depth))*B1.b[8];
 	}
 	else
 	{
-		B.x = (x1+x2)/2*Width;
-		B.y = (y1+y2)/2*Height;
-		B.b[0] = Width*(x2-x1);	B.b[4] = Height*(y2-y1);
-		B.b[8] = sqrt(B.b[0]*B.b[4]);
-		B.z = (1.f-B.b[8]/(2*Depth))*Depth;
+		M.x = (x1+x2)/2*Width;
+		M.y = (y1+y2)/2*Height;
+		M.b[0] = Width*(x2-x1);	M.b[4] = Height*(y2-y1);
+		M.b[8] = sqrt(M.b[0]*M.b[4]);
+		M.z = (1.f-M.b[8]/(2*Depth))*Depth;
 		B1=B;
 	}
-	inW=B.b[0];	inH=B.b[4];	ZMin=1;
+	inW=M.b[0];	inH=M.b[4];	ZMin=1;
 	inX=Width*x1;	inY=Height*y1;
-	font_factor = B.b[0] < B.b[4] ? B.b[0] : B.b[4];
+	font_factor = M.b[0] < M.b[4] ? M.b[0] : M.b[4];
 	mglPrim p;	p.id = ObjId;
 	p.n1=x1*Width;	p.n2=x2*Width;	p.n3=y1*Height;	p.n4=y2*Height;
+#pragma omp critical(sub)
 	MGL_PUSH(Sub,p,mutexSub);
 }
 //-----------------------------------------------------------------------------
@@ -570,7 +579,7 @@ void mglCanvas::StickPlot(int num, int id, mreal tet, mreal phi)
 	mglPoint p1(-1,0,0), p2(1,0,0);
 	// first iteration
 	InPlot(0,1,0,1,true);	Rotate(tet, phi);
-	PostScale(p1);	PostScale(p2);	f1 = B.pf;
+	PostScale(GetB(),p1);	PostScale(GetB(),p2);	f1 = B.pf;
 	dx=(p2.x-p1.x)*1.55/B1.b[0];	dy=(p2.y-p1.y)*1.55/B1.b[4];
 	wx=1/(1+(num-1)*fabs(dx));		wy=1/(1+(num-1)*fabs(dy));
 	x1=dx>0?dx*id:dx*(id-num+1);	y1=dy>0?dy*id:dy*(id-num+1);
@@ -586,6 +595,17 @@ void mglCanvas::StickPlot(int num, int id, mreal tet, mreal phi)
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Rotate(mreal tetz,mreal tetx,mreal tety)
+{
+	B.Rotate(tetz,tetx,tety);
+	if(get(MGL_AUTO_FACTOR))
+	{
+		mreal w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
+		mreal h=(fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B1.b[0];
+		B.pf = 1.55+0.6147*(w<h ? (h-1):(w-1));
+	}
+}
+//-----------------------------------------------------------------------------
+void mglMatrix::Rotate(mreal tetz,mreal tetx,mreal tety)
 {
 //	RotateN(TetX,1.,0.,0.);
 //	RotateN(TetY,0.,1.,0.);
@@ -595,16 +615,21 @@ void mglCanvas::Rotate(mreal tetz,mreal tetx,mreal tety)
 	R[0] = cx*cy;			R[1] = -cy*sx;			R[2] = sy;
 	R[3] = cx*sy*sz+cz*sx;	R[4] = cx*cz-sx*sy*sz;	R[5] =-cy*sz;
 	R[6] = sx*sz-cx*cz*sy;	R[7] = cx*sz+cz*sx*sy;	R[8] = cy*cz;
-	memcpy(O,B.b,9*sizeof(mreal));
-	B.b[0] = R[0]*O[0] + R[3]*O[1] + R[6]*O[2];
-	B.b[1] = R[1]*O[0] + R[4]*O[1] + R[7]*O[2];
-	B.b[2] = R[2]*O[0] + R[5]*O[1] + R[8]*O[2];
-	B.b[3] = R[0]*O[3] + R[3]*O[4] + R[6]*O[5];
-	B.b[4] = R[1]*O[3] + R[4]*O[4] + R[7]*O[5];
-	B.b[5] = R[2]*O[3] + R[5]*O[4] + R[8]*O[5];
-	B.b[6] = R[0]*O[6] + R[3]*O[7] + R[6]*O[8];
-	B.b[7] = R[1]*O[6] + R[4]*O[7] + R[7]*O[8];
-	B.b[8] = R[2]*O[6] + R[5]*O[7] + R[8]*O[8];
+	memcpy(O,b,9*sizeof(mreal));
+	b[0] = R[0]*O[0] + R[3]*O[1] + R[6]*O[2];
+	b[1] = R[1]*O[0] + R[4]*O[1] + R[7]*O[2];
+	b[2] = R[2]*O[0] + R[5]*O[1] + R[8]*O[2];
+	b[3] = R[0]*O[3] + R[3]*O[4] + R[6]*O[5];
+	b[4] = R[1]*O[3] + R[4]*O[4] + R[7]*O[5];
+	b[5] = R[2]*O[3] + R[5]*O[4] + R[8]*O[5];
+	b[6] = R[0]*O[6] + R[3]*O[7] + R[6]*O[8];
+	b[7] = R[1]*O[6] + R[4]*O[7] + R[7]*O[8];
+	b[8] = R[2]*O[6] + R[5]*O[7] + R[8]*O[8];
+}
+//-----------------------------------------------------------------------------
+void mglCanvas::RotateN(mreal Tet,mreal x,mreal y,mreal z)
+{
+	B.RotateN(Tet,x,y,z);
 	if(get(MGL_AUTO_FACTOR))
 	{
 		mreal w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
@@ -613,51 +638,27 @@ void mglCanvas::Rotate(mreal tetz,mreal tetx,mreal tety)
 	}
 }
 //-----------------------------------------------------------------------------
-void mglCanvas::RotateN(mreal Tet,mreal x,mreal y,mreal z)
+void mglMatrix::RotateN(mreal Tet,mreal x,mreal y,mreal z)
 {
 	mreal R[9],T[9],c=cos(Tet*M_PI/180),s=-sin(Tet*M_PI/180),r=1-c,n=sqrt(x*x+y*y+z*z);
 	x/=n;	y/=n;	z/=n;
 	T[0] = x*x*r+c;		T[1] = x*y*r-z*s;	T[2] = x*z*r+y*s;
 	T[3] = x*y*r+z*s;	T[4] = y*y*r+c;		T[5] = y*z*r-x*s;
 	T[6] = x*z*r-y*s;	T[7] = y*z*r+x*s;	T[8] = z*z*r+c;
-	memcpy(R,B.b,9*sizeof(mreal));
-	B.b[0] = T[0]*R[0] + T[3]*R[1] + T[6]*R[2];
-	B.b[1] = T[1]*R[0] + T[4]*R[1] + T[7]*R[2];
-	B.b[2] = T[2]*R[0] + T[5]*R[1] + T[8]*R[2];
-	B.b[3] = T[0]*R[3] + T[3]*R[4] + T[6]*R[5];
-	B.b[4] = T[1]*R[3] + T[4]*R[4] + T[7]*R[5];
-	B.b[5] = T[2]*R[3] + T[5]*R[4] + T[8]*R[5];
-	B.b[6] = T[0]*R[6] + T[3]*R[7] + T[6]*R[8];
-	B.b[7] = T[1]*R[6] + T[4]*R[7] + T[7]*R[8];
-	B.b[8] = T[2]*R[6] + T[5]*R[7] + T[8]*R[8];
-	if(get(MGL_AUTO_FACTOR))
-	{
-		mreal w=(fabs(B.b[3])+fabs(B.b[4])+fabs(B.b[5]))/B1.b[4];
-		mreal h=(fabs(B.b[0])+fabs(B.b[1])+fabs(B.b[2]))/B1.b[0];
-		B.pf = 1.55+0.6147*(w<h ? (h-1):(w-1));
-	}
+	memcpy(R,b,9*sizeof(mreal));
+	b[0] = T[0]*R[0] + T[3]*R[1] + T[6]*R[2];
+	b[1] = T[1]*R[0] + T[4]*R[1] + T[7]*R[2];
+	b[2] = T[2]*R[0] + T[5]*R[1] + T[8]*R[2];
+	b[3] = T[0]*R[3] + T[3]*R[4] + T[6]*R[5];
+	b[4] = T[1]*R[3] + T[4]*R[4] + T[7]*R[5];
+	b[5] = T[2]*R[3] + T[5]*R[4] + T[8]*R[5];
+	b[6] = T[0]*R[6] + T[3]*R[7] + T[6]*R[8];
+	b[7] = T[1]*R[6] + T[4]*R[7] + T[7]*R[8];
+	b[8] = T[2]*R[6] + T[5]*R[7] + T[8]*R[8];
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::View(mreal tetx,mreal tetz,mreal tety)
-{
-	mreal R[9], A[9];
-	mreal cx=cos(tetx*M_PI/180), sx=sin(tetx*M_PI/180);
-	mreal cy=cos(tety*M_PI/180), sy=sin(tety*M_PI/180);
-	mreal cz=cos(tetz*M_PI/180), sz=sin(tetz*M_PI/180);
-	R[0] = cx*cy;			R[1] = -cy*sx;			R[2] = sy;
-	R[3] = cx*sy*sz+cz*sx;	R[4] = cx*cz-sx*sy*sz;	R[5] =-cy*sz;
-	R[6] = sx*sz-cx*cz*sy;	R[7] = cx*sz+cz*sx*sy;	R[8] = cy*cz;
-	memcpy(A,Bp.b,9*sizeof(mreal));		ClfZB();
-	Bp.b[0] = R[0]*A[0] + R[3]*A[1] + R[6]*A[2];
-	Bp.b[1] = R[1]*A[0] + R[4]*A[1] + R[7]*A[2];
-	Bp.b[2] = R[2]*A[0] + R[5]*A[1] + R[8]*A[2];
-	Bp.b[3] = R[0]*A[3] + R[3]*A[4] + R[6]*A[5];
-	Bp.b[4] = R[1]*A[3] + R[4]*A[4] + R[7]*A[5];
-	Bp.b[5] = R[2]*A[3] + R[5]*A[4] + R[8]*A[5];
-	Bp.b[6] = R[0]*A[6] + R[3]*A[7] + R[6]*A[8];
-	Bp.b[7] = R[1]*A[6] + R[4]*A[7] + R[7]*A[8];
-	Bp.b[8] = R[2]*A[6] + R[5]*A[7] + R[8]*A[8];
-}
+{	Bp.Rotate(-tetz,-tetx,-tety);	}
 //-----------------------------------------------------------------------------
 void mglCanvas::Zoom(mreal x1, mreal y1, mreal x2, mreal y2)
 {
@@ -685,12 +686,15 @@ void mglCanvas::Aspect(mreal Ax,mreal Ay,mreal Az)
 {
 	if(mgl_isnan(Ax))
 	{
-		mreal dy = (Max.y-Min.y), dx = (Max.x-Min.x);
+		mreal dy = (Max.y-Min.y), dx = (Max.x-Min.x), dz = (Max.z-Min.z);
 		if(islog(Min.x,Max.x) && fx)	dx = log10(Max.x/Min.x);
 		if(islog(Min.y,Max.y) && fy)	dy = log10(Max.y/Min.y);
-		mreal f=exp(M_LN10*floor(0.5+log10(fabs(dy/dx))));
-		if(!mgl_isnan(Ay))	f=Ay;
-		Ax = Height*dx*f;	Ay = Width*dy;	Az = Depth;
+		if(islog(Min.z,Max.z) && fz)	dz = log10(Max.z/Min.z);
+		mreal fy=exp(M_LN10*floor(0.5+log10(fabs(dy/dx))));
+		mreal fz=exp(M_LN10*floor(0.5+log10(fabs(dz/dx))));
+		if(Ay>0)	fy*=Ay;
+		if(Az>0)	fz*=Az;
+		Ax = Height*dx;	Ay = Width*dy*fy;	Az = Depth*dz*fz;
 	}
 	mreal a = fabs(Ax) > fabs(Ay) ? fabs(Ax) : fabs(Ay);
 	a = a > fabs(Az) ? a : fabs(Az);
@@ -723,15 +727,19 @@ void mglCanvas::arrow_plot(long n1, long n2, char st)
 {
 	if(n1<0 || n2<0 || !strchr("AVKSDTIO",st))	return;
 	float ll = PenWidth*ArrowSize*0.35*font_factor;
+	uint64_t m=mask;	int ma=MaskAn;
+	ResetMask();
 	if((Quality&3)==3)
 		arrow_plot_3d(n1, n2, st, ll);
 	else
 		arrow_draw(n1, n2, st, ll);
+	mask=m;	MaskAn=ma;
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const char *font, const char *opt)
 {
-	long n=leg.size(), iw, ih;
+	long n=leg.size();
+	mreal iw, ih;
 	if(n<1)	{	SetWarn(mglWarnLeg,"Legend");	return;	}
 	mreal ll = SaveState(opt);	if(mgl_isnan(ll))	ll=0.1;
 	if(saved)	MarkSize=MSS;	// restore back size of marks
@@ -765,11 +773,20 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 		nrow = 1+(n-1)/j;
 		ncol = (n+nrow-1)/nrow;
 	}
-	x = x*(iw-w*ncol-2*dx)+B.x-iw/2+dx;
-	y = y*(ih-h*nrow-2*dy)+B.y-ih/2+dy;
+	if(mglchr(font,'^'))	// use "external" positioning
+	{
+		x = x>=0.5 ? x*iw : x*iw-w*ncol-2*dx;
+		y = y>=0.5 ? y*ih : y*ih-h*nrow-2*dy;
+	}
+	else
+	{
+		x *= iw-w*ncol-2*dx;
+		y *= ih-h*nrow-2*dy;
+	}
+	x += B.x-iw/2+dx;	y += B.y-ih/2+dy;
 	// draw it
 	long k1=0,k2=0,k3=0,k4=0;
-	mglPoint p,q=mglPoint(NAN,NAN);
+	mglPoint p,q=mglPoint(NAN,NAN,NAN);
 
 	for(i=0;ff[i] && ff[i]!=':';i++)	if(strchr(MGL_COLORS,ff[i]))
 	{
@@ -783,13 +800,14 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 	mreal c1=AddTexture(char(k1?k1:'w')), c2=AddTexture(char(k2?k2:'k'));
 	if((Flag&3)==2)	{	mreal cc=c1;	c2=c1;	c1=cc;	}
 
+	mglMatrix M=B;	M.norot=true;
 	if(strchr(ff,'#'))	// draw bounding box
 	{
 		SetPenPal("k-");
-		k1=AddPnt(mglPoint(x,y,Depth/MGL_FEPSILON),c1,q,-1,0);
-		k2=AddPnt(mglPoint(x+w*ncol,y,Depth/MGL_FEPSILON),c1,q,-1,0);
-		k3=AddPnt(mglPoint(x,y+h*nrow,Depth/MGL_FEPSILON),c1,q,-1,0);
-		k4=AddPnt(mglPoint(x+w*ncol,y+h*nrow,Depth/MGL_FEPSILON),c1,q,-1,0);
+		k1=AddPnt(&M,mglPoint(x,y,Depth/MGL_FEPSILON),c1,q,-1,0);
+		k2=AddPnt(&M,mglPoint(x+w*ncol,y,Depth/MGL_FEPSILON),c1,q,-1,0);
+		k3=AddPnt(&M,mglPoint(x,y+h*nrow,Depth/MGL_FEPSILON),c1,q,-1,0);
+		k4=AddPnt(&M,mglPoint(x+w*ncol,y+h*nrow,Depth/MGL_FEPSILON),c1,q,-1,0);
 		quad_plot(k1,k2,k3,k4);
 		k1=CopyNtoC(k1,c2);	k2=CopyNtoC(k2,c2);
 		k3=CopyNtoC(k3,c2);	k4=CopyNtoC(k4,c2);
@@ -800,16 +818,16 @@ void mglCanvas::Legend(const std::vector<mglText> &leg, mreal x, mreal y, const
 	{
 		register long iy=nrow-(i%nrow)-1,ix=i/nrow;
 		char m=SetPenPal(leg[i].stl.c_str());
-		k1=AddPnt(mglPoint(x+ix*w+0.1*ll,y+iy*h+0.45*h,Depth),CDef,q,-1,0);
-		k2=AddPnt(mglPoint(x+ix*w+0.9*ll,y+iy*h+0.45*h,Depth),CDef,q,-1,0);	pPos=0;
+		k1=AddPnt(&M,mglPoint(x+ix*w+0.1*ll,y+iy*h+0.45*h,Depth),CDef,q,-1,0);
+		k2=AddPnt(&M,mglPoint(x+ix*w+0.9*ll,y+iy*h+0.45*h,Depth),CDef,q,-1,0);	pPos=0;
 		if(!leg[i].stl.empty())	line_plot(k1,k2);
 		if(m)	for(j=0;j<LegendMarks;j++)
 		{
 			p = mglPoint(x+ix*w+0.1f*ll + (j+1)*0.8f*ll/(1.+LegendMarks),y+iy*h+0.45*h,Depth);
-			mark_plot(AddPnt(p,CDef,q,-1,0),m);
+			mark_plot(AddPnt(&M,p,CDef,q,-1,0),m);
 		}
 		p = mglPoint(x+ix*w+((!leg[i].stl.empty())?ll:0.01*iw), y+iy*h+0.15*h, Depth);
-		text_plot(AddPnt(p,-1,q,-1,0), leg[i].text.c_str(), ff, size);
+		text_plot(AddPnt(&M,p,-1,q,-1,0), leg[i].text.c_str(), ff, size);
 	}
 	Pop();	EndGroup();	delete []ff;
 }
@@ -865,24 +883,24 @@ void mglCanvas::Table(mreal x, mreal y, HCDT val, const wchar_t *text, const cha
 	if(grid)	// draw bounding box
 	{
 		SetPenPal("k-");
-		k1=AddPnt(mglPoint(x,y,Depth),-1,q,-1,0);
-		k2=AddPnt(mglPoint(x,y+m*h,Depth),-1,q,-1,0);
+		k1=AddPnt(&B,mglPoint(x,y,Depth),-1,q,-1,0);
+		k2=AddPnt(&B,mglPoint(x,y+m*h,Depth),-1,q,-1,0);
 		line_plot(k1,k2);
 		ww = *text ? TextWidth(text,frm,-1)+sp:0;
-		k1=AddPnt(mglPoint(x+ww,y,Depth),-1,q,-1,0);
-		k2=AddPnt(mglPoint(x+ww,y+m*h,Depth),-1,q,-1,0);
+		k1=AddPnt(&B,mglPoint(x+ww,y,Depth),-1,q,-1,0);
+		k2=AddPnt(&B,mglPoint(x+ww,y+m*h,Depth),-1,q,-1,0);
 		line_plot(k1,k2);
 		for(i=0,xx=x+ww,yy=y;i<n;i++)
 		{
 			xx += eqd ? w1:(TextWidth(str[i].c_str(),frm,-1)+sp);
-			k1=AddPnt(mglPoint(xx,yy,Depth),-1,q,-1,0);
-			k2=AddPnt(mglPoint(xx,yy+m*h,Depth),-1,q,-1,0);
+			k1=AddPnt(&B,mglPoint(xx,yy,Depth),-1,q,-1,0);
+			k2=AddPnt(&B,mglPoint(xx,yy+m*h,Depth),-1,q,-1,0);
 			line_plot(k1,k2);
 		}
 		for(i=0,xx=x,yy=y;i<=m;i++)
 		{
-			k1=AddPnt(mglPoint(xx,yy,Depth),-1,q,-1,0);
-			k2=AddPnt(mglPoint(xx+w,yy,Depth),-1,q,-1,0);
+			k1=AddPnt(&B,mglPoint(xx,yy,Depth),-1,q,-1,0);
+			k2=AddPnt(&B,mglPoint(xx+w,yy,Depth),-1,q,-1,0);
 			line_plot(k1,k2);	yy += h;
 		}
 	}
@@ -890,14 +908,14 @@ void mglCanvas::Table(mreal x, mreal y, HCDT val, const wchar_t *text, const cha
 	if(*text)
 	{
 		ww = TextWidth(text,frm,-1)+sp;
-		k1=AddPnt(mglPoint(x+ww*align/2.,y+h*(m-0.99),Depth),-1,q,-1,0);
+		k1=AddPnt(&B,mglPoint(x+ww*align/2.,y+h*(m-0.99),Depth),-1,q,-1,0);
 		text_plot(k1,text,frm);
 	}
 	else 	ww = 0;
 	for(i=0,xx=x+ww,yy=y+h*(m-0.99);i<n;i++)	// draw lines and legend
 	{
 		ww = eqd ? w1:(TextWidth(str[i].c_str(),frm,-1)+sp);
-		k1=AddPnt(mglPoint(xx+ww*align/2.,yy,Depth),-1,q,-1,0);
+		k1=AddPnt(&B,mglPoint(xx+ww*align/2.,yy,Depth),-1,q,-1,0);
 		text_plot(k1,str[i].c_str(),frm);	xx += ww;
 	}
 	FontSize = fsize;	EndGroup();
@@ -906,11 +924,7 @@ void mglCanvas::Table(mreal x, mreal y, HCDT val, const wchar_t *text, const cha
 void mglCanvas::Title(const char *title,const char *stl,mreal size)
 {
 	if(!title)	title="";
-	size_t s = mbstowcs(0,title,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,title,s);
-	Title(wcs, stl,size);
-	delete []wcs;
+	MGL_TO_WCS(title,Title(wcs, stl,size));
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::Title(const wchar_t *title,const char *stl,mreal size)
@@ -919,19 +933,22 @@ void mglCanvas::Title(const wchar_t *title,const char *stl,mreal size)
 	if(h>=inH)	{	SetWarn(mglWarnSpc,"Title");	return;	}
 	static int cgid=1;	StartGroup("Title",cgid++);
 	bool box=mglchr(stl,'#');
-	int align;	mglGetStyle(stl,0,&align);	align = align&3;
+	int align;
+	char col = mglGetStyle(stl,0,&align);	align = align&3;
+	if(col==0)	col = 'k';
 	mreal y=inY+inH-h;
-	mglPoint p(inX + inW*align/2.,y,Depth),q(NAN);
-	if(title)	text_plot(AddPnt(p,-1,q,-1,0),title,stl,size);
+	mglPoint p(inX + inW*align/2.,y,3*Depth),q(NAN,NAN,NAN);
+	mglMatrix M=B;	M.norot=true;
+	if(title)	text_plot(AddPnt(&M,p,-1,q,-1,0),title,stl,size);
 	if(box)	//	draw boungind box
 	{
-		mreal c1=AddTexture('w'), c2=AddTexture('k');
+		mreal c1=AddTexture('w'), c2=AddTexture(col);
 		if((Flag&3)==2)	{	mreal cc=c1;	c2=c1;	c1=cc;	}
 		long k1,k2,k3,k4;
-		k1=AddPnt(mglPoint(inX,y,Depth),c1,q,-1,0);
-		k2=AddPnt(mglPoint(inX+inW,y,Depth),c1,q,-1,0);
-		k3=AddPnt(mglPoint(inX,y+h,Depth),c1,q,-1,0);
-		k4=AddPnt(mglPoint(inX+inW,y+h,Depth),c1,q,-1,0);
+		k1=AddPnt(&M,mglPoint(inX,y-h*0.4,3*Depth),c1,q,-1,0);
+		k2=AddPnt(&M,mglPoint(inX+inW,y-h*0.4,3*Depth),c1,q,-1,0);
+		k3=AddPnt(&M,mglPoint(inX,y+h,3*Depth),c1,q,-1,0);
+		k4=AddPnt(&M,mglPoint(inX+inW,y+h,3*Depth),c1,q,-1,0);
 		quad_plot(k1,k2,k3,k4);
 		k1=CopyNtoC(k1,c2);	k2=CopyNtoC(k2,c2);
 		k3=CopyNtoC(k3,c2);	k4=CopyNtoC(k4,c2);
@@ -952,15 +969,17 @@ void mglCanvas::StartAutoGroup (const char *lbl)
 	if(ObjId<0)	{	ObjId = -id;	id++;	}
 	register size_t len = Grp.size();
 	if(ObjId>=0 && len>0 && ObjId!=Grp[len-1].Id)
-	{	MGL_PUSH(Grp,mglGroup(lbl,ObjId),mutexGrp);	}
+#pragma omp critical(grp)
+	{	MGL_PUSH(Grp,mglGroup(lbl,ObjId),mutexGrp);}
 	else if(ObjId<0)
-	{	MGL_PUSH(Grp,mglGroup(lbl,ObjId),mutexGrp);	}
+#pragma omp critical(grp)
+	{	MGL_PUSH(Grp,mglGroup(lbl,ObjId),mutexGrp);}
 }
 //-----------------------------------------------------------------------------
 void mglCanvas::EndGroup()
 {
 	LoadState();
-	if(Quality&4)
+	if(Quality&MGL_DRAW_LMEM)
 	{
 		Pnt.clear();		Prm.clear();		Ptx.clear();
 		Glf.clear();		Act.clear(); 	Grp.clear();
@@ -980,3 +999,9 @@ int mglCanvas::IsActive(int xs, int ys,int &n)
 	n=-1;	return GetObjId(xs,ys);
 }
 //-----------------------------------------------------------------------------
+void mglCanvas::Push()
+{
+#pragma omp critical(stk)
+	{MGL_PUSH(stack,B,mutexStk);}
+}
+//-----------------------------------------------------------------------------
