@@ -1,6 +1,6 @@
 /***************************************************************************
  * canvas_cf.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -99,6 +99,8 @@ void MGL_EXPORT mgl_mat_pop(HMGL gr)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->Pop();	}
 void MGL_EXPORT mgl_clf(HMGL gr)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->Clf();	}
+void MGL_EXPORT mgl_clf_chr(HMGL gr, char ch)
+{	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->Clf(mglColor(ch));	}
 void MGL_EXPORT mgl_clf_rgb(HMGL gr, double r, double g, double b)
 {	mglCanvas *gg = dynamic_cast<mglCanvas *>(gr);	if(gg)	gg->Clf(mglColor(r,g,b));	}
 //-----------------------------------------------------------------------------
@@ -147,6 +149,8 @@ void MGL_EXPORT mgl_gridplot(HMGL gr, int nx, int ny, int i, double dd)
 	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);
 	if(g)	g->InPlot(ix*wx,wx*(ix+1-dd),1-wy*(iy+1-dd),1-iy*wy,true);
 }
+void MGL_EXPORT mgl_gridplot_(uintptr_t *gr, int *nx, int *ny, int *m, mreal *d)
+{	mgl_gridplot(_GR_,*nx,*ny,*m,*d);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_stickplot(HMGL gr, int num, int i, double tet, double phi)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->StickPlot(num, i, tet, phi);	}
@@ -194,6 +198,8 @@ void MGL_EXPORT mgl_mat_push_(uintptr_t *gr)	{	_GR_->Push();	}
 void MGL_EXPORT mgl_mat_pop_(uintptr_t *gr)	{	_GR_->Pop();	}
 void MGL_EXPORT mgl_clf_(uintptr_t *gr)
 {	_GR_->Clf();	}
+void MGL_EXPORT mgl_clf_chr_(uintptr_t *gr, const char *ch, int)
+{	_GR_->Clf(mglColor(*ch));	}
 void MGL_EXPORT mgl_clf_rgb_(uintptr_t *gr, mreal *r, mreal *g, mreal *b)
 {	_GR_->Clf(mglColor(*r,*g,*b));	}
 //-----------------------------------------------------------------------------
@@ -367,7 +373,7 @@ void MGL_EXPORT mgl_axis_grid_(uintptr_t *gr, const char *dir,const char *pen, c
 	char *p=new char[n+1];	memcpy(p,pen,n);	p[n]=0;
 	char *o=new char[m+1];	memcpy(o,opt,m);	o[m]=0;
 	_GR_->Grid(s,p,o);	delete []s;	delete []p;	delete []o;	}
-	void MGL_EXPORT mgl_label_(uintptr_t *gr, const char *dir, const char *text, mreal *pos, const char *opt,int,int l,int m)
+void MGL_EXPORT mgl_label_(uintptr_t *gr, const char *dir, const char *text, mreal *pos, const char *opt,int,int l,int m)
 {	char *s=new char[l+1];	memcpy(s,text,l);	s[l]=0;
 	char *o=new char[m+1];	memcpy(o,opt,m);	o[m]=0;
 	_GR_->Label(*dir, s, *pos, o);	delete []s;	delete []o;	}
@@ -408,34 +414,6 @@ double MGL_EXPORT mgl_wnd_get_delay(HMGL gr)
 void MGL_EXPORT mgl_wnd_set_delay_(uintptr_t *gr, mreal *dt)	{	_GR_->SetDelay(*dt);	}
 double MGL_EXPORT mgl_wnd_get_delay_(uintptr_t *gr)	{	return _GR_->GetDelay();	}
 //-----------------------------------------------------------------------------
-HMEX MGL_EXPORT mgl_create_expr(const char *expr)	{	return new mglFormula(expr);	}
-void MGL_EXPORT mgl_delete_expr(HMEX ex)	{	delete ex;	}
-double MGL_EXPORT mgl_expr_eval(HMEX ex, double x, double y,double z)
-{	return ex->Calc(x,y,z);	}
-double MGL_EXPORT mgl_expr_eval_v(HMEX ex, mreal *var)
-{	return ex->Calc(var);	}
-double MGL_EXPORT mgl_expr_diff(HMEX ex, char dir, double x, double y,double z)
-{	return ex->CalcD(dir,x,y,z);	}
-double MGL_EXPORT mgl_expr_diff_v(HMEX ex, char dir, mreal *var)
-{	return ex->CalcD(var, dir);		}
-//-----------------------------------------------------------------------------
-uintptr_t MGL_EXPORT mgl_create_expr_(const char *expr, int l)
-{	char *s=new char[l+1];	memcpy(s,expr,l);	s[l]=0;
-	uintptr_t res = uintptr_t(mgl_create_expr(s));
-	delete []s;	return res;	}
-void MGL_EXPORT mgl_delete_expr_(uintptr_t *ex)	{	mgl_delete_expr((HMEX)ex);	}
-double MGL_EXPORT mgl_eval_expr_(uintptr_t *ex, mreal *x, mreal *y, mreal *z)
-{	return mgl_expr_eval((HMEX) ex, *x,*y,*z);		}
-double MGL_EXPORT mgl_diff_expr_(uintptr_t *ex, const char *dir, mreal *x, mreal *y, mreal *z, int)
-{	return mgl_expr_diff((HMEX) ex, *dir,*x,*y,*z);	}
-//-----------------------------------------------------------------------------
-HAEX MGL_EXPORT mgl_create_cexpr(const char *expr)	{	return new mglFormulaC(expr);	}
-void MGL_EXPORT mgl_delete_cexpr(HAEX ex)	{	delete ex;	}
-dual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z)
-{	return ex->Calc(x,y,z);	}
-dual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *var)
-{	return ex->Calc(var);	}
-//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_set_plotfactor(HMGL gr, double val)
 {	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);	if(g)	g->SetPlotFactor(val);	}
 void MGL_EXPORT mgl_set_plotfactor_(uintptr_t *gr, mreal *val)
