@@ -1,6 +1,6 @@
 /***************************************************************************
  * data_io.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -25,6 +25,7 @@
 
 #include "mgl2/datac.h"
 #include "mgl2/evalc.h"
+#include "mgl2/thread.h"
 
 #if MGL_HAVE_HDF5
 #define H5_USE_16_API
@@ -55,6 +56,7 @@ void mglFromStr(HADT d,char *buf,long NX,long NY,long NZ)	// TODO: add multithre
 	mgl_datac_create(d, NX,NY,NZ);
 	long nb = strlen(buf);
 	register long i=0, j=0;
+	setlocale(LC_NUMERIC, "C");
 	while(j<nb)
 	{
 		while(buf[j]<=' ' && j<nb)	j++;
@@ -85,6 +87,7 @@ void mglFromStr(HADT d,char *buf,long NX,long NY,long NZ)	// TODO: add multithre
 		d->a[i] = dual(re,im);
 		i++;	if(i>=NX*NY*NZ)	break;
 	}
+	setlocale(LC_NUMERIC, "");
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_datac_set(HADT d, HCDT a)
@@ -140,33 +143,23 @@ void MGL_EXPORT mgl_datac_set_float(HADT d, const float *A,long NX,long NY,long
 {
 	if(NX<=0 || NY<=0 || NZ<=0)	return;
 	mgl_datac_create(d, NX,NY,NZ);	if(!A)	return;
-#if MGL_USE_DOUBLE
+#pragma omp parallel for
 	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
-#else
-	memcpy(d->a,A,NX*NY*NZ*sizeof(float));
-#endif
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_datac_set_double(HADT d, const double *A,long NX,long NY,long NZ)
 {
 	if(NX<=0 || NY<=0 || NZ<=0)	return;
 	mgl_datac_create(d, NX,NY,NZ);	if(!A)	return;
-#if MGL_USE_DOUBLE
-	memcpy(d->a,A,NX*NY*NZ*sizeof(double));
-#else
+#pragma omp parallel for
 	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
-#endif
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_datac_set_complex(HADT d, const dual *A,long NX,long NY,long NZ)
 {
 	if(NX<=0 || NY<=0 || NZ<=0)	return;
 	mgl_datac_create(d, NX,NY,NZ);	if(!A)	return;
-	#if MGL_USE_DOUBLE
-	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
-	#else
 	memcpy(d->a,A,NX*NY*NZ*sizeof(float));
-	#endif
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_datac_set_float_(uintptr_t *d, const float *A,int *NX,int *NY,int *NZ)
@@ -197,10 +190,11 @@ void MGL_EXPORT mgl_datac_save(HCDT d, const char *fname,long ns)
 {
 	const mglDataC *dd = dynamic_cast<const mglDataC*>(d);
 	if(!dd)	{	mgl_data_save(d,fname,ns);	return;	}
-	FILE *fp;
-	fp = fopen(fname,"w");
+	FILE *fp = fopen(fname,"w");
+	if(!fp)	return;
 	register long i,j,k;
 	long nx=dd->nx, ny=dd->ny, nz=dd->nz;
+	setlocale(LC_NUMERIC, "C");
 	if(ns<0 || (ns>=nz && nz>1))	for(k=0;k<nz;k++)
 	{	// save whole data
 		for(i=0;i<ny;i++)
@@ -220,6 +214,7 @@ void MGL_EXPORT mgl_datac_save(HCDT d, const char *fname,long ns)
 		else if(ns<ny)	for(j=0;j<nx;j++)
 			fprintf(fp,"%g+i%g\t", real(dd->a[j+nx*ns]), imag(dd->a[j+nx*ns]));
 	}
+	setlocale(LC_NUMERIC, "");
 	fclose(fp);
 }
 void MGL_EXPORT mgl_datac_save_(uintptr_t *d, const char *fname,int *ns,int l)
@@ -378,10 +373,13 @@ int MGL_EXPORT mgl_datac_read_mat_(uintptr_t *d, const char *fname,int *dim,int
 MGL_NO_EXPORT void *mgl_cfill_x(void *par)
 {
 	mglThreadC *t=(mglThreadC *)par;
-	register long i0, nx=t->p[0],ny=t->p[1];
+	long nx=t->p[0],ny=t->p[1];
 	dual *b=t->a, x1=t->b[0], dx=t->b[1];
-	register char dir = t->s[0];
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+	char dir = t->s[0];
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
 		if(dir=='x') b[i0] = x1+dx*mreal(i0%nx);
 		else if(dir=='y') b[i0] = x1+dx*mreal((i0/nx)%ny);
@@ -405,8 +403,7 @@ void MGL_EXPORT mgl_datac_fill_(uintptr_t *d, dual *x1,dual *x2,const char *dir,
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_datac_squeeze(HADT d, long rx,long ry,long rz,long smooth)
 {
-	long kx,ky,kz,i,j,k;
-	long nx=d->nx, ny=d->ny, nz=d->nz;
+	long kx,ky,kz, nx=d->nx, ny=d->ny, nz=d->nz;
 	dual *b;
 
 	// simple checking
@@ -416,19 +413,23 @@ void MGL_EXPORT mgl_datac_squeeze(HADT d, long rx,long ry,long rz,long smooth)
 	// new sizes
 	kx = 1+(nx-1)/rx;	ky = 1+(ny-1)/ry;	kz = 1+(nz-1)/rz;
 	b = new dual[kx*ky*kz];
-	if(!smooth)	for(k=0;k<kz;k++)	for(j=0;j<ky;j++)	for(i=0;i<kx;i++)
-		b[i+kx*(j+ky*k)] = d->a[i*rx+nx*(j*ry+ny*rz*k)];
-	else		for(k=0;k<kz;k++)	for(j=0;j<ky;j++)	for(i=0;i<kx;i++)
-	{
-		long dx,dy,dz,i1,j1,k1;
-		dx = (i+1)*rx<=nx ? rx : nx-i*rx;
-		dy = (j+1)*ry<=ny ? ry : ny-j*ry;
-		dz = (k+1)*rz<=nz ? rz : nz-k*rz;
-		dual s = 0;
-		for(k1=k*rz;k1<k*rz+dz;k1++)	for(j1=j*ry;j1<j*ry+dz;j1++)	for(i1=i*rx;i1<i*rx+dx;i1++)
-			s += d->a[i1+nx*(j1+ny*k1)];
-		b[i+kx*(j+ky*k)] = s/mreal(dx*dy*dz);
-	}
+	if(!smooth)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<kz;k++)	for(long j=0;j<ky;j++)	for(long i=0;i<kx;i++)
+			b[i+kx*(j+ky*k)] = d->a[i*rx+nx*(j*ry+ny*rz*k)];
+	else
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<kz;k++)	for(long j=0;j<ky;j++)	for(long i=0;i<kx;i++)
+		{
+			long dx,dy,dz,i1,j1,k1;
+			dx = (i+1)*rx<=nx ? rx : nx-i*rx;
+			dy = (j+1)*ry<=ny ? ry : ny-j*ry;
+			dz = (k+1)*rz<=nz ? rz : nz-k*rz;
+			dual s = 0;
+			for(k1=k*rz;k1<k*rz+dz;k1++)	for(j1=j*ry;j1<j*ry+dz;j1++)	for(i1=i*rx;i1<i*rx+dx;i1++)
+				s += d->a[i1+nx*(j1+ny*k1)];
+			b[i+kx*(j+ky*k)] = s/mreal(dx*dy*dz);
+		}
 	if(!d->link)	delete [](d->a);
 	d->a=b;	d->nx = kx;  d->ny = ky;  d->nz = kz;	d->NewId();	d->link=false;
 }
@@ -441,29 +442,35 @@ void MGL_EXPORT mgl_datac_extend(HADT d, long n1, long n2)
 	if(nz>2 || n1==0)	return;
 	long mx, my, mz;
 	dual *b=0;
-	register long i,j;
 	if(n1>0) // extend to higher dimension(s)
 	{
 		n2 = n2>0 ? n2:1;
 		mx = nx;	my = ny>1?ny:n1;	mz = ny>1 ? n1 : n2;
 		b = new dual[mx*my*mz];
-		if(ny>1)	for(i=0;i<n1;i++)
-			memcpy(b+i*nx*ny, d->a, nx*ny*sizeof(dual));
-		else		for(i=0;i<n1*n2;i++)
-			memcpy(b+i*nx, d->a, nx*sizeof(dual));
+		if(ny>1)
+#pragma omp parallel for
+			for(long i=0;i<n1;i++)	memcpy(b+i*nx*ny, d->a, nx*ny*sizeof(dual));
+		else
+#pragma omp parallel for
+			for(long i=0;i<n1*n2;i++)	memcpy(b+i*nx, d->a, nx*sizeof(dual));
 	}
 	else
 	{
 		mx = -n1;	my = n2<0 ? -n2 : nx;	mz = n2<0 ? nx : ny;
 		if(n2>0 && ny==1)	mz = n2;
 		b = new dual[mx*my*mz];
-		register dual v;
-		if(n2<0)	for(j=0;j<nx;j++)	for(i=0,v=d->a[j];i<mx*my;i++)
-			b[i+mx*my*j] = v;
-		else	for(j=0;j<nx*ny;j++)	for(i=0,v=d->a[j];i<mx;i++)
-			b[i+mx*j] = v;
-		if(n2>0 && ny==1)	for(i=0;i<n2;i++)
-			memcpy(b+i*mx*my, d->a, mx*my*sizeof(dual));
+		dual v;
+		if(n2<0)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<nx;j++)	for(long i=0;i<mx*my;i++)
+				b[i+mx*my*j] = d->a[j];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<nx*ny;j++)	for(long i=0;i<mx;i++)
+				b[i+mx*j] = d->a[j];
+		if(n2>0 && ny==1)
+#pragma omp parallel for
+			for(long i=0;i<n2;i++)	memcpy(b+i*mx*my, d->a, mx*my*sizeof(dual));
 	}
 	if(!d->link)	delete [](d->a);
 	d->a=b;	d->nx=mx;	d->ny=my;	d->nz=mz;
@@ -474,37 +481,41 @@ void MGL_EXPORT mgl_datac_extend_(uintptr_t *d, int *n1, int *n2)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_datac_transpose(HADT d, const char *dim)
 {
-	long nx=d->nx, ny=d->ny, nz=d->nz;
+	long nx=d->nx, ny=d->ny, nz=d->nz, n;
 	dual *b=new dual[nx*ny*nz], *a=d->a;
-	register long i,j,k,n;
 	if(!strcmp(dim,"xyz"))	memcpy(b,a,nx*ny*nz*sizeof(dual));
 	else if(!strcmp(dim,"xzy") || !strcmp(dim,"zy"))
 	{
-		for(j=0;j<ny;j++)	for(k=0;k<nz;k++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(3)
+		for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 			b[i+nx*(k+nz*j)] = a[i+nx*(j+ny*k)];
 		n=nz;	nz=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"yxz") || !strcmp(dim,"yx"))
 	{
-		for(k=0;k<nz;k++)	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			b[j+ny*(i+nx*k)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"yzx"))
 	{
-		for(k=0;k<nz;k++)	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			b[j+ny*(k+nz*i)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=ny;	ny=nz;	nz=n;
 	}
 	else if(!strcmp(dim,"zxy"))
 	{
-		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)	for(k=0;k<nz;k++)
+#pragma omp parallel for collapse(3)
+		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)
 			b[k+nz*(i+nx*j)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=nz;	nz=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"zyx") || !strcmp(dim,"zx"))
 	{
-		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)	for(k=0;k<nz;k++)
+#pragma omp parallel for collapse(3)
+		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)
 			b[k+nz*(j+ny*i)] = a[i+nx*(j+ny*k)];
 		n=nz;	nz=nx;	nx=n;
 	}
@@ -520,14 +531,17 @@ MGL_NO_EXPORT void *mgl_cmodify(void *par)
 {
 	mglThreadC *t=(mglThreadC *)par;
 	const mglFormulaC *f = (const mglFormulaC *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1],nz=t->p[2];
+	long nx=t->p[0],ny=t->p[1],nz=t->p[2];
 	dual *b=t->a;
 	mreal dx,dy,dz;
 	const dual *v=t->b, *w=t->c;
 	dx=nx>1?1/(nx-1.):0;	dy=ny>1?1/(ny-1.):0;	dz=nz>1?1/(nz-1.):0;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
+		register long i=i0%nx, j=((i0/nx)%ny), k=i0/(nx*ny);
 		b[i0] = f->Calc(i*dx, j*dy, k*dz, b[i0], v?v[i0]:dual(0,0), w?w[i0]:dual(0,0));
 	}
 	return 0;
@@ -556,14 +570,17 @@ MGL_NO_EXPORT void *mgl_cmodify_gen(void *par)
 {
 	mglThreadV *t=(mglThreadV *)par;
 	const mglFormulaC *f = (const mglFormulaC *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1],nz=t->p[2];
+	long nx=t->p[0],ny=t->p[1],nz=t->p[2];
 	dual *b=t->aa;
 	mreal dx,dy,dz;
 	HCDT v=(HCDT)t->b, w=(HCDT)t->c;
 	dx=nx>1?1/(nx-1.):0;	dy=ny>1?1/(ny-1.):0;	dz=nz>1?1/(nz-1.):0;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
+		register long i=i0%nx, j=((i0/nx)%ny), k=i0/(nx*ny);
 		b[i0] = f->Calc(i*dx, j*dy, k*dz, b[i0], v?v->vthr(i0):0, w?w->vthr(i0):0);
 	}
 	return 0;
@@ -586,63 +603,6 @@ void MGL_EXPORT mgl_datac_modify_vw_(uintptr_t *d, const char *eq, uintptr_t *v,
 {	char *s=new char[l+1];	memcpy(s,eq,l);	s[l]=0;
 	mgl_datac_modify_vw(_DC_,s,_DA_(v),_DA_(w));	delete []s;	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_cfill_f(void *par)
-{
-	mglThreadC *t=(mglThreadC *)par;
-	const mglFormulaC *f = (const mglFormulaC *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1];
-	dual *b=t->a;
-	const dual *v=t->b, *w=t->c, *x=t->d;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
-	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
-		b[i0] = f->Calc(x[0]+mreal(i)*x[1], x[2]+mreal(j)*x[3], x[4]+mreal(k)*x[5],
-						b[i0], v?v[i0]:dual(0,0), w?w[i0]:dual(0,0));
-	}
-	return 0;
-}
-MGL_NO_EXPORT void *mgl_cfill_fgen(void *par)
-{
-	mglThreadV *t=(mglThreadV *)par;
-	const mglFormulaC *f = (const mglFormulaC *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1];
-	dual *b=t->aa;
-	HCDT v=(HCDT)t->b, w=(HCDT)t->c;
-	const mreal *x=t->d;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
-	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
-		b[i0] = f->Calc(x[0]+i*x[1], x[2]+j*x[3], x[4]+k*x[5],
-						b[i0], v?v->vthr(i0):0, w?w->vthr(i0):0);
-	}
-	return 0;
-}
-void MGL_EXPORT mgl_datac_fill_eq(HMGL gr, HADT d, const char *eq, HCDT vdat, HCDT wdat, const char *opt)
-{
-	const mglDataC *v = dynamic_cast<const mglDataC *>(vdat);
-	const mglDataC *w = dynamic_cast<const mglDataC *>(wdat);
-	long nn = d->nx*d->ny*d->nz, par[3]={d->nx,d->ny,d->nz};
-	if(v && v->nx*v->ny*v->nz!=nn)	return;
-	if(w && w->nx*w->ny*w->nz!=nn)	return;
-	gr->SaveState(opt);
-	mreal xx[6]={gr->Min.x,0, gr->Min.y,0, gr->Min.z,0};
-	if(d->nx>1)	xx[1] = (gr->Max.x-gr->Min.x)/(d->nx-1.);
-	if(d->ny>1)	xx[3] = (gr->Max.y-gr->Min.y)/(d->ny-1.);
-	if(d->nz>1)	xx[5] = (gr->Max.z-gr->Min.z)/(d->nz-1.);
-	dual cc[6]={xx[0],xx[1],xx[2],xx[3],xx[4],xx[5]};
-	mglFormulaC f(eq);
-	if(v && w)	mglStartThreadC(mgl_cfill_f,0,nn,d->a,v->a,w->a,par,&f,cc);
-	else if(vdat && wdat)	mglStartThreadV(mgl_cfill_fgen,nn,d->a,vdat,wdat,par,&f,xx);
-	else if(v)	mglStartThreadC(mgl_cfill_f,0,nn,d->a,v->a,0,par,&f,cc);
-	else if(vdat)	mglStartThreadV(mgl_cfill_fgen,nn,d->a,vdat,0,par,&f,xx);
-	else	mglStartThreadC(mgl_cfill_f,0,nn,d->a,0,0,par,&f,cc);
-	gr->LoadState();
-}
-void MGL_EXPORT mgl_datac_fill_eq_(uintptr_t *gr, uintptr_t *d, const char *eq, uintptr_t *v, uintptr_t *w, const char *opt,int l,int lo)
-{	char *s=new char[l+1];	memcpy(s,eq,l);	s[l]=0;
-	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
-	mgl_datac_fill_eq(_GR_,_DC_,s,_DA_(v),_DA_(w),o);	delete []o;	delete []s;	}
-//-----------------------------------------------------------------------------
 bool MGL_NO_EXPORT mgl_add_file(long &kx,long &ky, long &kz, dual *&b, mglDataC *d,bool as_slice)
 {
 	if(as_slice && d->nz==1)
@@ -692,7 +652,7 @@ int MGL_EXPORT mgl_datac_read_range(HADT dat, const char *templ, double from, do
 	//read first file
 	do{	snprintf(fname,n,templ,t);	t+= step;	} while(!mgl_datac_read(&d,fname) && t<=to);
 
-	if(t>to)	return false;
+	if(t>to)	{	delete []fname;	return false;	}
 	kx = d.nx;	ky = d.ny;	kz = d.nz;
 	b = (dual *)malloc(kx*ky*kz*sizeof(dual));
 	memcpy(b,d.a,kx*ky*kz*sizeof(dual));
@@ -721,14 +681,13 @@ int MGL_EXPORT mgl_datac_read_all(HADT dat, const char *templ, int as_slice)
 	size_t i;
 	dual *b;
 	long kx,ky,kz;
-	char *fname = new char[256];
 	glob (templ, GLOB_TILDE, NULL, &res);
 
 	//read first file
 	for(i=0;i<res.gl_pathc;i++)
 		if(mgl_datac_read(&d,res.gl_pathv[i]))	break;
 
-	if(i>=res.gl_pathc)	{	delete []fname;	return false;	}
+	if(i>=res.gl_pathc)	{	globfree (&res);	return false;	}
 	kx = d.nx;	ky = d.ny;	kz = d.nz;
 	b = (dual *)malloc(kx*ky*kz*sizeof(dual));
 	memcpy(b,d.a,kx*ky*kz*sizeof(dual));
@@ -737,12 +696,11 @@ int MGL_EXPORT mgl_datac_read_all(HADT dat, const char *templ, int as_slice)
 	{
 		if(mgl_datac_read(&d,res.gl_pathv[i]))
 			if(!mgl_add_file(kx,ky,kz,b,&d,as_slice))
-			{	delete []fname;		return false;	}
+			{	globfree (&res);	free(b);	return false;	}
 	}
 	dat->Set(b,kx,ky,kz);
 
-	globfree (&res);
-	delete []fname;		free(b);
+	globfree (&res);	free(b);
 	return true;
 #else
 	return false;
@@ -757,8 +715,9 @@ HMDT MGL_EXPORT mgl_datac_real(HCDT d)
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	mglData *r=new mglData(nx,ny,nz);
 	const mglDataC *dd = dynamic_cast<const mglDataC*>(d);
-	register long i;
-	if(dd)	for(i=0;i<nx*ny*nz;i++)	r->a[i] = real(dd->a[i]);
+	if(dd)
+#pragma omp parallel for
+		for(long i=0;i<nx*ny*nz;i++)	r->a[i] = real(dd->a[i]);
 	else		r->Set(d);
 	return r;
 }
@@ -770,8 +729,9 @@ HMDT MGL_EXPORT mgl_datac_imag(HCDT d)
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	mglData *r=new mglData(nx,ny,nz);
 	const mglDataC *dd = dynamic_cast<const mglDataC*>(d);
-	register long i;
-	if(dd)	for(i=0;i<nx*ny*nz;i++)	r->a[i] = imag(dd->a[i]);
+	if(dd)
+#pragma omp parallel for
+		for(long i=0;i<nx*ny*nz;i++)	r->a[i] = imag(dd->a[i]);
 	return r;
 }
 uintptr_t MGL_EXPORT mgl_datac_imag_(uintptr_t *d)
@@ -782,9 +742,12 @@ HMDT MGL_EXPORT mgl_datac_abs(HCDT d)
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	mglData *r=new mglData(nx,ny,nz);
 	const mglDataC *dd = dynamic_cast<const mglDataC*>(d);
-	register long i;
-	if(dd)	for(i=0;i<nx*ny*nz;i++)	r->a[i] = abs(dd->a[i]);
-	else	for(i=0;i<nx*ny*nz;i++)	r->a[i] = fabs(d->vthr(i));
+	if(dd)
+#pragma omp parallel for
+		for(long i=0;i<nx*ny*nz;i++)	r->a[i] = abs(dd->a[i]);
+	else
+#pragma omp parallel for
+		for(long i=0;i<nx*ny*nz;i++)	r->a[i] = fabs(d->vthr(i));
 	return r;
 }
 uintptr_t MGL_EXPORT mgl_datac_abs_(uintptr_t *d)
@@ -795,8 +758,9 @@ HMDT MGL_EXPORT mgl_datac_arg(HCDT d)
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	mglData *r=new mglData(nx,ny,nz);
 	const mglDataC *dd = dynamic_cast<const mglDataC*>(d);
-	register long i;
-	if(dd)	for(i=0;i<nx*ny*nz;i++)	r->a[i] = arg(dd->a[i]);
+	if(dd)
+#pragma omp parallel for
+		for(long i=0;i<nx*ny*nz;i++)	r->a[i] = arg(dd->a[i]);
 	return r;
 }
 uintptr_t MGL_EXPORT mgl_datac_arg_(uintptr_t *d)
@@ -806,8 +770,8 @@ void MGL_EXPORT mgl_datac_set_ri(HADT d, HCDT re, HCDT im)
 {
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	d->Create(nx,ny,nz);
-	register long i;
-	for(i=0;i<nx*ny*nz;i++)	d->a[i] = dual(re->vthr(i),im->vthr(i));
+#pragma omp parallel for
+	for(long i=0;i<nx*ny*nz;i++)	d->a[i] = dual(re->vthr(i),im->vthr(i));
 }
 void MGL_EXPORT mgl_datac_set_ri_(uintptr_t *d, uintptr_t *re, uintptr_t *im)
 {	mgl_datac_set_ri(_DC_,_DA_(re),_DA_(im));	}
@@ -816,11 +780,10 @@ void MGL_EXPORT mgl_datac_set_ap(HADT d, HCDT a, HCDT p)
 {
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	d->Create(nx,ny,nz);
-	register long i;
-	register mreal aa,pp;
-	for(i=0;i<nx*ny*nz;i++)
+#pragma omp parallel for
+	for(long i=0;i<nx*ny*nz;i++)
 	{
-		aa=a->vthr(i);	pp=p->vthr(i);
+		register mreal aa=a->vthr(i), pp=p->vthr(i);
 		d->a[i] = dual(aa*cos(pp), aa*sin(pp));
 	}
 }
