@@ -1,6 +1,6 @@
 /***************************************************************************
  * cont.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -17,7 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-// NOTE: Borland before 2007 (i.e. < 0x0600) use <algorithm.h>, fter 0x0630 it use <algorithm>.
+// NOTE: Borland before 2007 (i.e. < 0x0600) use <algorithm.h>, after 0x0630 it use <algorithm>.
 // I don't find information about 2009, 2010 versions (i.e. 0x0610 and 0x0620).
 // May be condition below can be rewritten as (__CODEGEARC__ >=  0x0600)
 #if !defined(__BORLANDC__) || (__CODEGEARC__ >=  0x0630)
@@ -31,12 +31,13 @@
 #include "mgl2/data.h"
 #include "mgl2/eval.h"
 #include "mgl2/font.h"
+#include "mgl2/base.h"
 //-----------------------------------------------------------------------------
 //
 //	Text printing along a curve
 //
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,long *ff,long *nn,const wchar_t *text, const char *font, mreal size)
+void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,const long *ff,const long *nn,const wchar_t *text, const char *font, mreal size)
 {
 	if(f<0 || nn[f]==-1)	return;	// do nothing since there is no curve
 	if(!font)	font="";
@@ -75,7 +76,7 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,long *ff,long *nn,c
 		else		{	strcat(fnt,":L");	align=0;	}
 	}
 	if(rev)	reverse(qa.begin(),qa.end());
-	long len = wcslen(text);
+	long len = mgl_wcslen(text);
 	mreal *wdt=new mreal[len+1];
 	for(j=0;j<len;j++)	{	L[0]=text[j];	wdt[j]=1.2*gr->TextWidth(L,font,size);	}
 	wdt[len]=0;
@@ -84,9 +85,6 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,long *ff,long *nn,c
 	mglPoint *pt=new mglPoint[len+1];
 	pt[0] = qa[0];	m = qa.size();
 
-	// next string for testing curve position
-	//for(i=0;i<m;i++) gr->mark_plot(gr->AddPnt(qa[i],c,s,-1,-1),'.');
-	
 	mreal a,b,d,w,t1,t2;
 	for(i=j=0,tt=0;j<len;j++)
 	{
@@ -105,8 +103,10 @@ void MGL_NO_EXPORT mgl_string_curve(mglBase *gr,long f,long ,long *ff,long *nn,c
 	}
 	if(rev)	pos=-pos;
 	for(j=0;j<len;j++)	// draw text
-	{	L[0] = text[align!=2?j:len-1-j];	s = pt[j+1]-pt[j];	l = !s;
-	gr->text_plot(gr->AddPnt(pt[j]+(pos*h)*l,c,s,-1,-1),L,font,size,0.05,c);	}
+	{
+		L[0] = text[align!=2?j:len-1-j];	s = pt[j+1]-pt[j];	l = !s;
+		gr->text_plot(gr->AddPnt(pt[j]+(pos*h)*l,c,s,-1,-1),L,font,size,0.05,c);
+	}
 	delete []wdt;	delete []pt;	delete []fnt;
 }
 //-----------------------------------------------------------------------------
@@ -119,16 +119,19 @@ void MGL_EXPORT mgl_textw_xyz(HMGL gr, HCDT x, HCDT y, HCDT z,const wchar_t *tex
 	static int cgid=1;	gr->StartGroup("TextC",cgid++);
 
 	long *nn = new long[n], *ff = new long[n];
-	mglPoint p;
-	register long i;
 	const mglData *mdx = dynamic_cast<const mglData *>(x);
 	const mglData *mdy = dynamic_cast<const mglData *>(y);
 	const mglData *mdz = dynamic_cast<const mglData *>(z);
-	if(mdx && mdy && mdz)	for(i=0;i<n;i++)
-	{	p = mglPoint(mdx->a[i],mdy->a[i],mdz->a[i]);	ff[i] = gr->AddPnt(p,-1);	}
-	else	for(i=0;i<n;i++)
-	{	p = mglPoint(x->v(i),y->v(i),z->v(i));	ff[i] = gr->AddPnt(p,-1);	}
-	for(i=1;i<n;i++)	nn[i-1] = i;
+	if(mdx && mdy && mdz)
+#pragma omp parallel for
+		for(long i=0;i<n;i++)
+			ff[i] = gr->AddPnt(mglPoint(mdx->a[i],mdy->a[i],mdz->a[i]),-1);
+	else
+#pragma omp parallel for
+		for(long i=0;i<n;i++)
+			ff[i] = gr->AddPnt(mglPoint(x->v(i),y->v(i),z->v(i)),-1);
+#pragma omp parallel for
+	for(long i=1;i<n;i++)	nn[i-1] = i;
 	nn[n-1]=-1;
 	mgl_string_curve(gr,0,n,ff,nn,text,font,-1);
 	gr->EndGroup();
@@ -141,7 +144,6 @@ void MGL_EXPORT mgl_textw_xy(HMGL gr, HCDT x, HCDT y, const wchar_t *text, const
 	mglData z(y->GetNx());
 	mreal zm = gr->AdjustZMin();	z.Fill(zm,zm);
 	mgl_textw_xyz(gr,x,y,&z,text,font,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textw_y(HMGL gr, HCDT y, const wchar_t *text, const char *font, const char *opt)
@@ -151,16 +153,11 @@ void MGL_EXPORT mgl_textw_y(HMGL gr, HCDT y, const wchar_t *text, const char *fo
 	x.Fill(gr->Min.x,gr->Max.x);
 	mreal zm = gr->AdjustZMin();	z.Fill(zm,zm);
 	mgl_textw_xyz(gr,&x,y,&z,text,font,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_text_xyz(HMGL gr, HCDT x, HCDT y, HCDT z,const char *text, const char *font, const char *opt)
 {
-	size_t s = mbstowcs(0,text,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,text,s);
-	mgl_textw_xyz(gr,x,y,z, wcs, font, opt);
-	delete []wcs;
+	MGL_TO_WCS(text,mgl_textw_xyz(gr,x,y,z, wcs, font, opt));
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_text_xy(HMGL gr, HCDT x, HCDT y, const char *text, const char *font, const char *opt)
@@ -268,19 +265,19 @@ long *mgl_cont_prep(mreal val, HCDT a,long ak, std::vector<mglPnt2> &kk)
 	const mglData *ma = dynamic_cast<const mglData *>(a);
 	if(ma)
 	{
-		for(i=0;i<n-1;i++)	for(j=0;j<m;j++)
+		for(j=0;j<m;j++)	for(i=0;i<n-1;i++)
 		{
 			d = mgl_d(val,ma->a[i+n*(j+m*ak)],ma->a[i+1+n*(j+m*ak)]);
 			if(d>=0 && d<1)	kk.push_back(mglPnt2(i+d,j));
 		}
 		// add intersection point of isoline and X axis
-		for(i=0;i<n;i++)	for(j=0;j<m-1;j++)
+		for(j=0;j<m-1;j++)	for(i=0;i<n;i++)
 		{
 			d = mgl_d(val,ma->a[i+n*(j+m*ak)],ma->a[i+n*(j+1+m*ak)]);
 			if(d>=0 && d<1)	kk.push_back(mglPnt2(i,j+d));
 		}
 	}
-	else	for(i=0;i<n-1;i++)	for(j=0;j<m;j++)
+	else	for(j=0;j<m-1;j++)	for(i=0;i<n-1;i++)
 	{
 		register mreal vv = a->v(i,j,ak);
 		d = (i<n-1)?mgl_d(val,vv,a->v(i+1,j,ak)):-1;
@@ -396,7 +393,7 @@ void MGL_EXPORT mgl_cont_gen(HMGL gr, double val, HCDT a, HCDT x, HCDT y, HCDT z
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Cont"))	return;
 
 	gr->SaveState(opt);
@@ -407,28 +404,36 @@ void MGL_EXPORT mgl_cont_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const c
 	long s=gr->AddTexture(sch);
 	gr->SetPenPal(sch);
 
-	mglData xx, yy, zz(z->GetNx(), z->GetNy());
+	mglData xx, yy;
 	if(x->GetNx()*x->GetNy()!=m*n || y->GetNx()*y->GetNy()!=m*n)	// make
 	{
 		xx.Create(n, m);		yy.Create(n, m);
 		const mglData *mx = dynamic_cast<const mglData *>(x);
 		const mglData *my = dynamic_cast<const mglData *>(y);
-		if(mx && my)	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
-		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
+		if(mx && my)
+#pragma omp parallel for collapse(2)
+			for(long i=0;i<n;i++)	for(long j=0;j<m;j++)
+			{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
+		else
+#pragma omp parallel for collapse(2)
+			for(long i=0;i<n;i++)	for(long j=0;j<m;j++)
+			{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
 		x = &xx;	y = &yy;
 	}
 	// x, y -- have the same size z
-	mreal z0, v0;
-	for(j=0;j<z->GetNz();j++)	for(i=0;i<v->GetNx();i++)
+#pragma omp parallel
 	{
-		if(gr->Stop)	return;
-		v0 = v->v(i);		z0 = fixed ? gr->Min.z : v0;
-		if(z->GetNz()>1)
-			z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
-		zz.Fill(z0,z0);
-		mgl_cont_gen(gr,v0,z,x,y,&zz,gr->GetC(s,v0),text,j);
+		mglData zz(n, m);
+#pragma omp for collapse(2)
+		for(long j=0;j<z->GetNz();j++)	for(long i=0;i<v->GetNx();i++)
+		{
+			if(gr->Stop)	continue;
+			mreal v0 = v->v(i), z0 = fixed ? gr->Min.z : v0;
+			if(z->GetNz()>1)
+				z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
+			zz.Fill(z0,z0);
+			mgl_cont_gen(gr,v0,z,x,y,&zz,gr->GetC(s,v0),text,j);
+		}
 	}
 	gr->EndGroup();
 }
@@ -442,29 +447,26 @@ void MGL_EXPORT mgl_cont_val(HMGL gr, HCDT v, HCDT z, const char *sch, const cha
 	x.Fill(gr->Min.x,gr->Max.x,'x');
 	y.Fill(gr->Min.y,gr->Max.y,'y');
 	mgl_cont_xy_val(gr,v,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont_xy_val(gr,&v,x,y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont_val(gr,&v,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_xy_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -604,7 +606,7 @@ void MGL_EXPORT mgl_contf_gen(HMGL gr, double v1, double v2, HCDT a, HCDT x, HCD
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"ContF"))	return;
 
 	gr->SaveState(opt);
@@ -612,28 +614,36 @@ void MGL_EXPORT mgl_contf_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const
 	long s=gr->AddTexture(sch);
 
 	bool fixed=(mglchr(sch,'_')) || (gr->Min.z==gr->Max.z);
-	mglData xx, yy, zz(n, m);
+	mglData xx, yy;
 	if(x->GetNx()*x->GetNy()!=m*n || y->GetNx()*y->GetNy()!=m*n)	// make
 	{
 		xx.Create(n, m);		yy.Create(n, m);
 		const mglData *mx = dynamic_cast<const mglData *>(x);
 		const mglData *my = dynamic_cast<const mglData *>(y);
-		if(mx && my)	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
-		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
+		if(mx && my)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
 		x = &xx;	y = &yy;
 	}
 	// x, y -- have the same size z
-	mreal z0, v0;
-	for(j=0;j<z->GetNz();j++)	for(i=0;i<v->GetNx()-1;i++)
+#pragma omp parallel
 	{
-		if(gr->Stop)	return;
-		v0 = v->v(i);		z0 = fixed ? gr->Min.z : v0;
-		if(z->GetNz()>1)
-			z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
-		zz.Fill(z0,z0);
-		mgl_contf_gen(gr,v0,v->v(i+1),z,x,y,&zz,gr->GetC(s,v0),j);
+		mglData zz(n, m);
+#pragma omp for collapse(2)
+		for(long j=0;j<z->GetNz();j++)	for(long i=0;i<v->GetNx()-1;i++)
+		{
+			if(gr->Stop)	continue;
+			mreal v0 = v->v(i), z0 = fixed ? gr->Min.z : v0;
+			if(z->GetNz()>1)
+				z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
+			zz.Fill(z0,z0);
+			mgl_contf_gen(gr,v0,v->v(i+1),z,x,y,&zz,gr->GetC(s,v0),j);
+		}
 	}
 	gr->EndGroup();
 }
@@ -647,27 +657,24 @@ void MGL_EXPORT mgl_contf_val(HMGL gr, HCDT v, HCDT z, const char *sch, const ch
 	x.Fill(gr->Min.x,gr->Max.x,'x');
 	y.Fill(gr->Min.y,gr->Max.y,'y');
 	mgl_contf_xy_val(gr,v,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
 	mglData v(Num+2);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf_xy_val(gr,&v,x,y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
 	mglData v(Num+2);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf_val(gr,&v,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_xy_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -706,7 +713,7 @@ int MGL_NO_EXPORT mgl_get_ncol(const char *sch, char *res)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contd_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j=0,n=z->GetNx(),m=z->GetNy();
+	long i,j=0,n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"ContD"))	return;
 
 	gr->SaveState(opt);
@@ -717,28 +724,37 @@ void MGL_EXPORT mgl_contd_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const
 	if(j==0)	sch = MGL_DEF_PAL;
 	long s = gr->AddTexture(sch,1);
 	int nc = gr->GetNumPal(s*256);
-	mglData xx, yy, zz(n, m);
+	mglData xx, yy;
 	if(x->GetNx()*x->GetNy()!=m*n || y->GetNx()*y->GetNy()!=m*n)	// make
 	{
 		xx.Create(n, m);		yy.Create(n, m);
 		const mglData *mx = dynamic_cast<const mglData *>(x);
 		const mglData *my = dynamic_cast<const mglData *>(y);
-		if(mx && my)	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
-		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
+		if(mx && my)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
 		x = &xx;	y = &yy;
 	}
 	// x, y -- have the same size z
-	mreal z0, v0, dc = nc>1 ? 1/(MGL_FEPSILON*(nc-1)) : 0;
-	for(j=0;j<z->GetNz();j++)	for(i=0;i<v->GetNx()-1;i++)
+	mreal dc = nc>1 ? 1/(MGL_FEPSILON*(nc-1)) : 0;
+#pragma omp parallel
 	{
-		if(gr->Stop)	return;
-		v0 = v->v(i);		z0 = fixed ? gr->Min.z : v0;
-		if(z->GetNz()>1)
-			z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
-		zz.Fill(z0,z0);
-		mgl_contf_gen(gr,v0,v->v(i+1),z,x,y,&zz,s+i*dc,j);
+		mglData zz(n, m);
+#pragma omp for collapse(2)
+		for(long j=0;j<z->GetNz();j++)	for(long i=0;i<v->GetNx()-1;i++)
+		{
+			if(gr->Stop)	continue;
+			mreal v0 = v->v(i), z0 = fixed ? gr->Min.z : v0;
+			if(z->GetNz()>1)
+				z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
+			zz.Fill(z0,z0);
+			mgl_contf_gen(gr,v0,v->v(i+1),z,x,y,&zz,s+i*dc,j);
+		}
 	}
 	gr->EndGroup();
 }
@@ -752,7 +768,6 @@ void MGL_EXPORT mgl_contd_val(HMGL gr, HCDT v, HCDT z, const char *sch, const ch
 	x.Fill(gr->Min.x,gr->Max.x,'x');
 	y.Fill(gr->Min.y,gr->Max.y,'y');
 	mgl_contd_xy_val(gr,v,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contd_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
@@ -761,7 +776,6 @@ void MGL_EXPORT mgl_contd_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, c
 	mglData v(mgl_get_ncol(sch,0)+1);
 	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contd_xy_val(gr,&v,x,y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contd(HMGL gr, HCDT z, const char *sch, const char *opt)
@@ -770,7 +784,6 @@ void MGL_EXPORT mgl_contd(HMGL gr, HCDT z, const char *sch, const char *opt)
 	mglData v(mgl_get_ncol(sch,0)+1);
 	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contd_val(gr,&v,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contd_xy_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -826,7 +839,7 @@ void MGL_EXPORT mgl_contv_gen(HMGL gr, mreal val, mreal dval, HCDT a, HCDT x, HC
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contv_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"ContV"))	return;
 
 	gr->SaveState(opt);
@@ -835,31 +848,39 @@ void MGL_EXPORT mgl_contv_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const
 	long s=gr->AddTexture(sch);
 	gr->SetPenPal(sch);
 
-	mglData xx, yy, zz(n, m);
+	mglData xx, yy;
 	if(x->GetNx()*x->GetNy()!=m*n || y->GetNx()*y->GetNy()!=m*n)	// make
 	{
 		xx.Create(n, m);		yy.Create(n, m);
 		const mglData *mx = dynamic_cast<const mglData *>(x);
 		const mglData *my = dynamic_cast<const mglData *>(y);
-		if(mx && my)	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
-		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
+		if(mx && my)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
 		x = &xx;	y = &yy;
 	}
 	// x, y -- have the same size z
-	mreal z0, v0;
-	for(j=0;j<z->GetNz();j++)	for(i=0;i<v->GetNx();i++)
+#pragma omp parallel
 	{
-		if(gr->Stop)	return;
-		v0 = v->v(i);		z0 = fixed ? gr->Min.z : v0;
-		if(z->GetNz()>1)	z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
-		zz.Fill(z0,z0);
-		mreal dv = (gr->Max.c-gr->Min.c)/8;
-		if(i>0)	dv = v->v(i-1)-v->v(i);
-		else if(i<v->GetNx()-1)	dv = v->v(i)-v->v(i+1);
-		if(fixed)	dv=-dv;
-		mgl_contv_gen(gr,v0,dv,z,x,y,&zz,gr->GetC(s,v0),j);
+		mglData zz(n, m);
+#pragma omp for collapse(2)
+		for(long j=0;j<z->GetNz();j++)	for(long i=0;i<v->GetNx();i++)
+		{
+			if(gr->Stop)	continue;
+			mreal v0 = v->v(i), z0 = fixed ? gr->Min.z : v0;
+			if(z->GetNz()>1)	z0 = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(j)/(z->GetNz()-1);
+			zz.Fill(z0,z0);
+			mreal dv = (gr->Max.c-gr->Min.c)/8;
+			if(i>0)	dv = v->v(i-1)-v->v(i);
+			else if(i<v->GetNx()-1)	dv = v->v(i)-v->v(i+1);
+			if(fixed)	dv=-dv;
+			mgl_contv_gen(gr,v0,dv,z,x,y,&zz,gr->GetC(s,v0),j);
+		}
 	}
 	gr->EndGroup();
 }
@@ -873,29 +894,26 @@ void MGL_EXPORT mgl_contv_val(HMGL gr, HCDT v, HCDT z, const char *sch, const ch
 	x.Fill(gr->Min.x,gr->Max.x,'x');
 	y.Fill(gr->Min.y,gr->Max.y,'y');
 	mgl_contv_xy_val(gr,v,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contv_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_contv_xy_val(gr,&v,x,y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contv(HMGL gr, HCDT z, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_contv_val(gr,&v,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contv_xy_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -1004,7 +1022,7 @@ void MGL_NO_EXPORT mgl_get_slice(_mgl_slice &s, HCDT x, HCDT y, HCDT z, HCDT a,
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_get_slice_md(_mgl_slice &s, const mglData *x, const mglData *y, const mglData *z, const mglData *a, char dir, mreal d, bool both)
 {
-	register long i,j,i0,i1,n=a->nx,m=a->ny,l=a->nz, nx=1,ny=1,p;
+	long n=a->nx,m=a->ny,l=a->nz, nx=1,ny=1,p;
 
 	if(dir=='x')	{	nx = m;	ny = l;	if(d<0)	d = n/2.;	}
 	if(dir=='y')	{	nx = n;	ny = l;	if(d<0)	d = m/2.;	}
@@ -1019,60 +1037,69 @@ void MGL_NO_EXPORT mgl_get_slice_md(_mgl_slice &s, const mglData *x, const mglDa
 
 	if(both)
 	{
-		if(dir=='x')	for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
-		{
-			i0 = i+nx*j;	i1 = p+n*(i+m*j);
-			s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+1]*d;
-			s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+1]*d;
-			s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+1]*d;
-			s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+1]*d;
-		}
-		if(dir=='y')	for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
-		{
-			i0 = i+nx*j;	i1 = i+n*(p+m*j);
-			s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+n]*d;
-			s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+n]*d;
-			s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+n]*d;
-			s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+n]*d;
-		}
-		if(dir=='z')	for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
-		{
-			i0 = i+nx*j;	i1 = i+n*(j+m*p);
-			s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+n*m]*d;
-			s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+n*m]*d;
-			s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+n*m]*d;
-			s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+n*m]*d;
-		}
+		if(dir=='x')
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
+			{
+				register long i0 = i+nx*j, i1 = p+n*(i+m*j);
+				s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+1]*d;
+				s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+1]*d;
+				s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+1]*d;
+				s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+1]*d;
+			}
+		if(dir=='y')
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
+			{
+				register long i0 = i+nx*j, i1 = i+n*(p+m*j);
+				s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+n]*d;
+				s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+n]*d;
+				s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+n]*d;
+				s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+n]*d;
+			}
+		if(dir=='z')
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
+			{
+				register long i0 = i+nx*j, i1 = i+n*(j+m*p);
+				s.x.a[i0] = x->a[i1]*(1-d) + x->a[i1+n*m]*d;
+				s.y.a[i0] = y->a[i1]*(1-d) + y->a[i1+n*m]*d;
+				s.z.a[i0] = z->a[i1]*(1-d) + z->a[i1+n*m]*d;
+				s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+n*m]*d;
+			}
 	}
 	else	// x, y, z -- vectors
 	{
 		if(dir=='x')
 		{
 			v = x->a[p]*(1-d)+x->a[p+1]*d;
-			for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			{
-				i0 = i+nx*j;	s.x.a[i0] = v;	i1 = p+n*(i+m*j);
-				s.y.a[i0] = y->a[i];	s.z.a[i0] = z->a[j];
+				register long i0 = i+nx*j, i1 = p+n*(i+m*j);
+				s.x.a[i0] = v;	s.y.a[i0] = y->a[i];	s.z.a[i0] = z->a[j];
 				s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+1]*d;
 			}
 		}
 		if(dir=='y')
 		{
 			v = y->a[p]*(1-d)+y->a[p+1]*d;
-			for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			{
-				i0 = i+nx*j;	s.y.a[i0] = v;	i1 = i+n*(p+m*j);
-				s.x.a[i0] = x->a[i];	s.z.a[i0] = z->a[j];
+				register long i0 = i+nx*j, i1 = i+n*(p+m*j);
+				s.x.a[i0] = x->a[i];	s.y.a[i0] = v;	s.z.a[i0] = z->a[j];
 				s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+n]*d;
 			}
 		}
 		if(dir=='z')
 		{
 			v = z->a[p]*(1-d)+z->a[p+1]*d;
-			for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			{
-				i0 = i+nx*j;	s.z.a[i0] = v;	i1 = i+n*(j+m*p);
-				s.x.a[i0] = x->a[i];	s.y.a[i0] = y->a[j];
+				register long i0 = i+nx*j, i1 = i+n*(j+m*p);
+				s.x.a[i0] = x->a[i];	s.y.a[i0] = y->a[j];	s.z.a[i0] = v;
 				s.a.a[i0] = a->a[i1]*(1-d) + a->a[i1+n*m]*d;
 			}
 		}
@@ -1101,9 +1128,10 @@ void MGL_EXPORT mgl_cont3_xyz_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, HCDT
 	const mglData *ma = dynamic_cast<const mglData *>(a);
 	if(mx&&my&&mz&&ma)	mgl_get_slice_md(s,mx,my,mz,ma,dir,sVal,both);
 	else mgl_get_slice(s,x,y,z,a,dir,sVal,both);
+#pragma omp parallel for
 	for(long i=0;i<v->GetNx();i++)
 	{
-		mreal v0 = v->v(i);
+		register mreal v0 = v->v(i);
 		mgl_cont_gen(gr,v0,&s.a,&s.x,&s.y,&s.z,gr->GetC(ss,v0),text,0);
 	}
 	gr->EndGroup();
@@ -1117,29 +1145,26 @@ void MGL_EXPORT mgl_cont3_val(HMGL gr, HCDT v, HCDT a, const char *sch, double s
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
 	mgl_cont3_xyz_val(gr,v,&x,&y,&z,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont3_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, double sVal, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont3");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont3");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont3_xyz_val(gr,&v,x,y,z,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont3(HMGL gr, HCDT a, const char *sch, double sVal, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont3");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Cont3");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont3_val(gr,&v,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont3_xyz_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch, mreal *sVal, const char *opt,int l,int lo)
@@ -1198,7 +1223,6 @@ void MGL_EXPORT mgl_dens3(HMGL gr, HCDT a, const char *sch, double sVal, const c
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
 	mgl_dens3_xyz(gr,&x,&y,&z,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dens3_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch, mreal *sVal, const char *opt,int l,int lo)
@@ -1246,7 +1270,6 @@ void MGL_EXPORT mgl_grid3(HMGL gr, HCDT a, const char *sch, double sVal, const c
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
 	mgl_grid3_xyz(gr,&x,&y,&z,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_grid3_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch, mreal *sVal, const char *opt,int l,int lo)
@@ -1283,9 +1306,10 @@ void MGL_EXPORT mgl_contf3_xyz_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, HCDT
 	const mglData *ma = dynamic_cast<const mglData *>(a);
 	if(mx&&my&&mz&&ma)	mgl_get_slice_md(s,mx,my,mz,ma,dir,sVal,both);
 	else mgl_get_slice(s,x,y,z,a,dir,sVal,both);
+#pragma omp parallel for
 	for(long i=0;i<v->GetNx()-1;i++)
 	{
-		mreal v0 = v->v(i);
+		register mreal v0 = v->v(i);
 		mgl_contf_gen(gr,v0,v->v(i+1),&s.a,&s.x,&s.y,&s.z,gr->GetC(ss,v0),0);
 	}
 	gr->EndGroup();
@@ -1299,27 +1323,24 @@ void MGL_EXPORT mgl_contf3_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	y.Fill(gr->Min.y,gr->Max.y);
 	z.Fill(gr->Min.z,gr->Max.z);
 	mgl_contf3_xyz_val(gr,v,&x,&y,&z,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf3_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, double sVal, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"ContF3");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"ContF3");	return;	}
 	mglData v(Num+2);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf3_xyz_val(gr,&v,x,y,z,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf3(HMGL gr, HCDT a, const char *sch, double sVal, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?7:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"ContF3");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"ContF3");	return;	}
 	mglData v(Num+2);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf3_val(gr,&v,a,sch,sVal,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf3_xyz_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch, mreal *sVal, const char *opt,int l,int lo)
@@ -1363,7 +1384,6 @@ void MGL_NO_EXPORT mgl_axial_plot(mglBase *gr,long pc, mglPoint *ff, long *nn,ch
 	b = !a;	c = a^b;
 
 	register long i,j,k;
-	mreal fi,si,co;
 	long p1,p2,p3,p4;
 	gr->Reserve(pc*82);
 	for(i=0;i<pc;i++)
@@ -1380,11 +1400,12 @@ void MGL_NO_EXPORT mgl_axial_plot(mglBase *gr,long pc, mglPoint *ff, long *nn,ch
 		p2 = wire ? gr->AddPnt(p,cc) : gr->AddPnt(p,cc,(a*q2.y + c*q2.x)^b);
 		if(wire==1)	gr->line_plot(p1,p2);
 		else if(wire)	{	gr->mark_plot(p1,'.');	gr->mark_plot(p2,'.');	}
-		
+
 		for(j=1;j<41;j++)
 		{
 			p3 = p1;	p4 = p2;
-			fi = j*M_PI/20;		si = sin(fi);	co = cos(fi);
+			register float co = mgl_cos[(j*18)%360], si = mgl_cos[(270+j*18)%360];
+//			fi = j*M_PI/20;		si = sin(fi);	co = cos(fi);
 			p = a*ff[i].y + b*(si*ff[i].x) +  c*(co*ff[i].x);
 			p1 = wire ?	gr->AddPnt(p,cc) : gr->AddPnt(p,cc,(a*q1.y + b*(si*q1.x) +  c*(co*q1.x))^(b*co-c*si));
 			p = a*ff[nn[i]].y + b*(si*ff[nn[i]].x) +  c*(co*ff[nn[i]].x);
@@ -1495,12 +1516,10 @@ void MGL_EXPORT mgl_axial_gen(HMGL gr, mreal val, HCDT a, HCDT x, HCDT y, mreal
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_axial_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Axial"))	return;
-
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Axial",cgid++);
-
 	long s=gr->AddTexture(sch);
 	char dir='y';
 	if(mglchr(sch,'x'))	dir = 'x';
@@ -1512,20 +1531,24 @@ void MGL_EXPORT mgl_axial_xy_val(HMGL gr, HCDT v, HCDT x, HCDT y, HCDT z, const
 		xx.Create(n, m);		yy.Create(n, m);
 		const mglData *mx = dynamic_cast<const mglData *>(x);
 		const mglData *my = dynamic_cast<const mglData *>(y);
-		if(mx && my)	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
-		else	for(i=0;i<n;i++)	for(j=0;j<m;j++)
-		{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
+		if(mx && my)
+#pragma omp parallel for collapse(2)
+			for(long i=0;i<n;i++)	for(long j=0;j<m;j++)
+			{	xx.a[i+n*j] = mx->a[i];	yy.a[i+n*j] = my->a[j];	}
+		else
+#pragma omp parallel for collapse(2)
+			for(long i=0;i<n;i++)	for(long j=0;j<m;j++)
+			{	xx.a[i+n*j] = x->v(i);	yy.a[i+n*j] = y->v(j);	}
 		x = &xx;	y = &yy;
 	}
 	// x, y -- have the same size z
-	mreal v0;
 	int wire = mglchr(sch,'#')?1:0;
 	if(mglchr(sch,'.'))	wire = 2;
-	for(j=0;j<z->GetNz();j++)	for(i=0;i<v->GetNx();i++)
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<z->GetNz();j++)	for(long i=0;i<v->GetNx();i++)
 	{
-		if(gr->Stop)	return;
-		v0 = v->v(i);
+		if(gr->Stop)	continue;
+		register mreal v0 = v->v(i);
 		mgl_axial_gen(gr,v0,z,x,y,gr->GetC(s,v0),dir,j,wire);
 	}
 	gr->EndGroup();
@@ -1541,29 +1564,26 @@ void MGL_EXPORT mgl_axial_val(HMGL gr, HCDT v, HCDT a, const char *sch, const ch
 	else	x.Fill(0,gr->Max.x,'x');
 	y.Fill(gr->Min.y,gr->Max.y,'y');
 	mgl_axial_xy_val(gr,v,&x,&y,a,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_axial_xy(HMGL gr, HCDT x, HCDT y, HCDT a, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?3:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Axial");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Axial");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_axial_xy_val(gr,&v,x,y,a,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_axial(HMGL gr, HCDT a, const char *sch, const char *opt)
 {
 	mreal r = gr->SaveState(opt);
 	long Num = mgl_isnan(r)?3:long(r+0.5);
-	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Axial");	gr->LoadState();	return;	}
+	if(Num<1)	{	gr->SetWarn(mglWarnCnt,"Axial");	return;	}
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_axial_val(gr,&v,a,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_axial_xy_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
