@@ -1,6 +1,6 @@
 /***************************************************************************
  * crust.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -20,6 +20,8 @@
 #include <float.h>
 #include "mgl2/other.h"
 #include "mgl2/data.h"
+#include "mgl2/thread.h"
+#include "mgl2/base.h"
 //-----------------------------------------------------------------------------
 //
 //	TriPlot series
@@ -33,24 +35,24 @@ void MGL_EXPORT mgl_triplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 	long ss=gr->AddTexture(sch);
 	gr->SaveState(opt);	gr->SetPenPal("-");
 	static int cgid=1;	gr->StartGroup("TriPlot",cgid++);
-	mglPoint p1,p2,p3,q=mglPoint(NAN,NAN);
+	mglPoint p1,p2,p3,q;
 
-	register long i,k1,k2,k3;
 	bool wire = mglchr(sch,'#');
 	long nc = a->GetNx();
 	if(nc!=n && nc>=m)	// colors per triangle
 	{
 		gr->Reserve(m*3);
-		for(i=0;i<m;i++)
+#pragma omp parallel for private(p1,p2,p3,q)
+		for(long i=0;i<m;i++)
 		{
-			if(gr->Stop)	return;
-			k1 = long(nums->v(0,i)+0.5);
+			if(gr->Stop)	continue;
+			register long k1 = long(nums->v(0,i)+0.5);
 			p1 = mglPoint(x->v(k1), y->v(k1), z->v(k1));
-			k2 = long(nums->v(1,i)+0.5);
+			register long k2 = long(nums->v(1,i)+0.5);
 			p2 = mglPoint(x->v(k2), y->v(k2), z->v(k2));
-			k3 = long(nums->v(2,i)+0.5);
+			register long k3 = long(nums->v(2,i)+0.5);
 			p3 = mglPoint(x->v(k3), y->v(k3), z->v(k3));
-			if(!wire)	q = (p2-p1) ^ (p3-p1);
+			q = wire ? mglPoint(NAN,NAN) : (p2-p1) ^ (p3-p1);
 			k1 = gr->AddPnt(p1,gr->GetC(ss,a->v(k1)),q);
 			k2 = gr->AddPnt(p2,gr->GetC(ss,a->v(k2)),q);
 			k3 = gr->AddPnt(p3,gr->GetC(ss,a->v(k3)),q);
@@ -62,34 +64,38 @@ void MGL_EXPORT mgl_triplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 		gr->Reserve(n);
 		long *kk = new long[n];
 		mglPoint *pp = new mglPoint[n];
-		for(i=0;i<m;i++)	// add averaged normales
+#pragma omp parallel for
+		for(long i=0;i<m;i++)	// add averaged normales
 		{
-			if(gr->Stop)	{	delete []kk;	delete []pp;	return;	}
-			k1 = long(nums->v(0,i)+0.5);
-			k2 = long(nums->v(1,i)+0.5);
-			k3 = long(nums->v(2,i)+0.5);
+			if(gr->Stop)	continue;
+			register long k1 = long(nums->v(0,i)+0.5);
+			register long k2 = long(nums->v(1,i)+0.5);
+			register long k3 = long(nums->v(2,i)+0.5);
 			if(!wire)
 			{
-				q = mglPoint(x->v(k2)-x->v(k1), y->v(k2)-y->v(k1), z->v(k2)-z->v(k1)) ^
+				mglPoint q = mglPoint(x->v(k2)-x->v(k1), y->v(k2)-y->v(k1), z->v(k2)-z->v(k1)) ^
 					mglPoint(x->v(k3)-x->v(k1), y->v(k3)-y->v(k1), z->v(k3)-z->v(k1));
 				q.Normalize();
-				// try be sure that in the same direction ... 
+				// try be sure that in the same direction ...
 				if(q.z<0)	q *= -1;
-				pp[k1] += q;	pp[k2] += q;	pp[k3] += q;
+#pragma omp critical(quadplot)
+				{pp[k1] += q;	pp[k2] += q;	pp[k3] += q;}
 			}
+			else	pp[k1]=pp[k2]=pp[k3]=mglPoint(NAN,NAN);
 		}
-		for(i=0;i<n;i++)	// add points
+#pragma omp parallel for
+		for(long i=0;i<n;i++)	// add points
 		{
-			if(gr->Stop)	{	delete []kk;	delete []pp;	return;	}
-			q = mglPoint(x->v(i), y->v(i), z->v(i));
-			kk[i] = gr->AddPnt(q,gr->GetC(ss,a->v(i)),pp[i]);
+			if(gr->Stop)	continue;
+			kk[i] = gr->AddPnt(mglPoint(x->v(i), y->v(i), z->v(i)), gr->GetC(ss,a->v(i)), pp[i]);
 		}
-		for(i=0;i<m;i++)	// draw triangles
+#pragma omp parallel for
+		for(long i=0;i<m;i++)	// draw triangles
 		{
-			if(gr->Stop)	{	delete []kk;	delete []pp;	return;	}
-			k1 = long(nums->v(0,i)+0.5);
-			k2 = long(nums->v(1,i)+0.5);
-			k3 = long(nums->v(2,i)+0.5);
+			if(gr->Stop)	continue;
+			register long k1 = long(nums->v(0,i)+0.5);
+			register long k2 = long(nums->v(1,i)+0.5);
+			register long k3 = long(nums->v(2,i)+0.5);
 			if(wire)
 			{
 				gr->line_plot(kk[k1],kk[k2]);	gr->line_plot(kk[k1],kk[k3]);
@@ -111,7 +117,6 @@ void MGL_EXPORT mgl_triplot_xy(HMGL gr, HCDT nums, HCDT x, HCDT y, const char *s
 	mglData z(x->GetNx());
 	mreal zm = gr->AdjustZMin();	z.Fill(zm,zm);
 	mgl_triplot_xyzc(gr,nums,x,y,&z,&z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_triplot_xyzc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, const char *opt,int l,int lo)
@@ -143,26 +148,26 @@ void MGL_EXPORT mgl_quadplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HC
 	long ss=gr->AddTexture(sch);
 	gr->SaveState(opt);	gr->SetPenPal("-");
 	static int cgid=1;	gr->StartGroup("QuadPlot",cgid++);
-	mglPoint p1,p2,p3,p4,q=mglPoint(NAN,NAN);
+	mglPoint p1,p2,p3,p4;
 
-	register long i,k1,k2,k3,k4;
 	long nc = a->GetNx();
 	bool wire = mglchr(sch,'#');
 	if(nc!=n && nc>=m)	// colors per triangle
 	{
 		gr->Reserve(m*4);
-		for(i=0;i<m;i++)
+#pragma omp parallel for private(p1,p2,p3,p4)
+		for(long i=0;i<m;i++)
 		{
-			if(gr->Stop)	return;
-			k1 = long(nums->v(0,i)+0.5);
+			if(gr->Stop)	continue;
+			register long k1 = long(nums->v(0,i)+0.5);
 			p1 = mglPoint(x->v(k1), y->v(k1), z->v(k1));
-			k2 = long(nums->v(1,i)+0.5);
+			register long k2 = long(nums->v(1,i)+0.5);
 			p2 = mglPoint(x->v(k2), y->v(k2), z->v(k2));
-			k3 = long(nums->v(2,i)+0.5);
+			register long k3 = long(nums->v(2,i)+0.5);
 			p3 = mglPoint(x->v(k3), y->v(k3), z->v(k3));
-			k4 = floor(nums->v(3,i)+0.5);
+			register long k4 = floor(nums->v(3,i)+0.5);
 			p4 = mglPoint(x->v(k4), y->v(k4), z->v(k4));
-			if(!wire)	q = (p2-p1) ^ (p3-p1);
+			mglPoint q = wire ? mglPoint(NAN,NAN):(p2-p1) ^ (p3-p1);
 			k1 = gr->AddPnt(p1,gr->GetC(ss,a->v(k1)),q);
 			k2 = gr->AddPnt(p2,gr->GetC(ss,a->v(k2)),q);
 			k3 = gr->AddPnt(p3,gr->GetC(ss,a->v(k3)),q);
@@ -175,39 +180,44 @@ void MGL_EXPORT mgl_quadplot_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HC
 		gr->Reserve(n);
 		long *kk = new long[n];
 		mglPoint *pp = new mglPoint[n];
-		for(i=0;i<m;i++)	// add averaged normales
+#pragma omp parallel for private(p1,p2,p3,p4)
+		for(long i=0;i<m;i++)	// add averaged normales
 		{
-			if(gr->Stop)	{	delete []kk;	delete []pp;	return;	}
-			k1 = long(nums->v(0,i)+0.5);
+			if(gr->Stop)	continue;
+			register long k1 = long(nums->v(0,i)+0.5);
 			p1 = mglPoint(x->v(k1), y->v(k1), z->v(k1));
-			k2 = long(nums->v(1,i)+0.5);
+			register long k2 = long(nums->v(1,i)+0.5);
 			p2 = mglPoint(x->v(k2), y->v(k2), z->v(k2));
-			k3 = long(nums->v(2,i)+0.5);
+			register long k3 = long(nums->v(2,i)+0.5);
 			p3 = mglPoint(x->v(k3), y->v(k3), z->v(k3));
-			k4 = floor(nums->v(3,i)+0.5);
+			register long k4 = floor(nums->v(3,i)+0.5);
 			p4 = mglPoint(x->v(k4), y->v(k4), z->v(k4));
 
 			if(wire)	pp[k1]=pp[k2]=pp[k3]=pp[k4]=mglPoint(NAN,NAN);
 			else
 			{
-				q = (p2-p1) ^ (p3-p1);	if(q.z<0) q*=-1;	pp[k1] += q;
-				q = (p2-p4) ^ (p3-p4);	if(q.z<0) q*=-1;	pp[k2] += q;
-				q = (p1-p2) ^ (p4-p2);	if(q.z<0) q*=-1;	pp[k3] += q;
-				q = (p1-p4) ^ (p4-p3);	if(q.z<0) q*=-1;	pp[k4] += q;
+				mglPoint q1 = (p2-p1) ^ (p3-p1);	if(q1.z<0) q1*=-1;
+				mglPoint q2 = (p2-p4) ^ (p3-p4);	if(q2.z<0) q2*=-1;
+				mglPoint q3 = (p1-p2) ^ (p4-p2);	if(q3.z<0) q3*=-1;
+				mglPoint q4 = (p1-p4) ^ (p4-p3);	if(q4.z<0) q4*=-1;
+#pragma omp critical(quadplot)
+				{pp[k1] += q1;	pp[k2] += q2;	pp[k3] += q3;	pp[k4] += q4;}
 			}
 		}
-		for(i=0;i<n;i++)	// add points
+#pragma omp parallel for
+		for(long i=0;i<n;i++)	// add points
 		{
-			if(gr->Stop)	{	delete []kk;	delete []pp;	return;	}
+			if(gr->Stop)	continue;
 			kk[i] = gr->AddPnt(mglPoint(x->v(i), y->v(i), z->v(i)),gr->GetC(ss,a->v(i)), pp[i]);
 		}
-		for(i=0;i<m;i++)	// draw quads
+#pragma omp parallel for
+		for(long i=0;i<m;i++)	// draw quads
 		{
-			if(gr->Stop)	{	delete []kk;	delete []pp;	return;	}
-			k1 = floor(nums->v(0,i)+0.5);
-			k2 = floor(nums->v(1,i)+0.5);
-			k3 = floor(nums->v(2,i)+0.5);
-			k4 = floor(nums->v(3,i)+0.5);
+			if(gr->Stop)	continue;
+			register long k1 = floor(nums->v(0,i)+0.5);
+			register long k2 = floor(nums->v(1,i)+0.5);
+			register long k3 = floor(nums->v(2,i)+0.5);
+			register long k4 = floor(nums->v(3,i)+0.5);
 			if(wire)
 			{
 				gr->line_plot(kk[k1],kk[k2]);	gr->line_plot(kk[k1],kk[k3]);
@@ -228,7 +238,6 @@ void MGL_EXPORT mgl_quadplot_xy(HMGL gr, HCDT nums, HCDT x, HCDT y, const char *
 	gr->SaveState(opt);
 	mglData z(x->GetNx());	z.Fill(gr->Min.z,gr->Min.z);
 	mgl_quadplot_xyzc(gr,nums,x,y,&z,&z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_quadplot_xyzc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *sch, const char *opt,int l,int lo)
@@ -260,21 +269,17 @@ void MGL_EXPORT mgl_tricont_xyzcv(HMGL gr, HCDT v, HCDT nums, HCDT x, HCDT y, HC
 	long ss=gr->AddTexture(sch);
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("TriCont",cgid++);
-	mreal val, c;
-	register long i,k;
-	long k1,k2,k3;
 	bool zVal = !(mglchr(sch,'_'));
-	mreal d1,d2,d3;
 	mglPoint p1,p2,p3;
-	for(k=0;k<v->GetNx();k++)	for(i=0;i<m;i++)
+#pragma omp parallel for private(p1,p2,p3) collapse(2)
+	for(long k=0;k<v->GetNx();k++)	for(long i=0;i<m;i++)
 	{
-		if(gr->Stop)	return;
-		k1 = long(nums->v(0,i)+0.1);	if(k1<0 || k1>=n)	continue;
-		k2 = long(nums->v(1,i)+0.1);	if(k2<0 || k2>=n)	continue;
-		k3 = long(nums->v(2,i)+0.1);	if(k3<0 || k3>=n)	continue;
-		val = v->v(k);
-		c = gr->GetC(ss,val);
-		
+		if(gr->Stop)	continue;
+		register long k1 = long(nums->v(0,i)+0.1);	if(k1<0 || k1>=n)	continue;
+		register long k2 = long(nums->v(1,i)+0.1);	if(k2<0 || k2>=n)	continue;
+		register long k3 = long(nums->v(2,i)+0.1);	if(k3<0 || k3>=n)	continue;
+		register mreal val = v->v(k), c = gr->GetC(ss,val), d1,d2,d3;
+
 		d1 = mgl_d(val,a->v(k1),a->v(k2));
 		p1 = mglPoint(x->v(k1)*(1-d1)+x->v(k2)*d1, y->v(k1)*(1-d1)+y->v(k2)*d1,
 					  zVal?z->v(k1)*(1-d1)+z->v(k2)*d1:gr->Min.z);
@@ -309,7 +314,6 @@ void MGL_EXPORT mgl_tricont_xyzc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, HCD
 	mglData v(n);
 	for(long i=0;i<n;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(n+1);
 	mgl_tricont_xyzcv(gr,&v,nums,x,y,z,a,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tricont_xyc(HMGL gr, HCDT nums, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
@@ -346,36 +350,40 @@ void MGL_EXPORT mgl_tricont_xyc_(uintptr_t *gr, uintptr_t *nums, uintptr_t *x, u
 //	Dots series
 //
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_dots_a(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+void MGL_EXPORT mgl_dots_ca(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, HCDT a, const char *sch, const char *opt)
 {
 	long n = x->GetNN(), d, k=1;
 	if(x->GetNz()>1) 	k=3;		else if(x->GetNy()>1)	k=2;
 
-	if(y->GetNN()!=n || z->GetNN()!=n || a->GetNN()!=n)	{	gr->SetWarn(mglWarnDim,"Dots");	return;	}
+	if(y->GetNN()!=n || z->GetNN()!=n || c->GetNN()!=n || (a && a->GetNN()!=n))
+	{	gr->SetWarn(mglWarnDim,"Dots");	return;	}
 	gr->SaveState(opt);
 
 	d = gr->MeshNum>0 ? mgl_ipow(gr->MeshNum+1,k) : n;
 	d = n>d ? n/d:1;
-	
+
 	static int cgid=1;	gr->StartGroup("Dots",cgid++);
 	char mk=gr->SetPenPal(sch);
-	long ss=gr->AddTexture(sch), pp;
+	long ss=gr->AddTexture(sch);
 	if(mk==0)	mk='.';
 	gr->Reserve(n);
-	mglPoint p;
 
+#pragma omp parallel for
 	for(long i=0;i<n;i+=d)
 	{
-		if(gr->Stop)	return;
-		p = mglPoint(x->vthr(i),y->vthr(i),z->vthr(i));
-		pp = gr->AddPnt(p,gr->GetC(ss,a->vthr(i)));
+		if(gr->Stop)	continue;
+		mglPoint p = mglPoint(x->vthr(i),y->vthr(i),z->vthr(i));
+		long pp = gr->AddPnt(p,gr->GetC(ss,c->vthr(i)),mglPoint(NAN),a?gr->GetA(a->vthr(i)):-1);
 		gr->mark_plot(pp, mk);
 	}
 	gr->EndGroup();
 }
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_dots_a(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char *sch, const char *opt)
+{	mgl_dots_ca(gr, x, y, z, z, a, sch, opt);	}
+//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dots(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
-{	mgl_dots_a(gr, x, y, z, z, sch, opt);	}
+{	mgl_dots_ca(gr, x, y, z, z, NULL, sch, opt);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dots_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *sch, const char *opt,int l,int lo)
 {	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
@@ -387,6 +395,11 @@ void MGL_EXPORT mgl_dots_a_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t
 	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
 	mgl_dots_a(_GR_, _DA_(x),_DA_(y),_DA_(z),_DA_(a),s, o);	delete []o;	delete []s;	}
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_dots_ca_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
+{	char *s=new char[l+1];	memcpy(s,sch,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	mgl_dots_ca(_GR_, _DA_(x),_DA_(y),_DA_(z),_DA_(c),_DA_(a),s, o);	delete []o;	delete []s;	}
+//-----------------------------------------------------------------------------
 //
 //	mglTriangulation
 //
@@ -394,19 +407,20 @@ void MGL_EXPORT mgl_dots_a_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t
 long MGL_NO_EXPORT mgl_crust(long n,mglPoint *pp,long **nn,mreal ff);
 HMDT MGL_EXPORT mgl_triangulation_3d(HCDT x, HCDT y, HCDT z)
 {	// TODO: should be used s-hull or q-hull
-	mglData *nums=new mglData;
+	mglData *nums=0;
 	long n = x->GetNx(), m;
 	if(y->GetNx()!=n || z->GetNx()!=n)	return nums;
-	register long i;
 	mglPoint *pp = new mglPoint[n];
 	long *nn=0;
-	for(i=0;i<n;i++)	pp[i] = mglPoint(x->v(i), y->v(i), z->v(i));
+#pragma omp parallel for
+	for(long i=0;i<n;i++)	pp[i] = mglPoint(x->v(i), y->v(i), z->v(i));
 	m = mgl_crust(n,pp,&nn,0);
 
 	if(m>0)
 	{
-		nums->Create(3,m);
-		for(i=0;i<3*m;i++)	nums->a[i]=nn[i];
+		nums=new mglData(3,m);
+#pragma omp parallel for
+		for(long i=0;i<3*m;i++)	nums->a[i]=nn[i];
 	}
 	delete []pp;	free(nn);	return nums;
 }
@@ -414,26 +428,24 @@ HMDT MGL_EXPORT mgl_triangulation_3d(HCDT x, HCDT y, HCDT z)
 #include "s_hull/s_hull_pro.h"
 HMDT MGL_EXPORT mgl_triangulation_2d(HCDT x, HCDT y)
 {
-	mglData *nums=new mglData;
-	register long n = x->GetNx(), m,i;
-	if(y->GetNx()!=n)	return nums;
+	mglData *nums=0;
+	long n = x->GetNN();
+	if(y->GetNN()!=n)	return nums;
 	// use s-hull here
 	std::vector<Shx> pts;
 	std::vector<size_t> out;
 	Shx pt;
 
-	for(i=0;i<n;i++)
-	{
-		pt.r = x->v(i);	pt.c = y->v(i);
-		pt.id = i;	pts.push_back(pt);
-	}
+	for(long i=0;i<n;i++)
+	{	pt.r = x->vthr(i);	pt.c = y->vthr(i);	pt.id = i;	pts.push_back(pt);	}
 	std::vector<Triad> triads;
 	if(de_duplicate(pts, out))
 		mglGlobalMess += "There are duplicated points for triangulation.\n";
 	s_hull_pro(pts, triads);
-	m = triads.size();
-	nums->Create(3,m);
-	for(i=0;i<m;i++)
+	long m = triads.size();
+	nums=new mglData(3,m);
+#pragma omp parallel for
+	for(long i=0;i<m;i++)
 	{
 		nums->a[3*i]   = triads[i].a;
 		nums->a[3*i+1] = triads[i].b;
@@ -455,12 +467,14 @@ MGL_NO_EXPORT void *mgl_grid_t(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nx=t->p[0],ny=t->p[1];
-	register long i0, k1,k2,k3;
 	mreal *b=t->a;
 	const mreal *x=t->b, *y=t->c, *d=t->d, *z=t->e;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
-	{
-		k1 = long(d[3*i0]); k2 = long(d[3*i0+1]); k3 = long(d[3*i0+2]);
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
+	{	// TODO check if rounding needed
+		register long k1 = long(d[3*i0]), k2 = long(d[3*i0+1]), k3 = long(d[3*i0+2]);
 		mreal dxu,dxv,dyu,dyv;
 		mglPoint d1=mglPoint(x[k2]-x[k1],y[k2]-y[k1],z[k2]-z[k1]), d2=mglPoint(x[k3]-x[k1],y[k3]-y[k1],z[k3]-z[k1]), p;
 
@@ -490,30 +504,40 @@ MGL_NO_EXPORT void *mgl_grid_t(void *par)
 	}
 	return 0;
 }
-void MGL_EXPORT mgl_data_grid(HMGL gr, HMDT d, HCDT xdat, HCDT ydat, HCDT zdat, const char *opt)
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_data_grid_xy(HMDT d, HCDT xdat, HCDT ydat, HCDT zdat, mreal x1, mreal x2, mreal y1, mreal y2)
 { // NOTE: only for mglData
 	const mglData *x = dynamic_cast<const mglData *>(xdat);
 	const mglData *y = dynamic_cast<const mglData *>(ydat);
 	const mglData *z = dynamic_cast<const mglData *>(zdat);
 	if(!x || !y || !z) return;
-	long n=x->nx;
-	if((n<3) || (y->nx!=n) || (z->nx!=n))	return;
+	long n=x->GetNN();
+	if((n<3) || (y->GetNN()!=n) || (z->GetNN()!=n))	return;
 
-	gr->SaveState(opt);
 	mglData *nums = mgl_triangulation_2d(x,y);
 	if(nums->nx<3)	{	delete nums;	return;	}
 	long nn = nums->ny, par[3]={d->nx,d->ny,d->nz};
-	mreal xx[4]={gr->Min.x,0, gr->Min.y,0};
-	if(d->nx>1) xx[1] = (d->nx-1.)/(gr->Max.x-gr->Min.x);
-	if(d->ny>1) xx[3] = (d->ny-1.)/(gr->Max.y-gr->Min.y);
+	mreal xx[4]={x1,0, y1,0};
+	if(d->nx>1) xx[1] = (d->nx-1.)/(x2-x1);
+	if(d->ny>1) xx[3] = (d->ny-1.)/(y2-y1);
 
-	register long i;
 	mreal *xc=new mreal[n], *yc=new mreal[n];
-	for(i=0;i<n;i++)	{	xc[i]=xx[1]*(x->a[i]-xx[0]);	yc[i]=xx[3]*(y->a[i]-xx[2]);	}
+#pragma omp parallel for
+	for(long i=0;i<n;i++)	{	xc[i]=xx[1]*(x->a[i]-xx[0]);	yc[i]=xx[3]*(y->a[i]-xx[2]);	}
+#pragma omp parallel for
 	for(long i=0;i<d->nx*d->ny*d->nz;i++) d->a[i] = NAN;
-	
+
 	mglStartThread(mgl_grid_t,0,nn,d->a,xc,yc,par,0,nums->a,z->a);
-	gr->LoadState();	delete nums;	delete []xc;	delete []yc;
+	delete nums;	delete []xc;	delete []yc;
+}
+void MGL_EXPORT mgl_data_grid_xy_(uintptr_t *d, uintptr_t *x, uintptr_t *y, uintptr_t *z, mreal *x1, mreal *x2, mreal *y1, mreal *y2)
+{	mgl_data_grid_xy(_DT_,_DA_(x),_DA_(y),_DA_(z),*x1,*x2,*y1,*y2);	}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_data_grid(HMGL gr, HMDT d, HCDT xdat, HCDT ydat, HCDT zdat, const char *opt)
+{
+	gr->SaveState(opt);
+	mgl_data_grid_xy(d,xdat,ydat,zdat,gr->Min.x,gr->Max.x,gr->Min.y,gr->Max.y);
+	gr->LoadState();
 }
 void MGL_EXPORT mgl_data_grid_(uintptr_t *gr, uintptr_t *d, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *opt,int lo)
 {	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
