@@ -1,6 +1,6 @@
 /***************************************************************************
  * data_io.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -26,6 +26,7 @@
 #include "mgl2/data.h"
 #include "mgl2/datac.h"
 #include "mgl2/eval.h"
+#include "mgl2/thread.h"
 
 #if MGL_HAVE_HDF5
 //#define H5_NO_DEPRECATED_SYMBOLS
@@ -62,6 +63,7 @@ void mglFromStr(HMDT d,char *buf,long NX,long NY,long NZ)	// TODO: add multithre
 	mgl_data_create(d, NX,NY,NZ);
 	long nb = strlen(buf);
 	register long i=0, j=0;
+	setlocale(LC_NUMERIC, "C");
 	while(j<nb)
 	{
 		while(buf[j]<=' ' && j<nb)	j++;
@@ -86,6 +88,7 @@ void mglFromStr(HMDT d,char *buf,long NX,long NY,long NZ)	// TODO: add multithre
 		d->a[i] = strstr(s,"NAN")?NAN:atof(s);
 		i++;	if(i>=NX*NY*NZ)	break;
 	}
+	setlocale(LC_NUMERIC, "");
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_set(HMDT d, HCDT a)
@@ -96,11 +99,9 @@ void MGL_EXPORT mgl_data_set(HMDT d, HCDT a)
 	if(dd)	// this one should be much faster
 		memcpy(d->a, dd->a, d->nx*d->ny*d->nz*sizeof(mreal));
 	else	// very inefficient!!!
-	{
-		register long i,j,k;
-		for(k=0;k<d->nz;k++)	for(j=0;j<d->ny;j++)	for(i=0;i<d->nx;i++)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<d->nz;k++)	for(long j=0;j<d->ny;j++)	for(long i=0;i<d->nx;i++)
 			d->a[i+d->nx*(j+d->ny*k)] = a->v(i,j,k);
-	}
 }
 void MGL_EXPORT mgl_data_set_(uintptr_t *d, uintptr_t *a)	{	mgl_data_set(_DT_,_DA_(a));	}
 //-----------------------------------------------------------------------------
@@ -122,6 +123,7 @@ void MGL_EXPORT mgl_data_set_vector(HMDT d, gsl_vector *v)
 #if MGL_HAVE_GSL
 	if(!v || v->size<1)	return;
 	mgl_data_create(d, v->size,1,1);
+#pragma omp parallel for
 	for(long i=0;i<d->nx;i++)	d->a[i] = v->data[i*v->stride];
 #endif
 }
@@ -131,8 +133,8 @@ void MGL_EXPORT mgl_data_set_matrix(HMDT d, gsl_matrix *m)
 #if MGL_HAVE_GSL
 	if(!m || m->size1<1 || m->size2<1)	return;
 	mgl_data_create(d, m->size1,m->size2,1);
-	register long i,j;
-	for(j=0;j<d->ny;j++)	for(i=0;i<d->nx;i++)
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<d->ny;j++)	for(long i=0;i<d->nx;i++)
 		d->a[i+j*d->nx] = m->data[i * m->tda + j];
 #endif
 }
@@ -142,6 +144,7 @@ void MGL_EXPORT mgl_data_set_float(HMDT d, const float *A,long NX,long NY,long N
 	if(NX<=0 || NY<=0 || NZ<=0)	return;
 	mgl_data_create(d, NX,NY,NZ);	if(!A)	return;
 #if MGL_USE_DOUBLE
+#pragma omp parallel for
 	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
 #else
 	memcpy(d->a,A,NX*NY*NZ*sizeof(float));
@@ -155,53 +158,62 @@ void MGL_EXPORT mgl_data_set_double(HMDT d, const double *A,long NX,long NY,long
 #if MGL_USE_DOUBLE
 	memcpy(d->a,A,NX*NY*NZ*sizeof(double));
 #else
+#pragma omp parallel for
 	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
 #endif
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_data_set_float2(HMDT d, const float **A,long N1,long N2)
+void MGL_EXPORT mgl_data_set_float2(HMDT d, float const * const *A,long N1,long N2)
 {
 	if(N1<=0 || N2<=0)	return;
 	mgl_data_create(d, N2,N1,1);	if(!A)	return;
 #if MGL_USE_DOUBLE
+#pragma omp parallel for collapse(2)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	d->a[j+i*N2] = A[i][j];
 #else
+#pragma omp parallel for
 	for(long i=0;i<N1;i++)	memcpy(d->a+i*N2,A[i],N2*sizeof(float));
 #endif
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_data_set_double2(HMDT d, const double **A,long N1,long N2)
+void MGL_EXPORT mgl_data_set_double2(HMDT d, double const *const *A,long N1,long N2)
 {
 	if(N1<=0 || N2<=0)	return;
 	mgl_data_create(d, N2,N1,1);	if(!A)	return;
 #if MGL_USE_DOUBLE
+#pragma omp parallel for
 	for(long i=0;i<N1;i++)	memcpy(d->a+i*N2,A[i],N2*sizeof(double));
 #else
+#pragma omp parallel for collapse(2)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	d->a[j+i*N2] = A[i][j];
 #endif
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_data_set_float3(HMDT d, const float ***A,long N1,long N2,long N3)
+void MGL_EXPORT mgl_data_set_float3(HMDT d, float const * const * const *A,long N1,long N2,long N3)
 {
 	if(N1<=0 || N2<=0 || N3<=0)	return;
 	mgl_data_create(d, N3,N2,N1);	if(!A)	return;
 #if MGL_USE_DOUBLE
+#pragma omp parallel for collapse(3)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	for(long k=0;k<N3;k++)
 		d->a[k+N3*(j+i*N2)] = A[i][j][k];
 #else
+#pragma omp parallel for collapse(2)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)
 		memcpy(d->a+N3*(j+i*N2),A[i][j],N3*sizeof(float));
 #endif
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_data_set_double3(HMDT d, const double ***A,long N1,long N2,long N3)
+void MGL_EXPORT mgl_data_set_double3(HMDT d, double const * const * const *A,long N1,long N2,long N3)
 {
 	if(N1<=0 || N2<=0 || N3<=0)	return;
 	mgl_data_create(d, N3,N2,N1);	if(!A)	return;
 #if MGL_USE_DOUBLE
+#pragma omp parallel for collapse(2)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)
 		memcpy(d->a+N3*(j+i*N2),A[i][j],N3*sizeof(double));
 #else
+#pragma omp parallel for collapse(3)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	for(long k=0;k<N3;k++)
 		d->a[k+N3*(j+i*N2)] = A[i][j][k];
 #endif
@@ -243,16 +255,17 @@ void MGL_EXPORT mgl_data_set_id_(uintptr_t *d, const char *eq,int l)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_save(HCDT d, const char *fname,long ns)
 {
-	FILE *fp;
-	fp = fopen(fname,"w");
+	FILE *fp = fopen(fname,"w");
+	if(!fp)	return;
 	register long i,j,k;
 	long nx=d->GetNx(), ny=d->GetNy(), nz=d->GetNz();
+	setlocale(LC_NUMERIC, "C");
 	if(ns<0 || (ns>=nz && nz>1))	for(k=0;k<nz;k++)
 	{	// save whole data
 		const mglData *dr = dynamic_cast<const mglData *>(d);
-		if(dr)	fprintf(fp,"## %s\n",dr->id.c_str());
+		if(dr && !dr->id.empty())	fprintf(fp,"## %s\n",dr->id.c_str());
 		const mglDataC *dc = dynamic_cast<const mglDataC *>(d);
-		if(dc)	fprintf(fp,"## %s\n",dc->id.c_str());
+		if(dc && !dc->id.empty())	fprintf(fp,"## %s\n",dc->id.c_str());
 		for(i=0;i<ny;i++)
 		{
 			for(j=0;j<nx-1;j++)	fprintf(fp,"%g\t",d->v(j,i,k));
@@ -270,6 +283,7 @@ void MGL_EXPORT mgl_data_save(HCDT d, const char *fname,long ns)
 		else if(ns<ny)	for(j=0;j<nx;j++)
 			fprintf(fp,"%g\t",d->v(j,ns));
 	}
+	setlocale(LC_NUMERIC, "");
 	fclose(fp);
 }
 void MGL_EXPORT mgl_data_save_(uintptr_t *d, const char *fname,int *ns,int l)
@@ -444,41 +458,115 @@ int MGL_EXPORT mgl_data_read_mat_(uintptr_t *d, const char *fname,int *dim,int l
 //-----------------------------------------------------------------------------
 mreal MGL_EXPORT mgl_data_max(HCDT d)
 {
-	register mreal m=-1e10, v;
-	register long nn=d->GetNN();
+	mreal m1=-INFINITY;
+	long nn=d->GetNN();
 	const mglData *b = dynamic_cast<const mglData *>(d);
-	if(b)	for(long i=0;i<nn;i++)
-	{	v = b->a[i];	if(!mgl_isnan(v))	m = m>v ? m:v;	}
-	else	for(long i=0;i<nn;i++)
-	{	v = d->vthr(i);	if(!mgl_isnan(v))	m = m>v ? m:v;	}
-	return m;
+#pragma omp parallel
+	{
+		register mreal m=-INFINITY, v;
+		if(b)
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = b->a[i];	m = m<v ? v:m;	}
+		else
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = d->vthr(i);	m = m<v ? v:m;	}
+#pragma omp critical(max_dat)
+		{	m1 = m1>m ? m1:m;	}
+	}
+	return m1;
 }
 mreal MGL_EXPORT mgl_data_max_(uintptr_t *d)	{	return mgl_data_max(_DT_);	}
 //-----------------------------------------------------------------------------
 mreal MGL_EXPORT mgl_data_min(HCDT d)
 {
-	register mreal m=1e10, v;
-	register long nn=d->GetNN();
+	mreal m1=INFINITY;
+	long nn=d->GetNN();
 	const mglData *b = dynamic_cast<const mglData *>(d);
-	if(b)	for(long i=0;i<nn;i++)
-	{	v = b->a[i];	if(!mgl_isnan(v))	m = m<v ? m:v;	}
-	else	for(long i=0;i<nn;i++)
-	{	v = d->vthr(i);	if(!mgl_isnan(v))	m = m<v ? m:v;	}
-	return m;
+#pragma omp parallel
+	{
+		register mreal m=INFINITY, v;
+		if(b)
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = b->a[i];	m = m>v ? v:m;	}
+		else
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = d->vthr(i);	m = m>v ? v:m;	}
+#pragma omp critical(min_dat)
+		{	m1 = m1<m ? m1:m;	}
+	}
+	return m1;
 }
 mreal MGL_EXPORT mgl_data_min_(uintptr_t *d)	{	return mgl_data_min(_DT_);	}
 //-----------------------------------------------------------------------------
+mreal MGL_EXPORT mgl_data_neg_max(HCDT d)
+{
+	mreal m1=0;
+	long nn=d->GetNN();
+	const mglData *b = dynamic_cast<const mglData *>(d);
+#pragma omp parallel
+	{
+		register mreal m=0, v;
+		if(b)
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = b->a[i];	m = m<v && v<0 ? v:m;	}
+		else
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = d->vthr(i);	m = m<v && v<0 ? v:m;	}
+#pragma omp critical(max_dat)
+		{	m1 = m1>m ? m1:m;	}
+	}
+	return m1;
+}
+mreal MGL_EXPORT mgl_data_neg_max_(uintptr_t *d)	{	return mgl_data_neg_max(_DT_);	}
+//-----------------------------------------------------------------------------
+mreal MGL_EXPORT mgl_data_pos_min(HCDT d)
+{
+	mreal m1=INFINITY;
+	long nn=d->GetNN();
+	const mglData *b = dynamic_cast<const mglData *>(d);
+#pragma omp parallel
+	{
+		register mreal m=INFINITY, v;
+		if(b)
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = b->a[i];	m = m>v && v>0 ? v:m;	}
+		else
+#pragma omp for nowait
+			for(long i=0;i<nn;i++)
+			{	v = d->vthr(i);	m = m>v && v>0 ? v:m;	}
+#pragma omp critical(min_dat)
+		{	m1 = m1<m ? m1:m;	}
+	}
+	return m1;
+}
+mreal MGL_EXPORT mgl_data_pos_min_(uintptr_t *d)	{	return mgl_data_pos_min(_DT_);	}
+//-----------------------------------------------------------------------------
 mreal MGL_EXPORT mgl_data_max_int(HCDT d, long *i, long *j, long *k)
 {
-	register mreal m=-1e10, v;
+	mreal m1=-INFINITY;
 	long nx=d->GetNx(), ny=d->GetNy(), nn=d->GetNN();
-	for(long ii=0;ii<nn;ii++)
+#pragma omp parallel
 	{
-		v = d->vthr(ii);
-		if(!mgl_isnan(v) && m < v)
-		{	m=v;	*i=ii%nx;	*j=(ii/nx)%ny;	*k=ii/(nx*ny);   }
+		register mreal m=-INFINITY, v;
+		long im=-1,jm=-1,km=-1;
+#pragma omp for nowait
+		for(long ii=0;ii<nn;ii++)
+		{
+			v = d->vthr(ii);
+			if(m < v)
+			{	m=v;	im=ii%nx;	jm=(ii/nx)%ny;	km=ii/(nx*ny);   }
+		}
+#pragma omp critical(max_int)
+		if(m1 < m)	{	m1=m;	*i=im;	*j=jm;	*k=km;   }
 	}
-	return m;
+	return m1;
 }
 mreal MGL_EXPORT mgl_data_max_int_(uintptr_t *d, int *i, int *j, int *k)
 {	long ii,jj,kk;	mreal res=mgl_data_max_int(_DT_,&ii,&jj,&kk);
@@ -486,15 +574,23 @@ mreal MGL_EXPORT mgl_data_max_int_(uintptr_t *d, int *i, int *j, int *k)
 //-----------------------------------------------------------------------------
 mreal MGL_EXPORT mgl_data_min_int(HCDT d, long *i, long *j, long *k)
 {
-	register mreal m=1e10, v;
+	mreal m1=INFINITY;
 	long nx=d->GetNx(), ny=d->GetNy(), nn=d->GetNN();
-	for(long ii=0;ii<nn;ii++)
+#pragma omp parallel
 	{
-		v = d->vthr(ii);
-		if(!mgl_isnan(v) && m > v)
-		{	m=v;	*i=ii%nx;	*j=(ii/nx)%ny;	*k=ii/(nx*ny);   }
+		register mreal m=INFINITY, v;
+		long im=-1,jm=-1,km=-1;
+#pragma omp for nowait
+		for(long ii=0;ii<nn;ii++)
+		{
+			v = d->vthr(ii);
+			if(m > v)
+			{	m=v;	im=ii%nx;	jm=(ii/nx)%ny;	km=ii/(nx*ny);   }
+		}
+#pragma omp critical(min_int)
+		if(m1 > m)	{	m1=m;	*i=im;	*j=jm;	*k=km;   }
 	}
-	return m;
+	return m1;
 }
 mreal MGL_EXPORT mgl_data_min_int_(uintptr_t *d, int *i, int *j, int *k)
 {	long ii,jj,kk;	mreal res=mgl_data_min_int(_DT_,&ii,&jj,&kk);
@@ -571,15 +667,24 @@ mreal MGL_EXPORT mgl_data_min_real_(uintptr_t *d, mreal *x, mreal *y, mreal *z)
 MGL_NO_EXPORT void *mgl_fill_x(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
-	register long i0, nx=t->p[0],ny=t->p[1];
+	long nx=t->p[0],ny=t->p[1];
 	mreal *b=t->a, x1=t->b[0], dx=t->b[1];
 	register char dir = t->s[0];
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
-	{
-		if(dir=='x') b[i0] = x1+dx*(i0%nx);
-		else if(dir=='y') b[i0] = x1+dx*((i0/nx)%ny);
-		else if(dir=='z') b[i0] = x1+dx*(i0/(nx*ny));
-	}
+	if(dir=='x')
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+		for(long i0=t->id;i0<t->n;i0+=mglNumThr)	b[i0] = x1+dx*(i0%nx);
+	else if(dir=='y')
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+		for(long i0=t->id;i0<t->n;i0+=mglNumThr)	b[i0] = x1+dx*((i0/nx)%ny);
+	else if(dir=='z')
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+		for(long i0=t->id;i0<t->n;i0+=mglNumThr)	b[i0] = x1+dx*(i0/(nx*ny));
 	return 0;
 }
 void MGL_EXPORT mgl_data_fill(HMDT d, mreal x1,mreal x2,char dir)
@@ -598,15 +703,11 @@ void MGL_EXPORT mgl_data_fill_(uintptr_t *d, mreal *x1,mreal *x2,const char *dir
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_norm(HMDT d, mreal v1,mreal v2,long sym,long dim)
 {
-	long i,s,nn=d->nx*d->ny*d->nz;
-	mreal a1=1e20,a2=-1e20,v,*a=d->a;
-	if(d->nz>1)	s = dim*d->nx*d->ny;
-	else		s = dim*d->ny;
-	for(i=s;i<nn;i++)	// determines borders of existing data
-	{
-		if(mgl_isnan(a[i]))	continue;
-		a1 = (a1<a[i] ? a1 : a[i]);	a2 = (a2>a[i] ? a2 : a[i]);
-	}
+	long s,nn=d->nx*d->ny*d->nz;
+	mreal a1=INFINITY,a2=-INFINITY,v,*a=d->a;
+	s = dim*d->ny*(d->nz>1 ? d->nx : 1);
+	for(long i=s;i<nn;i++)	// determines borders of existing data
+	{	a1 = a1>a[i] ? a[i]:a1;	a2 = a2<a[i] ? a[i]:a2;	}
 	if(a1==a2)  {  if(a1!=0)	a1=0.;  else a2=1;  }
 	if(v1>v2)	{	v=v1;	v1=v2;	v2=v;	}	// swap if uncorrect
 	if(sym)				// use symmetric
@@ -614,16 +715,16 @@ void MGL_EXPORT mgl_data_norm(HMDT d, mreal v1,mreal v2,long sym,long dim)
 		v2 = -v1>v2 ? -v1:v2;	v1 = -v2;
 		a2 = -a1>a2 ? -a1:a2;	a1 = -a2;
 	}
-	for(i=s;i<nn;i++)	// normalize
-		a[i] = v1 + (v2-v1)*(a[i]-a1)/(a2-a1);
+	v2 = (v2-v1)/(a2-a1);	v1 = v1-a1*v2;
+#pragma omp parallel for
+	for(long i=s;i<nn;i++)	a[i] = v1 + v2*a[i];
 }
 void MGL_EXPORT mgl_data_norm_(uintptr_t *d, mreal *v1,mreal *v2,int *sym,int *dim)
 {	mgl_data_norm(_DT_,*v1,*v2,*sym,*dim);	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_squeeze(HMDT d, long rx,long ry,long rz,long smooth)
 {
-	long kx,ky,kz,i,j,k;
-	long nx=d->nx, ny=d->ny, nz=d->nz;
+	long kx,ky,kz, nx=d->nx, ny=d->ny, nz=d->nz;
 	mreal *b;
 
 	// simple checking
@@ -633,19 +734,23 @@ void MGL_EXPORT mgl_data_squeeze(HMDT d, long rx,long ry,long rz,long smooth)
 	// new sizes
 	kx = 1+(nx-1)/rx;	ky = 1+(ny-1)/ry;	kz = 1+(nz-1)/rz;
 	b = new mreal[kx*ky*kz];
-	if(!smooth)	for(k=0;k<kz;k++)	for(j=0;j<ky;j++)	for(i=0;i<kx;i++)
-		b[i+kx*(j+ky*k)] = d->a[i*rx+nx*(j*ry+ny*rz*k)];
-	else		for(k=0;k<kz;k++)	for(j=0;j<ky;j++)	for(i=0;i<kx;i++)
-	{
-		long dx,dy,dz,i1,j1,k1;
-		dx = (i+1)*rx<=nx ? rx : nx-i*rx;
-		dy = (j+1)*ry<=ny ? ry : ny-j*ry;
-		dz = (k+1)*rz<=nz ? rz : nz-k*rz;
-		mreal s = 0;
-		for(k1=k*rz;k1<k*rz+dz;k1++)	for(j1=j*ry;j1<j*ry+dz;j1++)	for(i1=i*rx;i1<i*rx+dx;i1++)
-			s += d->a[i1+nx*(j1+ny*k1)];
-		b[i+kx*(j+ky*k)] = s/(dx*dy*dz);
-	}
+	if(!smooth)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<kz;k++)	for(long j=0;j<ky;j++)	for(long i=0;i<kx;i++)
+			b[i+kx*(j+ky*k)] = d->a[i*rx+nx*(j*ry+ny*rz*k)];
+	else
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<kz;k++)	for(long j=0;j<ky;j++)	for(long i=0;i<kx;i++)
+		{
+			long dx,dy,dz,i1,j1,k1;
+			dx = (i+1)*rx<=nx ? rx : nx-i*rx;
+			dy = (j+1)*ry<=ny ? ry : ny-j*ry;
+			dz = (k+1)*rz<=nz ? rz : nz-k*rz;
+			mreal s = 0;
+			for(k1=k*rz;k1<k*rz+dz;k1++)	for(j1=j*ry;j1<j*ry+dz;j1++)	for(i1=i*rx;i1<i*rx+dx;i1++)
+				s += d->a[i1+nx*(j1+ny*k1)];
+			b[i+kx*(j+ky*k)] = s/(dx*dy*dz);
+		}
 	if(!d->link)	delete [](d->a);
 	d->a=b;	d->nx = kx;  d->ny = ky;  d->nz = kz;	d->NewId();	d->link=false;
 }
@@ -658,29 +763,35 @@ void MGL_EXPORT mgl_data_extend(HMDT d, long n1, long n2)
 	if(nz>2 || n1==0)	return;
 	long mx, my, mz;
 	mreal *b=0;
-	register long i,j;
 	if(n1>0) // extend to higher dimension(s)
 	{
 		n2 = n2>0 ? n2:1;
 		mx = nx;	my = ny>1?ny:n1;	mz = ny>1 ? n1 : n2;
 		b = new mreal[mx*my*mz];
-		if(ny>1)	for(i=0;i<n1;i++)
-			memcpy(b+i*nx*ny, d->a, nx*ny*sizeof(mreal));
-		else		for(i=0;i<n1*n2;i++)
-			memcpy(b+i*nx, d->a, nx*sizeof(mreal));
+		if(ny>1)
+#pragma omp parallel for
+			for(long i=0;i<n1;i++)	memcpy(b+i*nx*ny, d->a, nx*ny*sizeof(mreal));
+		else
+#pragma omp parallel for
+			for(long i=0;i<n1*n2;i++)	memcpy(b+i*nx, d->a, nx*sizeof(mreal));
 	}
 	else
 	{
 		mx = -n1;	my = n2<0 ? -n2 : nx;	mz = n2<0 ? nx : ny;
 		if(n2>0 && ny==1)	mz = n2;
 		b = new mreal[mx*my*mz];
-		register mreal v;
-		if(n2<0)	for(j=0;j<nx;j++)	for(i=0,v=d->a[j];i<mx*my;i++)
-			b[i+mx*my*j] = v;
-		else	for(j=0;j<nx*ny;j++)	for(i=0,v=d->a[j];i<mx;i++)
-			b[i+mx*j] = v;
-		if(n2>0 && ny==1)	for(i=0;i<n2;i++)
-			memcpy(b+i*mx*my, d->a, mx*my*sizeof(mreal));
+		if(n2<0)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<nx;j++)	for(long i=0;i<mx*my;i++)
+				b[i+mx*my*j] = d->a[j];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<nx*ny;j++)	for(long i=0;i<mx;i++)
+				b[i+mx*j] = d->a[j];
+		if(n2>0 && ny==1)
+#pragma omp parallel for
+			for(long i=0;i<n2;i++)
+				memcpy(b+i*mx*my, d->a, mx*my*sizeof(mreal));
 	}
 	if(!d->link)	delete [](d->a);
 	d->a=b;	d->nx=mx;	d->ny=my;	d->nz=mz;
@@ -691,37 +802,41 @@ void MGL_EXPORT mgl_data_extend_(uintptr_t *d, int *n1, int *n2)
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_data_transpose(HMDT d, const char *dim)
 {
-	long nx=d->nx, ny=d->ny, nz=d->nz;
+	long nx=d->nx, ny=d->ny, nz=d->nz, n;
 	mreal *b=new mreal[nx*ny*nz], *a=d->a;
-	register long i,j,k,n;
 	if(!strcmp(dim,"xyz"))	memcpy(b,a,nx*ny*nz*sizeof(mreal));
 	else if(!strcmp(dim,"xzy") || !strcmp(dim,"zy"))
 	{
-		for(j=0;j<ny;j++)	for(k=0;k<nz;k++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(3)
+		for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 			b[i+nx*(k+nz*j)] = a[i+nx*(j+ny*k)];
 		n=nz;	nz=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"yxz") || !strcmp(dim,"yx"))
 	{
-		for(k=0;k<nz;k++)	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			b[j+ny*(i+nx*k)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"yzx"))
 	{
-		for(k=0;k<nz;k++)	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			b[j+ny*(k+nz*i)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=ny;	ny=nz;	nz=n;
 	}
 	else if(!strcmp(dim,"zxy"))
 	{
-		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)	for(k=0;k<nz;k++)
+#pragma omp parallel for collapse(3)
+		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)
 			b[k+nz*(i+nx*j)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=nz;	nz=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"zyx") || !strcmp(dim,"zx"))
 	{
-		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)	for(k=0;k<nz;k++)
+#pragma omp parallel for collapse(3)
+		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)
 			b[k+nz*(j+ny*i)] = a[i+nx*(j+ny*k)];
 		n=nz;	nz=nx;	nx=n;
 	}
@@ -737,13 +852,16 @@ MGL_NO_EXPORT void *mgl_modify(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	const mglFormula *f = (const mglFormula *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1],nz=t->p[2];
+	long nx=t->p[0],ny=t->p[1],nz=t->p[2];
 	mreal *b=t->a, dx,dy,dz;
 	const mreal *v=t->b, *w=t->c;
 	dx=nx>1?1/(nx-1.):0;	dy=ny>1?1/(ny-1.):0;	dz=nz>1?1/(nz-1.):0;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
+		register long i=i0%nx, j=((i0/nx)%ny), k=i0/(nx*ny);
 		b[i0] = f->Calc(i*dx, j*dy, k*dz, b[i0], v?v[i0]:0, w?w[i0]:0);
 	}
 	return 0;
@@ -772,13 +890,16 @@ MGL_NO_EXPORT void *mgl_modify_gen(void *par)
 {
 	mglThreadV *t=(mglThreadV *)par;
 	const mglFormula *f = (const mglFormula *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1],nz=t->p[2];
+	register long nx=t->p[0],ny=t->p[1],nz=t->p[2];
 	mreal *b=t->a, dx,dy,dz;
 	HCDT v=(HCDT)t->b, w=(HCDT)t->c;
 	dx=nx>1?1/(nx-1.):0;	dy=ny>1?1/(ny-1.):0;	dz=nz>1?1/(nz-1.):0;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
+		register long i=i0%nx, j=((i0/nx)%ny), k=i0/(nx*ny);
 		b[i0] = f->Calc(i*dx, j*dy, k*dz, b[i0], v?v->vthr(i0):0, w?w->vthr(i0):0);
 	}
 	return 0;
@@ -801,60 +922,6 @@ void MGL_EXPORT mgl_data_modify_vw_(uintptr_t *d, const char *eq, uintptr_t *v,
 {	char *s=new char[l+1];	memcpy(s,eq,l);	s[l]=0;
 	mgl_data_modify_vw(_DT_,s,_DA_(v),_DA_(w));	delete []s;	}
 //-----------------------------------------------------------------------------
-MGL_NO_EXPORT void *mgl_fill_f(void *par)
-{
-	mglThreadD *t=(mglThreadD *)par;
-	const mglFormula *f = (const mglFormula *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1];
-	mreal *b=t->a;
-	const mreal *v=t->b, *w=t->c, *x=t->d;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
-	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
-		b[i0] = f->Calc(x[0]+i*x[1], x[2]+j*x[3], x[4]+k*x[5], b[i0], v?v[i0]:0, w?w[i0]:0);
-	}
-	return 0;
-}
-MGL_NO_EXPORT void *mgl_fill_fgen(void *par)
-{
-	mglThreadV *t=(mglThreadV *)par;
-	const mglFormula *f = (const mglFormula *)(t->v);
-	register long i,j,k,i0, nx=t->p[0],ny=t->p[1];
-	mreal *b=t->a;
-	HCDT v=(HCDT)t->b, w=(HCDT)t->c;
-	const mreal *x=t->d;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
-	{
-		i=i0%nx;	j=((i0/nx)%ny);	k=i0/(nx*ny);
-		b[i0] = f->Calc(x[0]+i*x[1], x[2]+j*x[3], x[4]+k*x[5], b[i0], v?v->vthr(i0):0, w?w->vthr(i0):0);
-	}
-	return 0;
-}
-void MGL_EXPORT mgl_data_fill_eq(HMGL gr, HMDT d, const char *eq, HCDT vdat, HCDT wdat, const char *opt)
-{
-	const mglData *v = dynamic_cast<const mglData *>(vdat);
-	const mglData *w = dynamic_cast<const mglData *>(wdat);
-	long nn = d->nx*d->ny*d->nz, par[3]={d->nx,d->ny,d->nz};
-	if(vdat && vdat->GetNN()!=nn)	return;
-	if(wdat && wdat->GetNN()!=nn)	return;
-	gr->SaveState(opt);
-	mreal xx[6]={gr->Min.x,0, gr->Min.y,0, gr->Min.z,0};
-	if(d->nx>1)	xx[1] = (gr->Max.x-gr->Min.x)/(d->nx-1.);
-	if(d->ny>1)	xx[3] = (gr->Max.y-gr->Min.y)/(d->ny-1.);
-	if(d->nz>1)	xx[5] = (gr->Max.z-gr->Min.z)/(d->nz-1.);
-	mglFormula f(eq);
-	if(v && w)	mglStartThread(mgl_fill_f,0,nn,d->a,v->a,w->a,par,&f,xx);
-	else if(vdat && wdat)	mglStartThreadV(mgl_fill_fgen,nn,d->a,vdat,wdat,par,&f,xx);
-	else if(v)	mglStartThread(mgl_fill_f,0,nn,d->a,v->a,0,par,&f,xx);
-	else if(vdat)	mglStartThreadV(mgl_fill_fgen,nn,d->a,vdat,0,par,&f,xx);
-	else	mglStartThread(mgl_fill_f,0,nn,d->a,0,0,par,&f,xx);
-	gr->LoadState();
-}
-void MGL_EXPORT mgl_data_fill_eq_(uintptr_t *gr, uintptr_t *d, const char *eq, uintptr_t *v, uintptr_t *w, const char *opt,int l,int lo)
-{	char *s=new char[l+1];	memcpy(s,eq,l);	s[l]=0;
-	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
-	mgl_data_fill_eq(_GR_,_DT_,s,_DA_(v),_DA_(w),o);	delete []o;	delete []s;	}
-//-----------------------------------------------------------------------------
 #if MGL_HAVE_HDF4
 int MGL_EXPORT mgl_data_read_hdf4(HMDT d,const char *fname,const char *data)
 {
@@ -1045,7 +1112,7 @@ int MGL_EXPORT mgl_data_read_range(HMDT dat, const char *templ, double from, dou
 	//read first file
 	do{	snprintf(fname,n,templ,t);	t+= step;	} while(!mgl_data_read(&d,fname) && t<=to);
 
-	if(t>to)	return false;
+	if(t>to)	{	delete []fname;	return false;	}
 	kx = d.nx;	ky = d.ny;	kz = d.nz;
 	b = (mreal *)malloc(kx*ky*kz*sizeof(mreal));
 	memcpy(b,d.a,kx*ky*kz*sizeof(mreal));
@@ -1074,14 +1141,14 @@ int MGL_EXPORT mgl_data_read_all(HMDT dat, const char *templ, int as_slice)
 	size_t i;
 	mreal *b;
 	long kx,ky,kz;
-	char *fname = new char[256];
 	glob (templ, GLOB_TILDE, NULL, &res);
 
 	//read first file
 	for(i=0;i<res.gl_pathc;i++)
 		if(mgl_data_read(&d,res.gl_pathv[i]))	break;
 
-	if(i>=res.gl_pathc)	{	delete []fname;	return false;	}
+	if(i>=res.gl_pathc)
+	{	globfree (&res);	return false;	}
 	kx = d.nx;	ky = d.ny;	kz = d.nz;
 	b = (mreal *)malloc(kx*ky*kz*sizeof(mreal));
 	memcpy(b,d.a,kx*ky*kz*sizeof(mreal));
@@ -1090,12 +1157,11 @@ int MGL_EXPORT mgl_data_read_all(HMDT dat, const char *templ, int as_slice)
 	{
 		if(mgl_data_read(&d,res.gl_pathv[i]))
 			if(!mgl_add_file(kx,ky,kz,b,&d,as_slice))
-			{	delete []fname;		return false;	}
+			{	globfree (&res);	free(b);	return false;	}
 	}
 	dat->Set(b,kx,ky,kz);
 
-	globfree (&res);
-	delete []fname;		free(b);
+	globfree (&res);	free(b);
 	return true;
 #else
 	return false;
