@@ -1,6 +1,6 @@
 /***************************************************************************
  * eval.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -49,6 +49,7 @@ EQ_POW,		// power x^y
 EQ_MOD,		// x modulo y
 EQ_LOG,		// logarithm of x on base a, log_a(x) = ln(x)/ln(a)
 EQ_ARG,		// argument of complex number arg(x,y) = atan2(x,y)
+EQ_HYPOT,	// sqrt(x^2+y^2)=hypot(x,y)
 // special functions of 2 arguments
 EQ_BESJ,		// regular cylindrical Bessel function of fractional order
 EQ_BESY,		// irregular cylindrical Bessel function of fractional order
@@ -118,7 +119,7 @@ EQ_DC,		// Jacobian elliptic function dn(u|m)/cn(u|m)
 			// MUST BE LAST ELLIPTIC FUNCTION
 // not-ready
 EQ_EN,
-EQ_CL,		// Clausen function
+EQ_CL		// Clausen function
 };
 //-----------------------------------------------------------------------------
 #ifndef M_PI
@@ -147,20 +148,36 @@ void MGL_EXPORT mgl_srnd(long seed)
 }
 void MGL_EXPORT mgl_srnd_(int *seed)	{	mgl_srnd(*seed);	}
 //-----------------------------------------------------------------------------
+double MGL_EXPORT mgl_hypot(double x, double y)	{	return hypot(x,y);	}
+//-----------------------------------------------------------------------------
+#if MGL_HAVE_PTHREAD
+pthread_mutex_t mutexRnd;
+#endif
 double MGL_EXPORT mgl_rnd()
 {
-#if MGL_HAVE_GSL
-	if(mgl_rng==0)
-	{
-		gsl_rng_env_setup();
-		mgl_rng = gsl_rng_alloc(gsl_rng_default);
-		gsl_rng_set(mgl_rng, time(0));
-	}
-	return gsl_rng_uniform(mgl_rng);
-//	gsl_rng_free(r);
-#else
-	return rand()/(RAND_MAX-1.);
+#if MGL_HAVE_PTHREAD
+	pthread_mutex_lock(&mutexRnd);
 #endif
+	double res;
+#pragma omp critical(rnd)
+	{
+#if MGL_HAVE_GSL
+		if(mgl_rng==0)
+		{
+			gsl_rng_env_setup();
+			mgl_rng = gsl_rng_alloc(gsl_rng_default);
+			gsl_rng_set(mgl_rng, time(0));
+		}
+		res = gsl_rng_uniform(mgl_rng);
+//		gsl_rng_free(r);
+#else
+		res = rand()/(RAND_MAX-1.);
+#endif
+	}
+#if MGL_HAVE_PTHREAD
+	pthread_mutex_unlock(&mutexRnd);
+#endif
+	return res;
 }
 double MGL_EXPORT mgl_rnd_()	{	return mgl_rnd();	}
 //-----------------------------------------------------------------------------
@@ -349,6 +366,7 @@ mglFormula::mglFormula(const char *string)
 			else if(!strcmp(name+1,"anh"))	Kod=EQ_TANH;
 			else if(!strcmp(name+1,"h"))	Kod=EQ_TANH;
 		}
+		else if(!strcmp(name,"hypot"))	Kod=EQ_HYPOT;
 		else if(!strcmp(name,"pow"))	Kod=EQ_POW;
 		else if(!strcmp(name,"mod"))	Kod=EQ_MOD;
 		else if(!strcmp(name,"i"))		Kod=EQ_BESI;
@@ -483,14 +501,14 @@ typedef double (*func_2)(double, double);
 // evaluation of embedded (included) expressions
 mreal mglFormula::CalcIn(const mreal *a1) const
 {
-	mreal z2[22] = {3,3,3,3,0,3,3,0,0,0,0,0,NAN,0
+	mreal z2[EQ_SIN-EQ_LT] = {3,3,3,3,0,3,3,0,0,0,0,0,NAN,0
 #if MGL_HAVE_GSL
 			,3,NAN, 3,NAN, 0,0,3,1
 #else
 			,0,0,0,0,0,0,0,0
 #endif
 		};
-	func_2 f2[22] = {clt,cgt,ceq,cor,cand,add,sub,mul,del,ipw,pow,fmod,llg,arg
+	func_2 f2[EQ_SIN-EQ_LT] = {clt,cgt,ceq,cor,cand,add,sub,mul,del,ipw,pow,fmod,llg,arg,hypot
 #if MGL_HAVE_GSL
 			,gsl_sf_bessel_Jnu,gsl_sf_bessel_Ynu,
 			gsl_sf_bessel_Inu,gsl_sf_bessel_Knu,
@@ -499,7 +517,7 @@ mreal mglFormula::CalcIn(const mreal *a1) const
 			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
 #endif
 		};
-	func_1 f1[42] = {sin,cos,tan,asin,acos,atan,sinh,cosh,tanh,
+	func_1 f1[EQ_SN-EQ_SIN] = {sin,cos,tan,asin,acos,atan,sinh,cosh,tanh,
 					asinh,acosh,atanh,sqrt,exp,log,log10,sgn,stp,floor,fabs
 #if MGL_HAVE_GSL
 			,gsl_sf_dilog,gslEllEc,gslEllFc,gslAi,gslBi,gsl_sf_erf,
@@ -519,21 +537,23 @@ mreal mglFormula::CalcIn(const mreal *a1) const
 		else	return (Kod==EQ_A) ? a1[int(Res)] : Res;
 	}
 	double a = Left->CalcIn(a1);
-	if(!mgl_isnan(a))
+	if(mgl_isfin(a))
 	{
 		if(Kod<EQ_SIN)
 		{
 			// try to bypass calc b if a==0
 			if(a==0 && z2[Kod-EQ_LT]!=3)	return z2[Kod-EQ_LT];
 			double b = Right->CalcIn(a1);
-			return !mgl_isnan(b) ? f2[Kod-EQ_LT](a,b) : NAN;
+			b = mgl_isfin(b) ? f2[Kod-EQ_LT](a,b):NAN;
+			return mgl_isfin(b) ? b : NAN;
 		}
-		else if(Kod<EQ_SN)	return f1[Kod-EQ_SIN](a);
+		else if(Kod<EQ_SN)
+		{	a = f1[Kod-EQ_SIN](a);	return mgl_isfin(a)?a:NAN;	}	
 #if MGL_HAVE_GSL
 		else if(Kod<=EQ_DC)
 		{
 			double sn=0,cn=0,dn=0,b = Right->CalcIn(a1);
-			if(mgl_isnan(b))	return NAN;
+			if(mgl_isbad(b))	return NAN;
 			gsl_sf_elljac_e(a,b, &sn, &cn, &dn);
 			switch(Kod)
 			{
@@ -605,21 +625,21 @@ double MGL_NO_EXPORT gamma_d(double a)	{return gsl_sf_psi(a)*gsl_sf_gamma(a);}
 // evaluation of derivative of embedded (included) expressions
 mreal mglFormula::CalcDIn(int id, const mreal *a1) const
 {
-	func_2 f21[22] = {mgz2,mgz2,mgz2, mgz2,mgz2,mgp, mgp,mul1,div1, ipw1,pow1,mgp,llg1, mgz2
+	func_2 f21[EQ_SIN-EQ_LT] = {mgz2,mgz2,mgz2, mgz2,mgz2,mgp, mgp,mul1,div1, ipw1,pow1,mgp,llg1, mgz2
 #if MGL_HAVE_GSL
 			,mgz2,mgz2,mgz2, mgz2,gslEllE1,gslEllF2, mgz2,mgz2
 #else
 			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
 #endif
 		};
-		func_2 f22[22] = {mgz2,mgz2,mgz2,mgz2,mgz2,mgp,mgm,mul2,div2,pow2,pow2,mgz2,llg2, mgz2
+		func_2 f22[EQ_SIN-EQ_LT] = {mgz2,mgz2,mgz2,mgz2,mgz2,mgp,mgm,mul2,div2,pow2,pow2,mgz2,llg2, mgz2
 #if MGL_HAVE_GSL
 			,gslJnuD,gslYnuD,gslInuD,gslKnuD,gslEllE2,gslEllF2,mgz2/*gslLegP*/,mgz2
 #else
 			,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2,mgz2
 #endif
 		};
-	func_1 f11[42] = {cos,cos_d,tan_d,asin_d,acos_d,atan_d,cosh,sinh,tanh_d,
+	func_1 f11[EQ_SN-EQ_SIN] = {cos,cos_d,tan_d,asin_d,acos_d,atan_d,cosh,sinh,tanh_d,
 					asinh_d,acosh_d,atanh_d,sqrt_d,exp,log_d,log10_d,mgz1,mgz1,mgz1,sgn
 #if MGL_HAVE_GSL
 			,dilog_d,gslE_d,gslK_d,gslAi_d,gslBi_d,erf_d,exp3_d,ei_d,e1_d,e2_d,
@@ -632,19 +652,21 @@ mreal mglFormula::CalcDIn(int id, const mreal *a1) const
 	if(Kod<EQ_LT)	return (Kod==EQ_A && id==(int)Res)?1:0;
 
 	double a = Left->CalcIn(a1), d = Left->CalcDIn(id,a1);
-	if(!mgl_isnan(a) && !mgl_isnan(d))
+	if(mgl_isfin(a) && mgl_isfin(d))
 	{
 		if(Kod<EQ_SIN)
 		{
 			double b = Right->CalcIn(a1), c = Right->CalcDIn(id,a1);
-			return !mgl_isnan(b) ? (d?f21[Kod-EQ_LT](a,b)*d:0) + (c?f22[Kod-EQ_LT](a,b)*c:0) : NAN;
+			b = mgl_isfin(b) ? (d?f21[Kod-EQ_LT](a,b)*d:0) + (c?f22[Kod-EQ_LT](a,b)*c:0) : NAN;
+			return mgl_isfin(b) ? b : NAN;
 		}
-		else if(Kod<EQ_SN)	return (d?f11[Kod-EQ_SIN](a)*d:0);
+		else if(Kod<EQ_SN)
+		{	a = (d?f11[Kod-EQ_SIN](a)*d:0);	return mgl_isfin(a)?a:NAN;	}
 #if MGL_HAVE_GSL
 		else if(Kod<=EQ_DC)
 		{
 			double sn=0,cn=0,dn=0,b = Right->CalcIn(a1);
-			if(mgl_isnan(b))	return NAN;
+			if(mgl_isbad(b))	return NAN;
 			gsl_sf_elljac_e(a,b, &sn, &cn, &dn);
 			switch(Kod)	// At this moment parse only differentiation or argument NOT mu !!!
 			{
@@ -693,3 +715,24 @@ int MGL_NO_EXPORT mglFindInText(char *str,const char *lst)
 	return -1;
 }
 //-----------------------------------------------------------------------------
+HMEX MGL_EXPORT mgl_create_expr(const char *expr)	{	return new mglFormula(expr);	}
+void MGL_EXPORT mgl_delete_expr(HMEX ex)	{	delete ex;	}
+double MGL_EXPORT mgl_expr_eval(HMEX ex, double x, double y,double z)
+{	return ex->Calc(x,y,z);	}
+double MGL_EXPORT mgl_expr_eval_v(HMEX ex, mreal *var)
+{	return ex->Calc(var);	}
+double MGL_EXPORT mgl_expr_diff(HMEX ex, char dir, double x, double y,double z)
+{	return ex->CalcD(dir,x,y,z);	}
+double MGL_EXPORT mgl_expr_diff_v(HMEX ex, char dir, mreal *var)
+{	return ex->CalcD(var, dir);		}
+//-----------------------------------------------------------------------------
+uintptr_t MGL_EXPORT mgl_create_expr_(const char *expr, int l)
+{	char *s=new char[l+1];	memcpy(s,expr,l);	s[l]=0;
+	uintptr_t res = uintptr_t(mgl_create_expr(s));
+	delete []s;	return res;	}
+void MGL_EXPORT mgl_delete_expr_(uintptr_t *ex)	{	mgl_delete_expr((HMEX)ex);	}
+double MGL_EXPORT mgl_expr_eval_(uintptr_t *ex, mreal *x, mreal *y, mreal *z)
+{	return mgl_expr_eval((HMEX) ex, *x,*y,*z);		}
+double MGL_EXPORT mgl_expr_diff_(uintptr_t *ex, const char *dir, mreal *x, mreal *y, mreal *z, int)
+{	return mgl_expr_diff((HMEX) ex, *dir,*x,*y,*z);	}
+//-----------------------------------------------------------------------------
