@@ -1,6 +1,6 @@
 /***************************************************************************
  * evalc.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -18,9 +18,8 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <time.h>
-#include "mgl2/data_cf.h"
+#include "mgl2/datac_cf.h"
 #include "mgl2/evalc.h"
-#include "mgl2/addon.h"
 #if MGL_HAVE_GSL
 #include <gsl/gsl_sf.h>
 #endif
@@ -56,7 +55,8 @@ EQ_EXP,		// exponential function \exp(x)
 EQ_EXPI,		// exponential function \exp(i*x)
 EQ_LN,		// logarithm of x, ln(x)
 EQ_LG,		// decimal logarithm of x, lg(x) = ln(x)/ln(10)
-EQ_ABS		// absolute value
+EQ_ABS,		// absolute value
+EQ_LAST		// id of last entry
 };
 //-----------------------------------------------------------------------------
 int mglFormulaC::Error=0;
@@ -238,8 +238,8 @@ typedef dual (*func_2)(dual, dual);
 // evaluation of embedded (included) expressions
 dual mglFormulaC::CalcIn(const dual *a1) const
 {
-	func_2 f2[7] = {addc,subc,mulc,divc,ipwc,powc,llgc};
-	func_1 f1[18] = {sinc,cosc,tanc,asinc,acosc,atanc,sinhc,coshc,tanhc,
+	func_2 f2[EQ_SIN-EQ_ADD] = {addc,subc,mulc,divc,ipwc,powc,llgc};
+	func_1 f1[EQ_LAST-EQ_SIN] = {sinc,cosc,tanc,asinc,acosc,atanc,sinhc,coshc,tanhc,
 					asinhc,acoshc,atanhc,sqrtc,expc,expi,logc,lgc,absc};
 //	if(Error)	return 0;
 	if(Kod==EQ_A)	return a1[(int)Res.real()];
@@ -247,16 +247,18 @@ dual mglFormulaC::CalcIn(const dual *a1) const
 	if(Kod==EQ_NUM) return Res;
 
 	dual a = Left->CalcIn(a1);
-	if(mgl_isnan(a.real()) || mgl_isnan(a.imag()))	return NAN;
-
-	if(Kod<EQ_SIN)
+	if(mgl_isfin(a))
 	{
-		dual b = Right->CalcIn(a1);
-		if(mgl_isnan(b.real()) || mgl_isnan(b.imag()))	return NAN;
-		return f2[Kod-EQ_ADD](a,b);
+		if(Kod<EQ_SIN)
+		{
+			dual b = Right->CalcIn(a1);
+			b = mgl_isfin(b)?f2[Kod-EQ_ADD](a,b):NAN;
+			return mgl_isfin(b)?b:NAN;
+		}
+		else
+		{	a = f1[Kod-EQ_SIN](a);	return mgl_isfin(a)?a:NAN;	}
 	}
-	else	return f1[Kod-EQ_SIN](a);
-	return Res;
+	return NAN;
 }
 //-----------------------------------------------------------------------------
 dual MGL_EXPORT mgl_ipowc(dual x,int n)
@@ -270,4 +272,19 @@ dual MGL_EXPORT mgl_ipowc(dual x,int n)
 	if(n%2==1)	t *= x;
 	return t;
 }
+dual MGL_EXPORT mgl_ipowc_(dual *x,int *n)	{	return mgl_ipowc(*x,*n);	}
+//-----------------------------------------------------------------------------
+HAEX MGL_EXPORT mgl_create_cexpr(const char *expr)	{	return new mglFormulaC(expr);	}
+uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int l)
+{	char *s=new char[l+1];	memcpy(s,expr,l);	s[l]=0;
+	uintptr_t res = uintptr_t(mgl_create_cexpr(s));
+	delete []s;	return res;	}
+void MGL_EXPORT mgl_delete_cexpr(HAEX ex)	{	delete ex;	}
+void MGL_EXPORT mgl_delete_cexpr_(uintptr_t *ex)	{	mgl_delete_cexpr((HAEX)ex);	}
+dual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z)
+{	return ex->Calc(x,y,z);	}
+dual MGL_EXPORT mgl_cexpr_eval_(uintptr_t *ex, dual *x, dual *y, dual *z)
+{	return mgl_cexpr_eval((HAEX) ex, *x,*y,*z);		}
+dual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *var)
+{	return ex->Calc(var);	}
 //-----------------------------------------------------------------------------
