@@ -1,6 +1,6 @@
 /***************************************************************************
  * export.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -198,17 +198,28 @@ int MGL_NO_EXPORT mgl_jpeg_save(const char *fname, int w, int h, unsigned char *
 #endif
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_printf(void *fp, bool gz, const char *str, ...)
+void MGL_NO_EXPORT mgl_printf(void *fp, bool gz, const char *str, ...)	// NOTE This function is not thread-safe
 {
-	char buf[512];
+	static char buf[1024];
 	va_list lst;
 	va_start(lst,str);
-	vsnprintf(buf,512,str,lst);
+	vsnprintf(buf,1023,str,lst);
 	va_end(lst);
 	if(gz)	gzprintf((gzFile)fp, "%s", buf);
 	else	fprintf((FILE *)fp, "%s", buf);
 }
 //---------------------------------------------------------------------------
+std::string MGL_NO_EXPORT mgl_sprintf(const char *str, ...)
+{
+	char *buf=new char[1024];
+	va_list lst;
+	va_start(lst,str);
+	vsnprintf(buf,1023,str,lst);
+	va_end(lst);
+	std::string res = buf;	delete []buf;
+	return res;
+}
+//---------------------------------------------------------------------------
 int MGL_NO_EXPORT mgl_bps_save(const char *fname, int w, int h, unsigned char **p)
 {
 	time_t now;	time(&now);
@@ -303,9 +314,9 @@ void mglCanvas::StartGIF(const char *fname, int ms)
 	// get picture sizes
 	// NOTE: you shouldn't call SetSize() after StartGIF() !!!
 	long width, height;
-	unsigned char *f=0;
-	GetRGBLines(width, height, f);
+	unsigned char *f=0, **l=GetRGBLines(width, height, f);
 	if(f)	free(f);
+	if(l)	free(l);
 	// define colormap
 	GifColorType col[256];
 	memset(col,0,256*sizeof(GifColorType));
@@ -367,11 +378,11 @@ void mglCanvas::EndFrame()
 	Finish();
 	if(get(MGL_VECT_FRAME))	PushDrwDat();
 #if MGL_HAVE_GIF
+	if(!gif)	return;
 	long width, height, n;
-	unsigned char *f=0, **l=0;
-	l = GetRGBLines(width, height, f);
+	unsigned char *f=0, **l=GetRGBLines(width, height, f);
 	n = width*height;
-	if(!l || !gif)	return;
+	if(!l)	return;
 	EGifPutImageDesc(gif, 0, 0, width, height, 0, 0);
 	GifPixelType *line = new GifPixelType[n];
 	register long m;
@@ -387,8 +398,8 @@ void mglCanvas::EndFrame()
 	EGifPutLine(gif, line, n);
 	delete []line;	free(l);
 	if(f)	free(f);
-#else
-	mglGlobalMess += "GIF support was disabled. Please, enable it and rebuild MathGL.\n";
+//#else
+//	mglGlobalMess += "GIF support was disabled. Please, enable it and rebuild MathGL.\n";
 #endif
 }
 //-----------------------------------------------------------------------------
@@ -396,9 +407,11 @@ void mglCanvas::DelFrame(long i)
 {
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_lock(&mutexDrw);
+#pragma omp critical(drw)
 	if(get(MGL_VECT_FRAME))	DrwDat.erase(DrwDat.begin()+i);
 	pthread_mutex_unlock(&mutexDrw);
 #else
+#pragma omp critical(drw)
 	if(get(MGL_VECT_FRAME))	DrwDat.erase(DrwDat.begin()+i);
 #endif
 	CurFrameId--;
@@ -542,24 +555,29 @@ void MGL_EXPORT mgl_write_frame(HMGL gr, const char *fname,const char *descr)
 	if(!fname || !fname[0])
 	{	snprintf(buf,64,"%s%04d.jpg",_Gr_->PlotId.c_str(),_Gr_->GetNumFrame());	fname = buf;	}
 	int len=strlen(fname);
-	if(!strcmp(fname+len-4,".jpg"))	mgl_write_jpg(gr,fname,descr);
-	if(!strcmp(fname+len-5,".jpeg"))mgl_write_jpg(gr,fname,descr);
-	if(!strcmp(fname+len-4,".prc")) mgl_write_prc(gr,fname,descr,1);
-	if(!strcmp(fname+len-4,".pdf")) mgl_write_prc(gr,fname,descr,1);
-	if(!strcmp(fname+len-4,".png"))	mgl_write_png(gr,fname,descr);
-	if(!strcmp(fname+len-4,".eps"))	mgl_write_eps(gr,fname,descr);
-	if(!strcmp(fname+len-4,".svg"))	mgl_write_svg(gr,fname,descr);
-	if(!strcmp(fname+len-4,".gif"))	mgl_write_gif(gr,fname,descr);
-	if(!strcmp(fname+len-4,".bmp"))	mgl_write_bmp(gr,fname,descr);
-	if(!strcmp(fname+len-4,".tga"))	mgl_write_tga(gr,fname,descr);
-	if(!strcmp(fname+len-5,".mgld"))mgl_export_mgld(gr,fname,descr);
-	if(!strcmp(fname+len-4,".json")) mgl_write_json(gr,fname,descr);
-	if(!strcmp(fname+len-4,".obj")) mgl_write_obj(gr,fname,descr,1);
-	if(!strcmp(fname+len-4,".tex")) mgl_write_tex(gr,fname,descr);
-	if(!strcmp(fname+len-4,".xyz")) mgl_write_xyz(gr,fname,descr);
-	if(!strcmp(fname+len-4,".stl")) mgl_write_stl(gr,fname,descr);
-	if(!strcmp(fname+len-4,".off")) mgl_write_off(gr,fname,descr,0);
-//	if(!strcmp(fname+len-4,".x3d")) mgl_write_x3d(gr,fname,descr,1);
+	if(!strcmp(fname+len-4,".jpg")) 	mgl_write_jpg(gr,fname,descr);
+	if(!strcmp(fname+len-5,".jpeg"))	mgl_write_jpg(gr,fname,descr);
+	if(!strcmp(fname+len-4,".prc")) 	mgl_write_prc(gr,fname,descr,1);
+	if(!strcmp(fname+len-4,".pdf")) 	mgl_write_prc(gr,fname,descr,1);
+	if(!strcmp(fname+len-4,".png")) 	mgl_write_png(gr,fname,descr);
+	if(!strcmp(fname+len-4,".eps")) 	mgl_write_eps(gr,fname,descr);
+	if(!strcmp(fname+len-5,".epsz"))	mgl_write_eps(gr,fname,descr);
+	if(!strcmp(fname+len-4,".bps")) 	mgl_write_bps(gr,fname,descr);
+	if(!strcmp(fname+len-5,".bpsz"))	mgl_write_bps(gr,fname,descr);
+	if(!strcmp(fname+len-4,".svg")) 	mgl_write_svg(gr,fname,descr);
+	if(!strcmp(fname+len-5,".svgz"))	mgl_write_svg(gr,fname,descr);
+	if(!strcmp(fname+len-4,".gif")) 	mgl_write_gif(gr,fname,descr);
+	if(!strcmp(fname+len-4,".bmp")) 	mgl_write_bmp(gr,fname,descr);
+	if(!strcmp(fname+len-4,".tga")) 	mgl_write_tga(gr,fname,descr);
+	if(!strcmp(fname+len-5,".mgld"))	mgl_export_mgld(gr,fname,descr);
+	if(!strcmp(fname+len-5,".json"))	mgl_write_json(gr,fname,descr);
+	if(!strcmp(fname+len-6,".jsonz"))	mgl_write_json(gr,fname,descr);
+	if(!strcmp(fname+len-4,".obj")) 	mgl_write_obj(gr,fname,descr,1);
+	if(!strcmp(fname+len-4,".tex")) 	mgl_write_tex(gr,fname,descr);
+	if(!strcmp(fname+len-4,".xyz")) 	mgl_write_xyz(gr,fname,descr);
+	if(!strcmp(fname+len-4,".stl")) 	mgl_write_stl(gr,fname,descr);
+	if(!strcmp(fname+len-4,".off")) 	mgl_write_off(gr,fname,descr,0);
+//	if(!strcmp(fname+len-4,".x3d")) 	mgl_write_x3d(gr,fname,descr,1);
 }
 void MGL_EXPORT mgl_write_frame_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
