@@ -1,6 +1,6 @@
 /***************************************************************************
  * export_2d.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -52,17 +52,17 @@ MGL_NO_EXPORT const char *mgl_get_dash(unsigned short d, mreal w)
 	return s.c_str();
 }
 //-----------------------------------------------------------------------------
-bool MGL_NO_EXPORT mgl_is_same(HMGL gr, const mglPrim &pr,mreal wp,mglColor cp,int st)
+bool MGL_NO_EXPORT mgl_is_same(HMGL gr, long i, mreal wp,uint32_t cp, int st)
 {
+	const mglPrim &pr=_Gr_->GetPrm(i);
 	if(abs(pr.type)!=1)	return false;
 	if(pr.w>=1 && wp!=pr.w)	return false;
 	if(pr.w<1 && wp!=1)	return false;
-	if(st!=pr.n3)			return false;
-	mglColor c=_Gr_->GetColor(pr);
-	return (cp==c);
+	if(st!=pr.n3)	return false;
+	return (cp==_Gr_->GetPrmCol(i));
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT put_line(HMGL gr, void *fp, bool gz, long i, mreal wp, mglColor cp,int st, const char *ifmt, const char *nfmt, bool neg, mreal fc)
+void MGL_NO_EXPORT put_line(HMGL gr, void *fp, bool gz, long i, mreal wp, uint32_t cp,int st, const char *ifmt, const char *nfmt, bool neg, mreal fc)
 {
 	long n1=gr->GetPrm(i).n1, n2=gr->GetPrm(i).n2;
 	if(n1>n2)	{	n1=gr->GetPrm(i).n2;	n2=gr->GetPrm(i).n1;	}
@@ -78,7 +78,7 @@ void MGL_NO_EXPORT put_line(HMGL gr, void *fp, bool gz, long i, mreal wp, mglCol
 		{
 			mglPrim &q = gr->GetPrm(j);
 			if(q.type>1)	break;
-			if(mgl_is_same(gr, q,wp,cp,st) && q.type==1 && q.n1>=0 && q.n2>=0)	// previous point
+			if(mgl_is_same(gr, j, wp,cp,st) && q.type==1 && q.n1>=0 && q.n2>=0)	// previous point
 			{
 				const mglPnt &p1 = gr->GetPnt(q.n1);
 				const mglPnt &p2 = gr->GetPnt(q.n2);
@@ -104,7 +104,7 @@ void MGL_NO_EXPORT put_line(HMGL gr, void *fp, bool gz, long i, mreal wp, mglCol
 		{
 			mglPrim &q = gr->GetPrm(j);
 			if(q.type>1)	break;
-			if(mgl_is_same(gr, q,wp,cp,st) && q.type==1 && q.n1>=0 && q.n2>=0)	// next point
+			if(mgl_is_same(gr, j,wp,cp,st) && q.type==1 && q.n1>=0 && q.n2>=0)	// next point
 			{
 				const mglPnt &p1 = gr->GetPnt(q.n1);
 				const mglPnt &p2 = gr->GetPnt(q.n2);
@@ -169,44 +169,11 @@ void MGL_NO_EXPORT put_desc(HMGL gr, void *fp, bool gz, const char *pre, const c
 	delete []g;		delete []s;
 }
 //-----------------------------------------------------------------------------
-mglColor mglCanvas::GetColor(const mglPrim &p)
-{
-	unsigned char res[4],buf[4];
-	mglPnt pp=Pnt[p.type==1?p.n2:p.n1];
-	if(p.type==4)	pp.u=pp.v=NAN;
-	col2int(pp,res,p.id);
-	if(p.type==2 || p.type==3)
-	{
-		col2int(Pnt[p.n2],buf,p.id);	res[0]=(1L*res[0]+buf[0])/2;
-		res[1]=(1L*res[1]+buf[1])/2;	res[2]=(1L*res[2]+buf[2])/2;
-		col2int(Pnt[p.n3],buf,p.id);	res[0]=(2L*res[0]+buf[0])/3;
-		res[1]=(2L*res[1]+buf[1])/3;	res[2]=(2L*res[2]+buf[2])/3;
-	}
-	if(p.type==3)
-	{
-		col2int(Pnt[p.n4],buf,p.id);	res[0]=(3L*res[0]+buf[0])/4;
-		res[1]=(3L*res[1]+buf[1])/4;	res[2]=(3L*res[2]+buf[2])/4;
-	}
-	if(p.type==6)
-	{	res[0]=p.n2&0xff;	res[1]=(p.n2/256)&0xff;	res[2]=(p.n2/65536)&0xff;	}
-	
-	// add fog into resulting color
-	float zf = FogDist*(p.z/Depth-0.5-FogDz);
-	if(zf<0)	// add fog
-	{
-		int d = int(255.f-255.f*exp(5.f*zf));
-		unsigned char cb[4] = {BDef[0], BDef[1], BDef[2], (unsigned char)d};
-		if(d<255)	combine(res,cb);
-	}
-
-	return mglColor(res[0]/255.,res[1]/255.,res[2]/255.,res[3]/255.);
-}
-//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_write_eps(HMGL gr, const char *fname,const char *descr)
 {
 	if(!fname || *fname==0)	return;
 	if(gr->GetPrmNum()<1)	return;
-	_Gr_->clr(MGL_FINISHED);	_Gr_->Finish(false);
+	_Gr_->clr(MGL_FINISHED);	_Gr_->PreparePrim(1);
 	time_t now;	time(&now);
 
 	bool gz = fname[strlen(fname)-1]=='z';
@@ -214,6 +181,7 @@ void MGL_EXPORT mgl_write_eps(HMGL gr, const char *fname,const char *descr)
 	if(!strcmp(fname,"-"))	fp = stdout;		// allow to write in stdout
 	else		fp = gz ? (void*)gzopen(fname,"wt") : (void*)fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	mgl_printf(fp, gz, "%%!PS-Adobe-3.0 EPSF-3.0\n%%%%BoundingBox: 0 0 %d %d\n", _Gr_->GetWidth(), _Gr_->GetHeight());
 	mgl_printf(fp, gz, "%%%%Created by MathGL library\n%%%%Title: %s\n",descr ? descr : fname);
 	mgl_printf(fp, gz, "%%%%CreationDate: %s\n",ctime(&now));
@@ -277,21 +245,21 @@ void MGL_EXPORT mgl_write_eps(HMGL gr, const char *fname,const char *descr)
 	// write primitives
 	mreal wp=-1;
 	float qs_old=gr->mark_size()/gr->FontFactor();
-	mglColor cp;
+	mglRGBA cp;
 	int st=0;
 	char str[256]="";
 	for(i=0;i<gr->GetPrmNum();i++)
 	{
 		const mglPrim &q = gr->GetPrm(i);
 		if(q.type<0)	continue;	// q.n1>=0 always
-		cp = _Gr_->GetColor(q);
+		cp.c = _Gr_->GetPrmCol(i);
 		const mglPnt p1 = gr->GetPnt(q.n1);
-		if(q.type>1)	snprintf(str,256,"%.2g %.2g %.2g rgb ", cp.r,cp.g,cp.b);
+		if(q.type>1)	snprintf(str,256,"%.2g %.2g %.2g rgb ", cp.r[0]/255.,cp.r[1]/255.,cp.r[2]/255.);
 
 		if(q.type==0)	// mark
 		{
 			mreal x0 = p1.x,y0 = p1.y;
-			snprintf(str,256,"%.2g lw %.2g %.2g %.2g rgb ", 50*q.s*q.w>1?50*q.s*q.w:1, cp.r,cp.g,cp.b);
+			snprintf(str,256,"%.2g lw %.2g %.2g %.2g rgb ", 50*q.s*q.w>1?50*q.s*q.w:1, cp.r[0]/255.,cp.r[1]/255.,cp.r[2]/255.);
 			wp=1;	// NOTE: this may renew line style if a mark inside!
 			if(q.s!=qs_old)
 			{
@@ -329,22 +297,18 @@ void MGL_EXPORT mgl_write_eps(HMGL gr, const char *fname,const char *descr)
 		else if(q.type==3)	// quad
 		{
 			const mglPnt &p2=gr->GetPnt(q.n2), &p3=gr->GetPnt(q.n3), &p4=gr->GetPnt(q.n4);
-			if(p1.a>mgl_min_a || p2.a>mgl_min_a || p3.a>mgl_min_a || p4.a>mgl_min_a)
-				mgl_printf(fp, gz, "np %g %g mt %g %g ll %g %g ll %g %g ll cp %sfill\n",
-					   p1.x, p1.y, p2.x, p2.y, p4.x, p4.y, p3.x, p3.y, str);
+			if(cp.r[3])	mgl_printf(fp, gz, "np %g %g mt %g %g ll %g %g ll %g %g ll cp %sfill\n", p1.x, p1.y, p2.x, p2.y, p4.x, p4.y, p3.x, p3.y, str);
 		}
 		else if(q.type==2)	// trig
 		{
 			const mglPnt &p2=gr->GetPnt(q.n2), &p3=gr->GetPnt(q.n3);
-			if(p1.a>mgl_min_a || p2.a>mgl_min_a || p3.a>mgl_min_a)
-				mgl_printf(fp, gz, "np %g %g mt %g %g ll %g %g ll cp %sfill\n",
-					   p1.x, p1.y, p2.x, p2.y, p3.x, p3.y, str);
+			if(cp.r[3])	mgl_printf(fp, gz, "np %g %g mt %g %g ll %g %g ll cp %sfill\n", p1.x, p1.y, p2.x, p2.y, p3.x, p3.y, str);
 		}
 		else if(q.type==1)	// line
 		{
-			snprintf(str,256,"%.2g lw %.2g %.2g %.2g rgb ", q.w>1 ? q.w:1., cp.r,cp.g,cp.b);
+			snprintf(str,256,"%.2g lw %.2g %.2g %.2g rgb ", q.w>1 ? q.w:1., cp.r[0]/255.,cp.r[1]/255.,cp.r[2]/255.);
 			wp = q.w>1  ? q.w:1;	st = q.n3;
-			put_line(gr,fp,gz,i,wp,cp,st, "np %g %g mt ", "%g %g ll ", false, 1);
+			put_line(gr,fp,gz,i,wp,cp.c,st, "np %g %g mt ", "%g %g ll ", false, 1);
 			const char *sd = mgl_get_dash(q.n3,q.w);
 			if(sd && sd[0])	mgl_printf(fp, gz, "%s [%s] %g sd dr\n",str,sd,q.w*q.s);
 			else			mgl_printf(fp, gz, "%s d0 dr\n",str);
@@ -377,6 +341,7 @@ void MGL_EXPORT mgl_write_eps(HMGL gr, const char *fname,const char *descr)
 	}
 	mgl_printf(fp, gz, "\nshowpage\n%%%%EOF\n");
 	if(strcmp(fname,"-"))	{	if(gz)	gzclose((gzFile)fp);	else	fclose((FILE *)fp);	}
+	setlocale(LC_NUMERIC, "");
 }
 void MGL_EXPORT mgl_write_eps_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
@@ -387,7 +352,7 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 {
 	if(!fname || *fname==0)	return;
 	if(gr->GetPrmNum()<1)	return;
-	_Gr_->clr(MGL_FINISHED);	_Gr_->Finish(false);
+	_Gr_->clr(MGL_FINISHED);	_Gr_->PreparePrim(1);
 	time_t now;	time(&now);
 
 	bool gz = fname[strlen(fname)-1]=='z';
@@ -396,6 +361,7 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 	if(!strcmp(fname,"-"))	fp = stdout;		// allow to write in stdout
 	else		fp = gz ? (void*)gzopen(fname,"wt") : (void*)fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	mgl_printf(fp, gz, "<?xml version=\"1.0\" standalone=\"no\"?>\n");
 	mgl_printf(fp, gz, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20000303 Stylable//EN\" \"http://www.w3.org/TR/2000/03/WD-SVG-20000303/DTD/svg-20000303-stylable.dtd\">\n");
 	mgl_printf(fp, gz, "<svg width=\"%d\" height=\"%d\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n", _Gr_->GetWidth(), hh);
@@ -412,13 +378,13 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 	mreal wp=-1;
 	register long i;
 	int st=0;
-	mglColor cp;
+	mglRGBA cp;
 
 	for(i=0;i<gr->GetPrmNum();i++)
 	{
 		const mglPrim &q = gr->GetPrm(i);
 		if(q.type<0)	continue;	// q.n1>=0 always
-		cp = _Gr_->GetColor(q);
+		cp.c = _Gr_->GetPrmCol(i);
 		const mglPnt p1=gr->GetPnt(q.n1);
 		if(q.type==0)
 		{
@@ -426,9 +392,9 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 			if(!strchr("xsSoO",q.n4))	s *= 1.1;
 			wp = 1;
 			if(strchr("SDVTLR",q.n4))
-				mgl_printf(fp, gz, "<g fill=\"#%02x%02x%02x\">\n", int(255*cp.r),int(255*cp.g),int(255*cp.b));
+				mgl_printf(fp, gz, "<g fill=\"#%02x%02x%02x\">\n", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
 			else
-				mgl_printf(fp, gz, "<g stroke=\"#%02x%02x%02x\"  stroke-width=\"%g\">\n", int(255*cp.r),int(255*cp.g),int(255*cp.b), 50*q.s*q.w>1?50*q.s*q.w:1);
+				mgl_printf(fp, gz, "<g stroke=\"#%02x%02x%02x\"  stroke-width=\"%g\">\n", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]), 50*q.s*q.w>1?50*q.s*q.w:1);
 			switch(q.n4)
 			{
 			case 'P':
@@ -463,24 +429,24 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 				mgl_printf(fp, gz, "<path d=\"M %g %g L %g %g L %g %g M %g %g L %g %g\"/>\n", x,y-s, x,y, x+s,y+s, x,y, x-s,y+s);	break;
 			case 'C':
 				mgl_printf(fp, gz, "<circle style=\"fill:#%02x%02x%02x\" cx=\"%g\" cy=\"%g\" r=\"0.15\"/>\n<circle cx=\"%g\" cy=\"%g\" r=\"%g\"/>\n",
-							int(255*cp.r),int(255*cp.g),int(255*cp.b),x,y,x,y,s);	break;
+							int(cp.r[0]),int(cp.r[1]),int(cp.r[2]),x,y,x,y,s);	break;
 			case 'o':
 				mgl_printf(fp, gz, "<circle cx=\"%g\" cy=\"%g\" r=\"%g\"/>\n", x,y,s);	break;
 			case 'O':
 				mgl_printf(fp, gz, "<circle style=\"fill:#%02x%02x%02x\" cx=\"%g\" cy=\"%g\" r=\"%g\"/>\n",
-							int(255*cp.r),int(255*cp.g),int(255*cp.b),x,y,s);	break;
+							int(cp.r[0]),int(cp.r[1]),int(cp.r[2]),x,y,s);	break;
 			case '*':
 				mgl_printf(fp, gz, "<path d=\"M %g %g L %g %g M %g %g L %g %g M %g %g L %g %g\"/>\n",
 							x-s,y,x+s,y,x-0.6*s,y-0.8*s,x+0.6*s,y+0.8*s,x+0.6*s,y-0.8*s,x-0.6*s,y+0.8*s);	break;
 			default:
 				mgl_printf(fp, gz, "<circle style=\"fill:#%02x%02x%02x\" cx=\"%g\" cy=\"%g\" r=\"0.15\"/>\n",
-							int(255*cp.r),int(255*cp.g),int(255*cp.b),x,y);	break;
+							int(cp.r[0]),int(cp.r[1]),int(cp.r[2]),x,y);	break;
 			}
 			mgl_printf(fp, gz, "</g>\n");
 		}
 		else if(q.type==1)
 		{
-			mgl_printf(fp,gz,"<g stroke=\"#%02x%02x%02x\"",int(255*cp.r),int(255*cp.g),int(255*cp.b));
+			mgl_printf(fp,gz,"<g stroke=\"#%02x%02x%02x\"",int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
 			if(q.n3)
 			{
 				mgl_printf(fp, gz, " stroke-dasharray=\"%s\"", mgl_get_dash(q.n3,q.w));
@@ -488,19 +454,19 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 			}
 			if(q.w>1)	mgl_printf(fp, gz, " stroke-width=\"%g\"", q.w);
 			wp = q.w>1  ? q.w:1;	st = q.n3;
-			put_line(gr,fp,gz,i,wp,cp,st, "><path d=\" M %g %g", " L %g %g", true, 1);
+			put_line(gr,fp,gz,i,wp,cp.c,st, "><path d=\" M %g %g", " L %g %g", true, 1);
 			mgl_printf(fp, gz, "\"/> </g>\n");
 		}
-		else if(q.type==2 && cp.a>mgl_min_a)
+		else if(q.type==2 && cp.r[3])
 		{
 			const mglPnt &p2=gr->GetPnt(q.n2), &p3=gr->GetPnt(q.n3);
-			mgl_printf(fp, gz, "<g fill=\"#%02x%02x%02x\" opacity=\"%g\">\n", int(255*cp.r),int(255*cp.g),int(255*cp.b),cp.a);
+			mgl_printf(fp, gz, "<g fill=\"#%02x%02x%02x\" opacity=\"%g\">\n", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]),cp.r[3]/255.);
 			mgl_printf(fp, gz, "<path d=\"M %g %g L %g %g L %g %g Z\"/> </g>\n", p1.x, hh-p1.y, p2.x, hh-p2.y, p3.x, hh-p3.y);
 		}
-		else if(q.type==3 && cp.a>mgl_min_a)
+		else if(q.type==3 && cp.r[3])
 		{
 			const mglPnt &p2=gr->GetPnt(q.n2), &p3=gr->GetPnt(q.n3), &p4=gr->GetPnt(q.n4);
-			mgl_printf(fp, gz, "<g fill=\"#%02x%02x%02x\" opacity=\"%g\">\n", int(255*cp.r),int(255*cp.g),int(255*cp.b),cp.a);
+			mgl_printf(fp, gz, "<g fill=\"#%02x%02x%02x\" opacity=\"%g\">\n", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]),cp.r[3]/255.);
 			mgl_printf(fp, gz, "<path d=\"M %g %g L %g %g L %g %g L %g %g Z\"/> </g>\n", p1.x, hh-p1.y, p2.x, hh-p2.y, p4.x, hh-p4.y, p3.x, hh-p3.y);
 		}
 		else if(q.type==4)
@@ -512,9 +478,9 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 			{
 				mgl_printf(fp, gz, "<g transform=\"translate(%g,%g) scale(%.3g,%.3g) rotate(%g)\"", p1.x, hh-p1.y, ss, -ss, -phi);
 				if(q.n3&4)
-					mgl_printf(fp, gz, " stroke=\"#%02x%02x%02x\">", int(255*cp.r),int(255*cp.g),int(255*cp.b));
+					mgl_printf(fp, gz, " stroke=\"#%02x%02x%02x\">", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
 				else
-					mgl_printf(fp, gz, " fill=\"#%02x%02x%02x\">", int(255*cp.r),int(255*cp.g),int(255*cp.b));
+					mgl_printf(fp, gz, " fill=\"#%02x%02x%02x\">", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
 				mreal dy = 0.004,f=fabs(zz);
 				mgl_printf(fp, gz, "<path d=\"M %g %g L %g %g L %g %g L %g %g\"/></g>\n", xx,yy+dy, xx+f,yy+dy, xx+f,yy-dy, xx,yy-dy);
 			}
@@ -523,9 +489,9 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 				ss *= zz;
 				mgl_printf(fp, gz, "<g transform=\"translate(%g,%g) scale(%.3g,%.3g) rotate(%g)\"", p1.x, hh-p1.y, ss, -ss, -q.w);
 				if(q.n3&4)
-					mgl_printf(fp, gz, " stroke=\"#%02x%02x%02x\">", int(255*cp.r),int(255*cp.g),int(255*cp.b));
+					mgl_printf(fp, gz, " stroke=\"#%02x%02x%02x\">", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
 				else
-					mgl_printf(fp, gz, " fill=\"#%02x%02x%02x\">", int(255*cp.r),int(255*cp.g),int(255*cp.b));
+					mgl_printf(fp, gz, " fill=\"#%02x%02x%02x\">", int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
 				mgl_printf(fp, gz, "<use x=\"%g\" y=\"%g\" xlink:href=\"#%c%c_%04x\"/></g>\n", xx/zz, yy/zz, q.n3&1?'b':'n', q.n3&2?'i':'n', q.n4);
 			}
 		}
@@ -535,219 +501,23 @@ void MGL_EXPORT mgl_write_svg(HMGL gr, const char *fname,const char *descr)
 	{	mglPrim &q=gr->GetPrm(i);	if(q.type==-1)	q.type = 1;	}
 	mgl_printf(fp, gz, "</g></svg>");
 	if(strcmp(fname,"-"))	{	if(gz)	gzclose((gzFile)fp);	else	fclose((FILE *)fp);	}
+	setlocale(LC_NUMERIC, "");
 }
 void MGL_EXPORT mgl_write_svg_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
 	char *d=new char[n+1];	memcpy(d,descr,n);	d[n]=0;
 	mgl_write_svg(_GR_,s,d);	delete []s;		delete []d;	}
 //-----------------------------------------------------------------------------
-/// Color names easely parsed by LaTeX
-struct mglSVGName	{	const char *name;	mreal r,g,b;	};
-MGL_NO_EXPORT mglSVGName mgl_names[]={{"AliceBlue",.94,.972,1},
-{"Apricot", 0.984, 0.725, 0.51},
-{"Aquamarine", 0, 0.71, 0.745},
-{"Bittersweet", 0.753, 0.31, 0.0902},
-{"Black", 0.133, 0.118, 0.122},
-{"Blue", 0.176, 0.184, 0.573},
-{"BlueGreen", 0, 0.702, 0.722},
-{"BlueViolet", 0.278, 0.224, 0.573},
-{"BrickRed", 0.714, 0.196, 0.11},
-{"Brown", 0.475, 0.145, 0},
-{"BurntOrange", 0.969, 0.573, 0.114},
-{"CadetBlue", 0.455, 0.447, 0.604},
-{"CarnationPink", 0.949, 0.51, 0.706},
-{"Cerulean", 0, 0.635, 0.89},
-{"CornflowerBlue", 0.255, 0.69, 0.894},
-{"Cyan", 0, 0.682, 0.937},
-{"Dandelion", 0.992, 0.737, 0.259},
-{"DarkOrchid", 0.643, 0.325, 0.541},
-{"Emerald", 0, 0.663, 0.616},
-{"ForestGreen", 0, 0.608, 0.333},
-{"Fuchsia", 0.549, 0.212, 0.549},
-{"Goldenrod", 1, 0.875, 0.259},
-{"Gray", 0.58, 0.588, 0.596},
-{"Green", 0, 0.651, 0.31},
-{"GreenYellow", 0.875, 0.902, 0.455},
-{"JungleGreen", 0, 0.663, 0.604},
-{"Lavender", 0.957, 0.62, 0.769},
-{"LimeGreen", 0.553, 0.78, 0.243},
-{"Magenta", 0.925, 0, 0.549},
-{"Mahogany", 0.663, 0.204, 0.122},
-{"Maroon", 0.686, 0.196, 0.208},
-{"Melon", 0.973, 0.62, 0.482},
-{"MidnightBlue", 0, 0.404, 0.584},
-{"Mulberry", 0.663, 0.235, 0.576},
-{"NavyBlue", 0, 0.431, 0.722},
-{"OliveGreen", 0.235, 0.502, 0.192},
-{"Orange", 0.961, 0.506, 0.216},
-{"OrangeRed", 0.929, 0.0745, 0.353},
-{"Orchid", 0.686, 0.447, 0.69},
-{"Peach", 0.969, 0.588, 0.353},
-{"Periwinkle", 0.475, 0.467, 0.722},
-{"PineGreen", 0, 0.545, 0.447},
-{"Plum", 0.573, 0.149, 0.561},
-{"ProcessBlue", 0, 0.69, 0.941},
-{"Purple", 0.6, 0.278, 0.608},
-{"RawSienna", 0.592, 0.251, 0.0235},
-{"Red", 0.929, 0.106, 0.137},
-{"RedOrange", 0.949, 0.376, 0.208},
-{"RedViolet", 0.631, 0.141, 0.42},
-{"Rhodamine", 0.937, 0.333, 0.624},
-{"RoyalBlue", 0, 0.443, 0.737},
-{"RoyalPurple", 0.38, 0.247, 0.6},
-{"RubineRed", 0.929, 0.00392, 0.49},
-{"Salmon", 0.965, 0.573, 0.537},
-{"SeaGreen", 0.247, 0.737, 0.616},
-{"Sepia", 0.404, 0.0941, 0},
-{"SkyBlue", 0.275, 0.773, 0.867},
-{"SpringGreen", 0.776, 0.863, 0.404},
-{"Tan", 0.855, 0.616, 0.463},
-{"TealBlue", 0, 0.682, 0.702},
-{"Thistle", 0.847, 0.514, 0.718},
-{"Turquoise", 0, 0.706, 0.808},
-{"Violet", 0.345, 0.259, 0.608},
-{"VioletRed", 0.937, 0.345, 0.627},
-{"White", 0.6, 0.6, 0.6},
-{"WildStrawberry", 0.933, 0.161, 0.404},
-{"Yellow", 1, 0.949, 0},
-{"YellowGreen", 0.596, 0.8, 0.439},
-{"YellowOrange", 0.98, 0.635, 0.102},
-{"white", 1,1,1},
-{"black", 0,0,0},
-{"red", 1,0,0},
-{"green", 0,1,0},
-{"blue", 0,0,1},
-{"cyan", 0,1,1},
-{"magenta", 1,0,1},
-{"yellow", 1,1,0},
-{"",-1,-1,-1}};
-//-----------------------------------------------------------------------------
-MGL_NO_EXPORT const char *mglColorName(mglColor c)	// return closest SVG color
-{
-	register long i;
-	register mreal d, dm=10;
-	const char *name="";
-	for(i=0;mgl_names[i].name[0];i++)
-	{
-		d = fabs(c.r-mgl_names[i].r)+fabs(c.g-mgl_names[i].g)+fabs(c.b-mgl_names[i].b);
-		if(d<dm)	{	dm=d;	name=mgl_names[i].name;	}
-	}
-	return name;
-}
-//-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_write_tex(HMGL gr, const char *fname,const char *descr)
 {
 	if(gr->GetPrmNum()<1)	return;
-	_Gr_->clr(MGL_FINISHED);	_Gr_->Finish(false);
+	_Gr_->clr(MGL_FINISHED);	_Gr_->PreparePrim(1);
 
 	FILE *fp = fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	fprintf(fp, "%% Created by MathGL library\n%% Title: %s\n\n",descr?descr:fname);
-	fprintf(fp, "\\begin{tikzpicture}\n");
-
-	// write primitives first
-	mreal wp=-1;
-	register long i;
-	register int ii,jj,kk;
-	int st=0;
-	mglColor cp;
-	char cname[16];
-
-	for(i=0;i<gr->GetPrmNum();i++)
-	{
-		const mglPrim &q = gr->GetPrm(i);
-		if(q.type<0)	continue;	// q.n1>=0 always
-		cp = _Gr_->GetColor(q);
-
-		ii = (cp.r*255+25)/51;
-		jj = (cp.g*255+25)/51;
-		kk = (cp.b*255+25)/51;
-		snprintf(cname,16,"mgl_%d",ii+6*(jj+6*kk));
-//		cname = mglColorName(cp);
-		const mglPnt p1=gr->GetPnt(q.n1);
-		mreal x=p1.x/100,y=p1.y/100,s=q.s/100;
-		if(q.type==0)
-		{
-			if(!strchr("xsSoO",q.n4))	s *= 1.1;
-			wp = 1;
-			switch(q.n4)	// NOTE: no thickness for marks in TeX
-			{
-				case 'P':
-					fprintf(fp, "\\mglp{%g}{%g}{%s}{%g} \\mgls{%g}{%g}{%s}{%g}\n", x,y,cname,s,x,y,cname,s);	break;
-				case 'X':
-					fprintf(fp, "\\mglx{%g}{%g}{%s}{%g} \\mgls{%g}{%g}{%s}{%g}\n", x,y,cname,s,x,y,cname,s);	break;
-				case 'C':
-					fprintf(fp, "\\mglc{%g}{%g}{%s}{%g} \\mglo{%g}{%g}{%s}{%g}\n", x,y,cname,s,x,y,cname,s);	break;
-				case '+':	fprintf(fp, "\\mglp{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'x':	fprintf(fp, "\\mglx{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 's':	fprintf(fp, "\\mgls{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'S':	fprintf(fp, "\\mglS{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'd':	fprintf(fp, "\\mgld{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'D':	fprintf(fp, "\\mglD{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case '^':	fprintf(fp, "\\mglt{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'T':	fprintf(fp, "\\mglT{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'v':	fprintf(fp, "\\mglv{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'V':	fprintf(fp, "\\mglV{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case '<':	fprintf(fp, "\\mgll{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'L':	fprintf(fp, "\\mglL{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case '>':	fprintf(fp, "\\mglr{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'R':	fprintf(fp, "\\mglR{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'Y':	fprintf(fp, "\\mglY{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'o':	fprintf(fp, "\\mglo{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case 'O':	fprintf(fp, "\\mglO{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				case '*':	fprintf(fp, "\\mgla{%g}{%g}{%s}{%g}\n", x,y,cname,s);	break;
-				default:	fprintf(fp, "\\mglc{%g}{%g}{%s}\n", x,y,cname);	break;
-			}
-		}
-		else if(q.type==2 && cp.a>mgl_min_a)
-		{
-			const mglPnt p2=gr->GetPnt(q.n2), p3=gr->GetPnt(q.n3);
-			fprintf(fp, "\\fill[%s, fill opacity=%g] (%g,%g) -- (%g,%g) -- (%g,%g) -- cycle;\n", cname,cp.a, x,y, p2.x/100,p2.y/100, p3.x/100,p3.y/100);
-		}
-		else if(q.type==3 && cp.a>mgl_min_a)
-		{
-			const mglPnt p2=gr->GetPnt(q.n2), p3=gr->GetPnt(q.n3), p4=gr->GetPnt(q.n4);
-			fprintf(fp, "\\fill[%s, fill opacity=%g] (%g,%g) -- (%g,%g) -- (%g,%g) -- (%g,%g) -- cycle;\n", cname,cp.a, x,y, p2.x/100,p2.y/100, p4.x/100,p4.y/100, p3.x/100,p3.y/100);
-		}
-		else if(q.type==1)	// lines
-		{
-			//const char *dash[]={"", "8 8","4 4","1 3","7 4 1 4","3 2 1 2"};
-			const char *w[]={"semithick","thick","very thick","ultra thick"};
-			register int iw=int(q.w-0.5);	if(iw>3)	iw=3;
-			if(iw<0)	fprintf(fp,"\\draw[%s] ",cname);
-			else		fprintf(fp,"\\draw[%s,%s] ",cname,w[iw]);
-			// TODO: add line dashing
-			wp = q.w>1  ? q.w:1;	st = q.n3;
-			put_line(gr,fp,false,i,wp,cp,st, "(%g,%g)", " -- (%g,%g)", false, 0.01);
-			fprintf(fp, ";\n");
-		}
-		else if(q.type==6 && !mgl_isnan(q.p))	// text
-		{
-			const mglText &t = gr->GetPtx(q.n3);
-			mreal dy = q.w*cos(q.p*M_PI/180)/100, dx = q.w*sin(q.p*M_PI/180)/100;
-			int f,a;	mglGetStyle(t.stl.c_str(), &f, &a);
-			std::string ss=cname;
-			if((a&3)==0)	ss.append(",anchor=base west");
-			if((a&3)==1)	ss.append(",anchor=base");
-			if((a&3)==2)	ss.append(",anchor=base east");
-			if(f&MGL_FONT_ITAL)	ss.append(",font=\\itshape");
-			if(f&MGL_FONT_BOLD)	ss.append(",font=\\bfshape");
-			if(t.text.find('\\')!=std::string::npos || t.text.find('{')!=std::string::npos || t.text.find('_')!=std::string::npos || t.text.find('^')!=std::string::npos)
-				fprintf(fp,"\\draw[%s] (%g,%g) node[rotate=%.2g]{$%ls$};\n", ss.c_str(),x-dx,y-dy, -q.p, t.text.c_str());
-			else
-				fprintf(fp,"\\draw[%s] (%g,%g) node[rotate=%.2g]{%ls};\n", ss.c_str(),x-dx,y-dy, -q.p, t.text.c_str());
-		}
-	}
-	fprintf(fp, "\\end{tikzpicture}\n");
-	for(i=0;i<gr->GetPrmNum();i++)
-	{	mglPrim &q=gr->GetPrm(i);	if(q.type==-1)	q.type = 1;	}
-	fclose(fp);
-
-	// provide colors used by figure
-	fp=fopen("mglcolors.tex","wt");
-	for(ii=0;ii<6;ii++)	for(jj=0;jj<6;jj++)	for(kk=0;kk<6;kk++)
-		fprintf(fp,"\\definecolor{mgl_%d}{RGB}{%d,%d,%d}\n",ii+6*(jj+6*kk),51*ii,51*jj,51*kk);
-	mreal ms=0.4*gr->mark_size()/100;	// also provide marks
+	// provide marks
 	fprintf(fp, "\\providecommand{\\mglp}[4]{\\draw[#3] (#1-#4, #2) -- (#1+#4,#2) (#1,#2-#4) -- (#1,#2+#4);}\n");
 	fprintf(fp, "\\providecommand{\\mglx}[4]{\\draw[#3] (#1-#4, #2-#4) -- (#1+#4,#2+#4) (#1+#4,#2-#4) -- (#1-#4,#2+#4);}\n");
 	fprintf(fp, "\\providecommand{\\mgls}[4]{\\draw[#3] (#1-#4, #2-#4) -- (#1+#4,#2-#4) -- (#1+#4,#2+#4) -- (#1-#4,#2+#4) -- cycle;}\n");
@@ -767,15 +537,121 @@ void MGL_EXPORT mgl_write_tex(HMGL gr, const char *fname,const char *descr)
 	fprintf(fp, "\\providecommand{\\mglY}[4]{\\draw[#3] (#1, #2-#4) -- (#1,#2) (#1-#4,#2+#4) -- (#1,#2) (#1+#4,#2+#4) -- (#1,#2);}\n");
 	fprintf(fp, "\\providecommand{\\mglo}[4]{\\draw[#3] (#1, #2) circle (#4);}\n");
 	fprintf(fp, "\\providecommand{\\mglO}[4]{\\fill[#3] (#1, #2) circle (#4);}\n");
-	fprintf(fp, "\\providecommand{\\mglc}[3]{\\draw[#3] (#1, #2) circle (%g);}\n\n", 0.1*ms);
+	fprintf(fp, "\\providecommand{\\mglc}[3]{\\draw[#3] (#1, #2) circle (%g);}\n\n", 4e-4*gr->mark_size());
+	fprintf(fp, "\\begin{tikzpicture}\n");
+
+	// write primitives first
+	mreal wp=-1;
+	int st=0;
+	mglRGBA cp;
+	char cname[128];
+
+	for(long i=0;i<gr->GetPrmNum();i++)
+	{
+		const mglPrim &q = gr->GetPrm(i);
+		if(q.type<0)	continue;	// q.n1>=0 always
+		cp.c = _Gr_->GetPrmCol(i);
+		snprintf(cname,128,"color={rgb,255:red,%d;green,%d;blue,%d}",cp.r[0],cp.r[1],cp.r[2]);
+
+		const mglPnt p1=gr->GetPnt(q.n1);
+		mreal x=p1.x/100,y=p1.y/100,s=q.s/100;
+		if(q.type==0)
+		{
+			if(!strchr("xsSoO",q.n4))	s *= 1.1;
+			wp = 1;
+			switch(q.n4)	// NOTE: no thickness for marks in TeX
+			{
+				case 'P':
+					fprintf(fp, "\\mglp{%.4g}{%.4g}{%s}{%.4g} \\mgls{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s,x,y,cname,s);	break;
+				case 'X':
+					fprintf(fp, "\\mglx{%.4g}{%.4g}{%s}{%.4g} \\mgls{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s,x,y,cname,s);	break;
+				case 'C':
+					fprintf(fp, "\\mglc{%.4g}{%.4g}{%s}{%.4g} \\mglo{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s,x,y,cname,s);	break;
+				case '+':	fprintf(fp, "\\mglp{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'x':	fprintf(fp, "\\mglx{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 's':	fprintf(fp, "\\mgls{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'S':	fprintf(fp, "\\mglS{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'd':	fprintf(fp, "\\mgld{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'D':	fprintf(fp, "\\mglD{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case '^':	fprintf(fp, "\\mglt{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'T':	fprintf(fp, "\\mglT{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'v':	fprintf(fp, "\\mglv{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'V':	fprintf(fp, "\\mglV{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case '<':	fprintf(fp, "\\mgll{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'L':	fprintf(fp, "\\mglL{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case '>':	fprintf(fp, "\\mglr{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'R':	fprintf(fp, "\\mglR{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'Y':	fprintf(fp, "\\mglY{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'o':	fprintf(fp, "\\mglo{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case 'O':	fprintf(fp, "\\mglO{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				case '*':	fprintf(fp, "\\mgla{%.4g}{%.4g}{%s}{%.4g}\n", x,y,cname,s);	break;
+				default:	fprintf(fp, "\\mglc{%.4g}{%.4g}{%s}\n", x,y,cname);	break;
+			}
+		}
+		else if(q.type==2 && cp.r[3])
+		{
+			const mglPnt p2=gr->GetPnt(q.n2), p3=gr->GetPnt(q.n3);
+			if(cp.r[3]<255)
+				fprintf(fp, "\\fill[%s, fill opacity=%.4g] (%.4g,%.4g) -- (%.4g,%.4g) -- (%.4g,%.4g) -- cycle;\n", cname,cp.r[3]/255., x,y, p2.x/100,p2.y/100, p3.x/100,p3.y/100);
+			else
+				fprintf(fp, "\\fill[%s, fill] (%.4g,%.4g) -- (%.4g,%.4g) -- (%.4g,%.4g) -- cycle;\n", cname, x,y, p2.x/100,p2.y/100, p3.x/100,p3.y/100);
+		}
+		else if(q.type==3 && cp.r[3])
+		{
+			const mglPnt p2=gr->GetPnt(q.n2), p3=gr->GetPnt(q.n3), p4=gr->GetPnt(q.n4);
+			if(cp.r[3]<255)
+				fprintf(fp, "\\fill[%s, fill opacity=%.4g] (%.4g,%.4g) -- (%.4g,%.4g) -- (%.4g,%.4g) -- (%.4g,%.4g) -- cycle;\n", cname,cp.r[3]/255., x,y, p2.x/100,p2.y/100, p4.x/100,p4.y/100, p3.x/100,p3.y/100);
+			else
+				fprintf(fp, "\\fill[%s, fill] (%.4g,%.4g) -- (%.4g,%.4g) -- (%.4g,%.4g) -- (%.4g,%.4g) -- cycle;\n", cname, x,y, p2.x/100,p2.y/100, p4.x/100,p4.y/100, p3.x/100,p3.y/100);
+
+		}
+		else if(q.type==1)	// lines
+		{
+			//const char *dash[]={"", "8 8","4 4","1 3","7 4 1 4","3 2 1 2"};
+			const char *w[]={"semithick","thick","very thick","ultra thick"};
+			register int iw=int(q.w-0.5);	if(iw>3)	iw=3;
+			if(iw<0)	fprintf(fp,"\\draw[%s] ",cname);
+			else		fprintf(fp,"\\draw[%s,%s] ",cname,w[iw]);
+			// TODO: add line dashing
+			wp = q.w>1  ? q.w:1;	st = q.n3;
+			put_line(gr,fp,false,i,wp,cp.c,st, "(%.4g,%.4g)", " -- (%.4g,%.4g)", false, 0.01);
+			fprintf(fp, ";\n");
+		}
+		else if(q.type==6 && mgl_isnum(q.p))	// text
+		{
+			const mglText &t = gr->GetPtx(q.n3);
+			mreal dy = q.w*cos(q.p*M_PI/180)/100, dx = q.w*sin(q.p*M_PI/180)/100;
+			int f,a;	mglGetStyle(t.stl.c_str(), &f, &a);
+			std::string ss=cname;
+			if((a&3)==0)	ss.append(",anchor=base west");
+			if((a&3)==1)	ss.append(",anchor=base");
+			if((a&3)==2)	ss.append(",anchor=base east");
+//			if(f&MGL_FONT_ITAL)	ss.append(",font=\\itshape");
+//			if(f&MGL_FONT_BOLD)	ss.append(",font=\\bfshape");
+			if(t.text.find('\\')!=std::string::npos || t.text.find('{')!=std::string::npos || t.text.find('_')!=std::string::npos || t.text.find('^')!=std::string::npos)
+				fprintf(fp,"\\draw[%s] (%.4g,%.4g) node[rotate=%.2g]{$%ls$};\n", ss.c_str(),x-dx,y-dy, -q.p, t.text.c_str());
+			else
+				fprintf(fp,"\\draw[%s] (%.4g,%.4g) node[rotate=%.2g]{%ls};\n", ss.c_str(),x-dx,y-dy, -q.p, t.text.c_str());
+		}
+	}
+	fprintf(fp, "\\end{tikzpicture}\n");
+	for(long i=0;i<gr->GetPrmNum();i++)
+	{	mglPrim &q=gr->GetPrm(i);	if(q.type==-1)	q.type = 1;	}
 	fclose(fp);
+	setlocale(LC_NUMERIC, "");
 
 	// provide main file for viewing figure
 	fp=fopen("mglmain.tex","wt");
-	fprintf(fp, "\\documentclass{article}\n\n");
-	fprintf(fp, "%% following lines should be placed before \\begin{document}\n");
-	fprintf(fp,"\\usepackage{tikz}\n\\input{mglcolors.tex}\n");
-	fprintf(fp, "\\begin{document}\n%% start figure itself\n\\input{%s}\\end{document}\n",fname);
-	fclose(fp);
+	if(fp)
+	{
+		fprintf(fp, "%% this file just show figure\n");
+		fprintf(fp, "\\documentclass{article}\n\\usepackage{tikz}\n");
+		fprintf(fp, "\\begin{document}\n\\input{%s}\n\\end{document}\n",fname);
+		fclose(fp);
+	}
 }
+void MGL_EXPORT mgl_write_tex_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
+{	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
+	char *d=new char[n+1];	memcpy(d,descr,n);	d[n]=0;
+	mgl_write_tex(_GR_,s,d);	delete []s;		delete []d;	}
 //-----------------------------------------------------------------------------
