@@ -1,6 +1,6 @@
 /***************************************************************************
  * export_3d.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -18,6 +18,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <time.h>
+#include <stdarg.h>
 #include "mgl2/canvas.h"
 #include "mgl2/canvas_cf.h"
 #undef _GR_
@@ -26,6 +27,7 @@
 int MGL_NO_EXPORT mgl_tga_save(const char *fname, int w, int h, unsigned char **p);
 int MGL_NO_EXPORT mgl_pnga_save(const char *fname, int w, int h, unsigned char **p);
 void MGL_NO_EXPORT mgl_printf(void *fp, bool gz, const char *str, ...);
+std::string MGL_NO_EXPORT mgl_sprintf(const char *str, ...);
 //-----------------------------------------------------------------------------
 void mglTexture::GetRGBA(unsigned char *f) const
 {
@@ -50,6 +52,7 @@ void MGL_EXPORT mgl_obj_glyph_old(HMGL gr, const mglPrim &q, const mglPnt &p, FI
 {
 	mreal f = q.p/2, dx=p.u/2, dy=p.v/2, x,y;
 	mreal c=q.s*cos(q.w*M_PI/180), s=-q.s*sin(q.w*M_PI/180);
+	if(mgl_isnan(q.s))	c=s=0;
 	double b[4] = {c,-s, s,c};
 	long i=q.n1+1, ik,il=0;
 
@@ -260,14 +263,14 @@ void MGL_EXPORT mgl_obj_prim_old(HMGL gr, const mglPrim &q, const mglPnt &p, FIL
 			fprintf(fp,"l -1/%ld -3/%ld\n", i,i);	break;
 		case 'O':
 			for(j=0;j<=20;j++)
-				fprintf(fp,"v %g %g %g\n",p.x+ss*cos(j*M_PI/10),p.y+ss*sin(j*M_PI/10),p.z);
+				fprintf(fp,"v %g %g %g\n",p.x+ss*mgl_cos[(j*36)%360],p.y+ss*mgl_cos[(270+j*36)%360],p.z);
 			for(j=0;j<20;j++)
 				fprintf(fp,"f %ld/%ld %ld/%ld %ld/%ld\n", j-21,i, j-20,i, i,i);
 			break;
 		case 'C':	fprintf(fp,"p %ld\n", i);
 		case 'o':
 			for(j=0;j<=20;j++)
-				fprintf(fp,"v %g %g %g\n",p.x+ss*cos(j*M_PI/10),p.y+ss*sin(j*M_PI/10),p.z);
+				fprintf(fp,"v %g %g %g\n",p.x+ss*mgl_cos[(j*36)%360],p.y+ss*mgl_cos[(270+j*36)%360],p.z);
 			for(j=0;j<20;j++)
 				fprintf(fp,"l %ld/%ld %ld/%ld\n", j-21,i, j-20,i);
 			break;
@@ -284,13 +287,12 @@ void MGL_EXPORT mgl_obj_prim_old(HMGL gr, const mglPrim &q, const mglPnt &p, FIL
 void MGL_EXPORT mgl_write_obj_old(HMGL gr, const char *fname,const char *descr, int use_png)
 {
 	if(gr->GetPrmNum()==0)	return;	// nothing to do
-	register size_t i,j;
-	long m1=0,m2=0,m;
-	for(i=0;i<gr->Grp.size();i++)	// prepare array of indirect indexing
+	long m1=0,m2=0,m,j;
+	for(size_t i=0;i<gr->Grp.size();i++)	// prepare array of indirect indexing
 	{	m = gr->Grp[i].Id;	if(m<m1) m1=m;	if(m>m2) m2=m;	}
 	long *ng = new long[m2-m1+1];
-	for(i=0;i<gr->Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
-	for(i=0;i<size_t(gr->GetPrmNum());i++)	// collect data for groups
+	for(size_t i=0;i<gr->Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
+	for(long i=0;i<gr->GetPrmNum();i++)	// collect data for groups
 	// it is rather expensive (extra 4b per primitive) but need for export to 3D
 	{
 		m = gr->GetPrm(i).id-m1;
@@ -299,11 +301,12 @@ void MGL_EXPORT mgl_write_obj_old(HMGL gr, const char *fname,const char *descr,
 	delete []ng;
 
 	size_t len=strlen(fname),ntxt=gr->GetTxtNum();
-	char *tname = new char[len+5];	strcpy(tname,fname);
 	FILE *fp=fopen(fname,"wt");
+	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
 	// vertices definition
+	setlocale(LC_NUMERIC, "C");
 	fprintf(fp,"# Created by MathGL library\n# Title: %s\n",(descr && *descr) ? descr : fname);
-	for(i=0;i<size_t(gr->GetPntNum());i++)
+	for(long i=0;i<gr->GetPntNum();i++)
 	{
 		const mglPnt &pp = gr->GetPnt(i);
 		fprintf(fp,"v %g %g %g\n",pp.x,pp.y,pp.z);
@@ -312,15 +315,16 @@ void MGL_EXPORT mgl_write_obj_old(HMGL gr, const char *fname,const char *descr,
 //		else fprintf(fp,"vn %g %g %g\n",pp.u,pp.v,pp.w);
 	}
 	// primitive definition in groups
+	char *tname = new char[len+5];	strcpy(tname,fname);
 	tname[len-4]=0;	fprintf(fp,"# Primitives Definitions\nmtllib %s.mtl\nusemtl %s\n",tname,tname);
-	for(i=0;i<gr->Grp.size();i++)
+	for(size_t i=0;i<gr->Grp.size();i++)
 	{
 		fprintf(fp,"g %s\n",gr->Grp[i].Lbl.c_str());
 		std::vector<long> &p = gr->Grp[i].p;
-		for(j=0;j<p.size();j++)
+		for(size_t j=0;j<p.size();j++)
 		{
 			const mglPrim &q=gr->GetPrm(p[j]);
-			mgl_obj_prim_old(gr, q, gr->GetPnt(q.n1), fp, q.s);
+			mgl_obj_prim_old(gr, q, gr->GetPnt(q.n1), fp, mgl_isnan(q.s)?0:q.s);
 		}
 		gr->Grp[i].p.clear();	// we don't need indexes anymore
 	}
@@ -344,11 +348,12 @@ void MGL_EXPORT mgl_write_obj_old(HMGL gr, const char *fname,const char *descr,
 	j=gr->GetTxtNum();
 	unsigned char *buf = new unsigned char[4*256*256*j];
 	unsigned char **pbuf= (unsigned char **)malloc(256*j*sizeof(unsigned char *));
-	for(i=0;i<256*j;i++)	pbuf[i] = buf+4*256*i;
-	for(i=0;i<j;i++)	gr->GetTxt(i).GetRGBA(buf+i*256*256*4);
+	for(long i=0;i<256*j;i++)	pbuf[i] = buf+4*256*i;
+	for(long i=0;i<j;i++)	gr->GetTxt(i).GetRGBA(buf+i*256*256*4);
 	if(use_png)	mgl_pnga_save(tname,256,256*j,pbuf);
 	else		mgl_tga_save(tname,256,256*j,pbuf);
 	free(pbuf);	delete []buf;	delete []tname;
+	setlocale(LC_NUMERIC, "");
 }
 void MGL_EXPORT mgl_write_obj_old_(uintptr_t *gr, const char *fname,const char *descr, int *use_png,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
@@ -359,6 +364,8 @@ void MGL_EXPORT mgl_write_stl(HMGL gr, const char *fname,const char *descr)
 {
 	if(gr->GetPrmNum()==0)	return;	// nothing to do
 	FILE *fp = fopen(fname,"wt");
+	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	fprintf(fp,"solid %s\n",(descr && *descr)?descr:"mathgl");
 	register long i;
 	mglPnt pp;
@@ -399,6 +406,7 @@ void MGL_EXPORT mgl_write_stl(HMGL gr, const char *fname,const char *descr)
 	}
 	fprintf(fp,"endsolid %s",(descr && *descr)?descr:"mathgl");
 	fclose(fp);
+	setlocale(LC_NUMERIC, "");
 }
 void MGL_EXPORT mgl_write_stl_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
@@ -411,6 +419,8 @@ void MGL_EXPORT mgl_write_xyz(HMGL gr, const char *fname,const char *descr)
 
 	register long i;
 	FILE *fp=fopen(fname,"wt"), *ff;	// vertices definition
+	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	fprintf(fp,"# Created by MathGL library\n# Title: %s\n",(descr && *descr) ? descr : fname);
 	fprintf(fp,"# List of Vertices, with (x,y,z) coordinates.\n");
 	for(i=0;i<gr->GetPntNum();i++)
@@ -437,6 +447,7 @@ void MGL_EXPORT mgl_write_xyz(HMGL gr, const char *fname,const char *descr)
 		if(q.type==3)	fprintf(ff,"%ld %ld %ld\n%ld %ld %ld\n",q.n1+1,q.n2+1,q.n3+1,q.n4+1,q.n2+1,q.n3+1);
 	}
 	fclose(fp);	fclose(ff);	delete []tname;
+	setlocale(LC_NUMERIC, "");
 }
 void MGL_EXPORT mgl_write_xyz_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
@@ -454,6 +465,8 @@ void MGL_EXPORT mgl_write_off(HMGL gr, const char *fname,const char *descr, int
 	if(nf<=0)	return;	// nothing to do
 
 	FILE *fp=fopen(fname,"wt");
+	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	// vertices definition
 	if(colored)
 		fprintf(fp,"COFF\n# Created by MathGL library\n# Title: %s\n",(descr && *descr) ? descr : fname);
@@ -498,35 +511,68 @@ void MGL_EXPORT mgl_write_off(HMGL gr, const char *fname,const char *descr, int
 		}
 	}
 	fclose(fp);
+	setlocale(LC_NUMERIC, "");
 }
 void MGL_EXPORT mgl_write_off_(uintptr_t *gr, const char *fname,const char *descr,int *colored,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
 	char *d=new char[n+1];	memcpy(d,descr,n);	d[n]=0;
 	mgl_write_off(_GR_,s,d,*colored);	delete []s;		delete []d;	}
 //-----------------------------------------------------------------------------
-bool mglCanvas::WriteJSON(const char *fname)
+bool mglCanvas::WriteJSON(const char *fname, bool force_zlib)
 {
 	bool fl = strcmp(fname,"-");
-	bool gz = fname[strlen(fname)-1]=='z';
+	bool gz = force_zlib || fname[strlen(fname)-1]=='z';
 	void *fp = fl ? (gz ? (void*)gzopen(fname,"wt") : (void*)fopen(fname,"wt")) : stdout;
 	if (!fp)	return true;
+	std::string s=GetJSON();
+	if(gz)	gzprintf((gzFile)fp, "%s", s.c_str());
+	else	fprintf((FILE *)fp, "%s", s.c_str());
+	if(fl)	{	if(gz)	gzclose((gzFile)fp);	else	fclose((FILE *)fp);	}
+	return false;
+}
+//-----------------------------------------------------------------------------
+MGL_EXPORT const char *mgl_get_json(HMGL gr)
+{
+	static std::string json;
+	mglCanvas *g = dynamic_cast<mglCanvas *>(gr);
+	if(g)	json = g->GetJSON();
+	return json.c_str();
+}
+//-----------------------------------------------------------------------------
+std::string mglCanvas::GetJSON()
+{
 	ClearUnused();	// clear unused points
-	size_t i,l=Pnt.size();
-	mgl_printf(fp, gz,"{\n\"width\":%d,\t\"height\":%d,\t\"depth\":%d,\t\"plotid\":\"%s\",\t\"npnts\":%lu,\t\"pnts\":[\n",
-			Width, Height, Depth, PlotId.c_str(), (unsigned long)l);
-	for(i=0;i<l;i++)
+	std::string res, buf;
+	long i,ll=0,l=(long)Pnt.size();
+	long factor = Width>1?10:10000;
+	setlocale(LC_NUMERIC, "C");
+	res = res + mgl_sprintf("{\n\"width\":%d,\t\"height\":%d,\t\"depth\":%d,\t\"plotid\":\"%s\",\t\"npnts\":%ld,\t\"pnts\":[\n",
+			factor*Width, factor*Height, factor*Depth, PlotId.c_str(), l);
+	std::string *tmp=new std::string[l];
+#pragma omp parallel for reduction(+:ll)
+	for(long i=0;i<l;i++)
 	{
 		const mglPnt &q=Pnt[i];
-//		fprintf(fp,"[%.4g,%.4g,%.4g]%c\n", q.xx, Height-q.yy, q.zz, i+1<l?',':' ');
-		mgl_printf(fp, gz,"[%d,%d,%d]%c\n", int(q.xx), int(Height-q.yy), int(q.zz), i+1<l?',':' ');
+		tmp[i] = mgl_sprintf("[%ld,%ld,%ld,%d]%c\n", long(factor*q.xx), long(factor*(Height-q.yy)), long(factor*q.zz),q.sub, i+1<l?',':' ');
+		ll += tmp[i].length();
 	}
-	l = Prm.size();
-	mgl_printf(fp, gz,"],\t\"nprim\":%lu,\t\"prim\":[\n",(unsigned long)l);
+	res.reserve(ll);
+	for(i=0;i<l;i++)	res = res + tmp[i];
+	delete []tmp;
+
+	l = (long)Prm.size();
+	PreparePrim(3);
+	for(ll=i=0;i<l;i++)
+	{	mglRGBA c;	c.c = GetPrmCol(i);	if(c.r[3])	ll++;	}
+
+	res = res + mgl_sprintf("],\t\"nprim\":%ld,\t\"prim\":[\n",ll+1);
 
 	std::vector<mglPoint> xy;	// vector for glyphs coordinates (to be separated from pnts)
-	for(i=0;i<l;i++)
+	res.reserve(60*(ll+1));
+#pragma omp parallel for private(buf)
+	for(long i=0;i<l;i++)
 	{
-		const mglPrim &p=Prm[i];		mglColor c = GetColor(p);
+		const mglPrim &p=Prm[i];	mglRGBA cp;	cp.c = GetPrmCol(i);
 		if(p.n1<0 || (p.type==1 && p.n2<0) || (p.type==2 && (p.n2<0 || p.n3<0)) || (p.type==3 && (p.n2<0 || p.n3<0 || p.n4<0)))
 			continue;
 		long n1=p.n1, n2=p.n2, n3=0, n4=(p.type==3||p.type==0)?p.n4:0;
@@ -534,52 +580,49 @@ bool mglCanvas::WriteJSON(const char *fname)
 		if(p.type==4)
 		{
 			const mglPnt &q = Pnt[p.n1];
-			xy.push_back(mglPoint(q.u,q.v,p.n2));
-			n2 = xy.size()-1;
+#pragma omp critical
+			{n2 = xy.size();	xy.push_back(mglPoint(q.u,q.v,p.n2));}
 			n3 = p.n3;	n4 = p.n4;
 		}
 		if(p.type==1 && n1>n2)	{	n1=p.n2;	n2=p.n1;	}
-		if(c.a==1 || p.type==0 || p.type==1 || p.type==4 || p.type==6)
-//			fprintf(fp,"[%d,%ld,%ld,%ld,%ld,%d,%.4g,%.4g,%.4g,%.4g,\"#%02x%02x%02x\"]%c\n",
-			mgl_printf(fp, gz,"[%d,%ld,%ld,%ld,%ld,%d,%.3g,%.2g,%.2g,%.2g,\"#%02x%02x%02x\"]%c\n",
-				p.type, n1, n2, n3, n4, p.id, p.s, p.w==p.w?p.w:0, p.p==p.p?p.p:0,
-				0., int(255*c.r), int(255*c.g), int(255*c.b), i+1<l?',':' ');
-		else
-//			fprintf(fp,"[%d,%ld,%ld,%ld,%ld,%d,%.4g,%.4g,%.4g,%.4g,\"rgba(%d,%d,%d,%.2g)\"]%c\n",
-			mgl_printf(fp, gz,"[%d,%ld,%ld,%ld,%ld,%d,%.3g,%.2g,%.2g,%.2g,\"rgba(%d,%d,%d,%.2g)\"]%c\n",
-				p.type, n1, n2, n3, n4, p.id, p.s, p.w==p.w?p.w:0, p.p==p.p?p.p:0,
-				0., int(255*c.r), int(255*c.g), int(255*c.b), c.a, i+1<l?',':' ');
+		register long ps=p.s==p.s?long(100*factor*p.s):0, pw=p.w==p.w?long(100*p.w):0, pp=p.p==p.p?long(1e5*p.p+0.5):0;
+		if(cp.r[3]==255 || p.type==0 || p.type==1 || p.type==4 || p.type==6)
+			buf = mgl_sprintf("[%d,%ld,%ld,%ld,%ld,%d,%ld,%ld,%ld,0,\"#%02x%02x%02x\"],\n",
+				p.type, n1, n2, n3, n4, p.id, ps,pw,pp, int(cp.r[0]),int(cp.r[1]),int(cp.r[2]));
+		else if(cp.r[3])
+			buf = mgl_sprintf("[%d,%ld,%ld,%ld,%ld,%d,%ld,%ld,%ld,0,\"rgba(%d,%d,%d,%.2g)\"],\n",
+				p.type, n1, n2, n3, n4, p.id, ps,pw,pp, int(cp.r[0]),int(cp.r[1]),int(cp.r[2]),cp.r[3]/255.);
+		else	buf = "";
+#pragma omp critical
+		res += buf;
 	}
-	
-	l = xy.size();
-	mgl_printf(fp, gz,"],\t\"ncoor\":%lu,\t\"coor\":[\n",(unsigned long)l);
+	res += "[-1,0,0,0,0,0,0,0,0,0,\"#000000\"]\n";	// need to add this empty block
+
+	l = (long)xy.size();
+	res = res + mgl_sprintf("],\t\"ncoor\":%lu,\t\"coor\":[\n",(unsigned long)l);
 	for(i=0;i<l;i++)
 	{
 		const mglPoint &p=xy[i];
 		const mglPnt &q=Pnt[int(0.5+p.z)];
-		if(q.u==q.u)
-			mgl_printf(fp, gz,"[%.3g,%.3g,%.3g,%.3g,%.3g]%c\n", p.x, p.y, q.u, q.v, q.w, i+1<l?',':' ');
-//			fprintf(fp,"[%.4g,%.4g,%.4g,%.4g,%.4g]%c\n", p.x, p.y, q.u, q.v, q.w, i+1<l?',':' ');
+		register long px=long(100*p.x), py=long(100*p.y);
+		if(q.u==q.u && q.v==q.v && q.w==q.w)
+			res = res + mgl_sprintf("[%ld,%ld,%ld,%ld,%ld]%c\n", px, py, long(100*q.u), long(100*q.v), long(100*q.w), i+1<l?',':' ');
 		else
-//			fprintf(fp,"[%.4g,%.4g,1e11,1e11,1e11]%c\n", p.x, p.y, i+1<l?',':' ');
-			mgl_printf(fp, gz,"[%.2g,%.2g,1e11,1e11,1e11]%c\n", p.x, p.y, i+1<l?',':' ');
+			res = res + mgl_sprintf("[%ld,%ld,1e11,1e11,1e11]%c\n", px, py, i+1<l?',':' ');
 	}
 
-	l = Glf.size();
-	mgl_printf(fp, gz,"],\t\"nglfs\":%lu,\t\"glfs\":[\n",(unsigned long)l);
+	l = (long)Glf.size();
+	res = res + mgl_sprintf("],\t\"nglfs\":%lu,\t\"glfs\":[\n",(unsigned long)l);
 	for(i=0;i<l;i++)
 	{
 		const mglGlyph &g=Glf[i];
-		mgl_printf(fp, gz,"[%ld,%ld,\n\t[", g.nt, g.nl);
-		register long j;
-		for(j=0;j<6*g.nt;j++)	mgl_printf(fp, gz,"%d%c", g.trig[j], j+1<6*g.nt?',':' ');
-		mgl_printf(fp, gz,"],\n\t[");
-		for(j=0;j<2*g.nl;j++)	mgl_printf(fp, gz,"%d%c", g.line[j], j+1<2*g.nl?',':' ');
-		mgl_printf(fp, gz,"]\n]%c\n", i+1<l?',':' ');
+		res = res + mgl_sprintf("[%ld,\n\t[", g.nl);
+		for(long j=0;j<2*g.nl;j++)	res = res + mgl_sprintf("%d%c", g.line[j], j+1<2*g.nl?',':' ');
+		res = res + mgl_sprintf("]\n]%c\n", i+1<l?',':' ');
 	}
-	mgl_printf(fp, gz,"]\n}\n");
-	if(fl)	{	if(gz)	gzclose((gzFile)fp);	else	fclose((FILE *)fp);	}
-	return false;
+	res = res + mgl_sprintf("]\n}\n");
+	setlocale(LC_NUMERIC, "");
+	return res;
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_write_json(HMGL gr, const char *fname,const char *)
@@ -589,34 +632,41 @@ void MGL_EXPORT mgl_write_json_(uintptr_t *gr, const char *fname,const char *des
 	char *f=new char[n+1];	memcpy(f,descr,n);	f[n]=0;
 	mgl_write_json(_GR_,s,f);	delete []s;		delete []f;	}
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_write_json_z(HMGL gr, const char *fname,const char *)
+{	_Gr_->WriteJSON(fname,true);	}
+void MGL_EXPORT mgl_write_json_z_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
+{	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
+	char *f=new char[n+1];	memcpy(f,descr,n);	f[n]=0;
+	mgl_write_json_z(_GR_,s,f);	delete []s;		delete []f;	}
+//-----------------------------------------------------------------------------
 bool mglCanvas::ExportMGLD(const char *fname, const char *descr)
 {
 	if(Pnt.size()<1 || Prm.size()<1)	return true;
 	FILE *fp=fopen(fname,"wt");
 	if(!fp)	return true;
+	setlocale(LC_NUMERIC, "C");
 	// NOTE: I'll save Ptx. So prim type=6 is useless,and no LaTeX
 	fprintf(fp,"MGLD %lu %lu %lu %lu %d %d\n# %s\n", (unsigned long)Pnt.size(), (unsigned long)Prm.size(), (unsigned long)Txt.size(), (unsigned long)Glf.size(), Width, Height, (descr && *descr) ? descr : fname);
-	register size_t i;
 	fprintf(fp,"# Vertexes: x y z c t ta u v w r g b a\n");
-	for(i=0;i<Pnt.size();i++)
+	for(size_t i=0;i<Pnt.size();i++)
 	{
 		const mglPnt &q=Pnt[i];
 		fprintf(fp,"%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\t%.4g\n", q.xx, q.yy, q.zz, q.c, q.t, q.ta, q.u, q.v, q.w, q.r, q.g, q.b, q.a);
 	}
 	fprintf(fp,"# Primitives: type n1 n2 n3 n4 id s w p\n");
-	for(i=0;i<Prm.size();i++)
+	for(size_t i=0;i<Prm.size();i++)
 	{
-		const mglPrim &p=Prm[i];
-		fprintf(fp,"%d\t%ld\t%ld\t%ld\t%ld\t%d\t%g\t%g\t%g\n", p.type, p.n1, p.n2, p.n3, p.n4, p.id, p.s, p.w, p.p);
+		const mglPrim &p=Prm[i];	// TODO: check if better save p.m instead of p.s,p.p
+		fprintf(fp,"%d\t%ld\t%ld\t%ld\t%ld\t%d\t%g\t%g\t%g\n", p.type, p.n1, p.n2, p.n3, p.n4, p.id, p.s==p.s?p.s:0, p.w==p.w?p.w:0, p.p==p.p?p.p:0);
 	}
 	fprintf(fp,"# Textures: smooth alpha colors\n");
-	for(i=0;i<Txt.size();i++)
+	for(size_t i=0;i<Txt.size();i++)
 	{
 		const mglTexture &t=Txt[i];
 		fprintf(fp,"%d\t%.4g\t%s\n",t.Smooth,t.Alpha,t.Sch);
 	}
 	fprintf(fp,"# Glyphs: nt nl [trig] [line]\n");
-	for(i=0;i<Glf.size();i++)
+	for(size_t i=0;i<Glf.size();i++)
 	{
 		const mglGlyph &g=Glf[i];
 		fprintf(fp,"%ld\t%ld\n", g.nt, g.nl);
@@ -632,6 +682,7 @@ bool mglCanvas::ExportMGLD(const char *fname, const char *descr)
 		}
 	}
 	fclose(fp);
+	setlocale(LC_NUMERIC, "");
 	return false;
 }
 //-----------------------------------------------------------------------------
@@ -649,70 +700,73 @@ bool mglCanvas::ImportMGLD(const char *fname, bool add)
 	char *buf=new char[512];
 	if(!fgets(buf,512,fp))	*buf=0;
 	if(strncmp(buf,"MGLD",4))	{	delete []buf;	fclose(fp);	return true;	}
-	register size_t i;
-	unsigned long n=0,m=0,l=0,k=0, npnt=0, nglf=0;
+	unsigned long i,n=0,m=0,l=0,k=0, npnt=0, nglf=0;
 	int w=0,h=0,d;
 	sscanf(buf+5,"%lu%lu%lu%lu%d%d",&n,&m,&l,&k,&w,&h);
 	if(w<=0 || h<=0)	{	w=Width;	h=Height;	}
 	d = long(sqrt(double(w*h)));
-	if(n<=0 || m<=0 || l<=0)	{	delete []buf;	fclose(fp);	return true;	}
+	if(n==0 || m==0 || l==0)	{	delete []buf;	fclose(fp);	return true;	}
+	setlocale(LC_NUMERIC, "C");
 	if(!add)	{	Clf();	Txt.clear();	}
 	else	{	ClfZB();	npnt=Pnt.size();	nglf=Glf.size();	}
-	Pnt.reserve(n);	Prm.reserve(m);	Txt.reserve(l);	Glf.reserve(k);
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_lock(&mutexGlf);
 	pthread_mutex_lock(&mutexPnt);
 	pthread_mutex_lock(&mutexPrm);
 	pthread_mutex_lock(&mutexTxt);
 #endif
-	mglPnt p;
-	for(i=0;i<n;i++)
+#pragma omp critical
 	{
-		do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#');
-		sscanf(buf,"%g%g%g%g%g%g%g%g%g%g%g%g%g", &p.xx, &p.yy, &p.zz, &p.c, &p.t, &p.ta, &p.u, &p.v, &p.w, &p.r, &p.g, &p.b, &p.a);
-		// rescale to current image size
-		p.xx *= Width/double(w);	p.yy *= Height/double(h);	p.zz *= Depth/double(d);
-		Pnt.push_back(p);
-	}
-	mglPrim q;
-	for(i=0;i<m;i++)
-	{
-		do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#');
-		sscanf(buf,"%d%ld%ld%ld%ld%d%g%g%g", &q.type, &q.n1, &q.n2, &q.n3, &q.n4, &q.id, &q.s, &q.w, &q.p);
-		q.n1 = q.n1>=0?q.n1+npnt:-1;
-		q.n2 = q.n2>=0?q.n2+npnt:-1;
-		if(q.type==2 || q.type==3)
-		{	q.n3 = q.n3>=0?q.n3+npnt:-1;	q.n4 = q.n4>=0?q.n4+npnt:-1;	}
-		if(q.type==4)
-		{	q.n4 = q.n4>=0?q.n4+nglf:-1;	q.s *= font_factor/(w<h?w:h);	}
-		if(q.type<5)	Prm.push_back(q);
-	}
-	mglTexture t;
-	for(i=0;i<l;i++)
-	{
-		int sm=0;	float a;
-		do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#');
-		register size_t j,k=0;
-		for(j=0;buf[j];j++)
+		Pnt.reserve(n);	Prm.reserve(m);	Txt.reserve(l);	Glf.reserve(k);
+		mglPnt p;
+		for(i=0;i<n;i++)
 		{
-			if(buf[j]<=' ' && k)	{	sm++;	k=0;	}
-			if(buf[j]>' ')	k=1;
-			if(sm==2 && k)	break;
+			do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#');
+			sscanf(buf,"%g%g%g%g%g%g%g%g%g%g%g%g%g", &p.xx, &p.yy, &p.zz, &p.c, &p.t, &p.ta, &p.u, &p.v, &p.w, &p.r, &p.g, &p.b, &p.a);
+			// rescale to current image size
+			p.xx *= Width/double(w);	p.yy *= Height/double(h);	p.zz *= Depth/double(d);
+			Pnt.push_back(p);
+		}
+		mglPrim q;
+		for(i=0;i<m;i++)
+		{
+			do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#');
+			sscanf(buf,"%hd%ld%ld%ld%ld%d%g%g%g", &q.type, &q.n1, &q.n2, &q.n3, &q.n4, &q.id, &q.s, &q.w, &q.p);
+			q.n1 = q.n1>=0?q.n1+npnt:-1;
+			q.n2 = q.n2>=0?q.n2+npnt:-1;
+			if(q.type==2 || q.type==3)
+			{	q.n3 = q.n3>=0?q.n3+npnt:-1;	q.n4 = q.n4>=0?q.n4+npnt:-1;	}
+			if(q.type==4)
+			{	q.n4 = q.n4>=0?q.n4+nglf:-1;	q.s *= font_factor/(w<h?w:h);	}
+			if(q.type<5)	Prm.push_back(q);
+		}
+		mglTexture t;
+		for(i=0;i<l;i++)
+		{
+			int sm=0;	float a;
+			do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#');
+			register size_t j,k=0;
+			for(j=0;buf[j];j++)
+			{
+				if(buf[j]<=' ' && k)	{	sm++;	k=0;	}
+				if(buf[j]>' ')	k=1;
+				if(sm==2 && k)	break;
+			}
+			sscanf(buf,"%d%g", &sm, &a);
+			t.Set(buf+j, sm, a);
+			Txt.push_back(t);
+		}
+		mglGlyph g;
+		for(i=0;i<k;i++)
+		{
+			do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#' || *buf==0);
+			long nt=0,nl=0;
+			sscanf(buf,"%ld%ld", &nt, &nl);	g.Create(nt,nl);
+			register long j;
+			for(j=0;j<6*nt;j++)	fscanf(fp,"%hd",g.trig+j);
+			for(j=0;j<2*nl;j++)	fscanf(fp,"%hd",g.line+j);
+			Glf.push_back(g);
 		}
-		sscanf(buf,"%d%g", &sm, &a);
-		t.Set(buf+j, sm, a);
-		Txt.push_back(t);
-	}
-	mglGlyph g;
-	for(i=0;i<k;i++)
-	{
-		do {	if(!fgets(buf,512,fp))	*buf=0;	mgl_strtrim(buf);	} while(*buf=='#' || *buf==0);
-		long nt=0,nl=0;
-		sscanf(buf,"%ld%ld", &nt, &nl);	g.Create(nt,nl);
-		register long j;
-		for(j=0;j<6*nt;j++)	fscanf(fp,"%hd",g.trig+j);
-		for(j=0;j<2*nl;j++)	fscanf(fp,"%hd",g.line+j);
-		Glf.push_back(g);
 	}
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexGlf);
@@ -720,6 +774,7 @@ bool mglCanvas::ImportMGLD(const char *fname, bool add)
 	pthread_mutex_unlock(&mutexPrm);
 	pthread_mutex_unlock(&mutexTxt);
 #endif
+	setlocale(LC_NUMERIC, "");
 	delete []buf;	fclose(fp);	return false;
 }
 //-----------------------------------------------------------------------------
@@ -871,14 +926,14 @@ void MGL_EXPORT mgl_import_mgld_(uintptr_t *gr, const char *fname, int *add, int
 			fprintf(fp,"l -1/%ld -3/%ld\n", i,i);	break;
 		case 'O':
 			for(long j=0;j<=20;j++)
-				fprintf(fp,"v %g %g %g\n",p.x+ss*cos(j*M_PI/10),p.y+ss*sin(j*M_PI/10),p.z);
+				fprintf(fp,"v %g %g %g\n",p.x+ss*mgl_cos[(j*36)%360],p.y+ss*mgl_cos[(270+j*36)%360],p.z);
 			for(long j=0;j<20;j++)
 				fprintf(fp,"f %ld/%ld %ld/%ld %ld/%ld\n", j-21,i, j-20,i, i,i);
 			break;
 		case 'C':	fprintf(fp,"p %ld\n", i);
 		case 'o':
 			for(long j=0;j<=20;j++)
-				fprintf(fp,"v %g %g %g\n",p.x+ss*cos(j*M_PI/10),p.y+ss*sin(j*M_PI/10),p.z);
+				fprintf(fp,"v %g %g %g\n",p.x+ss*mgl_cos[(j*36)%360],p.y+ss*mgl_cos[(270+j*36)%360],p.z);
 			for(long j=0;j<20;j++)
 				fprintf(fp,"l %ld/%ld %ld/%ld\n", j-21,i, j-20,i);
 			break;
@@ -972,7 +1027,7 @@ void MGL_EXPORT mgl_x3d_mdef(HMGL gr, void *fp, bool gz)
 	m_s=false,m_a=false,m_o=false,m_T=false,
 	m_V=false,m_S=false,m_D=false,m_Y=false,m_l=false,
 	m_L=false,m_r=false,m_R=false,m_X=false,m_P=false;
-	for(size_t i=0;i<gr->GetPrmNum();i++)
+	for(long i=0;i<gr->GetPrmNum();i++)
 	{
 		const mglPrim q = gr->GetPrm(i);
 		if(q.type>0)	continue;		if(q.n4=='+')	m_p = true;
@@ -1098,7 +1153,7 @@ void MGL_EXPORT mgl_x3d_prim(const mglPrim &q, const mglPnt &p, const long *pnt,
 			if(q.n3&4)	mgl_printf(fp, gz, "dr");
 			else	mgl_printf(fp, gz, "eofill");
 			mgl_printf(fp, gz, " grestore\n");
-		}*/	
+		}*/
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
@@ -1109,6 +1164,7 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 	bool gz = fname[strlen(fname)-1]=='z';
 	void *fp = gz ? (void*)gzopen(fname,"wt") : (void*)fopen(fname,"wt");
 	if(!fp)		{	gr->SetWarn(mglWarnOpen,fname);	return;	}
+	setlocale(LC_NUMERIC, "C");
 	mgl_printf(fp, gz, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 	mgl_printf(fp, gz, "<!DOCTYPE X3D PUBLIC \"ISO//Web3D//DTD X3D 3.0//EN\" \"http://www.web3d.org/specifications/x3d-3.0.dtd\">\n");
 	mgl_printf(fp, gz, "<X3D profile='Immersive'>\n<head>\n<meta name='filename' content='%s'/>\n",fname);
@@ -1124,12 +1180,11 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 
 	// 2. now find group for primitives
 	long m1=0,m2=0,m;
-	register size_t i,j;
-	for(i=0;i<gr->Grp.size();i++)	// prepare array of indirect indexing
+	for(size_t i=0;i<gr->Grp.size();i++)	// prepare array of indirect indexing
 	{	m = gr->Grp[i].Id;	if(m<m1) m1=m;	if(m>m2) m2=m;	}
 	long *ng = new long[m2-m1+1];
-	for(i=0;i<gr->Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
-	for(i=0;i<gr->GetPrmNum();i++)	// collect data for groups
+	for(size_t i=0;i<gr->Grp.size();i++)	ng[gr->Grp[i].Id-m1] = i;
+	for(long i=0;i<gr->GetPrmNum();i++)	// collect data for groups
 	// it is rather expensive (extra 4b per primitive) but need for export to 3D
 	{
 		m = gr->GetPrm(i).id-m1;
@@ -1139,19 +1194,19 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 
 	// resort in creation order for proper drawing of lines and faces
 	gr->resort();
-	
+
 	// primitive definition in groups
 	long npnt = gr->GetPntNum(), k;
 	long *pnt=new long[npnt];
 	mglPrim q;
-	for(i=0;i<gr->Grp.size();i++)
+	for(size_t i=0;i<gr->Grp.size();i++)
 	{
 		mgl_printf(fp,gz,"<Group><!--%s-->\n",gr->Grp[i].Lbl.c_str());
 		std::vector<long> &p = gr->Grp[i].p;
 
 		// define coordinates, colors and so on
-		long line=-1, face=-1, other=-1;
-		for(j=0,k=0;j<p.size();j++)	// find points for this group
+		long line=-1, face=-1, other=-1;	k=0;
+		for(size_t j=0;j<p.size();j++)	// find points for this group
 		{
 			const mglPrim &q=gr->GetPrm(p[j]);
 			if(q.type==1)	line=q.n1;	// find kind of primitives in the group
@@ -1164,13 +1219,13 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 		{
 			mglColor c=gr->GetPntC(line);
 			bool same=true;	// check if there are the same colors for all line segments
-			for(j=0;j<p.size();j++)
+			for(size_t j=0;j<p.size();j++)
 			{
 				const mglPrim &q=gr->GetPrm(p[j]);
 				if(q.type==1 && c!=gr->GetPntC(q.n1))	same=false;
 			}
 			memset(pnt,-1,npnt*sizeof(long));
-			for(j=0,k=0;j<p.size();j++)	// rearrange points for this group
+			for(size_t j=0,k=0;j<p.size();j++)	// rearrange points for this group
 			{
 				const mglPrim &q=gr->GetPrm(p[j]);
 				if(q.type!=1)	continue;
@@ -1178,16 +1233,16 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 				if(q.n2>=0 && pnt[q.n2]<0)	{	pnt[q.n2]=k;	k++;	}
 			}
 			mgl_printf(fp, gz, "<Shape><Coordinate DEF='Lpnts_%ld' point='",i);
-			for(j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
+			for(long j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
 			{	const mglPnt &p=gr->GetPnt(j);	mgl_printf(fp, gz, "%g %g %g, ", p.x,p.y,p.z);	}
 			mgl_printf(fp, gz, "0.0 0.0 0.0'/>");
 			mgl_printf(fp, gz, "<Color DEF='Lclrs_%ld' color='",i);
-			for(j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
+			for(long j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
 			{	const mglPnt &p=gr->GetPnt(j);	mgl_printf(fp, gz, "%g %g %g, ", p.r,p.g,p.b);	}
 			mgl_printf(fp, gz, "0.0 0.0 0.0'/>");
 
 			// TODO save IndexedLineSet here + manual color is same==true
-			
+
 			mgl_printf(fp, gz, "</Shape>");
 		}
 
@@ -1196,13 +1251,13 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 		{
 			mglColor c=gr->GetPntC(face);
 			bool same=true;	// check if there are the same colors for all line segments
-			for(j=0;j<p.size();j++)
+			for(size_t j=0;j<p.size();j++)
 			{
 				const mglPrim &q=gr->GetPrm(p[j]);
 				if((q.type==2 || q.type==3) && c!=gr->GetPntC(q.n1))	same=false;
 			}
 			memset(pnt,-1,npnt*sizeof(long));
-			for(j=0,k=0;j<p.size();j++)	// rearrange points for this group
+			for(size_t j=0,k=0;j<p.size();j++)	// rearrange points for this group
 			{
 				const mglPrim &q=gr->GetPrm(p[j]);
 				if(q.type!=2 && q.type!=3)	continue;
@@ -1212,11 +1267,11 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 				if(q.type==3 && q.n4>=0 && pnt[q.n4]<0)	{	pnt[q.n4]=k;	k++;	}
 			}
 			mgl_printf(fp, gz, "<Shape><Coordinate DEF='Fpnts_%ld' point='",i);
-			for(j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
+			for(long j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
 			{	const mglPnt &p=gr->GetPnt(j);	mgl_printf(fp, gz, "%g %g %g, ", p.x,p.y,p.z);	}
 			mgl_printf(fp, gz, "0.0 0.0 0.0'/>");
 			mgl_printf(fp, gz, "<Color DEF='Fclrs_%ld' color='",i);
-			for(j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
+			for(long j=0;j<gr->GetPntNum();j++)	if(pnt[j]>=0)
 			{	const mglPnt &p=gr->GetPnt(j);	mgl_printf(fp, gz, "%g %g %g, ", p.r,p.g,p.b);	}
 			mgl_printf(fp, gz, "0.0 0.0 0.0'/>");
 
@@ -1253,7 +1308,7 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 		}
 		// no normals since mathgl ones are "signless" -- x3d should calculate it by itself
 
-		for(j=0;j<p.size();j++)
+		for(size_t j=0;j<p.size();j++)
 		{
 			const mglPrim &q=gr->GetPrm(p[j]);	// TODO: collect by type (quads,trig,line) and draw together???
 			mgl_x3d_prim(q, gr->GetPnt(q.n1), pnt, fp,gz, q.s*gr->FontFactor());
@@ -1263,6 +1318,8 @@ void MGL_EXPORT mgl_write_x3d(HMGL gr, const char *fname,const char *descr)
 	}
 	mgl_printf(fp, gz, "</Scene>\n");
 	if(gz)	gzclose((gzFile)fp);	else	fclose((FILE *)fp);
+	setlocale(LC_NUMERIC, "");
+	delete []pnt;
 }
 void MGL_EXPORT mgl_write_x3d_(uintptr_t *gr, const char *fname,const char *descr,int l,int n)
 {	char *s=new char[l+1];	memcpy(s,fname,l);	s[l]=0;
