@@ -1,6 +1,6 @@
 /***************************************************************************
  * fit.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -22,6 +22,7 @@
 #include "mgl2/prim.h"
 #include "mgl2/eval.h"
 #include "mgl2/data.h"
+#include "mgl2/base.h"
 
 #if MGL_HAVE_GSL
 #include <gsl/gsl_multifit_nlin.h>
@@ -66,15 +67,18 @@ struct mglFitData
 int	mgl_fit__f (const gsl_vector *x, void *data, gsl_vector *f)
 {
 	mglFitData *fd = (mglFitData *)data;
-	register long i;
-	mreal val[MGL_VS];
-	for(i=0;i<fd->m;i++)	val[fd->var[i]-'a'] = gsl_vector_get(x,i);
-	for(i=0;i<fd->n;i++)
+#pragma omp parallel
 	{
-		val['x'-'a'] = fd->x[i];
-		val['y'-'a'] = fd->y ? fd->y[i] : 0;
-		val['z'-'a'] = fd->z ? fd->z[i] : 0;
-		gsl_vector_set (f, i, (fd->eq->Calc(val) - fd->a[i])/fd->s[i]);
+		mreal val[MGL_VS];
+		for(long i=0;i<fd->m;i++)	val[fd->var[i]-'a'] = gsl_vector_get(x,i);
+#pragma omp for
+		for(long i=0;i<fd->n;i++)
+		{
+			val['x'-'a'] = fd->x[i];
+			val['y'-'a'] = fd->y ? fd->y[i] : 0;
+			val['z'-'a'] = fd->z ? fd->z[i] : 0;
+			gsl_vector_set (f, i, (fd->eq->Calc(val) - fd->a[i])/fd->s[i]);
+		}
 	}
 	return GSL_SUCCESS;
 }
@@ -82,16 +86,19 @@ int	mgl_fit__f (const gsl_vector *x, void *data, gsl_vector *f)
 int MGL_NO_EXPORT mgl_fit__df (const gsl_vector * x, void *data, gsl_matrix * J)
 {
 	mglFitData *fd = (mglFitData *)data;
-	register long i,j;
-	mreal val[MGL_VS],s;
-	for(i=0;i<fd->m;i++)	val[fd->var[i]-'a'] = gsl_vector_get(x,i);
-	for(i=0;i<fd->n;i++)
+#pragma omp parallel
 	{
-		val['x'-'a'] = fd->x[i];	s = fd->s[i];
-		val['y'-'a'] = fd->y ? fd->y[i] : 0;
-		val['z'-'a'] = fd->z ? fd->z[i] : 0;
-		for(j=0;j<fd->m;j++)
-			gsl_matrix_set (J, i, j, fd->eq->CalcD(val, fd->var[j])/s);
+		mreal val[MGL_VS],s;
+		for(long i=0;i<fd->m;i++)	val[fd->var[i]-'a'] = gsl_vector_get(x,i);
+#pragma omp for
+		for(long i=0;i<fd->n;i++)
+		{
+			val['x'-'a'] = fd->x[i];	s = fd->s[i];
+			val['y'-'a'] = fd->y ? fd->y[i] : 0;
+			val['z'-'a'] = fd->z ? fd->z[i] : 0;
+			for(long j=0;j<fd->m;j++)
+				gsl_matrix_set (J, i, j, fd->eq->CalcD(val, fd->var[j])/s);
+		}
 	}
 	return GSL_SUCCESS;
 }
@@ -105,13 +112,13 @@ int MGL_NO_EXPORT mgl_fit__fdf (const gsl_vector * x, void *data, gsl_vector * f
 #endif
 //-----------------------------------------------------------------------------
 /// GSL based fitting procedure for formula/arguments specified by string
-mreal MGL_NO_EXPORT mgl_fit_base(mglFitData *fd, mreal *ini)
+mreal MGL_NO_EXPORT mgl_fit_base(mglFitData &fd, mreal *ini)
 {
 #if MGL_HAVE_GSL
-	register long i,m=fd->m,n=fd->n,iter=0;
-	if(n<1 || fd==0 || ini==0)	return -1;
+	register long i,m=fd.m,n=fd.n,iter=0;
+	if(n<1 || ini==0)	return -1;
 	// setup data
-	double *x_init = new double[fd->m];
+	double *x_init = new double[fd.m];
 	for(i=0;i<m;i++)	x_init[i] = ini[i];
 	// setup fitting
 	gsl_matrix *covar = gsl_matrix_alloc(m, m);
@@ -121,7 +128,7 @@ mreal MGL_NO_EXPORT mgl_fit_base(mglFitData *fd, mreal *ini)
 	gsl_multifit_function_fdf f;
 	f.f = mgl_fit__f;		f.df = mgl_fit__df;
 	f.fdf = mgl_fit__fdf;	f.n = n;	f.p = m;
-	f.params = fd;
+	f.params = &fd;
 	gsl_multifit_fdfsolver_set(s, &f, &vx.vector);
 	int status;	// start fitting
 	do
@@ -149,7 +156,8 @@ void mglPrepareFitEq(mglBase *gr,mreal chi, const char *eq, const char *var, mre
 {
 	char buf[32]="";
 	snprintf(mglFitRes,1024,"chi=%g",chi);
-	for(int i=0;i<int(strlen(var));i++)
+	size_t i,k,len=strlen(var);
+	for(i=0;i<len;i++)
 	{
 		snprintf(buf,32,", %c=%g",var[i],par[i]);
 		strcat(mglFitRes,buf);
@@ -157,7 +165,7 @@ void mglPrepareFitEq(mglBase *gr,mreal chi, const char *eq, const char *var, mre
 	gr->SetWarn(-1,mglFitRes);
 
 	memset(mglFitRes, 0, 1024);	//mglFitRes[0] = 0;
-	register long i,k,len=strlen(eq);
+	len=strlen(eq);
 	for(i=k=0;i<len;i++)
 	{
 		const char *c = strchr(var,eq[i]);
@@ -223,44 +231,57 @@ HMDT MGL_EXPORT mgl_fit_ys(HMGL gr, HCDT y, HCDT s, const char *eq, const char *
 	return mgl_fit_xys(gr,&x,y,s,eq,var,ini,0);
 }
 //-----------------------------------------------------------------------------
+void MGL_NO_EXPORT mgl_fill_fit(HMGL gr, mglData &fit, mglData &in, mglFitData &fd, const char *var, long nx, long ny, long nz, long k)
+{
+	long nn = nx*ny*nz;
+	mreal dx = nx>1?(gr->Max.x-gr->Min.x)/(nx-1):0;
+	mreal dy = ny>1?(gr->Max.y-gr->Min.y)/(ny-1):0;
+	mreal dz = nz>1?(gr->Max.z-gr->Min.z)/(nz-1):0;
+#pragma omp parallel
+	{
+		mreal val[MGL_VS];	memset(val,0,MGL_VS*sizeof(mreal));
+		for(long j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
+#pragma omp for collapse(3)
+		for(long jz=0;jz<nz;jz++)	for(long jy=0;jy<ny;jy++)	for(long jx=0;jx<nx;jx++)
+		{
+			val['x'-'a'] = gr->Min.x+jx*dx;
+			if(dy)	val['y'-'a'] = gr->Min.y+jy*dy;
+			if(dz)	val['z'-'a'] = gr->Min.z+jz*dz;
+			fit.a[jx+nx*(jy+ny*jz)+k*nn] = fd.eq->Calc(val);
+		}
+	}
+}
+//-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_fit_xys(HMGL gr, HCDT xx, HCDT yy, HCDT ss, const char *eq, const char *var, HMDT ini, const char *opt)
 {
-	mglData *fit=new mglData;
 	long m = yy->GetNx();
-	long nn = long(0.5+gr->SaveState(opt));
+	mreal rr = gr->SaveState(opt);
+	long nn = (mgl_isnan(rr) || rr<=0) ? mglFitPnts:long(rr+0.5);
 	if(xx->GetNx()!=m)
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	if(m<2)
-	{	gr->SetWarn(mglWarnLow,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnLow,"Fit[S]");	return 0;	}
 	if(ss->GetNx()*ss->GetNy()*ss->GetNz() != m*yy->GetNy()*yy->GetNz())
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	if(!var || *var==0)
-	{	gr->SetWarn(mglWarnNull,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnNull,"Fit[S]");	return 0;	}
 
-	if(nn<mglFitPnts)	nn = mglFitPnts;
 	mglData x(xx), y(yy), s(ss);
 	mglFitData fd;
 	fd.n = m;	fd.x = x.a;		fd.y = 0;
 	fd.z = 0;	fd.a = y.a;		fd.s = s.a;
 	fd.var = var;	fd.m = strlen(var);
 	fd.eq = new mglFormula(eq);
-	mglData in(fd.m);
-	fit->Create(nn, yy->GetNy(), yy->GetNz());
-	mreal val[MGL_VS],res=-1;
-	register long j;
+	mglData in(fd.m), *fit=new mglData(nn, yy->GetNy(), yy->GetNz());
+	mreal res=-1;
 	for(long i=0;i<yy->GetNy()*yy->GetNz();i++)
 	{
 		if(ini && ini->nx>=fd.m)	in.Set(ini->a,fd.m);
 		else in.Fill(0.,0);
 		fd.a = y.a+i*m;		fd.x = x.a+(i%x.ny)*m;
 		fd.s = s.a+i*m;
-		res = mgl_fit_base(&fd,in.a);
-		for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
-		for(j=0;j<nn;j++)
-		{
-			val['x'-'a'] = gr->Min.x+j*(gr->Max.x-gr->Min.x)/(nn-1);
-			fit->a[j+i*nn] = fd.eq->Calc(val);
-		}
+		res = mgl_fit_base(fd,in.a);
+		mgl_fill_fit(gr,*fit,in,fd,var,nn,1,1,i);
 		if(ini && ini->nx>=fd.m)	memcpy(ini->a,in.a,fd.m*sizeof(mreal));
 	}
 	mglPrepareFitEq(gr,res,eq,var,in.a);
@@ -269,24 +290,23 @@ HMDT MGL_EXPORT mgl_fit_xys(HMGL gr, HCDT xx, HCDT yy, HCDT ss, const char *eq,
 //-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_fit_xyzs(HMGL gr, HCDT xx, HCDT yy, HCDT zz, HCDT ss, const char *eq, const char *var, HMDT ini, const char *opt)
 {
-	mglData *fit=new mglData;
 	long m=zz->GetNx(),n=zz->GetNy();
-	long nn = long(0.5+gr->SaveState(opt));
+	mreal rr = gr->SaveState(opt);
+	long nn = (mgl_isnan(rr) || rr<=0) ? mglFitPnts:long(rr+0.5);
 	if(xx->GetNx()!=m)
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	if(ss->GetNx()*ss->GetNy()*ss->GetNz() != m*n*zz->GetNz())
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	if(yy->GetNx()!=n && (xx->GetNy()!=n || yy->GetNx()!=m || yy->GetNy()!=n))
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	if(m<2|| n<2)
-	{	gr->SetWarn(mglWarnLow,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnLow,"Fit[S]");	return 0;	}
 	if(!var || *var==0)
-	{	gr->SetWarn(mglWarnNull,"Fit[S]");	return fit;	}
-	
-	if(nn<mglFitPnts)	nn = mglFitPnts;
+	{	gr->SetWarn(mglWarnNull,"Fit[S]");	return 0;	}
+
 	mglData x(m, n), y(m, n), z(zz), s(ss);
-	register long i,j;
-	for(i=0;i<m;i++)	for(j=0;j<n;j++)	// ñîçäàåì ìàññèâ òî÷åê
+#pragma omp parallel for collapse(2)
+	for(long i=0;i<m;i++)	for(long j=0;j<n;j++)	// ñîçäàåì ìàññèâ òî÷åê
 	{
 		x.a[i+m*j] = GetX(xx,i,j,0).x;
 		y.a[i+m*j] = GetY(yy,i,j,0).x;
@@ -296,22 +316,15 @@ HMDT MGL_EXPORT mgl_fit_xyzs(HMGL gr, HCDT xx, HCDT yy, HCDT zz, HCDT ss, const
 	fd.z = 0;		fd.a = z.a;	fd.s = s.a;
 	fd.var = var;	fd.m = strlen(var);
 	fd.eq = new mglFormula(eq);
-	mglData in(fd.m);
-	fit->Create(nn, nn, zz->GetNz());
-	mreal val[MGL_VS], res = -1;
-	for(i=0;i<zz->GetNz();i++)
+	mglData in(fd.m), *fit=new mglData(nn, nn, zz->GetNz());
+	mreal res = -1;
+	for(long i=0;i<zz->GetNz();i++)
 	{
 		if(ini && ini->nx>=fd.m)	in.Set(ini->a,fd.m);
 		else in.Fill(0.,0);
 		fd.a = z.a+i*m*n;		fd.s = s.a+i*m*n;
-		res = mgl_fit_base(&fd,in.a);
-		for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
-		for(j=0;j<nn*nn;j++)
-		{
-			val['x'-'a'] = gr->Min.x+(j%nn)*(gr->Max.x-gr->Min.x)/(nn-1);
-			val['y'-'a'] = gr->Min.y+(j/nn)*(gr->Max.y-gr->Min.y)/(nn-1);
-			fit->a[j+i*nn*nn] = fd.eq->Calc(val);
-		}
+		res = mgl_fit_base(fd,in.a);
+		mgl_fill_fit(gr,*fit,in,fd,var,nn,nn,1,i);
 		if(ini && ini->nx>=fd.m)	memcpy(ini->a,in.a,fd.m*sizeof(mreal));
 	}
 	mglPrepareFitEq(gr,res, eq,var,in.a);
@@ -320,26 +333,24 @@ HMDT MGL_EXPORT mgl_fit_xyzs(HMGL gr, HCDT xx, HCDT yy, HCDT zz, HCDT ss, const
 //-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_fit_xyzas(HMGL gr, HCDT xx, HCDT yy, HCDT zz, HCDT aa, HCDT ss, const char *eq, const char *var, HMDT ini, const char *opt)
 {
-	mglData *fit=new mglData;
-	register long i,j,k,i0;
-	long m=aa->GetNx(), n=aa->GetNy(), l=aa->GetNz();
-	i = n*m*l;
-	long nn = long(0.5+gr->SaveState(opt));
+	long m=aa->GetNx(), n=aa->GetNy(), l=aa->GetNz(), i = n*m*l;
+	mreal rr = gr->SaveState(opt);
+	long nn = (mgl_isnan(rr) || rr<=0) ? mglFitPnts:long(rr+0.5);
 	if(m<2 || n<2 || l<2)
-	{	gr->SetWarn(mglWarnLow,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnLow,"Fit[S]");	return 0;	}
 	if(ss->GetNx()*ss->GetNy()*ss->GetNz() != i)
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	bool both = xx->GetNx()*xx->GetNy()*xx->GetNz()==i && yy->GetNx()*yy->GetNy()*yy->GetNz()==i && zz->GetNx()*zz->GetNy()*zz->GetNz()==i;
 	if(!(both || (xx->GetNx()==m && yy->GetNx()==n && zz->GetNx()==l)))
-	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return fit;	}
+	{	gr->SetWarn(mglWarnDim,"Fit[S]");	return 0;	}
 	if(!var || *var==0)
-	{	gr->SetWarn(mglWarnNull,"Fit[S]");	return fit;	}
-	
-	if(nn<mglFitPnts)	nn = mglFitPnts;
+	{	gr->SetWarn(mglWarnNull,"Fit[S]");	return 0;	}
+
 	mglData x(aa), y(aa), z(aa), a(aa), s(ss);
-	for(i=0;i<m;i++)	for(j=0;j<n;j++)	for(k=0;k<l;k++)	// ñîçäàåì ìàññèâ òî÷åê
+#pragma omp parallel for collapse(3)
+	for(long i=0;i<m;i++)	for(long j=0;j<n;j++)	for(long k=0;k<l;k++)	// ñîçäàåì ìàññèâ òî÷åê
 	{
-		i0 = i+m*(j+n*k);
+		register long i0 = i+m*(j+n*k);
 		x.a[i0] = GetX(xx,i,j,k).x;
 		y.a[i0] = GetY(yy,i,j,k).x;
 		z.a[i0] = GetZ(zz,i,j,k).x;
@@ -349,21 +360,13 @@ HMDT MGL_EXPORT mgl_fit_xyzas(HMGL gr, HCDT xx, HCDT yy, HCDT zz, HCDT aa, HCDT
 	fd.z = z.a;		fd.a = a.a;	fd.s = s.a;
 	fd.var = var;	fd.m = strlen(var);
 	fd.eq = new mglFormula(eq);
-	mglData in(fd.m);
-	fit->Create(nn, nn, nn);
-	mreal val[MGL_VS], res = -1;
+	mglData in(fd.m), *fit=new mglData(nn, nn, nn);
+	mreal res = -1;
 
 	if(ini && ini->nx>=fd.m)	in.Set(ini->a,fd.m);
 	else in.Fill(0.,0);
-	res = mgl_fit_base(&fd,in.a);
-	for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
-	for(i=0;i<nn;i++)	for(j=0;j<nn*nn;j++)
-	{
-		val['x'-'a'] = gr->Min.x+(j%nn)*(gr->Max.x-gr->Min.x)/(nn-1);
-		val['y'-'a'] = gr->Min.y+(j/nn)*(gr->Max.y-gr->Min.y)/(nn-1);
-		val['z'-'a'] = gr->Min.z+i*(gr->Max.y-gr->Min.y)/(nn-1);
-		fit->a[j+nn*nn*i] = fd.eq->Calc(val);
-	}
+	res = mgl_fit_base(fd,in.a);
+	mgl_fill_fit(gr,*fit,in,fd,var,nn,nn,nn,0);
 	if(ini && ini->nx>=fd.m)	memcpy(ini->a,in.a,fd.m*sizeof(mreal));
 
 	mglPrepareFitEq(gr,res, eq,var,in.a);
@@ -374,25 +377,28 @@ HMDT MGL_EXPORT mgl_hist_x(HMGL gr, HCDT x, HCDT a, const char *opt)
 {
 	long nn=a->GetNx()*a->GetNy()*a->GetNz();
 	if(nn!=x->GetNx()*x->GetNy()*x->GetNz())
-	{	gr->SetWarn(mglWarnDim,"Hist");	return (new mglData);	}
-	long n = long(0.5+gr->SaveState(opt));
-	if(n<mglFitPnts)	n = mglFitPnts;
+	{	gr->SetWarn(mglWarnDim,"Hist");	return 0;	}
+	mreal rr = gr->SaveState(opt);
+	long n = (mgl_isnan(rr) || rr<=0) ? mglFitPnts:long(rr+0.5);
 	mglData *res = new mglData(n);
-	register long i,j1;
 
 	const mglData *dx = dynamic_cast<const mglData *>(x);
 	const mglData *da = dynamic_cast<const mglData *>(a);
 	mreal vx = n/(gr->Max.x-gr->Min.x);
-	if(dx && da)	for(i=0;i<nn;i++)
-	{
-		j1 = long((dx->a[i]-gr->Min.x)*vx);
-		if(j1>=0 && j1<n)	res->a[j1] += da->a[i];
-	}
-	else	for(i=0;i<nn;i++)
-	{
-		j1 = long((x->vthr(i)-gr->Min.x)*vx);
-		if(j1>=0 && j1<n)	res->a[j1] += a->vthr(i);
-	}
+	if(dx && da)
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)
+		{
+			register long j1 = long((dx->a[i]-gr->Min.x)*vx);
+			if(j1>=0 && j1<n)	res->a[j1] += da->a[i];
+		}
+	else
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)
+		{
+			register long j1 = long((x->vthr(i)-gr->Min.x)*vx);
+			if(j1>=0 && j1<n)	res->a[j1] += a->vthr(i);
+		}
 	gr->LoadState();	return res;
 }
 //-----------------------------------------------------------------------------
@@ -400,28 +406,31 @@ HMDT MGL_EXPORT mgl_hist_xy(HMGL gr, HCDT x, HCDT y, HCDT a, const char *opt)
 {
 	long nn=a->GetNx()*a->GetNy()*a->GetNz();
 	if(nn!=x->GetNx()*x->GetNy()*x->GetNz() || nn!=y->GetNx()*y->GetNy()*y->GetNz())
-	{	gr->SetWarn(mglWarnDim,"Hist");	return (new mglData);	}
-	long n = long(0.5+gr->SaveState(opt));
-	if(n<mglFitPnts)	n = mglFitPnts;
+	{	gr->SetWarn(mglWarnDim,"Hist");	return 0;	}
+	mreal rr = gr->SaveState(opt);
+	long n = (mgl_isnan(rr) || rr<=0) ? mglFitPnts:long(rr+0.5);
 	mglData *res = new mglData(n, n);
-	register long i,j1,j2;
 	const mglData *dx = dynamic_cast<const mglData *>(x);
 	const mglData *dy = dynamic_cast<const mglData *>(y);
 	const mglData *da = dynamic_cast<const mglData *>(a);
 	mreal vx = n/(gr->Max.x-gr->Min.x);
 	mreal vy = n/(gr->Max.y-gr->Min.y);
-	if(dx && dy && da)	for(i=0;i<nn;i++)
-	{
-		j1 = long((dx->a[i]-gr->Min.x)*vx);
-		j2 = long((dy->a[i]-gr->Min.y)*vy);
-		if(j1>=0 && j1<n && j2>=0 && j2<n)	res->a[j1+n*j2] += da->a[i];
-	}
-	else	for(i=0;i<nn;i++)
-	{
-		j1 = long((x->vthr(i)-gr->Min.x)*vx);
-		j2 = long((y->vthr(i)-gr->Min.y)*vy);
-		if(j1>=0 && j1<n && j2>=0 && j2<n)	res->a[j1+n*j2] += a->vthr(i);
-	}
+	if(dx && dy && da)
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)
+		{
+			register long j1 = long((dx->a[i]-gr->Min.x)*vx);
+			register long j2 = long((dy->a[i]-gr->Min.y)*vy);
+			if(j1>=0 && j1<n && j2>=0 && j2<n)	res->a[j1+n*j2] += da->a[i];
+		}
+	else
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)
+		{
+			register long j1 = long((x->vthr(i)-gr->Min.x)*vx);
+			register long j2 = long((y->vthr(i)-gr->Min.y)*vy);
+			if(j1>=0 && j1<n && j2>=0 && j2<n)	res->a[j1+n*j2] += a->vthr(i);
+		}
 	gr->LoadState();	return res;
 }
 //-----------------------------------------------------------------------------
@@ -429,34 +438,37 @@ HMDT MGL_EXPORT mgl_hist_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT a, const char
 {
 	long nn=a->GetNx()*a->GetNy()*a->GetNz();
 	if(nn!=x->GetNx()*x->GetNy()*x->GetNz() || nn!=y->GetNx()*y->GetNy()*y->GetNz() || nn!=z->GetNx()*z->GetNy()*z->GetNz())
-	{	gr->SetWarn(mglWarnDim,"Hist");	return (new mglData);	}
-	long n = long(0.5+gr->SaveState(opt));
-	if(n<mglFitPnts)	n = mglFitPnts;
+	{	gr->SetWarn(mglWarnDim,"Hist");	return 0;	}
+	mreal rr = gr->SaveState(opt);
+	long n = (mgl_isnan(rr) || rr<=0) ? mglFitPnts:long(rr+0.5);
 	mglData *res = new mglData(n, n, n);
-	register long i,j1,j2,j3;
 	const mglData *dx = dynamic_cast<const mglData *>(x);
 	const mglData *dy = dynamic_cast<const mglData *>(y);
 	const mglData *dz = dynamic_cast<const mglData *>(z);
 	const mglData *da = dynamic_cast<const mglData *>(a);
 	mreal vx = n/(gr->Max.x-gr->Min.x), vy = n/(gr->Max.y-gr->Min.y), vz = n/(gr->Max.z-gr->Min.z);
-	if(dx && dy && dz && da)	for(i=0;i<nn;i++)
-	{
-		j1 = long((dx->a[i]-gr->Min.x)*vx);
-		j2 = long((dy->a[i]-gr->Min.y)*vy);
-		j3 = long((dz->a[i]-gr->Min.z)*vz);
-		if(j1>=0 && j1<n && j2>=0 && j2<n && j3>=0 && j3<n)
-			res->a[j1+n*(j2+n*j3)] += da->a[i];
-	}
-	else	for(i=0;i<nn;i++)
-	{
-		if(gr->Stop)	{	res->Create(1,1,1);	return res;	}
-		j1 = long((x->vthr(i)-gr->Min.x)*vx);
-		j2 = long((y->vthr(i)-gr->Min.y)*vy);
-		j3 = long((z->vthr(i)-gr->Min.z)*vz);
-		if(j1>=0 && j1<n && j2>=0 && j2<n && j3>=0 && j3<n)
-			res->a[j1+n*(j2+n*j3)] += a->vthr(i);
-	}
-	gr->LoadState();	return res;
+	if(dx && dy && dz && da)
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)
+		{
+			register long j1 = long((dx->a[i]-gr->Min.x)*vx);
+			register long j2 = long((dy->a[i]-gr->Min.y)*vy);
+			register long j3 = long((dz->a[i]-gr->Min.z)*vz);
+			if(j1>=0 && j1<n && j2>=0 && j2<n && j3>=0 && j3<n)
+				res->a[j1+n*(j2+n*j3)] += da->a[i];
+		}
+	else
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)
+		{
+			if(gr->Stop)	continue;
+			register long j1 = long((x->vthr(i)-gr->Min.x)*vx);
+			register long j2 = long((y->vthr(i)-gr->Min.y)*vy);
+			register long j3 = long((z->vthr(i)-gr->Min.z)*vz);
+			if(j1>=0 && j1<n && j2>=0 && j2<n && j3>=0 && j3<n)
+				res->a[j1+n*(j2+n*j3)] += a->vthr(i);
+		}
+	gr->LoadState();	return gr->Stop?0:res;
 }
 //-----------------------------------------------------------------------------
 uintptr_t MGL_EXPORT mgl_hist_x_(uintptr_t* gr, uintptr_t* x, uintptr_t* a, const char *opt, int lo)
