@@ -1,6 +1,6 @@
 /***************************************************************************
  * obj.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -35,10 +35,6 @@ void MGL_NO_EXPORT mgl_printf(void *fp, bool gz, const char *str, ...);
 #include <string>
 #include <vector>
 #include <deque>
-#include <list>
-#ifdef __GNUC__
-#include <ext/slist>
-#endif
 #include <map>
 #include <iostream>
 
