@@ -1,6 +1,6 @@
 /***************************************************************************
  * other.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -22,6 +22,7 @@
 #include "mgl2/cont.h"
 #include "mgl2/eval.h"
 #include "mgl2/data.h"
+#include "mgl2/base.h"
 //-----------------------------------------------------------------------------
 //
 //	DensX, DensY, DensZ series
@@ -29,7 +30,7 @@
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensX");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensX");	return;	}
@@ -40,14 +41,18 @@ void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	{
 		aa.Create(m,l);	xx.Create(m,l);	yy.Create(m,l);	zz.Create(m,l);
 		mreal d = (n-1)*(sv - gr->Min.x)/(gr->Max.x - gr->Min.x);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>n-2)	{	k=n-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<l;j++)	for(i=0;i<m;i++)
-			aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
-		else	for(j=0;j<l;j++)	for(i=0;i<m;i++)
-			aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+				aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+				aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
 		a = &aa;
 	}
 	else
@@ -60,7 +65,7 @@ void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensY");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensY");	return;	}
@@ -71,14 +76,18 @@ void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	{
 		aa.Create(n,l);	xx.Create(n,l);	yy.Create(n,l);	zz.Create(n,l);
 		mreal d = (m-1)*(sv - gr->Min.y)/(gr->Max.y - gr->Min.y);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>m-2)	{	k=m-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<l;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
-		else	for(j=0;j<l;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
 		a = &aa;
 	}
 	else
@@ -91,7 +100,7 @@ void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dens_z(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensZ");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensZ");	return;	}
@@ -103,14 +112,18 @@ void MGL_EXPORT mgl_dens_z(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	{
 		aa.Create(n,m);
 		mreal d = (l-1)*(sv - gr->Min.z)/(gr->Max.z - gr->Min.z);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>l-2)	{	k=l-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<m;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
-		else	for(j=0;j<m;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
 		a = &aa;
 	}
 	zz.Fill(sv, sv);
@@ -143,7 +156,7 @@ void MGL_EXPORT mgl_dens_z_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal
 void MGL_EXPORT mgl_cont_gen(HMGL gr, mreal val, HCDT a, HCDT x, HCDT y, HCDT z, mreal c, int text,long ak);
 void MGL_EXPORT mgl_cont_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContX");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContX");	return;	}
@@ -159,14 +172,18 @@ void MGL_EXPORT mgl_cont_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	{
 		aa.Create(m,l);	xx.Create(m,l);	yy.Create(m,l);	zz.Create(m,l);
 		mreal d = (n-1)*(sv - gr->Min.x)/(gr->Max.x - gr->Min.x);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>n-2)	{	k=n-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<l;j++)	for(i=0;i<m;i++)
-			aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
-		else	for(j=0;j<l;j++)	for(i=0;i<m;i++)
-			aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+				aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+				aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
 		a = &aa;
 	}
 	else
@@ -174,7 +191,8 @@ void MGL_EXPORT mgl_cont_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	xx.Fill(sv, sv);
 	yy.Fill(gr->Min.y, gr->Max.y,'x');
 	zz.Fill(gr->Min.z, gr->Max.z,'y');
-	for(i=0;i<v->GetNx();i++)
+#pragma omp parallel for
+	for(long i=0;i<v->GetNx();i++)
 	{
 		register mreal v0 = v->v(i);
 		mgl_cont_gen(gr,v0,a,&xx,&yy,&zz,gr->GetC(ss,v0),text,0);
@@ -184,7 +202,7 @@ void MGL_EXPORT mgl_cont_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContY");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContY");	return;	}
@@ -200,14 +218,18 @@ void MGL_EXPORT mgl_cont_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	{
 		aa.Create(n,l);	xx.Create(n,l);	yy.Create(n,l);	zz.Create(n,l);
 		mreal d = (m-1)*(sv - gr->Min.y)/(gr->Max.y - gr->Min.y);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>m-2)	{	k=m-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<l;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
-		else	for(j=0;j<l;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
 		a = &aa;
 	}
 	else
@@ -215,7 +237,8 @@ void MGL_EXPORT mgl_cont_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	yy.Fill(sv, sv);
 	xx.Fill(gr->Min.x, gr->Max.x,'x');
 	zz.Fill(gr->Min.z, gr->Max.z,'y');
-	for(i=0;i<v->GetNx();i++)
+#pragma omp parallel for
+	for(long i=0;i<v->GetNx();i++)
 	{
 		register mreal v0 = v->v(i);
 		mgl_cont_gen(gr,v0,a,&xx,&yy,&zz,gr->GetC(ss,v0),text,0);
@@ -225,7 +248,7 @@ void MGL_EXPORT mgl_cont_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_z_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContZ");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContZ");	return;	}
@@ -242,20 +265,25 @@ void MGL_EXPORT mgl_cont_z_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	{
 		aa.Create(n,m);
 		mreal d = (l-1)*(sv - gr->Min.z)/(gr->Max.z - gr->Min.z);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>l-2)	{	k=l-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<m;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
-		else	for(j=0;j<m;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
 		a = &aa;
 	}
 	zz.Fill(sv, sv);
 	yy.Fill(gr->Min.y, gr->Max.y,'y');
 	xx.Fill(gr->Min.x, gr->Max.x,'x');
-	for(i=0;i<v->GetNx();i++)
+#pragma omp parallel for
+	for(long i=0;i<v->GetNx();i++)
 	{
 		register mreal v0 = v->v(i);
 		mgl_cont_gen(gr,v0,a,&xx,&yy,&zz,gr->GetC(ss,v0),text,0);
@@ -270,7 +298,6 @@ void MGL_EXPORT mgl_cont_x(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont_x_val(gr,&v,a,sch,sv,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_y(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
@@ -280,7 +307,6 @@ void MGL_EXPORT mgl_cont_y(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont_y_val(gr,&v,a,sch,sv,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_z(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
@@ -290,7 +316,6 @@ void MGL_EXPORT mgl_cont_z(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	mglData v(Num);
 	for(long i=0;i<Num;i++)	v.a[i] = gr->Min.c + (gr->Max.c-gr->Min.c)*mreal(i+1)/(Num+1);
 	mgl_cont_z_val(gr,&v,a,sch,sv,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_x_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sv, const char *opt,int l,int lo)
@@ -330,7 +355,7 @@ void MGL_EXPORT mgl_cont_z_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const
 void MGL_EXPORT mgl_contf_gen(HMGL gr, mreal v1, mreal v2, HCDT a, HCDT x, HCDT y, HCDT z, mreal c, long ak);
 void MGL_EXPORT mgl_contf_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContFX");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFX");	return;	}
@@ -343,14 +368,18 @@ void MGL_EXPORT mgl_contf_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	{
 		aa.Create(m,l);	xx.Create(m,l);	yy.Create(m,l);	zz.Create(m,l);
 		mreal d = (n-1)*(sv - gr->Min.x)/(gr->Max.x - gr->Min.x);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>n-2)	{	k=n-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<l;j++)	for(i=0;i<m;i++)
-			aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
-		else	for(j=0;j<l;j++)	for(i=0;i<m;i++)
-			aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+				aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+				aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
 		a = &aa;
 	}
 	else
@@ -358,7 +387,8 @@ void MGL_EXPORT mgl_contf_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	xx.Fill(sv, sv);
 	yy.Fill(gr->Min.y, gr->Max.y,'x');
 	zz.Fill(gr->Min.z, gr->Max.z,'y');
-	for(i=0;i<v->GetNx()-1;i++)
+#pragma omp parallel for
+	for(long i=0;i<v->GetNx()-1;i++)
 	{
 		register mreal v0 = v->v(i);
 		mgl_contf_gen(gr,v0,v->v(i+1),a,&xx,&yy,&zz,gr->GetC(ss,v0),0);
@@ -368,7 +398,7 @@ void MGL_EXPORT mgl_contf_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContFY");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFY");	return;	}
@@ -381,14 +411,18 @@ void MGL_EXPORT mgl_contf_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	{
 		aa.Create(n,l);	xx.Create(n,l);	yy.Create(n,l);	zz.Create(n,l);
 		mreal d = (m-1)*(sv - gr->Min.y)/(gr->Max.y - gr->Min.y);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>m-2)	{	k=m-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<l;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
-		else	for(j=0;j<l;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
 		a = &aa;
 	}
 	else
@@ -396,7 +430,8 @@ void MGL_EXPORT mgl_contf_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	yy.Fill(sv, sv);
 	xx.Fill(gr->Min.x, gr->Max.x,'x');
 	zz.Fill(gr->Min.z, gr->Max.z,'y');
-	for(i=0;i<v->GetNx()-1;i++)
+#pragma omp parallel for
+	for(long i=0;i<v->GetNx()-1;i++)
 	{
 		register mreal v0 = v->v(i);
 		mgl_contf_gen(gr,v0,v->v(i+1),a,&xx,&yy,&zz,gr->GetC(ss,v0),0);
@@ -406,7 +441,7 @@ void MGL_EXPORT mgl_contf_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_z_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	register long i,j,k,n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
+	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
 	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContFZ");	return;	}
 	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFZ");	return;	}
@@ -420,20 +455,25 @@ void MGL_EXPORT mgl_contf_z_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 	{
 		aa.Create(n,m);
 		mreal d = (l-1)*(sv - gr->Min.z)/(gr->Max.z - gr->Min.z);
-		k = long(d);	d = d - k;
+		long k = long(d);	d = d - k;
 		if(k>l-2)	{	k=l-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
 		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)	for(j=0;j<m;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
-		else	for(j=0;j<m;j++)	for(i=0;i<n;i++)
-			aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
+		if(ma)
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
+		else
+#pragma omp parallel for collapse(2)
+			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+				aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
 		a = &aa;
 	}
 	zz.Fill(sv, sv);
 	yy.Fill(gr->Min.y, gr->Max.y,'y');
 	xx.Fill(gr->Min.x, gr->Max.x,'x');
-	for(i=0;i<v->GetNx()-1;i++)
+#pragma omp parallel for
+	for(long i=0;i<v->GetNx()-1;i++)
 	{
 		register mreal v0 = v->v(i);
 		mgl_contf_gen(gr,v0,v->v(i+1),a,&xx,&yy,&zz,gr->GetC(ss,v0),0);
@@ -447,7 +487,6 @@ void MGL_EXPORT mgl_contf_x(HMGL gr, HCDT a, const char *sch, double sv, const c
 	long Num = (mgl_isnan(r) || r<=0) ? 7:long(r+0.5);
 	mglData v(Num);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf_x_val(gr,&v,a,sch,sv,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_y(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
@@ -456,7 +495,6 @@ void MGL_EXPORT mgl_contf_y(HMGL gr, HCDT a, const char *sch, double sv, const c
 	long Num = (mgl_isnan(r) || r<=0) ? 7:long(r+0.5);
 	mglData v(Num);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf_y_val(gr,&v,a,sch,sv,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_z(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
@@ -465,7 +503,6 @@ void MGL_EXPORT mgl_contf_z(HMGL gr, HCDT a, const char *sch, double sv, const c
 	long Num = (mgl_isnan(r) || r<=0) ? 7:long(r+0.5);
 	mglData v(Num);	v.Fill(gr->Min.c, gr->Max.c);
 	mgl_contf_z_val(gr,&v,a,sch,sv,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_x_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal *sv, const char *opt,int l,int lo)
