@@ -1,6 +1,6 @@
 /***************************************************************************
  * parse.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -20,6 +20,7 @@
 #include <ctype.h>
 #include "mgl2/parser.h"
 #include "mgl2/canvas_cf.h"
+#include "mgl2/base.h"
 //-----------------------------------------------------------------------------
 #ifdef WIN32
 #include <io.h>
@@ -33,38 +34,22 @@ void MGL_EXPORT mgl_ask_gets(const wchar_t *quest, wchar_t *res)
 {	printf("%ls\n",quest);	if(!fgetws(res,1024,stdin))	*res=0;	}
 //-----------------------------------------------------------------------------
 mglFunc::mglFunc(const mglFunc &f)
-{	pos=f.pos;	narg=f.narg;	wcsncpy(func,f.func,64);	}
+{	pos=f.pos;	narg=f.narg;	func=f.func;	}
 mglFunc::mglFunc(long p, const wchar_t *f)
 {
-	pos = p;
-	register size_t i;
-	for(i=0;(isalnum(f[i]) || f[i]=='_') && i<63;i++)	func[i]=f[i];
-	func[i]=0;	narg = wcstol(f+i+1,0,0);
+	pos = p;	func = f;
+	size_t i;
+	for(i=0;(isalnum(f[i]) || f[i]=='_');i++);
+	narg = wcstol(f+i+1,0,0);	func = func.substr(0,i);
 	if(narg<0 || narg>9)	narg=0;
 }
 //-----------------------------------------------------------------------------
-size_t mglParser::GetParLen()
+long mglParser::IsFunc(const std::wstring &name, int *na)
 {
-	register size_t i,s=0,t;
-	for(i=0;i<40;i++)
-	{	t = par[i].length();	s = s<t?t:s;	}
-	return s;
-}
-//-----------------------------------------------------------------------------
-size_t mglParser::GetParLen(const wchar_t *str)
-{
-	register size_t nump=0, is;
-	for(is=0;is<wcslen(str);is++)	if(str[is]=='$')	nump++;
-	return nump*GetParLen();
-}
-//-----------------------------------------------------------------------------
-long mglParser::IsFunc(const wchar_t *name, int *na)
-{
-	register size_t i;
-	for(i=0;i<func.size();i++)
+	for(size_t i=0;i<func.size();i++)
 	{
 		const mglFunc &f = func[i];
-		if(!wcscmp(f.func, name))
+		if(f.func==name)
 		{	if(na)	*na=f.narg;	return f.pos;	}
 	}
 	return 0;
@@ -84,11 +69,10 @@ void mglParser::ScanFunc(const wchar_t *line)
 //-----------------------------------------------------------------------------
 MGL_NO_EXPORT wchar_t *mgl_str_copy(const char *s)
 {
-	wchar_t *str = new wchar_t[strlen(s)+1];
-	register long i;
-	for(i=0;i<int(strlen(s));i++)	str[i] = s[i];
-	str[i] = 0;
-	return str;
+	register size_t i,l=strlen(s);
+	wchar_t *str = new wchar_t[l+1];
+	for(i=0;i<l;i++)	str[i] = s[i];
+	str[i] = 0;	return str;
 }
 //-----------------------------------------------------------------------------
 int MGL_NO_EXPORT mgl_cmd_cmp(const void *a, const void *b)
@@ -98,9 +82,9 @@ int MGL_NO_EXPORT mgl_cmd_cmp(const void *a, const void *b)
 	return strcmp(aa->name, bb->name);
 }
 //-----------------------------------------------------------------------------
-bool check_for_name(const wchar_t *s)
+bool mgl_check_for_name(const std::wstring &s)
 {
-	return !isalpha(s[0])||wcschr(s,'.')||wcschr(s,':')||wcschr(s,'(')||wcschr(s,')');
+	return !isalpha(s[0])||s.find_first_of(L".:()")!=std::wstring::npos;
 }
 //-----------------------------------------------------------------------------
 mglCommand *mglParser::FindCommand(const char *com)
@@ -117,13 +101,13 @@ mglCommand *mglParser::FindCommand(const char *com)
 //-----------------------------------------------------------------------------
 mglCommand *mglParser::FindCommand(const wchar_t *com)
 {
-	size_t s = 15<wcslen(com)?15:wcslen(com);
+	size_t s = 15<mgl_wcslen(com)?15:mgl_wcslen(com);
 	char cmd[16];	wcstombs(cmd,com,s+1);	cmd[s]=0;
 	return FindCommand(cmd);
 }
 //-----------------------------------------------------------------------------
 // return values : 0 -- OK, 1 -- wrong arguments, 2 -- wrong command, 3 -- unclosed string
-int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const wchar_t *var, const wchar_t *opt)
+int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const std::wstring &var, const wchar_t *opt)
 {
 	int i;
 	const char *id="dsn";
@@ -131,14 +115,17 @@ int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const w
 	for(i=0;i<n;i++)
 	{
 		k += id[a[i].type];
-		a[i].s.assign(a[i].w.begin(),a[i].w.end());
+		size_t len = wcstombs(NULL,a[i].w.c_str(),0)+1;
+		char *buf = new char[len];	memset(buf,0,len);
+		wcstombs(buf,a[i].w.c_str(),len);	
+		a[i].s = buf;	delete []buf;
 	}
 	mglCommand *rts=FindCommand(com);
 	if(!rts || rts->type==6)	return 2;
 	if(rts->type == 4)
 	{
-		if(n<1 || check_for_name(var))	return 2;
-		mglVar *v = AddVar(var);
+		if(n<1 || mgl_check_for_name(var))	return 2;
+		mglVar *v = AddVar(var.c_str());
 		v->Create(1,1,1);
 		a[0].type = 0;	a[0].d = v;
 		a[0].w = var;	k[0] = 'd';
@@ -146,7 +133,7 @@ int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const w
 	char *o=0;
 	if(opt && *opt)	// TODO: parse arguments of options
 	{
-		long len = wcslen(opt);
+		long len = mgl_wcslen(opt);
 		o = new char[len+1];
 		for(i=0;i<len+1;i++)	o[i]=opt[i];
 	}
@@ -186,14 +173,12 @@ mglParser::mglParser(bool setsize)
 	Cmd = mgls_base_cmd;
 	AllowSetSize=setsize;	AllowFileIO=true;
 	Once = true;
-	op1 = new wchar_t[4096];	op2 = new wchar_t[4096];
 	fval = new mglData[40];
 }
 //-----------------------------------------------------------------------------
 mglParser::~mglParser()
 {
-	DeleteAll();
-	delete []op1;	delete []op2;	delete []fval;
+	DeleteAll();	delete []fval;
 	if(Cmd && Cmd!=mgls_base_cmd)	delete []Cmd;
 }
 //-----------------------------------------------------------------------------
@@ -213,60 +198,42 @@ void mglParser::DeleteAll()
 //-----------------------------------------------------------------------------
 void mglParser::AddParam(int n, const char *str)
 {
-	unsigned s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	AddParam(n,wcs);
-	delete []wcs;
+	MGL_TO_WCS(str,AddParam(n,wcs));
 }
 //-----------------------------------------------------------------------------
 int mglParser::Parse(mglGraph *gr, const char *str, long pos)
 {
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	int r = Parse(gr,wcs,pos);
-	delete []wcs;	return r;
+	int r=0;
+	MGL_TO_WCS(str,r = Parse(gr,wcs,pos));
+	return r;
 }
 //-----------------------------------------------------------------------------
 mglVar *mglParser::AddVar(const char *str)
 {
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	mglVar *v = AddVar(wcs);
-	delete []wcs;
+	mglVar *v=0;
+	MGL_TO_WCS(str,v = AddVar(wcs));
 	return v;
 }
 //-----------------------------------------------------------------------------
 mglVar *mglParser::FindVar(const char *str)
 {
 	if(!str || *str==0) 	return DataList;
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	mglVar *v = FindVar(wcs);
-	delete []wcs;
+	mglVar *v=DataList;
+	MGL_TO_WCS(str,v = FindVar(wcs));
 	return v;
 }
 //-----------------------------------------------------------------------------
 mglNum *mglParser::AddNum(const char *str)
 {
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	mglNum *v = AddNum(wcs);
-	delete []wcs;
+	mglNum *v=0;
+	MGL_TO_WCS(str,v = AddNum(wcs));
 	return v;
 }
 //-----------------------------------------------------------------------------
 mglNum *mglParser::FindNum(const char *str)
 {
-	size_t s = mbstowcs(0,str,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,str,s);
-	mglNum *v = FindNum(wcs);
-	delete []wcs;
+	mglNum *v=0;
+	MGL_TO_WCS(str,v = FindNum(wcs));
 	return v;
 }
 //-----------------------------------------------------------------------------
@@ -319,10 +286,10 @@ mglNum *mglParser::AddNum(const wchar_t *name)
 	return v;
 }
 //-----------------------------------------------------------------------------
-int mglFindArg(const wchar_t *str)
+int mglFindArg(const std::wstring &str)
 {
-	register long l=0,k=0,i;//,j,len=strlen(lst);
-	for(i=0;i<long(wcslen(str));i++)
+	register long l=0,k=0,i;
+	for(i=0;i<long(str.length());i++)
 	{
 		if(str[i]=='\'') l++;
 		if(str[i]=='{') k++;
@@ -336,96 +303,63 @@ int mglFindArg(const wchar_t *str)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-bool mgls_suffix(const wchar_t *p, mglData *d, mreal *v)
-{
-	mreal x,y,z,k;
-	bool ok=false;
-	if(!wcscmp(p,L"a"))			{	ok = true;	*v = d->a[0];	}
-	else if(!wcscmp(p,L"fst"))	{	ok = true;	long i=-1,j=-1,l=-1;	*v = d->Find(0,i,j,l);	}
-	else if(!wcscmp(p,L"lst"))	{	ok = true;	long i=-1,j=-1,l=-1;	*v = d->Last(0,i,j,l);	}
-	else if(!wcscmp(p,L"nx"))	{	ok = true;	*v=d->nx;	}
-	else if(!wcscmp(p,L"ny"))	{	ok = true;	*v=d->ny;	}
-	else if(!wcscmp(p,L"nz"))	{	ok = true;	*v=d->nz;	}
-	else if(!wcscmp(p,L"max"))	{	ok = true;	*v=d->Maximal();	}
-	else if(!wcscmp(p,L"min"))	{	ok = true;	*v=d->Minimal();	}
-	else if(!wcscmp(p,L"sum"))	{	ok = true;	*v=d->Momentum('x',x,y);	}
-	else if(!wcscmp(p,L"mx"))	{	ok = true;	d->Maximal(x,y,z);		*v=x/d->nx;	}
-	else if(!wcscmp(p,L"my"))	{	ok = true;	d->Maximal(x,y,z);		*v=y/d->ny;	}
-	else if(!wcscmp(p,L"mz"))	{	ok = true;	d->Maximal(x,y,z);		*v=z/d->nz;	}
-	else if(!wcscmp(p,L"ax"))	{	ok = true;	d->Momentum('x',x,y);	*v=x/d->nx;	}
-	else if(!wcscmp(p,L"ay"))	{	ok = true;	d->Momentum('y',x,y);	*v=x/d->ny;	}
-	else if(!wcscmp(p,L"az"))	{	ok = true;	d->Momentum('z',x,y);	*v=x/d->nz;	}
-	else if(!wcscmp(p,L"wx"))	{	ok = true;	d->Momentum('x',x,y);	*v=y/d->nx;	}
-	else if(!wcscmp(p,L"wy"))	{	ok = true;	d->Momentum('y',x,y);	*v=y/d->ny;	}
-	else if(!wcscmp(p,L"wz"))	{	ok = true;	d->Momentum('z',x,y);	*v=y/d->nz;	}
-	else if(!wcscmp(p,L"sx"))	{	ok = true;	d->Momentum('x',x,y,z,k);	*v=z/d->nx;	}
-	else if(!wcscmp(p,L"sy"))	{	ok = true;	d->Momentum('y',x,y,z,k);	*v=z/d->ny;	}
-	else if(!wcscmp(p,L"sz"))	{	ok = true;	d->Momentum('z',x,y,z,k);	*v=z/d->nz;	}
-	else if(!wcscmp(p,L"kx"))	{	ok = true;	d->Momentum('x',x,y,z,k);	*v=k/d->nx;	}
-	else if(!wcscmp(p,L"ky"))	{	ok = true;	d->Momentum('y',x,y,z,k);	*v=k/d->ny;	}
-	else if(!wcscmp(p,L"kz"))	{	ok = true;	d->Momentum('z',x,y,z,k);	*v=k/d->nz;	}
-	else if(!wcscmp(p,L"aa"))	{	ok = true;	d->Momentum('a',x,y);	*v=x;	}
-	else if(!wcscmp(p,L"wa"))	{	ok = true;	d->Momentum('a',x,y);	*v=y;	}
-	else if(!wcscmp(p,L"sa"))	{	ok = true;	d->Momentum('a',x,y,z,k);*v=z;	}
-	else if(!wcscmp(p,L"ka"))	{	ok = true;	d->Momentum('a',x,y,z,k);*v=k;	}
-	return ok;
-}
-//-----------------------------------------------------------------------------
 // convert substrings to arguments
-mglData mglFormulaCalc(const wchar_t *string, mglParser *arg);
-void mglParser::FillArg(mglGraph *gr, int k, wchar_t **arg, mglArg *a)
+mglData MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const mglVar *head);
+void mglParser::FillArg(mglGraph *gr, int k, std::wstring *arg, mglArg *a)
 {
 	register long n;
 	for(n=1;n<k;n++)
 	{
-		mglVar *v, *u;
-		mglNum *f;
+		mglVar *v, *u;	mglNum *f;
 		a[n-1].type = -1;
 		if(arg[n][0]=='|')	a[n-1].type = -1;
-		else if(arg[n][0]=='\'')
-		{	// this is string (simplest case)
+		else if(arg[n][0]=='\'')	// this is string (simplest case)
+		{
 			a[n-1].type = 1;
-			wchar_t *w=arg[n],ch,*f,buf[32];
-			long i,l,ll=wcslen(w);
-			for(l=i=1;i<ll;i++)
+			std::wstring &w=arg[n],f;
+			wchar_t buf[32];
+			long i,i1,ll=w.length();
+			for(i=1;i<ll;i++)
 			{
 				if(w[i]=='\'')
 				{
 					if(i==ll-1)	continue;
-					i++;	f = w+i;
+					i++;	i1 = i;
+					if(w[i1]==',')	i1++;
+					if(w[i1]==0)	continue;
 					for(;i<ll && w[i]!='\'';i++);
-					ch=w[i];	w[i]=0;
-					if(*f==',')	f++;
-					if(*f==0)	continue;
-					mglData d = mglFormulaCalc(f, this);	w[i]=ch;
-					mglprintf(buf,32,L"%g",d.a[0]);	a[n-1].w += buf;
+					if(i>i1)
+					{
+						mglData d = mglFormulaCalc(w.substr(i1,i-i1-(w[i]=='\''?1:0)), this, DataList);
+						mglprintf(buf,32,L"%g",d.a[0]);	a[n-1].w += buf;
+					}
 				}
 				else	a[n-1].w += w[i];
 			}
 		}
 		else if(arg[n][0]=='{')
 		{	// this is temp data
-			arg[n][wcslen(arg[n])-1] = 0;
 			u=new mglVar;	u->temp=true;
-			a[n-1].w = L"/*"+std::wstring(arg[n]+1)+L"*/";
+			std::wstring s = arg[n].substr(1,arg[n].length()-2);
+			a[n-1].w = L"/*"+s+L"*/";
 			if(DataList)	u->MoveAfter(DataList);
 			else			DataList = u;
 			a[n-1].type = 0;	a[n-1].d = u;
-			ParseDat(gr, arg[n]+1, *u);
+			ParseDat(gr, s, *u);
 		}
-		else if((v = FindVar(arg[n]))!=0)	// have to find normal variables (for data creation)
+		else if((v = FindVar(arg[n].c_str()))!=0)	// have to find normal variables (for data creation)
 		{	a[n-1].type=0;	a[n-1].d=v;	a[n-1].w=v->s;	}
-		else if((f = FindNum(arg[n]))!=0)	// have to find normal variables (for data creation)
+		else if((f = FindNum(arg[n].c_str()))!=0)	// have to find normal variables (for data creation)
 		{	a[n-1].type=2;	a[n-1].d=0;	a[n-1].v=f->d;	a[n-1].w = f->s;	}
 		else
 		{	// parse all numbers and formulas by unified way
-			mglData d = mglFormulaCalc(arg[n], this);
+			mglData d = mglFormulaCalc(arg[n], this, DataList);
 			if(d.nx*d.ny*d.nz==1)
 			{	a[n-1].type = 2;	a[n-1].v = d.a[0];	}
 			else
 			{
 				u=new mglVar;	u->temp=true;	u->Set(d);
-				a[n-1].w = L"/*"+std::wstring(arg[n])+L"*/";
+				a[n-1].w = L"/*"+arg[n]+L"*/";
 				if(DataList)	u->MoveAfter(DataList);
 				else			DataList = u;
 				a[n-1].type = 0;	a[n-1].d = u;
@@ -435,18 +369,17 @@ void mglParser::FillArg(mglGraph *gr, int k, wchar_t **arg, mglArg *a)
 }
 //-----------------------------------------------------------------------------
 // return values: 0 - not found, 1 - OK, 2 - wrong arguments, 3 - wrong command, 4 - string too long
-int mglParser::PreExec(mglGraph *, long k, wchar_t **arg, mglArg *a)
+int mglParser::PreExec(mglGraph *, long k, std::wstring *arg, mglArg *a)
 {
 	long n=0;
 	mglVar *v;
-	if(!wcscmp(L"delete",arg[0]))	// parse command "delete"
+	if(!arg[0].compare(L"delete") && k==2)	// parse command "delete"
 	{
-		if(k<2)	return 2;
-		DeleteVar(arg[1]);	n=1;
+		DeleteVar(arg[1].c_str());	n=1;
 	}
-	else if(!wcscmp(L"list",arg[0]))	// parse command "list"
+	else if(!arg[0].compare(L"list"))	// parse command "list"
 	{
-		if(k<3 || check_for_name(arg[1]))	return 2;
+		if(k<3 || mgl_check_for_name(arg[1]))	return 2;
 		long nx=0, ny=1,j=0,i,t=0;
 		char ch;
 		for(i=2;i<k;i++)
@@ -465,7 +398,7 @@ int mglParser::PreExec(mglGraph *, long k, wchar_t **arg, mglArg *a)
 			}
 			if(ch=='|' && t==1)		{	nx = j>nx ? j:nx;	j=0;	ny++;	}
 		}
-		v = AddVar(arg[1]);
+		v = AddVar(arg[1].c_str());
 		if(t==1)	nx = j>nx ? j:nx;
 		if(t==1)	// list of numeric values
 		{
@@ -513,135 +446,133 @@ int mglParser::PreExec(mglGraph *, long k, wchar_t **arg, mglArg *a)
 	return n;
 }
 //-----------------------------------------------------------------------------
-void mglParser::PutArg(const wchar_t *string, wchar_t *str, bool def)
+void mglParser::PutArg(std::wstring &str, bool def)
 {
-	wchar_t *sb = new wchar_t[wcslen(string)+1], *t;
-	if(def)	str = str+10;
-	register long n;
-	while((t=wcschr(str,'$'))!=0)
+	size_t pos = str.find('$',def?10:0);
+	while(pos<str.length())
 	{
-		wcscpy(sb,t+2);	t[0]=0;
-		n = t[1]-'0';	if(n>=0 && n<=9)	wcscat(str,par[n].c_str());
-		n = t[1]-'a';	if(n>=0 && n<='z'-'a')	wcscat(str,par[n+10].c_str());
-		if(t[1]=='$')	wcscat(str,L"\xffff");
-		wcscat(str,sb);
+		wchar_t ch = str[pos+1];
+		if(ch>='0' && ch<='9')	str.replace(pos,2,par[ch-'0']);
+		if(ch>='a' && ch<='z')	str.replace(pos,2,par[ch-'a'+10]);
+		if(ch=='$')	str.replace(pos,2,L"\xffff");
+		pos = str.find('$',def?10:0);
 	}
-	delete []sb;
-	while((t=wcschr(str,L'\xffff'))!=0)	*t='$';
+	while((pos = str.find(L'\xffff'))<str.length())	str[pos]='$';
+}
+//-----------------------------------------------------------------------------
+std::wstring mgl_trim_ws(const std::wstring &str)
+{
+	size_t n=str.length(), k, i;
+	for(k=0;k<n;k++)	if(str[k]>' ')	break;
+	for(i=n;i>k;i--)	if(str[i-1]>' ')	break;
+	return str.substr(k,i-k);
+}
+//-----------------------------------------------------------------------------
+int mglParser::ParseDef(std::wstring &str)
+{
+	if(!skip() && !str.compare(0,3,L"def") && (str[6]==' ' || str[6]=='\t'))
+	{
+		int res = 1;	mreal d;
+		PutArg(str,true);
+		const std::wstring s = mgl_trim_ws(str.substr(7));
+		if(!str.compare(3,3,L"ine"))
+		{
+			int nn = s[1]<='9' ? s[1]-'0' : (s[1]>='a' ? s[1]-'a'+10:-1);
+			if(s[0]=='$' && nn>=0 && nn<='z'-'a'+10)
+			{
+				AddParam(nn, mgl_trim_ws(s.substr(2)).c_str());	return 1;
+			}
+		}
+		if(!str.compare(3,3,L"num"))
+		{
+			int nn = s[1]<='9' ? s[1]-'0' : (s[1]>='a' ? s[1]-'a'+10:-1);
+			if(s[0]=='$' && nn>=0 && nn<='z'-'a'+10)
+			{
+				res = 0;
+				d = mglFormulaCalc(mgl_trim_ws(s.substr(2)), this, DataList).a[0];
+				char buf[32];	snprintf(buf,32,"%g",d);	AddParam(nn, buf);
+			}
+			return res+1;
+		}
+		if(!str.compare(3,3,L"chr"))
+		{
+			int nn = s[1]<='9' ? s[1]-'0' : (s[1]>='a' ? s[1]-'a'+10:-1);
+			if(s[0]=='$' && nn>=0 && nn<='z'-'a'+10)
+			{
+				res = 0;
+				d=mglFormulaCalc(mgl_trim_ws(s.substr(2)), this, DataList).a[0];
+				wchar_t buf[2]={0,0};	buf[0] = wchar_t(d);	AddParam(nn, buf);
+			}
+			return res+1;
+		}
+	}
+	if(!skip() && !str.compare(0,3,L"ask") && (str[3]==' ' || str[3]=='\t'))
+	{
+		PutArg(str,true);
+		std::wstring s = mgl_trim_ws(str.substr(4));
+		int nn = s[1]<='9' ? s[1]-'0' : (s[1]>='a' ? s[1]-'a'+10:-1);
+		if(s[0]=='$' && nn>=0 && nn<='z'-'a'+10)
+		{
+			static wchar_t res[1024];
+			s = mgl_trim_ws(s.substr(2));
+			if(s[0]=='\'')	s=s.substr(1,s.length()-2);
+			if(mgl_ask_func)
+			{	mgl_ask_func(s.c_str(),res);	if(*res)	AddParam(nn, res);	}
+			return mgl_ask_func?1:2;
+		}
+		else	return 2;
+	}
+	if(!skip() && !str.compare(0,3,L"for") && (str[3]==' ' || str[3]=='\t'))
+	{
+		size_t i;
+		for(i=4;str[i]<=' ';i++);
+		// if command have format 'for $N ...' then change it to 'for N ...'
+		if(str[i]=='$' && str[i+1]>='0' && str[i+1]<='9')	str[i] = ' ';
+		if(str[i]=='$' && str[i+1]>='a' && str[i+1]<='z')	str[i] = ' ';
+	}
+	return 0;
 }
 //-----------------------------------------------------------------------------
 // return values: 0 - OK, 1 - wrong arguments, 2 - wrong command, 3 - string too long, 4 -- unclosed string
-int mglParser::Parse(mglGraph *gr, const wchar_t *string, long pos)
+int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 {
 	if(Stop)	return 0;
-	size_t lstr = wcslen(string)+2+GetParLen(string);
-	wchar_t *str, *arg[1024],*t;
-	wchar_t *s = new wchar_t[lstr];
-	memset(s,0,lstr*sizeof(wchar_t));
-	wcscpy(s,string);	mgl_wcstrim(s);	str = s;
-	long n,k=0,m=0,mm=0;
+	std::wstring arg[1024];
+	str=mgl_trim_ws(str);
+	long n,k=0,m=0,mm=0,res;
 	// try parse ':' -- several commands in line
-	for(n=0;n<long(wcslen(str));n++)
+	for(n=0;n<long(str.length());n++)
 	{
 		if(str[n]=='\'' && (n==0 || str[n-1]!='\\'))	k++;
 		if(k%2)	continue;
-		if(str[n]=='(')	m++;
-		if(str[n]==')')	m--;
-		if(str[n]=='{')	mm++;
-		if(str[n]=='}')	mm--;
+		if(str[n]=='(')	m++;	if(str[n]==')')	m--;
+		if(str[n]=='{')	mm++;	if(str[n]=='}')	mm--;
 		if(str[n]=='#')	break;
 		if((str[n]==':' || str[n]=='\n') && k%2==0 && m==0 && mm==0)
 		{
-			str[n]=0;
-			int res=Parse(gr,str,pos);
-			if(!res)	res=Parse(gr,str+n+1,pos);
-			delete []s;	return res;
+			res=Parse(gr,str.substr(0,n),pos);
+			if(!res)	res=Parse(gr,str.substr(n+1),pos);
+			return res;
 		}
 	}
 	if(k%2 || m || mm)	return 4;	// strings is not closed
 	// define parameters or start cycle
-	if(!skip() && !wcsncmp(str,L"def",3) && (str[6]==' ' || str[6]=='\t'))
-	{
-		PutArg(string,str,true);
-		if(!wcsncmp(str+3,L"ine",3))
-		{
-			str += 7;	mgl_wcstrim(str);//	int res = 1;
-			int nn = str[1]<='9' ? str[1]-'0' : (str[1]>='a' ? str[1]-'a'+10:-1);
-			if(*str=='$' && nn>=0 && nn<='z'-'a'+10)
-			{
-				str +=2;	mgl_wcstrim(str);
-				AddParam(nn, str);	delete []s;	return 0;
-			}
-			else	str -= 7;
-		}
-		if(!wcsncmp(str+3,L"num",3))
-		{
-			str += 7;	mgl_wcstrim(str);	int res = 1;
-			int nn = str[1]<='9' ? str[1]-'0' : (str[1]>='a' ? str[1]-'a'+10:-1);
-			if(*str=='$' && nn>=0 && nn<='z'-'a'+10)
-			{
-				res = 0;	str +=2;	mgl_wcstrim(str);
-				mreal d = mglFormulaCalc(str, this).a[0];
-				char *buf=new char[128];
-				snprintf(buf,128,"%g",d);
-				AddParam(nn, buf);	delete []buf;
-			}
-			delete []s;	return res;
-		}
-		if(!wcsncmp(str+3,L"chr",3))
-		{
-			str += 7;	mgl_wcstrim(str);	int res = 1;
-			int nn = str[1]<='9' ? str[1]-'0' : (str[1]>='a' ? str[1]-'a'+10:-1);
-			if(*str=='$' && nn>=0 && nn<='z'-'a'+10)
-			{
-				res = 0;	str +=2;	mgl_wcstrim(str);
-				mreal d=mglFormulaCalc(str, this).a[0];
-				wchar_t buf[2]={0,0};	buf[0] = wchar_t(d);
-				AddParam(nn, buf);
-			}
-			delete []s;	return res;
-		}
-	}
-	if(!skip() && !wcsncmp(str,L"ask",3) && (str[3]==' ' || str[3]=='\t'))
-	{
-		PutArg(string,str,true);
-		str += 4;	mgl_wcstrim(str);//	int res = 1;
-		int nn = str[1]<='9' ? str[1]-'0' : (str[1]>='a' ? str[1]-'a'+10:-1);
-		if(*str=='$' && nn>=0 && nn<='z'-'a'+10)
-		{
-			static wchar_t res[1024];
-			str +=2;	mgl_wcstrim(str);
-			if(*str=='\'')	{	str++;	str[wcslen(str)-1]=0;	}
-			if(mgl_ask_func)
-			{	mgl_ask_func(str,res);	if(*res)	AddParam(nn, res);	}
-			delete []s;	return mgl_ask_func?0:1;
-		}
-		else	return 1;
-	}
-	wcscpy(str,string);			mgl_wcstrim(str);
-	if(!skip() && !wcsncmp(str,L"for",3) && (str[3]==' ' || str[3]=='\t'))
-	{
-		for(t=str+4;*t<=' ';t++);
-		// if command have format 'for $N ...' then change it to 'for N ...'
-		if(*t=='$' && t[1]>='0' && t[1]<='9')	*t = ' ';
-		if(*t=='$' && t[1]>='a' && t[1]<='z')	*t = ' ';
-	}
-
+	res = ParseDef(str);	if(res)	return res-1;
 	// parse arguments (parameters $1, ..., $9)
-	PutArg(string,str,false);	mgl_wcstrim(str);
+	PutArg(str,false);	str=mgl_trim_ws(str);
 
-	wchar_t *opt=0;
+	std::wstring opt;
 	for(k=0;k<1024;k++)	// parse string to substrings (by spaces)
 	{
 		n = mglFindArg(str);
 		if(n<1)	// this is option
 		{
-			if(str[-n]==';')	opt = str-n+1;
-			if(n<0)	str[-n]=0;
+			if(str[-n]==';')	opt = str.substr(-n+1);
+			if(n<0)	str = str.substr(0,-n);
 			break;
 		}
-		str[n]=0;	arg[k] = str;//	k++;
-		str = str+n+1;	mgl_wcstrim(str);
+		arg[k] = str.substr(0,n);
+		str = mgl_trim_ws(str.substr(n+1));
 	}
 	// try to find last argument
 	if(str[0]!=0 && str[0]!='#' && str[0]!=';')	{	arg[k] = str;	k++;	}
@@ -652,23 +583,23 @@ int mglParser::Parse(mglGraph *gr, const wchar_t *string, long pos)
 		mglArg *a = new mglArg[k];
 		FillArg(gr, k, arg, a);
 		// execute first special (program-flow-control) commands
-		if(!skip() && !wcscmp(arg[0],L"stop"))
-		{	Stop = true;	delete []s;	delete []a;	return 0;	}
-		if(!wcscmp(arg[0],L"func"))
-		{	delete []s;	delete []a;	return 0;	}
-		n = FlowExec(gr, arg[0],k-1,a);
-		if(n)		{	delete []s;	delete []a;	return n-1;	}
-		if(skip())	{	delete []s;	delete []a;	return 0;	}
-		if(!wcscmp(arg[0],L"define"))
+		if(!skip() && !arg[0].compare(L"stop"))
+		{	Stop = true;	delete []a;	return 0;	}
+		if(!arg[0].compare(L"func"))
+		{	delete []a;	return 0;	}
+		n = FlowExec(gr, arg[0].c_str(),k-1,a);
+		if(n)		{	delete []a;	return n-1;	}
+		if(skip())	{	delete []a;	return 0;	}
+		if(!arg[0].compare(L"define"))
 		{
 			if(k==3)
 			{
-				mglNum *v=AddNum(arg[1]);
-				v->d = mglFormulaCalc(arg[2],this).a[0];
+				mglNum *v=AddNum(arg[1].c_str());
+				v->d = mglFormulaCalc(arg[2],this, DataList).a[0];
 			}
-			delete []s;	delete []a;	return k==3?0:1;
+			delete []a;	return k==3?0:1;
 		}
-		if(!wcscmp(arg[0],L"call"))
+		if(!arg[0].compare(L"call"))
 		{
 			n = 1;
 			if(a[0].type==1)
@@ -687,7 +618,7 @@ int mglParser::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				{
 					mglFnStack fn;			fn.pos = pos;
 					for(int i=0;i<10;i++)	{	fn.par[i] = par[i];	par[i]=L"";	}
-					for(int i=1;i<k-1;i++)	AddParam(i,arg[i+1]);
+					for(int i=1;i<k-1;i++)	AddParam(i,arg[i+1].c_str());
 					fn_stack.push_back(fn);	n--;
 				}
 				else if(AllowFileIO)	// disable external scripts if AllowFileIO=false
@@ -708,16 +639,16 @@ int mglParser::Parse(mglGraph *gr, const wchar_t *string, long pos)
 					else	n=1;
 				}
 			}
-			delete []s;	delete []a;	return n;
+			delete []a;	return n;
 		}
-		if(!wcscmp(arg[0],L"for"))
+		if(!arg[0].compare(L"for"))
 		{
 			n = 1;
 			char ch = arg[1][0];
 			int r = ch-'0';
 			if(ch>='a' && ch<='z')	r = 10+ch-'a';
 //			int r = int(a[0].v);
-			if(arg[1][1]==0 && (r>=0 || r<=39))
+			if(arg[1][1]==0 && (r>=0 && r<40))	// TODO: check this
 			{
 				if(a[1].type==0)
 				{
@@ -744,13 +675,13 @@ int mglParser::Parse(mglGraph *gr, const wchar_t *string, long pos)
 					AddParam(r, buf);	fval[r].ny = 1;
 				}
 			}
-			delete []s;	delete []a;	return n;
+			delete []a;	return n;
 		}
 		// alocate new arrays and execute the command itself
 		n = PreExec(gr, k, arg, a);
 		if(n>0)	n--;
-		else if(!wcscmp(L"setsize",arg[0]) && !AllowSetSize)	n = 2;
-		else	n = Exec(gr, arg[0],k-1,a, arg[1], opt);
+		else if(!arg[0].compare(L"setsize") && !AllowSetSize)	n = 2;
+		else	n = Exec(gr, arg[0].c_str(),k-1,a, arg[1].c_str(), opt.c_str());
 		delete []a;
 	}
 	mglVar *v = DataList, *u;
@@ -760,25 +691,24 @@ int mglParser::Parse(mglGraph *gr, const wchar_t *string, long pos)
 		if(v->temp)	{	if(DataList==v)	DataList = v->next;		delete v;	}
 		v = u;
 	}
-	delete []s;	return n;
+	return n;
 }
 //-----------------------------------------------------------------------------
 // return values: 0 - OK, 1 - wrong arguments, 2 - wrong command, 3 - string too long, 4 -- unclosed string
-int mglParser::ParseDat(mglGraph *gr, const wchar_t *string, mglData &res)
+int mglParser::ParseDat(mglGraph *gr, std::wstring str, mglData &res)
 {
-	wchar_t *str, *s = new wchar_t[wcslen(string)+1+GetParLen(string)],*arg[32];
-	str = s;
-	wcscpy(str,string);	mgl_wcstrim(str);
+	std::wstring arg[32];
+	str = mgl_trim_ws(str);
 	long n,k=0;
 	for(k=0;k<32;k++)	// parse string to substrings (by spaces)
 	{
 		n = mglFindArg(str);
-		if(n<1)	{	if(n<0)	str[-n]=0;	break;	}
-		str[n]=0;	arg[k] = str;//	k++;
-		str = str+n+1;	mgl_wcstrim(str);
+		if(n<1)	{	if(n<0)	str=str.substr(0,-n);	break;	}
+		arg[k] = str.substr(0,n);//	k++;
+		str = str.substr(n+1);	str = mgl_trim_ws(str);
 	}
 	// try to find last argument
-	if(str[0]!=0 && str[0]!='#' && str[0]!=';')	{	arg[k] = str;	k++;	}
+	if(!str.empty())	{	arg[k] = str;	k++;	}
 	if(k<1) n =0;
 	else
 	{	// fill arguments by its values
@@ -793,18 +723,18 @@ int mglParser::ParseDat(mglGraph *gr, const wchar_t *string, mglData &res)
 			kk += id[a[i].type];
 			a[i].s.assign(a[i].w.begin(),a[i].w.end());
 		}
-		mglCommand *rts=FindCommand(arg[0]);
+		mglCommand *rts=FindCommand(arg[0].c_str());
 		if(!rts || rts->type!=4)	n = 2;
 		else n = rts->exec(gr, k, a, kk.c_str(), 0);
 		delete []a;
 	}
-	delete []s;	return n;
+	return n;
 }
 //-----------------------------------------------------------------------------
-int mglParser::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
+int mglParser::FlowExec(mglGraph *, const std::wstring &com, long m, mglArg *a)
 {
 	int n=-1;
-	if(!ifskip() && !wcscmp(com,L"once"))
+	if(!ifskip() && !com.compare(L"once"))
 	{
 		if(a[0].type==2)
 		{
@@ -814,7 +744,7 @@ int mglParser::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 		}
 		else n = 1;
 	}
-	else if(!Skip && !wcscmp(com,L"if"))
+	else if(!Skip && !com.compare(L"if"))
 	{
 		int cond;
 		if(a[0].type==2)
@@ -828,15 +758,15 @@ int mglParser::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 		if(n==0)
 		{	if_stack[if_pos] = cond;	if_pos = if_pos<39 ? if_pos+1 : 39;	}
 	}
-	else if(!Skip && !wcscmp(com,L"endif"))
+	else if(!Skip && !com.compare(L"endif"))
 	{	if_pos = if_pos>0 ? if_pos-1 : 0;	n = 0;	}
-	else if(!Skip && !wcscmp(com,L"else"))
+	else if(!Skip && !com.compare(L"else"))
 	{
 		if(if_pos>0)
 		{	n=0; if_stack[if_pos-1] = (if_stack[if_pos-1]&2)?2:3;	}
 		else n = 1;
 	}
-	else if(!Skip && !wcscmp(com,L"elseif"))
+	else if(!Skip && !com.compare(L"elseif"))
 	{
 		int cond;
 		if(if_pos<1 || m<1)	n = 1;
@@ -851,18 +781,18 @@ int mglParser::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 		else n = 1;
 		if(n==0)	if_stack[if_pos-1] = cond;
 	}
-	else if(!ifskip() && !Skip && !wcscmp(com,L"break"))
+	else if(!ifskip() && !Skip && !com.compare(L"break"))
 	{
 		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
 	}
-	else if(!skip() && !wcscmp(com, L"return"))	// parse command "delete"
+	else if(!skip() && !com.compare(L"return"))
 	{
 		if(fn_stack.size()<1)	return 2;
 		const mglFnStack &fn=fn_stack.back();
 		for(int i=0;i<10;i++)	par[i]=fn.par[i];
 		n = -fn.pos-1;	fn_stack.pop_back();
 	}
-	else if(!ifskip() && !Skip && !wcscmp(com,L"next"))
+	else if(!ifskip() && !Skip && !com.compare(L"next"))
 	{
 		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
 		int r = for_stack[0]-1;
@@ -883,7 +813,7 @@ int mglParser::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 			}
 		}
 	}
-	else if(!ifskip() && !Skip && !wcscmp(com,L"continue"))
+	else if(!ifskip() && !Skip && !com.compare(L"continue"))
 	{
 		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
 		int r = for_stack[0]-1;
@@ -917,7 +847,7 @@ void mglParser::Execute(mglGraph *gr, FILE *fp, bool print)
 void mglParser::Execute(mglGraph *gr, int n, const wchar_t **text)
 {
 	if(n<1 || text==0)	return;
-	long i, r;
+	long i, r, res=0;
 	char buf[64];
 	for_br=Skip=false;	if_pos=0;	ScanFunc(0);	fn_stack.clear();
 	for(i=0;i<n;i++)	ScanFunc(text[i]);
@@ -934,22 +864,24 @@ void mglParser::Execute(mglGraph *gr, int n, const wchar_t **text)
 		else if(gr->GetWarn()>0)	snprintf(buf,64," in line %ld\n", i+1);
 		else *buf=0;
 		if(*buf)	gr->SetWarn(-2,buf);
+		if(r>0 && r<5)	res=r;
 	}
+	int code[]={mglScrArg,	mglScrCmd,	mglScrLong,	mglScrStr};
+	if(res>0)	gr->SetWarn(code[res-1],"MGL Parser");
 }
 //-----------------------------------------------------------------------------
 void mglParser::Execute(mglGraph *gr, const wchar_t *text)
 {
-	size_t s = wcslen(text)+1;
+	size_t s = mgl_wcslen(text)+1, n=1;
 	wchar_t *wcs = new wchar_t[s];
 	const wchar_t **str;
-	register size_t i, n=1;
-	for(i=0;i<s;i++)	if(text[i]=='\n')	n++;
+	for(size_t i=0;i<s;i++)	if(text[i]=='\n')	n++;
 	str = (const wchar_t **)malloc(n*sizeof(wchar_t *));
 	memcpy(wcs, text, s*sizeof(wchar_t));
 	str[0] = wcs;	n=1;
 	long next=0;
 	Stop = false;
-	for(i=0;i<s;i++)
+	for(size_t i=0;i<s;i++)
 	{
 		if(text[i]=='\\')	next = i;
 		else if(text[i]>' ')next = 0;
@@ -970,12 +902,7 @@ void mglParser::Execute(mglGraph *gr, const wchar_t *text)
 //-----------------------------------------------------------------------------
 void mglParser::Execute(mglGraph *gr, const char *text)
 {
-	size_t s = mbstowcs(0,text,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	memset(wcs,0,s*sizeof(wchar_t));
-	mbstowcs(wcs,text,s);
-	Execute(gr, wcs);
-	delete []wcs;
+	MGL_TO_WCS(text,	Execute(gr, wcs));
 }
 //-----------------------------------------------------------------------------
 void mglParser::DeleteVar(mglVar *v)
@@ -1051,7 +978,7 @@ uintptr_t MGL_EXPORT mgl_parser_add_var_(uintptr_t* p, const char *name, int l)
 uintptr_t MGL_EXPORT mgl_parser_find_var_(uintptr_t* p, const char *name, int l)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
 	mglVar *v=_PR_->FindVar(s);	delete []s;	return uintptr_t(v);	}
-void MGL_EXPORT mgl_del_var_(uintptr_t* p, const char *name, int l)
+void MGL_EXPORT mgl_parser_del_var_(uintptr_t* p, const char *name, int l)
 {	char *s=new char[l+1];		memcpy(s,name,l);	s[l]=0;
 	_PR_->DeleteVar(s);	delete []s;	}
 int MGL_EXPORT mgl_parse_line_(uintptr_t* gr, uintptr_t* p, const char *str, int *pos, int l)
@@ -1097,14 +1024,12 @@ long MGL_EXPORT mgl_parser_cmd_num(HMPR pr)
 //---------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_parser_calc(HMPR pr, const char *formula)
 {
-	size_t s = mbstowcs(0,formula,0)+1;
-	wchar_t *wcs = new wchar_t[s];
-	mbstowcs(wcs,formula,s);
-	HMDT d = mgl_parser_calcw(pr,wcs);
-	delete []wcs;	return d;	
+	HMDT d=0;
+	MGL_TO_WCS(formula,d = mgl_parser_calcw(pr,wcs));
+	return d;
 }
 HMDT MGL_EXPORT mgl_parser_calcw(HMPR pr, const wchar_t *formula)
-{	mglData *d = new mglData(mglFormulaCalc(formula,pr)); 	return d;	}
+{	mglData *d = new mglData(mglFormulaCalc(formula,pr, pr->DataList)); 	return d;	}
 uintptr_t MGL_EXPORT mgl_parser_calc_(uintptr_t *p, const char *str,int l)
 {	char *s=new char[l+1];	memcpy(s,str,l);	s[l]=0;
 	uintptr_t d = (uintptr_t)mgl_parser_calc(_PR_, s);	delete []s;	return d;	}
