@@ -1,6 +1,6 @@
 /***************************************************************************
  * pde.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -18,9 +18,10 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include "mgl2/data.h"
+#include "mgl2/datac.h"
 #include "mgl2/eval.h"
-#include <complex>
-#define dual	std::complex<double>
+#include "mgl2/thread.h"
+#include "mgl2/base.h"
 #define GAMMA	0.1
 //-----------------------------------------------------------------------------
 struct mgl_pde_ham
@@ -36,54 +37,64 @@ MGL_NO_EXPORT void *mgl_pde_hprep(void *par)
 	mglThreadD *t=(mglThreadD *)par;
 	const mgl_pde_ham *f = (const mgl_pde_ham *)t->v;
 	mglFormula *eqs = f->eqs;
-	register long i,j,i0, nx=2*f->nx, ny=2*f->ny;
-	mreal var[MGL_VS];
-	dual *a = f->a,*h;
-	memset(var,0,('z'-'a')*sizeof(mreal));
-	var['z'-'a'] = f->zz;
-	for(i0=t->id,h=f->hxy;i0<t->n;i0+=mglNumThr)
+	long nx=2*f->nx, ny=2*f->ny;
+	dual *a = f->a;
+
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel
+#endif
 	{
-		i = i0%nx;	j = i0/nx;			var['u'-'a'] = abs(a[i0]);
-		var['x'-'a'] = f->xx+f->dx*i;	var['p'-'a'] = 0;
-		var['y'-'a'] = f->yy+f->dy*j;	var['q'-'a'] = 0;
-		h[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
-	}
-	if(f->ny>2)	for(i0=t->id,h=f->huy;i0<t->n;i0+=mglNumThr)	// step 3/2
-	{
-		i = i0%nx;	j = i0/nx;			var['u'-'a'] = abs(a[i0]);
-		var['x'-'a'] = f->xs;			var['p'-'a'] = f->dp*(i<nx/2 ? i:nx-i);
-		var['y'-'a'] = f->yy+f->dy*j;	var['q'-'a'] = 0;
-		h[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
-	}
-	for(i0=t->id,h=f->huv;i0<t->n;i0+=mglNumThr)	// step 2
-	{
-		i = i0%nx;	j = i0/nx;			var['u'-'a'] = abs(a[i0]);
-		var['x'-'a'] = f->xs;			var['p'-'a'] = f->dp*(i<nx/2 ? i:nx-i);
-		var['y'-'a'] = f->ys;			var['q'-'a'] = f->dq*(j<ny/2 ? j:ny-j);
-		h[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
-	}
-	if(f->ny>2)	for(i0=t->id,h=f->hxv;i0<t->n;i0+=mglNumThr)	// step 3/2
-	{
-		i = i0%nx;	j = i0/nx;			var['u'-'a'] = abs(a[i0]);
-		var['x'-'a'] = f->xx+f->dx*i;	var['p'-'a'] = 0;
-		var['y'-'a'] = f->ys;			var['q'-'a'] = f->dq*(j<ny/2 ? j:ny-j);
-		h[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
+		mreal var[MGL_VS];	memset(var,0,MGL_VS*sizeof(mreal));
+		var['z'-'a'] = f->zz;
+#pragma omp for nowait
+		for(long i0=t->id;i0<t->n;i0+=mglNumThr)
+		{
+			register long i = i0%nx, j = i0/nx;		var['u'-'a'] = abs(a[i0]);
+			var['x'-'a'] = f->xx+f->dx*i;	var['p'-'a'] = 0;
+			var['y'-'a'] = f->yy+f->dy*j;	var['q'-'a'] = 0;
+			f->hxy[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
+		}
+		if(f->ny>2)
+#pragma omp for nowait
+			for(long i0=t->id;i0<t->n;i0+=mglNumThr)	// step 3/2
+			{
+				register long i = i0%nx, j = i0/nx;		var['u'-'a'] = abs(a[i0]);
+				var['x'-'a'] = f->xs;			var['p'-'a'] = f->dp*(i<nx/2 ? i:nx-i);
+				var['y'-'a'] = f->yy+f->dy*j;	var['q'-'a'] = 0;
+				f->huy[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
+			}
+#pragma omp for nowait
+		for(long i0=t->id;i0<t->n;i0+=mglNumThr)	// step 2
+		{
+			register long i = i0%nx, j = i0/nx;		var['u'-'a'] = abs(a[i0]);
+			var['x'-'a'] = f->xs;			var['p'-'a'] = f->dp*(i<nx/2 ? i:nx-i);
+			var['y'-'a'] = f->ys;			var['q'-'a'] = f->dq*(j<ny/2 ? j:ny-j);
+			f->huv[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
+		}
+		if(f->ny>2)
+#pragma omp for nowait
+			for(long i0=t->id;i0<t->n;i0+=mglNumThr)	// step 3/2
+			{
+				register long i = i0%nx, j = i0/nx;		var['u'-'a'] = abs(a[i0]);
+				var['x'-'a'] = f->xx+f->dx*i;	var['p'-'a'] = 0;
+				var['y'-'a'] = f->ys;			var['q'-'a'] = f->dq*(j<ny/2 ? j:ny-j);
+				f->hxv[i0] = dual(-eqs->CalcD(var,'i'), eqs->Calc(var))*f->dd;
+			}
 	}
 	return 0;
 }
+//-----------------------------------------------------------------------------
 // Solve equation dx/dz = func(p,q,x,y,z,|u|)[u] where p=d/dx, q=d/dy. At this moment simplified form of ham is supported: ham = f(p,q,z) + g(x,y,z,'u'), where variable 'u'=|u| (for allowing solve nonlinear problems). You may specify imaginary part like ham = p^2 + i*x*(x>0) but only if dependence on variable 'i' is linear (i.e. ham = hre+i*him).
-HMDT MGL_EXPORT mgl_pde_solve(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im, mreal dz, mreal k0, const char *opt)
+HADT MGL_EXPORT mgl_pde_solve_c(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im, mreal dz, mreal k0, const char *opt)
 {
 	gr->SaveState(opt);
 	mglPoint Min=gr->Min, Max=gr->Max;
-	mglData *res=new mglData;
-	int nx=ini_re->GetNx(), ny=ini_re->GetNy();
-	int nz = int((Max.z-Min.z)/dz)+1;
+	long nx=ini_re->GetNx(), ny=ini_re->GetNy(), nz = long((Max.z-Min.z)/dz)+1;
 	if(nx<2 || nz<2 || Max.x==Min.x)			// Too small data
-	{	gr->SetWarn(mglWarnLow,"PDE");	return res;	}
+	{	gr->SetWarn(mglWarnLow,"PDE");	return 0;	}
 	if(ini_im->GetNx()*ini_im->GetNy() != nx*ny)// Wrong dimensions
-	{	gr->SetWarn(mglWarnDim,"PDE");	return res;	}
-	mgl_data_create(res, nz, nx, ny);
+	{	gr->SetWarn(mglWarnDim,"PDE");	return 0;	}
+	mglDataC *res=new mglDataC(nz, nx, ny);
 
 	mglFormula eqs(ham);
 	dual *a = new dual[4*nx*ny], hh0;	// Add "damping" area
@@ -93,17 +104,18 @@ HMDT MGL_EXPORT mgl_pde_solve(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im
 	dual *hy = new dual[2*ny], *hu = new dual[2*nx];
 	mreal *dmp = new mreal[4*nx*ny];
 	memset(a,0,4*nx*ny*sizeof(dual));
-	register int i,j,k,i0;
-	for(j=0;j<ny;j++)	for(i=0;i<nx;i++)	// Initial conditions
-	{
-		i0 = i+nx/2+2*nx*(j+ny/2);
-		a[i0] = dual(ini_re->v(i,j), ini_im->v(i,j));
-		res->a[nz*(i+nx*j)] = abs(a[i0]);
-	}
 	memset(dmp,0,4*nx*ny*sizeof(mreal));
-	for(j=0;j<2*ny;j++)	for(i=0;i<2*nx;i++)	// step 1
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)	// Initial conditions
 	{
-		i0 = i+2*nx*j;
+		register long i0 = i+nx/2+2*nx*(j+ny/2);
+		a[i0] = dual(ini_re->v(i,j), ini_im->v(i,j));
+		res->a[nz*(i+nx*j)] = a[i0];
+	}
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<2*ny;j++)	for(long i=0;i<2*nx;i++)	// step 1
+	{
+		register long i0 = i+2*nx*j;
 		if(i<nx/2)		dmp[i0] += GAMMA*mgl_ipow((nx/2-i)/(nx/2.),2);
 		if(i>3*nx/2)	dmp[i0] += GAMMA*mgl_ipow((i-3*nx/2-1)/(nx/2.),2);
 		if(j<ny/2)		dmp[i0] += GAMMA*mgl_ipow((ny/2-j)/(ny/2.),2);
@@ -113,56 +125,83 @@ HMDT MGL_EXPORT mgl_pde_solve(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im
 	mreal dp = M_PI/(Max.x-Min.x)/k0, dq = M_PI/(Max.y-Min.y)/k0;
 	mreal xs=(Min.x+Max.x)/2, ys=(Min.y+Max.y)/2;
 //	double xx = Min.x - dx*nx/2, yy = Min.x - dy*ny/2;
-	double ff = ny>1?4*nx*ny:2*nx, dd = k0*dz;
+	double dd = k0*dz;
 
 	mgl_pde_ham tmp;tmp.eqs = &eqs;
-	tmp.nx = nx;	tmp.ny = ny;	tmp.a=a;		tmp.hxy=hxy;
-	tmp.hxv=hxv;	tmp.huy=huy;	tmp.huv=huv;	tmp.dd = dd;
+	tmp.nx = nx;	tmp.ny = ny;	tmp.dd = dd;	tmp.a=a;
+	tmp.hxy=hxy;	tmp.hxv=hxv;	tmp.huy=huy;	tmp.huv=huv;
 	tmp.xx = Min.x-dx*(nx/2);	tmp.xs = xs;	tmp.dx = dx;	tmp.dp = dp;
 	tmp.yy = Min.y-dy*(ny/2);	tmp.ys = ys;	tmp.dy = dy;	tmp.dq = dq;
 
 	// prepare fft. NOTE: slow procedures due to unknown nx, ny.
-	void *wsx, *wtx = mgl_fft_alloc(2*nx,&wsx,1);
-	void *wsy, *wty = mgl_fft_alloc(2*ny,&wsy,1);
-	for(k=1;k<nz;k++)
+	void *wtx = mgl_fft_alloc(2*nx,0,0);
+	void *wty = mgl_fft_alloc(2*ny,0,0);
+	for(long k=1;k<nz;k++)
 	{
-		if(gr->Stop)
-		{
-			mgl_fft_free(wtx,&wsx,1);	mgl_fft_free(wty,&wsy,1);
-			delete []a;		delete []dmp;	delete []hxy;	delete []hxv;
-			delete []huy;	delete []huv;	delete []hx;	delete []hy;
-			delete []hu;	delete []hv;	return 0;
-		}
+		if(gr->Stop)	continue;
 		tmp.zz = Min.z+dz*k;
 		memset(hxy,0,4*nx*ny*sizeof(dual));	memset(hxv,0,4*nx*ny*sizeof(dual));
 		memset(huv,0,4*nx*ny*sizeof(dual));	memset(huy,0,4*nx*ny*sizeof(dual));
 		mglStartThread(mgl_pde_hprep,0,4*nx*ny,0,0,0,0,&tmp);
-		for(i=0;i<2*nx;i++)	{	hx[i] = hxv[i];			hu[i] = huv[i];		}
-		for(j=0;j<2*ny;j++)	{	hy[j] = huy[2*nx*j];	hv[j] = huv[2*nx*j];}
+#pragma omp parallel for
+		for(long i=0;i<2*nx;i++)	{	hx[i] = hxv[i];			hu[i] = huv[i];		}
+#pragma omp parallel for
+		for(long j=0;j<2*ny;j++)	{	hy[j] = huy[2*nx*j];	hv[j] = huv[2*nx*j];}
 		// rearrange arrays
-		hh0=hu[0]/2.;
-		if(ny>1)	for(i=0;i<2*nx;i++) for(j=0;j<2*ny;j++)
+		hh0=hu[0];
+		if(ny>1)
+#pragma omp parallel for collapse(2)
+			for(long i=0;i<2*nx;i++) for(long j=0;j<2*ny;j++)
+			{
+				register long i0 = i+2*nx*j;	huv[i0] -= hh0;
+				hxv[i0] -= hx[i]+hv[j]-hh0;
+				huy[i0] -= hu[i]+hy[j]-hh0;
+			}
+		else
+#pragma omp parallel for
+			for(long i=0;i<4*nx*ny;i++)	huv[i] -= hh0;
+			// solve equation
+		if(ny>1)
+#pragma omp parallel
 		{
-			i0 = i+2*nx*j;
-			huy[i0] -= (hu[i]+hy[j]-hh0)/2.;	huv[i0] -= (hu[i]+hv[j]-hh0)/2.;
-			hxy[i0] -= (hx[i]+hy[j]-hh0)/2.;	hxv[i0] -= (hx[i]+hv[j]-hh0)/2.;
+			void *wsx = mgl_fft_alloc_thr(2*nx), *wsy = mgl_fft_alloc_thr(2*ny);
+#pragma omp for
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i])*exp(-double(dmp[i]*dz));
+#pragma omp for
+			for(long i=0;i<2*ny;i++)	mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
+#pragma omp for
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(huy[i]);
+#pragma omp for
+			for(long i=0;i<2*nx;i++)	mgl_fft((double *)(a+i), 2*nx, 2*ny, wty, wsy, false);
+#pragma omp for
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(huv[i]);
+#pragma omp for
+			for(long i=0;i<2*ny;i++)	mgl_fft((double *)(a+2*i*nx), 1, 2*nx, wtx, wsx, true);
+#pragma omp for
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxv[i]);
+#pragma omp for
+			for(long i=0;i<2*nx;i++)	mgl_fft((double *)(a+i), 2*nx, 2*ny, wty, wsy, true);
+			mgl_fft_free_thr(wsx);	mgl_fft_free_thr(wsy);
 		}
-		// solve equation
-		for(i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i])*exp(-double(dmp[i]*dz))/ff;
-		for(i=0;i<2*ny;i++)		mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
-		for(i=0;i<4*nx*ny;i++)	a[i] *= exp(huy[i]);
-		if(ny>1) for(i=0;i<2*nx;i++)	mgl_fft((double *)(a+i), 2*nx, 2*ny, wty, wsy, false);
-		for(i=0;i<4*nx*ny;i++)	a[i] *= exp(huv[i]);
-		for(i=0;i<2*ny;i++)		mgl_fft((double *)(a+2*i*nx), 1, 2*nx, wtx, wsx, true);
-		for(i=0;i<4*nx*ny;i++)	a[i] *= exp(hxv[i]);
-		if(ny>1) for(i=0;i<2*nx;i++)	mgl_fft((double *)(a+i), 2*nx, 2*ny, wty, wsy, true);
-		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)	// save result
+		else
+#pragma omp parallel
 		{
-			i0 = i+nx/2+2*nx*(j+ny/2);
-			res->a[k+nz*(i+nx*j)] = abs(a[i0]);
+			void *wsx = mgl_fft_alloc_thr(2*nx);
+#pragma omp for
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(hxy[i])*exp(-double(dmp[i]*dz));
+#pragma omp for
+			for(long i=0;i<2*ny;i++)	mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
+#pragma omp for
+			for(long i=0;i<4*nx*ny;i++)	a[i] *= exp(huv[i]);
+#pragma omp for
+			for(long i=0;i<2*ny;i++)	mgl_fft((double *)(a+2*i*nx), 1, 2*nx, wtx, wsx, true);
+			mgl_fft_free_thr(wsx);
 		}
+#pragma omp parallel for collapse(2)
+		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	// save result
+			res->a[k+nz*(i+nx*j)] = a[i+nx/2+2*nx*(j+ny/2)];
 	}
-	mgl_fft_free(wtx,&wsx,1);	mgl_fft_free(wty,&wsy,1);
+	mgl_fft_free(wtx,0,0);	mgl_fft_free(wty,0,0);
 	delete []a;		delete []dmp;
 	delete []hxy;	delete []hxv;	delete []huy;	delete []huv;
 	delete []hx;	delete []hy;	delete []hu;	delete []hv;
@@ -170,12 +209,17 @@ HMDT MGL_EXPORT mgl_pde_solve(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im
 	return res;
 }
 //-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_pde_solve(HMGL gr, const char *ham, HCDT ini_re, HCDT ini_im, mreal dz, mreal k0, const char *opt)
+{
+	HADT res = mgl_pde_solve_c(gr,ham,ini_re,ini_im,dz,k0,opt);
+	HMDT out = mgl_datac_abs(res);	delete res;	return out;
+}
+//-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_ode_solve(void (*func)(const mreal *x, mreal *dx, void *par), int n, mreal *x0, mreal dt, mreal tmax, void *par)
 {
-	mglData *res=new mglData;
-	if(tmax<dt)	return res;	// nothing to do
+	if(tmax<dt)	return 0;	// nothing to do
 	int nt = int(tmax/dt)+1;
-	mgl_data_create(res,n,nt,1);
+	mglData *res=new mglData(n,nt);
 	mreal *x=new mreal[n], *k1=new mreal[n], *k2=new mreal[n], *k3=new mreal[n], *v=new mreal[n], hh=dt/2;
 	register long i,k;
 	// initial conditions
@@ -295,16 +339,18 @@ MGL_NO_EXPORT void *mgl_qo2d_hprep(void *par)
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo2d_ham *f = (mgl_qo2d_ham *)t->v;
 	mgl_ap *ra = f->ra;
-	
+
 	const mreal *r = f->r;
-	mreal tt, x1, hh;
-	register long i, nx=t->n;
-	for(i=t->id;i<nx;i+=mglNumThr)
+	long nx=t->n;
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i=t->id;i<nx;i+=mglNumThr)
 	{
 		// x terms
-		x1 = (2*i-nx+1)*f->dr;	hh = 1 - ra->t1*x1;
+		register mreal x1 = (2*i-nx+1)*f->dr, hh = 1 - ra->t1*x1;
 		hh = sqrt(sqrt(0.041+hh*hh*hh*hh));
-		tt = (ra->pt + ra->d1*x1)/hh - ra->pt;
+		register mreal tt = (ra->pt + ra->d1*x1)/hh - ra->pt;
 		f->hx[i] = f->ham(abs(f->a[i]), r[0]+ra->x1*x1, r[1]+ra->y1*x1, r[3]+ra->x0*tt, r[4]+ra->y0*tt, f->par) - f->h0/2.;
 		// u-y terms
 		x1 = f->dk/2*(i<nx/2 ? i:i-nx);
@@ -318,29 +364,28 @@ MGL_NO_EXPORT void *mgl_qo2d_hprep(void *par)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-HMDT MGL_EXPORT mgl_qo2d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal px, mreal py, void *par), void *par, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy)
+HADT MGL_EXPORT mgl_qo2d_func_c(dual (*ham)(mreal u, mreal x, mreal y, mreal px, mreal py, void *par), void *par, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy)
 {
-	mglData *res=new mglData;
 	const mglData *ray=dynamic_cast<const mglData *>(ray_dat);	// NOTE: Ray must be mglData!
-	if(!ray)	return res;
-	int nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;
-	if(nx<2 || ini_im->GetNx()!=nx || nt<2)	return res;
-	mgl_data_create(res,nx,nt,1);
+	if(!ray)	return 0;
+	long nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;
+	if(nx<2 || ini_im->GetNx()!=nx || nt<2)	return 0;
+	mglDataC *res=new mglDataC(nx,nt,1);
 
 	dual *a=new dual[2*nx], *hu=new dual[2*nx],  *hx=new dual[2*nx];
 	double *dmp=new double[2*nx];
 	mgl_ap *ra = new mgl_ap[nt];	mgl_init_ra(nt, n7, ray->a, ra);	// ray
-	register int i;
-
-	mreal dr = r/(nx-1), dk = M_PI*(nx-1)/(k0*r*nx), tt, x1, hh, B1, pt0;
 
+	mreal dr = r/(nx-1), dk = M_PI*(nx-1)/(k0*r*nx);
 	memset(dmp,0,2*nx*sizeof(double));
-	for(i=0;i<nx/2;i++)	// prepare damping
+#pragma omp parallel for
+	for(long i=0;i<nx/2;i++)	// prepare damping
 	{
-		x1 = (nx/2-i)/(nx/2.);
+		register mreal x1 = (nx/2-i)/(nx/2.);
 		dmp[2*nx-1-i] = dmp[i] = 30*GAMMA*x1*x1/k0;
 	}
-	for(i=0;i<nx;i++)	a[i+nx/2] = dual(ini_re->v(i),ini_im->v(i));	// init
+#pragma omp parallel for
+	for(long i=0;i<nx;i++)	a[i+nx/2] = dual(ini_re->v(i),ini_im->v(i));	// init
 	void *wsx, *wtx = mgl_fft_alloc(2*nx,&wsx,1);
 	if(xx && yy)	{	xx->Create(nx,nt);	yy->Create(nx,nt);	}
 
@@ -348,25 +393,30 @@ HMDT MGL_EXPORT mgl_qo2d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal px, m
 	tmp.hx=hx;	tmp.hu=hu;	tmp.dmp=dmp;	tmp.par=par;
 	tmp.dr=dr;	tmp.dk=dk;	tmp.ham=ham;	tmp.a=a;
 	// start calculation
-	for(int k=0;k<nt;k++)
+	for(long k=0;k<nt;k++)
 	{
-		for(i=0;i<nx;i++)	// "save"
-			res->a[i+k*nx]=abs(a[i+nx/2])*sqrt(ra[0].ch/ra[k].ch);
-		if(xx && yy)	for(i=0;i<nx;i++)	// prepare xx, yy
-		{
-			x1 = (2*i-nx+1)*dr;
-			xx->a[i+k*nx] = ray->a[n7*k] + ra[k].x1*x1;	// new coordinates
-			yy->a[i+k*nx] = ray->a[n7*k+1] + ra[k].y1*x1;
-		}
+#pragma omp parallel for
+		for(long i=0;i<nx;i++)	// "save"
+			res->a[i+k*nx]=a[i+nx/2]*sqrt(ra[0].ch/ra[k].ch);
+		if(xx && yy)
+#pragma omp parallel for
+			for(long i=0;i<nx;i++)	// prepare xx, yy
+			{
+				register mreal x1 = (2*i-nx+1)*dr;
+				xx->a[i+k*nx] = ray->a[n7*k] + ra[k].x1*x1;	// new coordinates
+				yy->a[i+k*nx] = ray->a[n7*k+1] + ra[k].y1*x1;
+			}
 		tmp.r=ray->a+n7*k;	tmp.ra=ra+k;
-		hh = ra[k].pt*(1/sqrt(sqrt(1.041))-1);	// 0.041=0.45^4 -- minimal value of h
+		mreal hh = ra[k].pt*(1/sqrt(sqrt(1.041))-1);	// 0.041=0.45^4 -- minimal value of h
 		tmp.h0 = ham(0, tmp.r[0], tmp.r[1], tmp.r[3]+ra[k].x0*hh, tmp.r[4]+ra[k].x0*hh, par);
 		mglStartThread(mgl_qo2d_hprep,0,2*nx,0,0,0,0,&tmp);
 		// Step for field
 		dual dt = dual(0, -ra[k].dt*k0);
-		for(i=0;i<2*nx;i++)		a[i] *= exp(hx[i]*dt);
+#pragma omp parallel for
+		for(long i=0;i<2*nx;i++)	a[i] *= exp(hx[i]*dt);
 		mgl_fft((double *)a, 1, 2*nx, wtx, wsx, false);
-		for(i=0;i<2*nx;i++)		a[i] *= exp(hu[i]*dt)/(2.*nx);
+#pragma omp parallel for
+		for(long i=0;i<2*nx;i++)	a[i] *= exp(hu[i]*dt);
 		mgl_fft((double *)a, 1, 2*nx, wtx, wsx, true);
 
 /*		// Calculate B1			// TODO make more general scheme later!!!
@@ -397,6 +447,12 @@ HMDT MGL_EXPORT mgl_qo2d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal px, m
 	return res;
 }
 //-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_qo2d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal px, mreal py, void *par), void *par, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy)
+{
+	HADT res = mgl_qo2d_func_c(ham,par,ini_re,ini_im,ray_dat,r,k0,xx,yy);
+	HMDT out = mgl_datac_abs(res);	delete res;	return out;
+}
+//-----------------------------------------------------------------------------
 dual MGL_NO_EXPORT mgl_ham2d(mreal u, mreal x, mreal y, mreal px, mreal py, void *par)
 {
 	mglFormula *h = (mglFormula *)par;
@@ -406,10 +462,16 @@ dual MGL_NO_EXPORT mgl_ham2d(mreal u, mreal x, mreal y, mreal px, mreal py, void
 	return dual(h->Calc(var), -h->CalcD(var,'i'));
 }
 //-----------------------------------------------------------------------------
-HMDT MGL_EXPORT mgl_qo2d_solve(const char *ham, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy)
+HADT MGL_EXPORT mgl_qo2d_solve_c(const char *ham, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy)
 {
 	mglFormula h(ham);
-	return mgl_qo2d_func(mgl_ham2d, &h, ini_re, ini_im, ray_dat, r, k0, xx, yy);
+	return mgl_qo2d_func_c(mgl_ham2d, &h, ini_re, ini_im, ray_dat, r, k0, xx, yy);
+}
+//-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_qo2d_solve(const char *ham, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy)
+{
+	HADT res = mgl_qo2d_solve_c(ham,ini_re,ini_im,ray_dat,r,k0,xx,yy);
+	HMDT out = mgl_datac_abs(res);	delete res;	return out;
 }
 //-----------------------------------------------------------------------------
 //
@@ -432,17 +494,18 @@ MGL_NO_EXPORT void *mgl_qo3d_hprep(void *par)
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo3d_ham *f = (mgl_qo3d_ham *)t->v;
 	mgl_ap *ra = f->ra;
-
 	const mreal *r = f->r;
-	mreal tt, x1, x2, hh;
-	register long ii,i,j, nx=t->n;
-	for(ii=t->id;ii<nx*nx;ii+=mglNumThr)
+	long nx=t->n;
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long ii=t->id;ii<nx*nx;ii+=mglNumThr)
 	{
-		i = ii%nx;	j = ii/nx;
+		register long i = ii%nx, j = ii/nx;
 		// x-y terms
-		x1 = (2*i-nx+1)*f->dr;	x2 = (2*j-nx+1)*f->dr;	hh = 1-ra->t1*x1-ra->t2*x2;
+		register mreal x1 = (2*i-nx+1)*f->dr, x2 = (2*j-nx+1)*f->dr, hh = 1-ra->t1*x1-ra->t2*x2;
 		hh = sqrt(sqrt(0.041+hh*hh*hh*hh));
-		tt = (ra->pt + ra->d1*x1 + ra->d2*x2)/hh - ra->pt;
+		register mreal tt = (ra->pt + ra->d1*x1 + ra->d2*x2)/hh - ra->pt;
 		f->hxy[ii] = f->ham(abs(f->a[i]), r[0]+ra->x1*x1+ra->x2*x2, r[1]+ra->y1*x1+ra->y2*x2, r[2]+ra->z1*x1+ra->z2*x2, r[3]+ra->x0*tt, r[4]+ra->y0*tt, r[5]+ra->z0*tt, f->par);
 		// x-v terms
 		x1 = (2*i-nx+1)*f->dr;	x2 = f->dk/2*(j<nx/2 ? j:j-nx);	hh = 1-ra->t1*x1;
@@ -465,11 +528,13 @@ MGL_NO_EXPORT void *mgl_qo3d_post(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	mgl_qo3d_ham *f = (mgl_qo3d_ham *)t->v;
-	
-	register long ii,i,j, nx=t->n;
-	for(ii=t->id;ii<nx*nx;ii+=mglNumThr)
+	long nx=t->n;
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long ii=t->id;ii<nx*nx;ii+=mglNumThr)
 	{
-		i = ii%nx;	j = ii/nx;
+		register long i = ii%nx, j = ii/nx;
 		f->hxy[ii] -= (f->hx[i]+f->hy[j]-f->h0/2.)/2.;
 		if(imag(f->hxy[ii])>0)	f->hxy[ii] = f->hxy[ii].real();
 		f->hxv[ii] -= (f->hx[i]+f->hv[j]-f->h0/2.)/2.;
@@ -484,35 +549,34 @@ MGL_NO_EXPORT void *mgl_qo3d_post(void *par)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-HMDT MGL_EXPORT mgl_qo3d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal z, mreal px, mreal py, mreal pz, void *par), void *par, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy, HMDT zz)
+HADT MGL_EXPORT mgl_qo3d_func_c(dual (*ham)(mreal u, mreal x, mreal y, mreal z, mreal px, mreal py, mreal pz, void *par), void *par, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy, HMDT zz)
 {
-	mglData *res=new mglData;
 	const mglData *ray=dynamic_cast<const mglData *>(ray_dat);	// NOTE: Ray must be mglData!
-	if(!ray)	return res;
-	int nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;	// NOTE: only square grids are supported now (for simplicity)
-	if(nx<2 || ini_re->GetNx()!=nx || ini_im->GetNx()*ini_im->GetNy()!=nx*nx || nt<2)	return res;
-	mgl_data_create(res,nx,nx,nt);
+	if(!ray)	return 0;
+	long nx=ini_re->GetNx(), nt=ray->ny, n7=ray->nx;	// NOTE: only square grids are supported now (for simplicity)
+	if(nx<2 || ini_re->GetNx()!=nx || ini_im->GetNx()*ini_im->GetNy()!=nx*nx || nt<2)	return 0;
+	mglDataC *res=new mglDataC(nx,nx,nt);
 
 	dual *a=new dual[4*nx*nx], *huv=new dual[4*nx*nx],  *hxy=new dual[4*nx*nx], *huy=new dual[4*nx*nx],  *hxv=new dual[4*nx*nx];
 	dual *hu=new dual[2*nx],  *hx=new dual[2*nx], *hy=new dual[2*nx],  *hv=new dual[2*nx];
 	mreal *dmp=new mreal[4*nx*nx];
 	mgl_ap *ra = new mgl_ap[nt];
 	mgl_init_ra(nt, n7, ray->a, ra);	// prepare ray
-	register int i,j,ii;
-
-	mreal dr = r/(nx-1), dk = M_PI*(nx-1)/(k0*r*nx), tt, x1, x2, hh, B1, pt0;
 
+	mreal dr = r/(nx-1), dk = M_PI*(nx-1)/(k0*r*nx);
 	memset(dmp,0,4*nx*nx*sizeof(mreal));
-	for(i=0;i<nx/2;i++)	for(j=0;j<nx/2;j++)	// prepare damping
+#pragma omp parallel for collapse(2)
+	for(long i=0;i<nx/2;i++)	for(long j=0;j<nx/2;j++)	// prepare damping
 	{
-		x1 = (nx/2-i)/(nx/2.);	x2 = (nx/2-j)/(nx/2.);
+		register mreal x1 = (nx/2-i)/(nx/2.), x2 = (nx/2-j)/(nx/2.);
 		dmp[2*nx-1-i] = dmp[i] = 30*GAMMA*x1*x1/k0;
 		dmp[(2*nx-1-j)*2*nx] += 30*GAMMA*x2*x2/k0;
 		dmp[j*2*nx] += 30*GAMMA*x2*x2/k0;
 	}
-	for(i=0;i<nx;i++)	for(j=0;j<nx;j++)	// init
+#pragma omp parallel for collapse(2)
+	for(long i=0;i<nx;i++)	for(long j=0;j<nx;j++)	// init
 		a[i+nx/2+2*nx*(j+nx/2)] = dual(ini_re->v(i,j),ini_im->v(i,j));
-	void *wsx, *wtx = mgl_fft_alloc(2*nx,&wsx,1);
+	void *wtx = mgl_fft_alloc(2*nx,0,0);
 	if(xx && yy && zz)	{	xx->Create(nx,nx,nt);	yy->Create(nx,nx,nt);	zz->Create(nx,nx,nt);	}
 
 	mgl_qo3d_ham tmp;	// parameters for Hamiltonian calculation
@@ -521,20 +585,24 @@ HMDT MGL_EXPORT mgl_qo3d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal z, mr
 	tmp.dmp=dmp;	tmp.par=par;
 	tmp.dr=dr;	tmp.dk=dk;	tmp.ham=ham;	tmp.a=a;
 	// start calculation
-	for(int k=0;k<nt;k++)
+	for(long k=0;k<nt;k++)
 	{
-		for(i=0;i<nx;i++)	for(j=0;j<nx;j++)	// "save"
-			res->a[i+nx*(j+k*nx)]=abs(a[i+nx/2+2*nx*(j+nx/2)])*sqrt(ra[0].ch/ra[k].ch);
-		if(xx && yy && zz)	for(i=0;i<nx;i++)	for(j=0;j<nx;j++)	// prepare xx, yy, zz
-		{
-			x1 = (2*i-nx+1)*dr;	x2 = (2*j-nx+1)*dr;
-			xx->a[i+nx*(j+k*nx)] = ray->a[n7*k] + ra[k].x1*x1 + ra[k].x2*x2;	// new coordinates
-			yy->a[i+nx*(j+k*nx)] = ray->a[n7*k+1] + ra[k].y1*x1 + ra[k].y2*x2;
-			zz->a[i+nx*(j+k*nx)] = ray->a[n7*k+2] + ra[k].z1*x1 + ra[k].z2*x2;
-		}
+#pragma omp parallel for collapse(2)
+		for(long i=0;i<nx;i++)	for(long j=0;j<nx;j++)	// "save"
+			res->a[i+nx*(j+k*nx)]=a[i+nx/2+2*nx*(j+nx/2)]*sqrt(ra[0].ch/ra[k].ch);
+		if(xx && yy && zz)
+#pragma omp parallel for collapse(2)
+			for(long i=0;i<nx;i++)	for(long j=0;j<nx;j++)	// prepare xx, yy, zz
+			{
+				register mreal x1 = (2*i-nx+1)*dr, x2 = (2*j-nx+1)*dr;
+				xx->a[i+nx*(j+k*nx)] = ray->a[n7*k] + ra[k].x1*x1 + ra[k].x2*x2;	// new coordinates
+				yy->a[i+nx*(j+k*nx)] = ray->a[n7*k+1] + ra[k].y1*x1 + ra[k].y2*x2;
+				zz->a[i+nx*(j+k*nx)] = ray->a[n7*k+2] + ra[k].z1*x1 + ra[k].z2*x2;
+			}
 		tmp.r=ray->a+n7*k;	tmp.ra=ra+k;
 		mglStartThread(mgl_qo3d_hprep,0,2*nx,0,0,0,0,&tmp);	tmp.h0 = huv[0];
-		for(i=0;i<2*nx;i++)	// fill intermediate arrays
+#pragma omp parallel for
+		for(long i=0;i<2*nx;i++)	// fill intermediate arrays
 		{
 			tmp.hx[i] = hxv[i];	tmp.hy[i] = huy[i*2*nx];
 			tmp.hv[i] = huv[i];	tmp.hu[i] = huv[i*2*nx];
@@ -542,19 +610,32 @@ HMDT MGL_EXPORT mgl_qo3d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal z, mr
 		mglStartThread(mgl_qo3d_post,0,2*nx,0,0,0,0,&tmp);
 		// Step for field
 		dual dt = dual(0, -ra[k].dt*k0);	// TODO: this part can be paralleled
-		for(i=0;i<4*nx*nx;i++)	a[i] *= exp(hxy[i]*dt);		// x-y
-		for(i=0;i<2*nx;i++)	// x->u
-			mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
-		for(i=0;i<4*nx*nx;i++)	a[i] *= exp(huy[i]*dt);		// u-y
-		for(i=0;i<2*nx;i++)	// y->v
-			mgl_fft((double *)(a+i), 2*nx, 2*nx, wtx, wsx, false);
-		for(i=0;i<4*nx*nx;i++)	a[i] *= exp(huv[i]*dt)/(4.*nx*nx);	// u-v
-		for(i=0;i<2*nx;i++)	// u->x
-			mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, true);
-		for(i=0;i<4*nx*nx;i++)	a[i] *= exp(hxv[i]*dt);		// x-v
-		for(i=0;i<2*nx;i++)	// v->y
-			mgl_fft((double *)(a+i), 2*nx, 2*nx, wtx, wsx, true);
-		
+#pragma omp parallel
+		{
+			void *wsx = mgl_fft_alloc_thr(2*nx);
+#pragma omp for
+			for(long i=0;i<4*nx*nx;i++)	a[i] *= exp(hxy[i]*dt);		// x-y
+#pragma omp for
+			for(long i=0;i<2*nx;i++)	// x->u
+				mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, false);
+#pragma omp for
+			for(long i=0;i<4*nx*nx;i++)	a[i] *= exp(huy[i]*dt);		// u-y
+#pragma omp for
+			for(long i=0;i<2*nx;i++)	// y->v
+				mgl_fft((double *)(a+i), 2*nx, 2*nx, wtx, wsx, false);
+#pragma omp for
+			for(long i=0;i<4*nx*nx;i++)	a[i] *= exp(huv[i]*dt);		// u-v
+#pragma omp for
+			for(long i=0;i<2*nx;i++)	// u->x
+				mgl_fft((double *)(a+i*2*nx), 1, 2*nx, wtx, wsx, true);
+#pragma omp for
+			for(long i=0;i<4*nx*nx;i++)	a[i] *= exp(hxv[i]*dt);		// x-v
+#pragma omp for
+			for(long i=0;i<2*nx;i++)	// v->y
+				mgl_fft((double *)(a+i), 2*nx, 2*nx, wtx, wsx, true);
+			mgl_fft_free_thr(wsx);
+		}
+
 /*		// Calculate B1			// TODO make more general scheme later!!!
 		hh = ra[k].pt*(1/sqrt(sqrt(1.041))-1);
 		var['x'-'a'] = ray->a[n7*k];	// new coordiantes
@@ -578,13 +659,19 @@ HMDT MGL_EXPORT mgl_qo3d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal z, mr
 		a1 = sqrt(a1/a2);
 		for(i=0;i<2*nx;i++)	a[i] *= a1;*/
 	}
-	mgl_fft_free(wtx,&wsx,1);
+	mgl_fft_free(wtx,0,0);
 	delete []a;		delete []ra;	delete []dmp;
 	delete []huv;	delete []hxy;	delete []hxv;	delete []huy;
 	delete []hu;	delete []hx;	delete []hv;	delete []hy;
 	return res;
 }
 //-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_qo3d_func(dual (*ham)(mreal u, mreal x, mreal y, mreal z, mreal px, mreal py, mreal pz, void *par), void *par, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy, HMDT zz)
+{
+	HADT res = mgl_qo3d_func_c(ham,par,ini_re,ini_im,ray_dat,r,k0,xx,yy,zz);
+	HMDT out = mgl_datac_abs(res);	delete res;	return out;
+}
+//-----------------------------------------------------------------------------
 dual MGL_NO_EXPORT mgl_ham3d(mreal u, mreal x, mreal y, mreal z, mreal px, mreal py, mreal pz, void *par)
 {
 	mglFormula *h = (mglFormula *)par;
@@ -594,23 +681,32 @@ dual MGL_NO_EXPORT mgl_ham3d(mreal u, mreal x, mreal y, mreal z, mreal px, mreal
 	return dual(h->Calc(var), -h->CalcD(var,'i'));
 }
 //-----------------------------------------------------------------------------
-HMDT MGL_EXPORT mgl_qo3d_solve(const char *ham, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy, HMDT zz)
+HADT MGL_EXPORT mgl_qo3d_solve_c(const char *ham, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy, HMDT zz)
 {
 	mglFormula h(ham);
-	return mgl_qo3d_func(mgl_ham3d, &h, ini_re, ini_im, ray_dat, r, k0, xx, yy, zz);
+	return mgl_qo3d_func_c(mgl_ham3d, &h, ini_re, ini_im, ray_dat, r, k0, xx, yy, zz);
+}
+//-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_qo3d_solve(const char *ham, HCDT ini_re, HCDT ini_im, HCDT ray_dat, mreal r, mreal k0, HMDT xx, HMDT yy, HMDT zz)
+{
+	HADT res = mgl_qo3d_solve_c(ham,ini_re,ini_im,ray_dat,r,k0,xx,yy,zz);
+	HMDT out = mgl_datac_abs(res);	delete res;	return out;
 }
 //-----------------------------------------------------------------------------
 MGL_NO_EXPORT void *mgl_jacob2(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
-	register long i,j,i0,ip,im,jp,jm, nx=t->p[0], ny=t->p[1];
+	long nx=t->p[0], ny=t->p[1];
 	mreal *r=t->a;
 	const mreal *x=t->b, *y=t->c;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		i=i0%nx;	j=i0/nx;
-		ip = i<nx-1 ? 1:0;	jp = j<ny-1 ? nx:0;
-		im = i>0 ? -1:0;	jm = j>0 ? -nx:0;
+		register long i=i0%nx, j=i0/nx;
+		register long ip = i<nx-1 ? 1:0, jp = j<ny-1 ? nx:0;
+		register long im = i>0 ? -1:0, jm = j>0 ? -nx:0;
 		r[i0] = (x[i0+ip]-x[i0+im])*(y[i0+jp]-y[i0+jm]) -
 				(y[i0+ip]-y[i0+im])*(x[i0+jp]-x[i0+jm]);
 		r[i0] *= mreal((nx-1)*(ny-1)) / ((ip-im)*(jp-jm));
@@ -620,9 +716,8 @@ MGL_NO_EXPORT void *mgl_jacob2(void *par)
 HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 {
 	int nx = x->GetNx(), ny=x->GetNy();
-	mglData *r=new mglData;
-	if(nx!=y->GetNx() || ny!=y->GetNy() || nx<2 || ny<2)	return	r;
-	mgl_data_create(r,nx,ny,1);
+	if(nx!=y->GetNx() || ny!=y->GetNy() || nx<2 || ny<2)	return	0;
+	mglData *r=new mglData(nx,ny,1);
 	const mglData *xx=dynamic_cast<const mglData *>(x);
 	const mglData *yy=dynamic_cast<const mglData *>(y);
 	if(xx && yy)
@@ -632,11 +727,11 @@ HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 	}
 	else	// slow variant
 	{
-		register long i,j,ip,im,jp,jm;
-		for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(2)
+		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
-			im = i>0 ? i-1:i;	ip = i<nx-1 ? i+1:i;
-			jm = j>0 ? j-1:j;	jp = j<ny-1 ? j+1:j;
+			register long im = i>0 ? i-1:i, ip = i<nx-1 ? i+1:i;
+			register long jm = j>0 ? j-1:j, jp = j<ny-1 ? j+1:j;
 			r->a[i+nx*j] = (x->v(ip,j)-x->v(im,j))*(y->v(i,jp)-y->v(i,jm)) -
 						(y->v(ip,j)-y->v(im,j))*(x->v(i,jp)-x->v(i,jm));
 			r->a[i+nx*j] *= mreal((nx-1)*(ny-1)) / ((ip-im)*(jp-jm));
@@ -648,14 +743,17 @@ HMDT MGL_EXPORT mgl_jacobian_2d(HCDT x, HCDT y)
 MGL_NO_EXPORT void *mgl_jacob3(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
-	register long i,j,k,i0,ip,im,jp,jm,kp,km, nx=t->p[0], ny=t->p[1], nz=t->p[2];
+	long nx=t->p[0], ny=t->p[1], nz=t->p[2];
 	mreal *r=t->a;
 	const mreal *x=t->b, *y=t->c, *z=t->d;
-	for(i0=t->id;i0<t->n;i0+=mglNumThr)
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for
+#endif
+	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
-		i=i0%nx;	j=(i0/nx)%ny;	k=i0/(nx*ny);
-		ip = i<nx-1 ? 1:0;	jp = j<ny-1 ? nx:0;	kp = k<nz-1 ? nx*ny:0;
-		im = i>0 ? -1:0;	jm = j>0 ? -nx:0;	km = k>0 ? -nx*ny:0;
+		register long i=i0%nx, j=(i0/nx)%ny, k=i0/(nx*ny);
+		register long ip = i<nx-1 ? 1:0, jp = j<ny-1 ? nx:0, kp = k<nz-1 ? nx*ny:0;
+		register long im = i>0 ? -1:0, jm = j>0 ? -nx:0, km = k>0 ? -nx*ny:0;
 		r[i0] = (x[i0+ip]-x[i0+im])*(y[i0+jp]-y[i0+jm])*(z[i0+kp]-z[i0+km]) -
 				(x[i0+ip]-x[i0+im])*(y[i0+kp]-y[i0+km])*(z[i0+jp]-z[i0+jm]) -
 				(x[i0+jp]-x[i0+jm])*(y[i0+ip]-y[i0+im])*(z[i0+kp]-z[i0+km]) +
@@ -669,10 +767,9 @@ MGL_NO_EXPORT void *mgl_jacob3(void *par)
 HMDT MGL_EXPORT mgl_jacobian_3d(HCDT x, HCDT y, HCDT z)
 {
 	int nx = x->GetNx(), ny=x->GetNy(), nz=x->GetNz(), nn = nx*ny*nz;
-	mglData *r=new mglData;
-	if(nx<2 || ny<2 || nz<2)	return	r;
-	if(nn!=y->GetNx()*y->GetNy()*y->GetNz() || nn!=z->GetNx()*z->GetNy()*z->GetNz())	return r;
-	mgl_data_create(r,nx,ny,nz);
+	if(nx<2 || ny<2 || nz<2)	return 0;
+	if(nn!=y->GetNx()*y->GetNy()*y->GetNz() || nn!=z->GetNx()*z->GetNy()*z->GetNz())	return 0;
+	mglData *r=new mglData(nx,ny,nz);
 	const mglData *xx=dynamic_cast<const mglData *>(x);
 	const mglData *yy=dynamic_cast<const mglData *>(y);
 	const mglData *zz=dynamic_cast<const mglData *>(z);
@@ -683,13 +780,13 @@ HMDT MGL_EXPORT mgl_jacobian_3d(HCDT x, HCDT y, HCDT z)
 	}
 	else	// slow variant
 	{
-		register long i,j,k,ip,im,jp,jm,kp,km,i0;
-		for(k=0;k<nz;k++)	for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
+#pragma omp parallel for collapse(3)
+		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
-			im = i>0 ? i-1:i;	ip = i<nx-1 ? i+1:i;
-			jm = j>0 ? j-1:j;	jp = j<ny-1 ? j+1:j;
-			km = k>0 ? k-1:k;	kp = k<nz-1 ? k+1:k;
-			i0 = i+nx*(j+ny*k);
+			register long im = i>0 ? i-1:i, ip = i<nx-1 ? i+1:i;
+			register long jm = j>0 ? j-1:j, jp = j<ny-1 ? j+1:j;
+			register long km = k>0 ? k-1:k, kp = k<nz-1 ? k+1:k;
+			register long i0 = i+nx*(j+ny*k);
 			r->a[i0] = (x->v(ip,j,k)-x->v(im,j,k))*(y->v(i,jp,k)-y->v(i,jm,k))*(z->v(i,j,kp)-z->v(i,j,km)) -
 					(x->v(ip,j,k)-x->v(im,j,k))*(y->v(i,j,kp)-y->v(i,j,km))*(z->v(i,jp,k)-z->v(i,jm,k)) -
 					(x->v(i,jp,k)-x->v(i,jm,k))*(y->v(ip,j,k)-y->v(im,j,k))*(z->v(i,j,kp)-z->v(i,j,km)) +
@@ -703,6 +800,11 @@ HMDT MGL_EXPORT mgl_jacobian_3d(HCDT x, HCDT y, HCDT z)
 	return r;
 }
 //-----------------------------------------------------------------------------
+uintptr_t MGL_EXPORT mgl_pde_solve_c_(uintptr_t* gr, const char *ham, uintptr_t* ini_re, uintptr_t* ini_im, mreal *dz, mreal *k0, const char *opt, int l, int lo)
+{	char *s=new char[l+1];	memcpy(s,ham,l);	s[l]=0;
+	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
+	uintptr_t res = uintptr_t(mgl_pde_solve_c(_GR_, s, _DA_(ini_re), _DA_(ini_im), *dz, *k0, o));
+	delete []o;	delete []s;	return res;	}
 uintptr_t MGL_EXPORT mgl_pde_solve_(uintptr_t* gr, const char *ham, uintptr_t* ini_re, uintptr_t* ini_im, mreal *dz, mreal *k0, const char *opt, int l, int lo)
 {	char *s=new char[l+1];	memcpy(s,ham,l);	s[l]=0;
 	char *o=new char[lo+1];	memcpy(o,opt,lo);	o[lo]=0;
