@@ -1,6 +1,6 @@
 /***************************************************************************
  * prim.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -32,7 +32,7 @@ void MGL_EXPORT mgl_mark(HMGL gr, double x, double y, double z,const char *mark)
 	if(mgl_isnan(z))	z=2*gr->Max.z-gr->Min.z;
 	static int cgid=1;	gr->StartGroup("MarkS",cgid++);
 	long k = gr->AddPnt(mglPoint(x,y,z),gr->CDef,mglPoint(NAN),-1,3);
-	gr->mark_plot(k,mk); 	gr->AddActive(k);
+	gr->mark_plot(k,mk,gr->GetPenWidth()); 	gr->AddActive(k);
 	gr->EndGroup();
 }
 //-----------------------------------------------------------------------------
@@ -153,7 +153,7 @@ void MGL_EXPORT mgl_face(HMGL gr, double x0, double y0, double z0, double x1, do
 	if(mgl_isnan(z2))	z2 = zz;	if(mgl_isnan(z3))	z3 = zz;
 	mglPoint p1(x0,y0,z0), p2(x1,y1,z1), p3(x2,y2,z2), p4(x3,y3,z3);
 	if(gr->GetNumPal(pal)>=4)
-	{	c2=gr->NextColor(pal);	c3=gr->NextColor(pal);	c4=gr->NextColor(pal);	}
+	{	c2=gr->NextColor(pal,1);	c3=gr->NextColor(pal,2);	c4=gr->NextColor(pal,3);	}
 	mglPoint q1,q2,q3,q4;
 	q1 = (p2-p1)^(p3-p1);	q4 = (p2-p4)^(p3-p4);
 	q2 = (p1-p2)^(p4-p2);	q3 = (p1-p3)^(p4-p3);
@@ -213,10 +213,10 @@ void MGL_EXPORT mgl_cone(HMGL gr, double x1, double y1, double z1, double x2, do
 	if(r1==0 && r2==0)	return;
 
 	static int cgid=1;	gr->StartGroup("Cone",cgid++);
-	mglPoint p1(x1,y1,z1), p2(x2,y2,z2), p,q, d=p2-p1,a,b;
+	mglPoint p1(x1,y1,z1), p2(x2,y2,z2), p,q(NAN,NAN),t(NAN,NAN), d=p2-p1,a,b;
 	a=!d;	a.Normalize();		b=d^a;	b.Normalize();
 	long ss=gr->AddTexture(stl);
-	mreal c1=gr->GetC(ss,p1.z), c2=gr->GetC(ss,p2.z);
+	mreal c1=gr->GetC(ss,p1.z), c2=gr->GetC(ss,p2.z), dr=r2-r1;
 	long *kk=new long[164],k1=-1,k2=-1;
 	bool edge = mglchr(stl,'@');
 	bool wire = mglchr(stl,'#');
@@ -226,39 +226,47 @@ void MGL_EXPORT mgl_cone(HMGL gr, double x1, double y1, double z1, double x2, do
 		k1=gr->AddPnt(p1,c1,d,-1,3);
 		k2=gr->AddPnt(p2,c2,d,-1,3);
 	}
-	mreal f,si,co, dr=r2-r1;
-	register long i;
-	for(i=0;i<(wire?13:41);i++)
+	long n=wire?6:18;	//wire?6:18;
+	if(mglchr(stl,'4'))	n=2;
+	else if(mglchr(stl,'6'))	n=3;
+	else if(mglchr(stl,'8'))	n=4;
+	bool refr = n>6;
+	if(refr)	t=d;
+
+#pragma omp parallel for firstprivate(p,q)
+	for(long i=0;i<2*n+1;i++)
 	{
-		if(gr->Stop)	{	delete []kk;	return;	}
-		f = i*M_PI/(wire?6:20);	co = cos(f);	si = sin(f);
+		if(gr->Stop)	continue;
+		register int f = n!=4?(2*i+1)*90/n:45*i;
+		register mreal co = mgl_cos[f%360], si = mgl_cos[(f+270)%360];
 		p = p1+(r1*co)*a+(r1*si)*b;
-		q = (si*a-co*b)^(d + (dr*co)*a + (dr*si)*b);
-		if(wire)	q.x=q.y=NAN;
+		if(refr)	q = (si*a-co*b)^(d + (dr*co)*a + (dr*si)*b);
 		kk[i] = gr->AddPnt(p,c1,q,-1,3);
-		if(edge && !wire)	kk[i+82] = gr->AddPnt(p,c1,d,-1,3);
+		if(edge && !wire)	kk[i+82] = gr->AddPnt(p,c1,t,-1,3);
 		p = p2+(r2*co)*a+(r2*si)*b;
-		kk[i+(wire?13:41)] = gr->AddPnt(p,c2,q,-1,3);
-		if(edge && !wire)	kk[i+123] = gr->AddPnt(p,c2,d,-1,3);
+		kk[i+2*n+1] = gr->AddPnt(p,c2,q,-1,3);
+		if(edge && !wire)	kk[i+123] = gr->AddPnt(p,c2,t,-1,3);
 	}
-	if(wire)	for(i=0;i<12;i++)
-	{
-		if(gr->Stop)	{	delete []kk;	return;	}
-		gr->line_plot(kk[i],kk[i+1]);
-		gr->line_plot(kk[i],kk[i+13]);
-		gr->line_plot(kk[i+14],kk[i+1]);
-		gr->line_plot(kk[i+14],kk[i+13]);
-	}
-	else	for(i=0;i<40;i++)
-	{
-		if(gr->Stop)	{	delete []kk;	return;	}
-		gr->quad_plot(kk[i],kk[i+1],kk[i+41],kk[i+42]);
-		if(edge)
+	if(wire)
+//#pragma omp parallel for		// useless
+		for(long i=0;i<2*n;i++)
 		{
-			gr->trig_plot(k1,kk[i+82],kk[i+83]);
-			gr->trig_plot(k2,kk[i+123],kk[i+124]);
+			gr->line_plot(kk[i],kk[i+1]);
+			gr->line_plot(kk[i],kk[i+2*n+1]);
+			gr->line_plot(kk[i+2*n+2],kk[i+1]);
+			gr->line_plot(kk[i+2*n+2],kk[i+2*n+1]);
+		}
+	else
+#pragma omp parallel for
+		for(long i=0;i<2*n;i++)
+		{
+			gr->quad_plot(kk[i],kk[i+1],kk[i+2*n+1],kk[i+2*n+2]);
+			if(edge)
+			{
+				gr->trig_plot(k1,kk[i+82],kk[i+83]);
+				gr->trig_plot(k2,kk[i+123],kk[i+124]);
+			}
 		}
-	}
 	gr->EndGroup();	delete []kk;
 }
 //-----------------------------------------------------------------------------
@@ -272,7 +280,7 @@ void MGL_EXPORT mgl_cone_(uintptr_t* gr, mreal *x1, mreal *y1, mreal *z1, mreal
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cones_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen, const char *opt)
 {
-	long i,j,m,mx,my,mz,n=z->GetNx(),nx=x->GetNx(), nz=z->GetNy(), pal;
+	long i=5,j,m,mx,my,mz,n=z->GetNx(),nx=x->GetNx(), nz=z->GetNy(), pal;
 	if(mgl_check_dim1(gr,x,z,y,0,"Cones",true))	return;
 
 	gr->SaveState(opt);
@@ -281,12 +289,20 @@ void MGL_EXPORT mgl_cones_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen,
 
 	bool above= mglchr(pen,'a');
 	bool wire = mglchr(pen,'#');
-	mreal *dd=new mreal[2*n], x1,z0,zz,d, vx,vy,vz,v0,v1;
+	bool tube = mglchr(pen,'t');
+	mreal *dd=new mreal[2*n], x1,z0,zz,d, vx,vy,vz,v0,v1,dv=nx>n?1:0;
+	if(mglchr(pen,'<'))	dv = 1;
+	if(mglchr(pen,'^'))	dv = 0;
+	if(mglchr(pen,'>'))	dv = -1;
 
 	gr->SetPenPal(pen,&pal);
-	char c1[7];	memset(c1,0,7);	c1[0] ='@';
-	char c2[7];	memset(c2,0,7);	c2[0] ='@';
-	if(wire)	c1[5]=c2[5]='#';
+	char c1[8];	memset(c1,0,8);	c1[0] ='@';
+	char c2[8];	memset(c2,0,8);	c2[0] ='@';
+	if(wire)	{	c1[5]=c2[5]='#';	i++;	}
+	if(mglchr(pen,'&'))	c1[i]=c2[i]='4';
+	else if(mglchr(pen,'4'))	c1[i]=c2[i]='4';
+	else if(mglchr(pen,'6'))	c1[i]=c2[i]='6';
+	else if(mglchr(pen,'8'))	c1[i]=c2[i]='8';
 	memset(dd,0,2*n*sizeof(mreal));
 	z0 = gr->GetOrgZ('x');
 	for(i=0;i<n;i++)	for(j=0;j<m;j++)	dd[i] += z->v(i, j<nz ? j:0);
@@ -295,7 +311,7 @@ void MGL_EXPORT mgl_cones_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen,
 		gr->NextColor(pal);		memcpy(c1+1,gr->last_line(),4);
 		if(gr->GetNumPal(pal)==2*m)
 		{	gr->NextColor(pal);	memcpy(c2+1,gr->last_line(),4);	}
-		else	memcpy(c2,c1,7);
+		else	memcpy(c2,c1,8);
 		mx = j<x->GetNy() ? j:0;	my = j<y->GetNy() ? j:0;	mz = j<nz ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -303,15 +319,16 @@ void MGL_EXPORT mgl_cones_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *pen,
 			vx=x->v(i,mx);	vy=y->v(i,my);	vz=z->v(i,mz);
 			v0=y->v(i,0);	v1=i<nx-1 ? x->v(i+1,mx):x->v(i-1,mx);
 			d = i<nx-1 ? v1-vx : vx-v1;
-			x1 = (n<nx ? (vx+v1)/2 : vx) + d/2*(1-0.7*gr->BarWidth);
+			x1 = vx + d/2*(dv-0.*gr->BarWidth);	// TODO
+			d *= 0.7*gr->BarWidth;
 			if(above)
 			{
 				zz = j>0?dd[i+n]:z0;	dd[i+n] += vz;
 				mgl_cone(gr, x1,v0,zz, x1,v0,dd[i+n],
-						 0.7*gr->BarWidth*d*(dd[i]-zz)/(dd[i]-z0),
-						 0.7*gr->BarWidth*d*(dd[i]-dd[i+n])/(dd[i]-z0), c1);
+						 tube?d:d*(dd[i]-zz)/(dd[i]-z0),
+						 tube?d:d*(dd[i]-dd[i+n])/(dd[i]-z0), c1);
 			}
-			else	mgl_cone(gr, x1,vy,z0, x1,vy,vz, 0.7*gr->BarWidth*d,0, vz<0?c1:c2);
+			else	mgl_cone(gr, x1,vy,z0, x1,vy,vz, d,tube?d:0, vz<0?c1:c2);
 		}
 	}
 	gr->EndGroup();	delete []dd;
@@ -323,7 +340,6 @@ void MGL_EXPORT mgl_cones_xz(HMGL gr, HCDT x, HCDT z, const char *pen, const cha
 	mglData y(z);
 	y.Fill(gr->Min.y,gr->Max.y,'y');
 	mgl_cones_xyz(gr,x,&y,z,pen,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cones(HMGL gr, HCDT z, const char *pen, const char *opt)
@@ -332,7 +348,6 @@ void MGL_EXPORT mgl_cones(HMGL gr, HCDT z, const char *pen, const char *opt)
 	mglData x(z->GetNx()+1);
 	x.Fill(gr->Min.x,gr->Max.x);
 	mgl_cones_xz(gr,&x,z,pen,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cones_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen, const char *opt,int l,int lo)
@@ -378,8 +393,8 @@ void MGL_EXPORT mgl_ellipse(HMGL gr, double x1, double y1, double z1, double x2,
 	for(long i=0;i<n;i++)
 	{
 		if(gr->Stop)	return;
-		mreal t = i*2*M_PI/(n-1.);
-		p = s+v*cos(t)+u*sin(t);
+		register int t=i*360/(n-1);
+		p = s+v*mgl_cos[t%360]+u*mgl_cos[(270+t)%360];
 		n2 = n1;	n1 = gr->AddPnt(p,c,q,-1,11);
 		if(i==n/4)	gr->AddActive(n1,2);
 		m2 = m1;	m1 = gr->CopyNtoC(n1,k);
@@ -446,32 +461,32 @@ void MGL_EXPORT mgl_drop(HMGL gr, mglPoint p, mglPoint q, double r, double c, do
 	q.Normalize();	p1 = !q;	p2 = q^p1;	r /= 2;
 
 	static int cgid=1;	gr->StartGroup("Drop",cgid++);
-	const int n = 24, m = n/2;
-	register long i,j;
+	const int m=12, n=2*m+1;
 	gr->Reserve(n*m);
 	long *nn=new long[2*n],n1,n2;
-	mreal u,v,x,y,z,rr,dr, co,si;
+	mreal x,y,z,rr,dr;
 
 	z = r*(1+sh)*(1+sh);	n1 = gr->AddPnt(p + q*z,c,q,-1,3);
 	z = r*(1+sh)*(sh-1);	n2 = gr->AddPnt(p + q*z,c,q,-1,3);
 
-	for(i=0;i<m;i++)	for(j=0;j<n;j++)
+	for(long i=0;i<=m;i++)	for(long j=0;j<n;j++)	// NOTE use prev.points => not for omp
 	{
-		if(gr->Stop)	{	delete []nn;	return;	}
-		if(i>0 && i<m-1)
+		if(gr->Stop)	continue;
+		if(i>0 && i<m)
 		{
-			u = i*M_PI/(m-1.);	v = 2*M_PI*j/(n-1.)-1;
-			si = sin(u);	co = cos(u);
+			register int u=i*180/m, v=180*j/m+202;
+			register float co=mgl_cos[u%360], si=mgl_cos[(u+270)%360];
+			register float cv=mgl_cos[v%360], sv=mgl_cos[(v+270)%360];
 			rr = r*a*si*(1.+sh*co)/(1+sh);
 			dr = r*a/(1+sh)*(co*(1.+sh*co) - sh*si*si);
-			x = rr*cos(v);	y = rr*sin(v);
+			x = rr*cv;	y = rr*sv;
 			z = r*(1+sh)*(co+sh);
 			pp = p + p1*x + p2*y + q*z;
-			qq = (p1*sin(v)-p2*cos(v))^(p1*(dr*cos(v)) + p2*(dr*sin(v)) - q*(r*(1+sh)*si));
+			qq = (p1*sv-p2*cv)^(p1*(dr*cv) + p2*(dr*sv) - q*(r*(1+sh)*si));
 			nn[j+n]=nn[j];	nn[j]=gr->AddPnt(pp,c,qq,-1,3);
 		}
 		else if(i==0)	nn[j] = n1;
-		else if(i==m-1)	{	nn[j+n]=nn[j];	nn[j]=n2;	}
+		else if(i==m)	{	nn[j+n]=nn[j];	nn[j]=n2;	}
 		if(i*j>0)	gr->quad_plot(nn[j-1], nn[j], nn[j+n-1], nn[j+n]);
 	}
 	delete []nn;	gr->EndGroup();
@@ -493,43 +508,48 @@ void MGL_EXPORT mgl_drop_(uintptr_t* gr, mreal *x1, mreal *y1, mreal *z1, mreal
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dew_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
-	long i,j,n=ax->GetNx(),m=ax->GetNy(),k;
+	long n=ax->GetNx(),m=ax->GetNy();
 	if(mgl_check_dim2(gr,x,y,ax,ay,"Dew"))	return;
 
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("DewXY",cgid++);
 
-	mreal xm,ym,dx,dy,dd;
 	long ss = gr->AddTexture(sch);
 	bool inv = mglchr(sch,'i');
-	mreal	zVal = gr->Min.z;
+	mreal zVal = gr->Min.z, xm=0;
 	long tx=1,ty=1;
 	if(gr->MeshNum>1)	{	tx=(n-1)/(gr->MeshNum-1);	ty=(m-1)/(gr->MeshNum-1);	}
 	if(tx<1)	tx=1;	if(ty<1)	ty=1;
 
-	for(k=0,xm=0;k<ax->GetNz();k++)	for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel
 	{
-		ym = sqrt(ax->v(i,j,k)*ax->v(i,j,k)+ay->v(i,j,k)*ay->v(i,j,k));
-		xm = xm>ym ? xm : ym;
+		register mreal xm1=0,ym;
+#pragma omp for nowait collapse(3)
+		for(long k=0;k<ax->GetNz();k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+		{
+			ym = sqrt(ax->v(i,j,k)*ax->v(i,j,k)+ay->v(i,j,k)*ay->v(i,j,k));
+			xm1 = xm1>ym ? xm1 : ym;
+		}
+#pragma omp critical(max_vec)
+		{xm = xm>xm1 ? xm:xm1;}
 	}
 	xm = 1./MGL_FEPSILON/(xm==0 ? 1:xm);
-	mglPoint q;
 
-	for(k=0;k<ax->GetNz();k++)
+	for(long k=0;k<ax->GetNz();k++)
 	{
 		if(ax->GetNz()>1)	zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(ax->GetNz()-1);
-		for(i=0;i<n;i+=tx)	for(j=0;j<m;j+=ty)
+//#pragma omp parallel for collapse(2)	// gain looks negligible?!?
+		for(long i=0;i<n;i+=tx)	for(long j=0;j<m;j+=ty)
 		{
-			if(gr->Stop)	return;
-			mreal xx=GetX(x,i,j,k).x, yy=GetY(y,i,j,k).x;
-			dx = i<n-1 ? (GetX(x,i+1,j,k).x-xx) : (xx-GetX(x,i-1,j,k).x);
-			dy = j<m-1 ? (GetY(y,i,j+1,k).x-yy) : (yy-GetY(y,i,j-1,k).x);
+			if(gr->Stop)	continue;
+			register mreal xx=GetX(x,i,j,k).x, yy=GetY(y,i,j,k).x, dd;
+			register mreal dx = i<n-1 ? (GetX(x,i+1,j,k).x-xx) : (xx-GetX(x,i-1,j,k).x);
+			register mreal dy = j<m-1 ? (GetY(y,i,j+1,k).x-yy) : (yy-GetY(y,i,j-1,k).x);
 			dx *= tx;	dy *= ty;
 
-			q = mglPoint(ax->v(i,j,k),ay->v(i,j,k));	dd = q.norm();
+			mglPoint q = mglPoint(ax->v(i,j,k),ay->v(i,j,k));	dd = q.norm();
 			if(inv)	q = -q;
-			mreal ccc = gr->GetC(ss,dd*xm,false);
-			mgl_drop(gr,mglPoint(xx, yy, zVal),q,(dx<dy?dx:dy)/2,ccc,dd*xm,1);
+			mgl_drop(gr,mglPoint(xx, yy, zVal),q,(dx<dy?dx:dy)/2,gr->GetC(ss,dd*xm,false),dd*xm,1);
 		}
 	}
 	gr->EndGroup();
@@ -542,7 +562,6 @@ void MGL_EXPORT mgl_dew_2d(HMGL gr, HCDT ax, HCDT ay, const char *sch, const cha
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_dew_xy(gr,&x,&y,ax,ay,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dew_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *ax, uintptr_t *ay, const char *sch, const char *opt,int l,int lo)
@@ -566,11 +585,7 @@ void MGL_EXPORT mgl_putsw(HMGL gr, double x, double y, double z,const wchar_t *t
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_puts_dir(HMGL gr, double x, double y, double z, double dx, double dy, double dz, const char *text, const char *font, double size)
 {
-	long len = mbstowcs(0,text,0)+1;
-	wchar_t *buf = new wchar_t[len+1];
-	mbstowcs(buf,text,len);
-	mgl_putsw_dir(gr, x, y, z, dx, dy, dz, buf, font, size);
-	delete []buf;
+	MGL_TO_WCS(text,mgl_putsw_dir(gr, x, y, z, dx, dy, dz, wcs, font, size));
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_putsw_dir(HMGL gr, double x, double y, double z, double dx, double dy, double dz, const wchar_t *text, const char *font, double size)
@@ -614,7 +629,7 @@ void MGL_EXPORT mgl_puts_dir_(uintptr_t *gr, mreal *x, mreal *y, mreal *z, mreal
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmarkw_xyzr(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const wchar_t *text, const char *fnt, const char *opt)
 {
-	long j,m,mx,my,mz,mr,n=y->GetNx();
+	long m,mx,my,mz,mr,n=y->GetNx();
 	if(mgl_check_dim0(gr,x,y,z,r,"TextMark"))	return;
 
 	gr->SaveState(opt);
@@ -625,16 +640,16 @@ void MGL_EXPORT mgl_textmarkw_xyzr(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, cons
 	gr->Reserve(n*m);
 
 	mglPoint p,q(NAN);
-	for(j=0;j<m;j++)
+	for(long j=0;j<m;j++)
 	{
 		mx = j<x->GetNy() ? j:0;	my = j<y->GetNy() ? j:0;
 		mz = j<z->GetNy() ? j:0;	mr = j<r->GetNy() ? j:0;
-		register long i,k;
-		for(i=0;i<n;i++)
+#pragma omp parallel for private(p)	// NOTE this should be useless ?!?
+		for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	return;
+			if(gr->Stop)	continue;
 			p = mglPoint(x->v(i,mx), y->v(i,my), z->v(i,mz));
-			k = gr->AddPnt(p,-1,q);
+			register long k = gr->AddPnt(p,-1,q);
 			gr->text_plot(k, text, fnt, -0.5*fabs(r->v(i,mr)));
 		}
 	}
@@ -646,7 +661,6 @@ void MGL_EXPORT mgl_textmarkw_xyr(HMGL gr, HCDT x, HCDT y, HCDT r, const wchar_t
 	gr->SaveState(opt);
 	mglData z(y->GetNx());	z.Fill(gr->Min.z,gr->Min.z);
 	mgl_textmarkw_xyzr(gr,x,y,&z,r,text,fnt,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmarkw_yr(HMGL gr, HCDT y, HCDT r, const wchar_t *text, const char *fnt, const char *opt)
@@ -656,7 +670,6 @@ void MGL_EXPORT mgl_textmarkw_yr(HMGL gr, HCDT y, HCDT r, const wchar_t *text, c
 	mglData x(n);	x.Fill(gr->Min.x,gr->Max.x);
 	mglData z(n);	z.Fill(gr->Min.z,gr->Min.z);
 	mgl_textmarkw_xyzr(gr,&x,y,&z,r,text,fnt,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmarkw(HMGL gr, HCDT y, const wchar_t *text, const char *fnt, const char *opt)
@@ -667,24 +680,19 @@ void MGL_EXPORT mgl_textmarkw(HMGL gr, HCDT y, const wchar_t *text, const char *
 	mglData x(n);	x.Fill(gr->Min.x,gr->Max.x);
 	mglData z(n);	z.Fill(gr->Min.z,gr->Min.z);
 	mgl_textmarkw_xyzr(gr,&x,y,&z,&r,text,fnt,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmark_xyzr(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT r, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_textmarkw_xyzr(gr, x, y, z, r, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_textmarkw_xyzr(gr, x, y, z, r, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmark_xyr(HMGL gr, HCDT x, HCDT y, HCDT r, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_textmarkw_xyr(gr, x, y, r, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_textmarkw_xyr(gr, x, y, r, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmark_yr(HMGL gr, HCDT y, HCDT r, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_textmarkw_yr(gr, y, r, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_textmarkw_yr(gr, y, r, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmark(HMGL gr, HCDT y, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_textmarkw(gr, y, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_textmarkw(gr, y, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_textmark_xyzr_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *r, const char *text, const char *fnt, const char *opt, int l,int n,int lo)
 {	wchar_t *s=new wchar_t[l+1];	memcpy(s,text,l);	s[l]=0;
@@ -718,25 +726,24 @@ void MGL_EXPORT mgl_textmark_(uintptr_t *gr, uintptr_t *y, const char *text, con
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_labelw_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const wchar_t *text, const char *fnt, const char *opt)
 {
-	long j,m,mx,my,mz,n=y->GetNx();
+	long m,mx,my,mz,n=y->GetNx();
 	if(mgl_check_dim1(gr,x,y,z,0,"Label"))	return;
 
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Label",cgid++);
 	m = x->GetNy() > y->GetNy() ? x->GetNy() : y->GetNy();	m = z->GetNy() > m ? z->GetNy() : m;
 
-	register long i,k,kk,l,nn;
-	mglPoint p,q(NAN);
+	mglPoint q(NAN);
 	wchar_t tmp[32];
-	for(j=0;j<m;j++)
+	for(long j=0;j<m;j++)
 	{
 		mx = j<x->GetNy() ? j:0;	my = j<y->GetNy() ? j:0;	mz = j<z->GetNy() ? j:0;
-		for(i=0;i<n;i++)
+#pragma omp parallel for private(tmp)
+		for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	return;
+			if(gr->Stop)	continue;
 			mreal xx=x->v(i,mx), yy=y->v(i,my), zz=z->v(i,mz);
-			p = mglPoint(xx,yy,zz);
-			kk = gr->AddPnt(p,-1,q);
+			register long kk = gr->AddPnt(mglPoint(xx,yy,zz),-1,q),k,l;
 			std::wstring buf;
 			for(k=l=0;text[k];k++)
 			{
@@ -761,7 +768,6 @@ void MGL_EXPORT mgl_labelw_xy(HMGL gr, HCDT x, HCDT y, const wchar_t *text, cons
 	gr->SaveState(opt);
 	mglData z(y->GetNx());	z.Fill(gr->Min.z,gr->Min.z);
 	mgl_labelw_xyz(gr,x,y,&z,text,fnt,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_labelw_y(HMGL gr, HCDT y, const wchar_t *text, const char *fnt, const char *opt)
@@ -772,20 +778,16 @@ void MGL_EXPORT mgl_labelw_y(HMGL gr, HCDT y, const wchar_t *text, const char *f
 	mglData x(n);	x.Fill(gr->Min.x,gr->Max.x);
 	mglData z(n);	z.Fill(gr->Min.z,gr->Min.z);
 	mgl_labelw_xyz(gr,&x,y,&z,text,fnt,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_label_xyz(HMGL gr, HCDT x, HCDT y, HCDT z, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_labelw_xyz(gr, x, y, z, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_labelw_xyz(gr, x, y, z, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_label_xy(HMGL gr, HCDT x, HCDT y, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_labelw_xy(gr, x, y, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_labelw_xy(gr, x, y, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_label_y(HMGL gr, HCDT y, const char *str, const char *fnt, const char *opt)
-{	long s = mbstowcs(0,str,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,str,s);
-	mgl_labelw_y(gr, y, wcs, fnt, opt);	delete []wcs;	}
+{	MGL_TO_WCS(str,mgl_labelw_y(gr, y, wcs, fnt, opt));	}
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_label_xyz_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *text, const char *fnt, const char *opt, int l,int n,int lo)
 {	wchar_t *s=new wchar_t[l+1];	mbstowcs(s,text,l);	s[l]=0;
@@ -819,9 +821,7 @@ void MGL_EXPORT mgl_tablew(HMGL gr, double x, double y, HCDT val, const wchar_t
 void MGL_EXPORT mgl_table(HMGL gr, double x, double y, HCDT val, const char *text, const char *fnt, const char *opt)
 {
 	if(!text)	mgl_tablew(gr,x,y,val,L"",fnt,opt);
-	else
-	{	long s = mbstowcs(0,text,0)+1;	wchar_t *wcs = new wchar_t[s];	mbstowcs(wcs,text,s);
-		mgl_tablew(gr, x, y, val, wcs, fnt, opt);	delete []wcs;	}
+	else	MGL_TO_WCS(text,mgl_tablew(gr, x, y, val, wcs, fnt, opt));
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_table_(uintptr_t *gr, mreal *x, mreal *y, uintptr_t *val, const char *text, const char *fnt, const char *opt,int l,int n,int lo)
