@@ -1,6 +1,6 @@
 /***************************************************************************
  * surf.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -21,33 +21,39 @@
 #include "mgl2/surf.h"
 #include "mgl2/data.h"
 #include "mgl2/eval.h"
+#include "mgl2/base.h"
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_mesh_plot(mglBase *gr, long *pos, long n, long m, int how)
 {
 	int d = gr->MeshNum>0 ? gr->MeshNum+1 : n*m, dx = n>d?n/d:1, dy = m>d?m/d:1;
-	register long i,j,s;
 	// NOTE: number of lines in each direction can be reduced too
-	if(how&1)	for(j=0;j<m;j+=dy)
-	{
-		for(s=i=0;i<n-1;i++)	if(pos[n*j+i]>=0 && pos[n*j+i+1]>=0)	s++;
-		d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
-		for(i=0;i<n-s;i+=s)
-			gr->line_plot(pos[n*j+i],pos[n*j+i+s]);
+	if(how&1)
+#pragma omp parallel for
+		for(long j=0;j<m;j+=dy)
+		{
+			register long s,i;
+			for(s=i=0;i<n-1;i++)	if(pos[n*j+i]>=0 && pos[n*j+i+1]>=0)	s++;
+			d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
+			for(i=0;i<n-s;i+=s)
+				gr->line_plot(pos[n*j+i],pos[n*j+i+s]);
 
-	}
-	if(how&2)	for(i=0;i<n;i+=dx)
-	{
-		for(s=j=0;j<m-1;j++)	if(pos[n*j+i]>=0 && pos[n*j+i+n]>=0)	s++;
-		d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
-		for(j=0;j<m-s;j+=s)
-			gr->line_plot(pos[n*j+i],pos[n*j+i+s*n]);
-	}
+		}
+	if(how&2)
+#pragma omp parallel for
+		for(long i=0;i<n;i+=dx)
+		{
+			register long s,j;
+			for(s=j=0;j<m-1;j++)	if(pos[n*j+i]>=0 && pos[n*j+i+n]>=0)	s++;
+			d = gr->FaceNum>0 ? gr->FaceNum+1 : n;	s = s>d?s/d:1;
+			for(j=0;j<m-s;j+=s)
+				gr->line_plot(pos[n*j+i],pos[n*j+i+s*n]);
+		}
 }
 //-----------------------------------------------------------------------------
 void MGL_NO_EXPORT mgl_surf_plot(mglBase *gr, long *pos, long n, long m)
 {
-	register long i,j,s=0;
-	for(j=0;j<m-1;j++)	for(i=0;i<n-1;i++)
+	long s=0;
+	for(long j=0;j<m-1;j++)	for(long i=0;i<n-1;i++)
 		if(pos[n*j+i]>=0 && pos[n*j+i+1]>=0 && pos[n*j+i+n]>=0 && pos[n*j+i+n+1]>=0)
 			s++;
 	long dx=1,dy=1;
@@ -56,7 +62,8 @@ void MGL_NO_EXPORT mgl_surf_plot(mglBase *gr, long *pos, long n, long m)
 		int d = gr->FaceNum+1,ns=n*s/((n-1)*(m-1)),ms=m*s/((n-1)*(m-1));
 		dx = ns>d?ns/d:1;		dy = ms>d?ms/d:1;
 	}
-	for(j=0;j<m-dy;j+=dy)	for(i=0;i<n-dx;i+=dx)
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<m-dy;j+=dy)	for(long i=0;i<n-dx;i+=dx)
 		gr->quad_plot(pos[n*j+i],pos[n*j+i+dx],pos[n*j+i+n*dy],pos[n*j+i+n*dy+dx]);
 }
 //-----------------------------------------------------------------------------
@@ -71,11 +78,11 @@ void MGL_EXPORT mgl_fsurf(HMGL gr, const char *eqZ, const char *sch, const char
 	long n = (mgl_isnan(r) || r<=0) ? 100:long(r+0.5);
 	mglData z(n,n);
 	mglFormula *eq = new mglFormula(eqZ);
-	register int i,j;
 	mreal dx = (gr->Max.x - gr->Min.x)/(n-1.), dy = (gr->Max.y - gr->Min.y)/(n-1.);
-	for(j=0;j<n;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<n;j++)	for(long i=0;i<n;i++)
 	{
-		if(gr->Stop)	{	delete eq;	return;	}
+		if(gr->Stop)	continue;
 		z.a[i+n*j] = eq->Calc(gr->Min.x+i*dx, gr->Min.y+j*dy);
 	}
 	mgl_surf(gr, &z, sch,0);
@@ -93,12 +100,11 @@ void MGL_EXPORT mgl_fsurf_xyz(HMGL gr, const char *eqX, const char *eqY, const c
 	ex = new mglFormula(eqX ? eqX : "u");
 	ey = new mglFormula(eqY ? eqY : "v");
 	ez = new mglFormula(eqZ);
-	register int i,j;
-	register mreal u,v;
-	for(j=0;j<n;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<n;j++)	for(long i=0;i<n;i++)
 	{
-		if(gr->Stop)	{	delete ex;	delete ey;	delete ez;	return;	}
-		v = i/(n-1.);	u = j/(n-1.);
+		if(gr->Stop)	continue;
+		register mreal u = j/(n-1.), v = i/(n-1.);
 		x.a[i+n*j] = ex->Calc(0,v,0,u);
 		y.a[i+n*j] = ey->Calc(0,v,0,u);
 		z.a[i+n*j] = ez->Calc(0,v,0,u);
@@ -130,25 +136,24 @@ void MGL_EXPORT mgl_fsurf_xyz_(uintptr_t *gr, const char *fx, const char *fy, co
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_mesh_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Mesh"))	return;
 
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Mesh",cgid++);
-	gr->SetPenPal("-");
+	gr->SetPenPal(sch,0,false);
 	long ss = gr->AddTexture(sch);
-	long *pos = new long[n*m];
+	long *pos = new long[n*m];	memset(pos,-1L,n*m*sizeof(long));
 	gr->Reserve(n*m*z->GetNz());
 
-	mglPoint p;
-	mreal c;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
-			p = mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, z->v(i,j,k));
-			c = gr->GetC(ss,p.z);		pos[i+n*j] = gr->AddPnt(p,c);
+			if(gr->Stop)	continue;
+			register mreal zz = z->v(i,j,k);
+			pos[i+n*j] = gr->AddPnt(mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zz),gr->GetC(ss,zz));
 		}
 		mgl_mesh_plot(gr,pos,n,m,3);
 	}
@@ -162,7 +167,6 @@ void MGL_EXPORT mgl_mesh(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_mesh_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_mesh_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -181,25 +185,24 @@ void MGL_EXPORT mgl_mesh_(uintptr_t *gr, uintptr_t *a, const char *sch, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fall_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Fall"))	return;
 
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("Fall",cgid++);
-	gr->SetPenPal("-");
+	gr->SetPenPal(sch,0,false);
 	long ss = gr->AddTexture(sch);
 	long *pos = new long[n*m];
 	gr->Reserve(n*m*z->GetNz());
 
-	mglPoint p;
-	mreal c;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
-			p = mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, z->v(i,j,k));
-			c = gr->GetC(ss,p.z);	pos[i+n*j] = gr->AddPnt(p,c);
+			if(gr->Stop)	continue;
+			register mreal zz = z->v(i,j,k);
+			pos[i+n*j] = gr->AddPnt(mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zz),gr->GetC(ss,zz));
 		}
 		mgl_mesh_plot(gr,pos,n,m, (mglchr(sch,'x')) ? 2:1);
 	}
@@ -213,7 +216,6 @@ void MGL_EXPORT mgl_fall(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_fall_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_fall_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -232,7 +234,7 @@ void MGL_EXPORT mgl_fall_(uintptr_t *gr, uintptr_t *a, const char *sch, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_grid_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Grid"))	return;
 
 	gr->SaveState(opt);
@@ -242,15 +244,14 @@ void MGL_EXPORT mgl_grid_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	long *pos = new long[n*m];
 	gr->Reserve(n*m*z->GetNz());
 
-	mglPoint p;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
 		if(z->GetNz()>1)	zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(z->GetNz()-1);
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
-			p = mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zVal);
-			pos[i+n*j] = gr->AddPnt(p,gr->CDef);
+			if(gr->Stop)	continue;
+			pos[i+n*j] = gr->AddPnt(mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zVal),gr->CDef);
 		}
 		mgl_mesh_plot(gr,pos,n,m,3);
 	}
@@ -264,7 +265,6 @@ void MGL_EXPORT mgl_grid(HMGL gr, HCDT z,const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_grid_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_grid_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -283,7 +283,7 @@ void MGL_EXPORT mgl_grid_(uintptr_t *gr, uintptr_t *a,const char *sch, const cha
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surf_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Surf"))	return;
 
 	gr->SaveState(opt);
@@ -291,28 +291,30 @@ void MGL_EXPORT mgl_surf_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	long ss = gr->AddTexture(sch);
 	long *pos = new long[n*m];
 	bool wire = (mglchr(sch,'#'));
-	gr->Reserve(n*m*z->GetNz()*(wire?2:1));
+	gr->Reserve((n+1)*(m+1)*z->GetNz()*(wire?2:1));
 
 	mglPoint p,q,s,xx,yy;
-	mreal c;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for private(p,q,s,xx,yy) collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
+			if(gr->Stop)	continue;
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			p = mglPoint(xx.x, yy.x, z->v(i,j,k));
 			q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
 			s = mglPoint(xx.z, yy.z, z->dvy(i,j,k));
-			c = gr->GetC(ss,p.z);
-			pos[i+n*j] = gr->AddPnt(p,c,q^s);
+			pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,p.z),q^s);
 		}
-		if(sch && mglchr(sch,'.'))	for(i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
+		if(sch && mglchr(sch,'.'))
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(wire)
+		if(wire && !gr->Stop)
 		{
 			gr->SetPenPal("k-");
-			for(i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
 	}
@@ -326,7 +328,6 @@ void MGL_EXPORT mgl_surf(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_surf_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surf_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -345,7 +346,7 @@ void MGL_EXPORT mgl_surf_(uintptr_t *gr, uintptr_t *a, const char *sch, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_belt_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Belt"))	return;
 
 	gr->SaveState(opt);
@@ -357,35 +358,36 @@ void MGL_EXPORT mgl_belt_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	bool how = !mglchr(sch,'x');
 
 	mglPoint p1,p2,q,s,xx,yy;
-	mreal c;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
-		if(how)	for(i=0;i<n-dx;i+=dx)
+		if(how)	for(long i=0;i<n-dx;i+=dx)
 		{
-			for(j=0;j<m;j++)
+#pragma omp parallel for private(p1,p2,xx,yy,q,s)
+			for(long j=0;j<m;j++)
 			{
-				if(gr->Stop)	{	delete []pos;	return;	}
+				if(gr->Stop)	continue;
 				xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 				p1 = mglPoint(xx.x, yy.x, z->v(i,j,k));
 				s = mglPoint(xx.z, yy.z, z->dvy(i,j,k));
 				q = mglPoint(xx.y, yy.y, 0);	s = q^s;
-				c = gr->GetC(ss,p1.z);
+				register mreal c = gr->GetC(ss,p1.z);
 				p2 = mglPoint(GetX(x,i+dx,j,k).x,GetY(y,i+dx,j,k).x,p1.z);
 				pos[2*j] = gr->AddPnt(p1,c,s);
 				pos[2*j+1]=gr->AddPnt(p2,c,s);
 			}
 			mgl_surf_plot(gr,pos,2,m);
 		}
-		else	for(j=0;j<m-dy;j+=dy)
+		else	for(long j=0;j<m-dy;j+=dy)
 		{
-			for(i=0;i<n;i++)	// ñîçäàåì ìàññèâ òî÷åê
+#pragma omp parallel for private(p1,p2,xx,yy,q,s)
+			for(long i=0;i<n;i++)	// ñîçäàåì ìàññèâ òî÷åê
 			{
-				if(gr->Stop)	{	delete []pos;	return;	}
+				if(gr->Stop)	continue;
 				xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 				p1 = mglPoint(xx.x, yy.x, z->v(i,j,k));
 				q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
 				s = mglPoint(xx.z, yy.z, 0);	s = q^s;
-				c = gr->GetC(ss,p1.z);
+				register mreal c = gr->GetC(ss,p1.z);
 				p2 = mglPoint(GetX(x,i,j+dy,k).x,GetY(y,i,j+dy,k).x,p1.z);
 				pos[2*i] = gr->AddPnt(p1,c,s);
 				pos[2*i+1]=gr->AddPnt(p2,c,s);
@@ -403,7 +405,6 @@ void MGL_EXPORT mgl_belt(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_belt_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_belt_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -422,7 +423,7 @@ void MGL_EXPORT mgl_belt_(uintptr_t *gr, uintptr_t *a, const char *sch, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dens_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Dens"))	return;
 
 	gr->SaveState(opt);
@@ -431,29 +432,33 @@ void MGL_EXPORT mgl_dens_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 
 	long ss = gr->AddTexture(sch);
 	long *pos = new long[n*m];
-	gr->Reserve(n*m*z->GetNz());
+	bool wire = (mglchr(sch,'#'));
+	gr->Reserve((n+1)*(m+1)*z->GetNz()*(wire?2:1));
 
 	mglPoint p,s=mglPoint(0,0,1);
 	mreal zz, c;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
 		if(z->GetNz()>1)
 			zVal = gr->Min.z+(gr->Max.z-gr->Min.z)*mreal(k)/(z->GetNz()-1);
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)	// ñîçäàåì ìàññèâ òî÷åê
+#pragma omp parallel for private(p,c,zz) collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)	// ñîçäàåì ìàññèâ òî÷åê
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
+			if(gr->Stop)	continue;
 			p = mglPoint(GetX(x,i,j,k).x, GetY(y,i,j,k).x, zVal);
 			zz = z->v(i,j,k);	c = gr->GetC(ss,zz);
 			if(mgl_isnan(zz))	p.x = NAN;
 			pos[i+n*j] = gr->AddPnt(p,c,s);
 		}
-		if(sch && mglchr(sch,'.'))	for(i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
+		if(sch && mglchr(sch,'.'))
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(mglchr(sch,'#'))
+		if(wire && !gr->Stop)
 		{
-			gr->Reserve(n*m);	gr->SetPenPal("k-");
-			for(i=0;i<n*m;i++)
-				pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
+			gr->SetPenPal("k-");
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
 	}
@@ -467,7 +472,6 @@ void MGL_EXPORT mgl_dens(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x, gr->Max.x);
 	y.Fill(gr->Min.y, gr->Max.y);
 	mgl_dens_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_dens_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -506,35 +510,38 @@ void MGL_EXPORT mgl_stfa_(uintptr_t *gr, uintptr_t *re, uintptr_t *im, int *dn,
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surfc_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,c,"SurfC"))	return;
 
 	gr->SaveState(opt);
 	static int cgid=1;	gr->StartGroup("SurfC",cgid++);
 	long ss = gr->AddTexture(sch);
 	long *pos = new long[n*m];
-	gr->Reserve(n*m*z->GetNz());
-	mreal col;
+	bool wire = (mglchr(sch,'#'));
+	gr->Reserve((n+1)*(m+1)*z->GetNz()*(wire?2:1));
 
 	mglPoint p,q,s,xx,yy;
-	for(k=0;k<z->GetNz();k++)
+	for(long k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for private(p,xx,yy,q,s) collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
+			if(gr->Stop)	continue;
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			p = mglPoint(xx.x, yy.x, z->v(i,j,k));
 			q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
 			s = mglPoint(xx.z, yy.z, z->dvy(i,j,k));
-			col = gr->GetC(ss,c->v(i,j,k));
-			pos[i+n*j] = gr->AddPnt(p,col,q^s);
+			pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,c->v(i,j,k)),q^s);
 		}
-		if(sch && mglchr(sch,'.'))	for(i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
+		if(sch && mglchr(sch,'.'))
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(mglchr(sch,'#'))
+		if(wire && !gr->Stop)
 		{
-			gr->Reserve(n*m);	gr->SetPenPal("k-");
-			for(i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
+			gr->SetPenPal("k-");
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
 	}
@@ -548,7 +555,6 @@ void MGL_EXPORT mgl_surfc(HMGL gr, HCDT z, HCDT c, const char *sch, const char *
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_surfc_xy(gr,&x,&y,z,c,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surfc_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -567,7 +573,6 @@ void MGL_EXPORT mgl_surfc_(uintptr_t *gr, uintptr_t *z, uintptr_t *a, const char
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surfa_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char *sch, const char *opt)
 {
-	register long i,j;
 	long k,n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,c,"SurfA"))	return;
 
@@ -575,26 +580,31 @@ void MGL_EXPORT mgl_surfa_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT c, const char
 	static int cgid=1;	gr->StartGroup("SurfA",cgid++);
 	long ss = gr->AddTexture(sch);
 	long *pos = new long[n*m];
-	gr->Reserve(n*m*z->GetNz());
+	bool wire = (mglchr(sch,'#'));
+	gr->Reserve((n+1)*(m+1)*z->GetNz()*(wire?2:1));
 
 	mglPoint p,q,s,xx,yy;
 	for(k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for private(p,xx,yy,q,s) collapse(2)
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			if(gr->Stop)	{	delete []pos;	return;	}
+			if(gr->Stop)	continue;
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			mreal vv = z->v(i,j,k);	p = mglPoint(xx.x, yy.x, vv);
 			q = mglPoint(xx.y, yy.y, z->dvx(i,j,k));
 			s = mglPoint(xx.z, yy.z, z->dvy(i,j,k));
 			pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,vv),q^s,gr->GetA(c->v(i,j,k)));
 		}
-		if(sch && mglchr(sch,'.'))	for(i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
+		if(sch && mglchr(sch,'.'))
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 		else	mgl_surf_plot(gr,pos,n,m);
-		if(mglchr(sch,'#'))
+		if(wire && !gr->Stop)
 		{
-			gr->Reserve(n*m);	gr->SetPenPal("k-");
-			for(i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
+			gr->SetPenPal("k-");
+#pragma omp parallel for
+			for(long i=0;i<n*m;i++)	pos[i] = gr->CopyNtoC(pos[i],gr->CDef);
 			mgl_mesh_plot(gr,pos,n,m,3);
 		}
 	}
@@ -608,7 +618,6 @@ void MGL_EXPORT mgl_surfa(HMGL gr, HCDT z, HCDT c, const char *sch, const char *
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_surfa_xy(gr,&x,&y,z,c,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_surfa_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -647,7 +656,7 @@ void MGL_EXPORT mgl_boxs_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	{
 		for(i=0;i<n;i+=dx)	for(j=0;j<m;j+=dy)
 		{
-			if(gr->Stop)	return;
+			if(gr->Stop)	continue;
 			zz = z->v(i,j,k);		c  = gr->GetC(ss,zz);
 			xx = GetX(x,i,j,k);		yy = GetY(y,i,j,k);
 			x1 = i<lx-dx ? GetX(x,i+dx,j,k).x:NAN;
@@ -716,7 +725,6 @@ void MGL_EXPORT mgl_boxs(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_boxs_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_boxs_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -735,7 +743,7 @@ void MGL_EXPORT mgl_boxs_(uintptr_t *gr, uintptr_t *a, const char *sch, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tile_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,0,"Tile",true))	return;
 
 	gr->SaveState(opt);
@@ -746,26 +754,25 @@ void MGL_EXPORT mgl_tile_xy(HMGL gr, HCDT x, HCDT y, HCDT z, const char *sch, co
 	long ss = gr->AddTexture(sch);
 	gr->Reserve(4*n*m*z->GetNz());
 
-	mglPoint p1,p2,p3,p4,s=mglPoint(0,0,1);
-	mreal zz,x0,y0,x1,y1,x2,y2,x3,y3,c;
-	long k1,k2,k3,k4;
-	for(k=0;k<z->GetNz();k++)
+	mglPoint s=mglPoint(0,0,1);
+	for(long k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j+=dx)	for(i=0;i<n;i+=dy)
+#pragma omp parallel for collapse(2)
+		for(long j=0;j<m;j+=dx)	for(long i=0;i<n;i+=dy)
 		{
-			if(gr->Stop)	return;
-			zz = z->v(i,j,k);		c = gr->GetC(ss,zz);
-			x0 = GetX(x,i,j,k).x;	y0 = GetY(y,i,j,k).x;
-			x1 = i<lx-dx ? GetX(x,i+dx,j,k).x:NAN;
-			y1 = i<lx-dx ? GetY(y,i+dx,j,k).x:NAN;
-			x2 = j<ly-dy ? GetX(x,i,j+dy,k).x:NAN;
-			y2 = j<ly-dy ? GetY(y,i,j+dy,k).x:NAN;
-			x3 = i<lx-dx && j<ly-dy ? GetX(x,i+dx,j+dy,k).x:NAN;
-			y3 = i<lx-dx && j<ly-dy ? GetY(y,i+dx,j+dy,k).x:NAN;
-			p1 = mglPoint(x0,y0,zz);	k1 = gr->AddPnt(p1,c,s);
-			p2 = mglPoint(x1,y1,zz);	k2 = gr->AddPnt(p2,c,s);
-			p3 = mglPoint(x2,y2,zz);	k3 = gr->AddPnt(p3,c,s);
-			p4 = mglPoint(x3,y3,zz);	k4 = gr->AddPnt(p4,c,s);
+			if(gr->Stop)	continue;
+			register mreal zz = z->v(i,j,k), c = gr->GetC(ss,zz);
+			register mreal xx = GetX(x,i,j,k).x, yy = GetY(y,i,j,k).x;
+			register long k1 = gr->AddPnt(mglPoint(xx,yy,zz),c,s);
+			xx = i<lx-dx ? GetX(x,i+dx,j,k).x:NAN;
+			yy = i<lx-dx ? GetY(y,i+dx,j,k).x:NAN;
+			register long k2 = gr->AddPnt(mglPoint(xx,yy,zz),c,s);
+			xx = j<ly-dy ? GetX(x,i,j+dy,k).x:NAN;
+			yy = j<ly-dy ? GetY(y,i,j+dy,k).x:NAN;
+			register long k3 = gr->AddPnt(mglPoint(xx,yy,zz),c,s);
+			xx = i<lx-dx && j<ly-dy ? GetX(x,i+dx,j+dy,k).x:NAN;
+			yy = i<lx-dx && j<ly-dy ? GetY(y,i+dx,j+dy,k).x:NAN;
+			register long k4 = gr->AddPnt(mglPoint(xx,yy,zz),c,s);
 			gr->quad_plot(k1,k2,k3,k4);
 		}
 	}
@@ -779,7 +786,6 @@ void MGL_EXPORT mgl_tile(HMGL gr, HCDT z, const char *sch, const char *opt)
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_tile_xy(gr,&x,&y,z,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tile_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, const char *sch, const char *opt,int l,int lo)
@@ -798,7 +804,7 @@ void MGL_EXPORT mgl_tile_(uintptr_t *gr, uintptr_t *a, const char *sch, const ch
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tiles_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT s, const char *sch, const char *opt)
 {
-	register long i,j,k,n=z->GetNx(),m=z->GetNy();
+	long n=z->GetNx(),m=z->GetNy();
 	if(mgl_check_dim2(gr,x,y,z,s,"TileS",true))	return;
 
 	gr->SaveState(opt);
@@ -809,16 +815,16 @@ void MGL_EXPORT mgl_tiles_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT s, const char
 	long cc = gr->AddTexture(sch);
 	gr->Reserve(4*n*m*z->GetNz());
 
-	mglPoint p1,p2,p3,p4,t=mglPoint(0,0,1);
-	mreal zz,x1,x2,x3,x4,y1,y2,y3,y4,ss,sm,c;
-	long k1,k2,k3,k4;
-	for(k=0;k<z->GetNz();k++)
+	mglPoint t=mglPoint(0,0,1);
+	mreal x1,x2,x3,x4,y1,y2,y3,y4;
+	for(long k=0;k<z->GetNz();k++)
 	{
-		for(j=0;j<m;j+=dx)	for(i=0;i<n;i+=dy)
+#pragma omp parallel for private(x1,x2,x3,x4,y1,y2,y3,y4) collapse(2)
+		for(long j=0;j<m;j+=dx)	for(long i=0;i<n;i+=dy)
 		{
-			if(gr->Stop)	return;
-			zz = z->v(i,j,k);	c = gr->GetC(cc,zz);
-			ss = (1-gr->GetA(s->v(i,j,k)))/2;	sm = 1-ss;
+			if(gr->Stop)	continue;
+			register mreal zz = z->v(i,j,k), c = gr->GetC(cc,zz);
+			register mreal ss = (1-gr->GetA(s->v(i,j,k)))/2, sm = 1-ss;
 
 			x1 = GetX(x,i,j,k).x;	y1 = GetY(y,i,j,k).x;
 			x2 = x3 = x4 = y2 = y3 = y4 = NAN;
@@ -830,14 +836,10 @@ void MGL_EXPORT mgl_tiles_xy(HMGL gr, HCDT x, HCDT y, HCDT z, HCDT s, const char
 			{	x3 = GetX(x,i+dx,j+dy,k).x-x2-x4-x1;
 				y3 = GetY(y,i+dx,j+dy,k).x-y2-y4-y1;	}
 
-			p1 = mglPoint(x1+x2*ss+x4*ss+x3*ss*ss, y1+y2*ss+y4*ss+y3*ss*ss, zz);
-			k1 = gr->AddPnt(p1,c,t);
-			p2 = mglPoint(x1+x2*sm+x4*ss+x3*ss*sm, y1+y2*sm+y4*ss+y3*ss*sm, zz);
-			k2 = gr->AddPnt(p2,c,t);
-			p3 = mglPoint(x1+x2*ss+x4*sm+x3*ss*sm, y1+y2*ss+y4*sm+y3*ss*sm, zz);
-			k3 = gr->AddPnt(p3,c,t);
-			p4 = mglPoint(x1+x2*sm+x4*sm+x3*sm*sm, y1+y2*sm+y4*sm+y3*sm*sm, zz);
-			k4 = gr->AddPnt(p4,c,t);
+			register long k1 = gr->AddPnt(mglPoint(x1+x2*ss+x4*ss+x3*ss*ss, y1+y2*ss+y4*ss+y3*ss*ss, zz),c,t);
+			register long k2 = gr->AddPnt(mglPoint(x1+x2*sm+x4*ss+x3*ss*sm, y1+y2*sm+y4*ss+y3*ss*sm, zz),c,t);
+			register long k3 = gr->AddPnt(mglPoint(x1+x2*ss+x4*sm+x3*ss*sm, y1+y2*ss+y4*sm+y3*ss*sm, zz),c,t);
+			register long k4 = gr->AddPnt(mglPoint(x1+x2*sm+x4*sm+x3*sm*sm, y1+y2*sm+y4*sm+y3*sm*sm, zz),c,t);
 			gr->quad_plot(k1,k2,k3,k4);
 		}
 	}
@@ -851,7 +853,6 @@ void MGL_EXPORT mgl_tiles(HMGL gr, HCDT z, HCDT s, const char *sch, const char *
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_tiles_xy(gr,&x,&y,z,s,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_tiles_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, uintptr_t *r, const char *sch, const char *opt,int l,int lo)
@@ -870,7 +871,7 @@ void MGL_EXPORT mgl_tiles_(uintptr_t *gr, uintptr_t *a, uintptr_t *r, const char
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char *sch, const char *opt)
 {
-	register long i,j,n=ax->GetNx(),m=ax->GetNy();
+	long n=ax->GetNx(),m=ax->GetNy();
 	if(mgl_check_dim2(gr,x,y,ax,ay,"Map"))	return;
 
 	bool both = x->GetNx()==n && y->GetNx()==n && x->GetNy()==m && y->GetNy()==m;
@@ -878,25 +879,25 @@ void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char
 	static int cgid=1;	gr->StartGroup("Map",cgid++);
 
 	long ss = gr->AddTexture(mgl_have_color(sch)?sch:"rgb",2);
-	long s = both ? n:1, s1, s2;
+	long s = both ? n:1;
 
-	mreal xdy,xdx,ydx,ydy,xx,yy;
-	mglPoint p,t=mglPoint(NAN);
+	mglPoint t=mglPoint(NAN);
 	long *pos = new long[n*m];
 	gr->Reserve(n*m);
 
-	for(j=0;j<m;j++)	for(i=0;i<n;i++)
+#pragma omp parallel for collapse(2)
+	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 	{
-		if(gr->Stop)	{	delete []pos;	return;	}
-		s1 = i>0 ? 1:0;		s2 = i<n-1 ? 1:0;
-		xdx = (ax->v(i+s2,j)-ax->v(i-s1,j))/(GetX(x,i+s2,j).x-GetX(x,i-s1,j).x);
-		ydx = (ay->v(i+s2,j)-ay->v(i-s1,j))/(GetX(x,i+s2,j).x-GetX(x,i-s1,j).x);
+		if(gr->Stop)	continue;
+		register long s1 = i>0 ? 1:0, s2 = i<n-1 ? 1:0;
+		register mreal xdx = (ax->v(i+s2,j)-ax->v(i-s1,j))/(GetX(x,i+s2,j).x-GetX(x,i-s1,j).x);
+		register mreal ydx = (ay->v(i+s2,j)-ay->v(i-s1,j))/(GetX(x,i+s2,j).x-GetX(x,i-s1,j).x);
 		s1 = j>0 ? s:0;		s2 = j<m-1 ? s:0;
-		xdy = (ax->v(i,j+s2)-ax->v(i,j-s1))/(GetY(y,i,j+s2).x-GetY(y,i,j-s1).x);
-		ydy = (ay->v(i,j+s2)-ay->v(i,j-s1))/(GetY(y,i,j+s2).x-GetY(y,i,j-s1).x);
+		register mreal xdy = (ax->v(i,j+s2)-ax->v(i,j-s1))/(GetY(y,i,j+s2).x-GetY(y,i,j-s1).x);
+		register mreal ydy = (ay->v(i,j+s2)-ay->v(i,j-s1))/(GetY(y,i,j+s2).x-GetY(y,i,j-s1).x);
 		xdx = xdx*ydy - xdy*ydx;	// Jacobian
+		register mreal xx,yy;
 
-		p = mglPoint(ax->v(i,j), ay->v(i,j), xdx);
 		if(both)
 		{
 			xx = (x->v(i,j) - gr->Min.x)/(gr->Max.x - gr->Min.x);
@@ -909,9 +910,11 @@ void MGL_EXPORT mgl_map_xy(HMGL gr, HCDT x, HCDT y, HCDT ax, HCDT ay, const char
 		}
 		if(xx<0)	xx=0;	if(xx>=1)	xx=1/MGL_FEPSILON;
 		if(yy<0)	yy=0;	if(yy>=1)	yy=1/MGL_FEPSILON;
-		pos[i+n*j] = gr->AddPnt(p,gr->GetC(ss,xx,false),t,yy);
+		pos[i+n*j] = gr->AddPnt(mglPoint(ax->v(i,j), ay->v(i,j), xdx),gr->GetC(ss,xx,false),t,yy);
 	}
-	if(sch && mglchr(sch,'.'))	for(i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
+	if(sch && mglchr(sch,'.'))
+#pragma omp parallel for
+		for(long i=0;i<n*m;i++)	gr->mark_plot(pos[i],'.');
 	else	mgl_surf_plot(gr,pos,n,m);
 	delete []pos;	gr->EndGroup();
 }
@@ -923,7 +926,6 @@ void MGL_EXPORT mgl_map(HMGL gr, HCDT ax, HCDT ay, const char *sch, const char *
 	x.Fill(gr->Min.x,gr->Max.x);
 	y.Fill(gr->Min.y,gr->Max.y);
 	mgl_map_xy(gr,&x,&y,ax,ay,sch,0);
-	gr->LoadState();
 }
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_map_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, uintptr_t *a, uintptr_t *b, const char *sch, const char *opt,int l,int lo)
