@@ -19,11 +19,14 @@
  ***************************************************************************/
 var obj;
 var ctx;
+var cw,ch;
 var deg = Math.PI/180;  //0.017453293;
 
 var main = function()
 {
 	ctx = document.getElementById("canvas").getContext("2d");
+	cw = document.getElementById("canvas").width;
+	ch = document.getElementById("canvas").height;
 	ctx.lineCap="round";	// global setting
 
 	mgl_init("json/alpha.json");
@@ -40,7 +43,7 @@ var mglChange = function()
 	var name = document.getElementById("select").value;
 	mgl_init("json/"+name+".json");
 	var t1 = new Date();
-	ctx.clearRect(0,0,obj.width,obj.height);
+	ctx.clearRect(0,0,cw,ch);
 	mgl_draw_good(obj, ctx);
 //	draw_fast(obj, ctx);
 	var t2 = new Date();
@@ -50,7 +53,7 @@ var mglChange = function()
 // mouse handling functions
 var mglMouseUp = function()
 {	obj.button = 0;	obj.good = 0;
-	ctx.clearRect(0,0,obj.width,obj.height);
+	ctx.clearRect(0,0,cw,ch);
 	mgl_draw_good(obj, ctx);	}
 var mglMouseDown = function(event)
 {
@@ -66,11 +69,11 @@ var mglMouseMove = function(event)
 	switch(obj.button)
 	{
 	case 1: // rotate
-		mgl_rotate_down(obj, y*90/obj.height);
-		mgl_rotate_left(obj, x*90/obj.width);	break;
+		mgl_rotate_down(obj, y*180/ch);
+		mgl_rotate_left(obj, x*180/cw);	break;
 	case 2: // shift
-		mgl_shift_down(obj, y/obj.height);
-		mgl_shift_right(obj, x/obj.width);		break;
+		mgl_shift_down(obj, y/ch);
+		mgl_shift_right(obj, x/cw);		break;
 	case 3: // zoom
 		mgl_zoom_in(obj, Math.pow(1.003,x));	break;
 	 }
@@ -90,7 +93,7 @@ var mglMouseWheel = function(event)
 var mglRestore = function()
 {
 	mgl_restore(obj);
-	ctx.clearRect(0,0,obj.width,obj.height);
+	ctx.clearRect(0,0,cw,ch);
 	mgl_draw_good(obj,ctx);
 }
 
@@ -115,14 +118,14 @@ var mgl_init = function(name)
 		}
 	};*/
 	req.send(null);
-	if(name[name.length-1]!='z')
+//	if(name[name.length-1]!='z')
 		txt = req.responseText;
-	else
-	{
-console.debug("compressed=",req.responseText);
-		var inflate = new Zlib.Inflate(req.responseText);
-		txt = inflate.decompress();
-	}
+// 	else
+// 	{
+// console.debug("compressed=",req.responseText);
+// 		var inflate = new Zlib.Inflate(req.responseText);
+// 		txt = inflate.decompress();
+// 	}
 	obj = JSON.parse(txt);
 
 	// copy original data for transformation
@@ -136,10 +139,10 @@ console.debug("compressed=",req.responseText);
 	obj.phi= 0;		// current phi-angle for rotation
 	obj.bet= 0;		// current beta-angle for rotation
 	obj.button = 0;	// pressed mouse buttons (0-none, 1-left, 2-right)
-	obj.mouseX=0;
-	obj.mouseY=0;
-	obj.fast = 0;
-	obj.good = 0;
+	obj.mouseX=0;	obj.mouseY=0;
+	obj.fast = 0;	obj.good = 0;
+	obj.dx = cw/obj.width;
+	obj.dy = ch/obj.height;
 }
 
 // Functions for rotation, shifting and zooming of the picture as whole
@@ -183,7 +186,7 @@ var mgl_shift_right = function(obj,val)	// shift right
 // This function make drawing itself
 var mgl_draw = function(obj, ctx)
 {
-	ctx.clearRect(0,0,obj.width,obj.height);
+	ctx.clearRect(0,0,cw,ch);
 	if(obj.good==0)
 	{
 		obj.good = 1;
@@ -207,23 +210,17 @@ var mgl_draw_fast = function(obj, ctx, skip)
 {
 	if(obj.fast==0)	return;
 	mgl_prepare(obj,skip);	// update coordinates
-	var i,n1;
-	for(var i=0;i<obj.nprim;i++)	// for each primitive
+	// for each primitive skipping superfluous
+	for(var i=0;i<obj.nprim;i ++)
 	{
-		var n1 = obj.prim[i][1], n2 = obj.prim[i][2];
-		if(obj.prim[i][0]==1)
-		{
-			ctx.strokeStyle = obj.prim[i][10];
-			ctx.beginPath();
-			ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
-			ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
-			ctx.lineWidth = obj.prim[i][7];
-			ctx.stroke();
-		}
-		else
+		var prim = obj.prim[i];
+		var n1 = prim[1], nn = obj.pp[n1];
+		if(prim[0]==1 || obj.pnts[n1][3]<0)
+			mgl_draw_prim(obj,ctx,prim,Math.abs(obj.b[12]));
+		else if(obj.prim[i][0]<4)
 		{
 			ctx.fillStyle = obj.prim[i][10];
-			ctx.fillRect(obj.pp[n1][0], obj.pp[n1][1], 2, 2);
+			ctx.fillRect(nn[0], nn[1], 2, 2);
 		}
 	}
 	obj.fast = 0;
@@ -235,78 +232,87 @@ var mgl_draw_good = function(obj, ctx, skip)
 	obj.fast = 0;
 	mgl_prepare(obj,skip);	// update coordinates
 	// NOTE: this valid only for current zoom/view. In general case it should be more complicated
-	var scl = Math.sqrt(obj.b[0]*obj.b[0]+obj.b[1]*obj.b[1]+obj.b[2]*obj.b[2]);
+	var s1 = Math.sqrt(obj.b[0]*obj.b[0]+obj.b[1]*obj.b[1]+obj.b[2]*obj.b[2]), s2 = Math.abs(obj.b[12]);
 	for(var i=0;i<obj.nprim;i++)	// for each primitive
 	{
-		var n1 = obj.prim[i][1], n2 = obj.prim[i][2];
-		var n3 = obj.prim[i][3], n4 = obj.prim[i][4];
-		ctx.strokeStyle = obj.prim[i][10];
-		ctx.fillStyle = obj.prim[i][10];
-		ctx.lineWidth = 1;
-		switch(obj.prim[i][0])		// draw it depending on its type
-		{
-		case 0: // marks
-//			var d = 0.35*(obj.width>obj.height?obj.height:obj.width)*scl;
-			ctx.lineWidth = obj.prim[i][7]*obj.prim[i][6]*50;
-			mgl_draw_mark(ctx, obj.pp[n1][0], obj.pp[n1][1], n4, obj.prim[i][6], scl);
-			break;
-		case 1: // lines
-			ctx.beginPath();
-			ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
-			ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
-			ctx.lineWidth = obj.prim[i][7];
-			ctx.stroke();	break;
-		case 2: // triangles
-			ctx.beginPath();
-			ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
-			ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
-			ctx.lineTo(obj.pp[n3][0],obj.pp[n3][1]);
-			ctx.closePath();	ctx.fill();	break;
-		case 3: // quadrangles
-			ctx.beginPath();
-			ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
-			ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
-			ctx.lineTo(obj.pp[n4][0],obj.pp[n4][1]);
-			ctx.lineTo(obj.pp[n3][0],obj.pp[n3][1]);
-			ctx.closePath();
-			// NOTE: look as alpha is disabled for lines
-			// So, next code should be only for the case alpha=false
-			if(obj.prim[i][10].charAt(0)=='#')	ctx.stroke();
-			ctx.fill();	break;
-		case 4: // glyphs
-			var t=obj.prim[i][7]*deg;
-			var xx=obj.coor[n2][2],yy=-obj.coor[n2][3],zz=obj.coor[n2][4];
-			var xc = obj.b[0]*xx + obj.b[1]*yy + obj.b[2]*zz;
-			var yc = obj.b[3]*xx + obj.b[4]*yy + obj.b[5]*zz;
-			var zc = obj.b[6]*xx + obj.b[7]*yy + obj.b[8]*zz;
-			var ll = xc*xc+yc*yc;
-			if(ll < 1e-10)	break;
-			if(ll<1e10 && t/deg<1e4)
-			{
-				t = Math.atan2(yc,xc);
-				if(Math.abs(t)>Math.PI/2)	t += Math.PI;
-			}
-			else t=0;
-			var c=Math.cos(t), s=Math.sin(t), d=obj.prim[i][6]/2;
-
-			var b=[d*c, d*s, d*s, -d*c, obj.pp[n1][0],obj.pp[n1][1]];
-			var x=obj.coor[n2][0]*scl, y=obj.coor[n2][1]*scl, f=obj.prim[i][8]*scl;
-			if(n3&8)
-			{
-				if(!(n3&4))	mgl_line_glyph(ctx, x,y, f,1,b);
-				mgl_line_glyph(ctx, x,y, f,0,b);
-			}
-			else
-			{
-				if(!(n3&4)) mgl_fill_glyph(ctx, x,y, f,obj.glfs[n4],b);
-				mgl_wire_glyph(ctx, x,y, f,obj.glfs[n4],b);
-			}
-			break;
-		}
+		var prim = obj.prim[i];
+		mgl_draw_prim(obj,ctx,prim,obj.pnts[prim[1]][3]?s2:s1);
 	}
 	obj.good = 0;
 }
 
+/** perform high-quality drawing */
+var mgl_draw_prim = function(obj, ctx, prim, scl)
+{
+	var n1 = prim[1], n2 = prim[2];
+	var n3 = prim[3], n4 = prim[4];
+	var pp = obj.pp;
+	var deg = Math.PI/180;  //0.017453293;
+
+	ctx.strokeStyle = prim[10];
+	ctx.fillStyle = prim[10];
+	ctx.lineWidth = 1;
+	switch(prim[0])		// draw it depending on its type
+	{
+	case 0: // marks
+		ctx.lineWidth = prim[7]*prim[6]*5e-4;
+		mgl_draw_mark(ctx, obj.pp[n1][0], obj.pp[n1][1], n4, prim[6]/100, scl);
+		break;
+	case 1: // lines
+		ctx.beginPath();
+		ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
+		ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
+		ctx.lineWidth = prim[7]/100;
+		ctx.stroke();	break;
+	case 2: // triangles
+		ctx.beginPath();
+		ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
+		ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
+		ctx.lineTo(obj.pp[n3][0],obj.pp[n3][1]);
+		ctx.closePath();	ctx.fill();	break;
+	case 3: // quadrangles
+		ctx.beginPath();
+		ctx.moveTo(obj.pp[n1][0],obj.pp[n1][1]);
+		ctx.lineTo(obj.pp[n2][0],obj.pp[n2][1]);
+		ctx.lineTo(obj.pp[n4][0],obj.pp[n4][1]);
+		ctx.lineTo(obj.pp[n3][0],obj.pp[n3][1]);
+		ctx.closePath();
+		// NOTE: look as alpha is disabled for lines
+		// So, next code should be only for the case alpha=false
+		if(prim[10].charAt(0)=='#')	ctx.stroke();
+		ctx.fill();	break;
+	case 4: // glyphs
+		var t=prim[7]*deg/100;
+		var xx=obj.coor[n2][2]/100,yy=-obj.coor[n2][3]/100,zz=obj.coor[n2][4]/100;
+		var xc = obj.b[0]*xx + obj.b[1]*yy + obj.b[2]*zz;
+		var yc = obj.b[3]*xx + obj.b[4]*yy + obj.b[5]*zz;
+//			var zc = obj.b[6]*xx + obj.b[7]*yy + obj.b[8]*zz;
+		var ll = xc*xc+yc*yc;
+		if(ll < 1e-10)	break;
+		if(ll<1e10 && t/deg<1e4)
+		{
+			t = Math.atan2(yc,xc);
+			if(Math.abs(t)>Math.PI/2)	t += Math.PI;
+		}
+		else t=0;
+		var c=Math.cos(t), s=Math.sin(t), d=prim[6]/200;
+
+		var b=[d*c, d*s, d*s, -d*c, obj.pp[n1][0],obj.pp[n1][1]];
+		var x=obj.coor[n2][0]*scl/100, y=obj.coor[n2][1]*scl/100, f=prim[8]*scl/1e5;
+		if(n3&8)
+		{
+			if(!(n3&4))	mgl_line_glyph(ctx, x,y, f,1,b);
+			else	mgl_line_glyph(ctx, x,y, f,0,b);
+		}
+		else
+		{
+			if(!(n3&4)) mgl_fill_glyph(ctx, x,y, f,obj.glfs[n4],b);
+			else	mgl_wire_glyph(ctx, x,y, f,obj.glfs[n4],b);
+		}
+		break;
+	}
+}
+
 // This function change coordinates according current transformations
 // Usually this Function is called internally by draw()
 var mgl_prepare = function(obj, skip)
@@ -319,11 +325,11 @@ var mgl_prepare = function(obj, skip)
 		var cx=Math.cos(obj.tet*deg), sx=Math.sin(obj.tet*deg);	// tetx
 		var cy=Math.cos(obj.phi*deg), sy=Math.sin(obj.phi*deg);	// tety
 		var cz=Math.cos(obj.bet*deg), sz=Math.sin(obj.bet*deg);	// tetz
-		obj.b = [dx*cx*cy, -dx*cy*sx, dx*sy,
-					dy*(cx*sy*sz+cz*sx), dy*(cx*cz-sx*sy*sz), -dy*cy*sz,
-					sx*sz-cx*cz*sy, cx*sz+cz*sx*sy, cy*cz,
-					obj.width/2*(1+dx-obj.z[1]-obj.z[0])/dx,
-					obj.height/2*(1+dy-obj.z[3]-obj.z[2])/dy, obj.depth/2];
+		obj.b = [obj.dx*dx*cx*cy, -obj.dx*dx*cy*sx, obj.dx*dx*sy,
+				obj.dy*dy*(cx*sy*sz+cz*sx), obj.dy*dy*(cx*cz-sx*sy*sz), -obj.dy*dy*cy*sz,
+				sx*sz-cx*cz*sy, cx*sz+cz*sx*sy, cy*cz,
+				cw/2*(1+dx-obj.z[1]-obj.z[0])/dx,
+				ch/2*(1+dy-obj.z[3]-obj.z[2])/dy, obj.depth/2, obj.dx*dx,obj.dy*dy,1];
 	}
 	// now transform points for found transformation matrix
 	var b = obj.b, i;
@@ -332,15 +338,21 @@ var mgl_prepare = function(obj, skip)
 		var x = obj.pnts[i][0]-obj.width/2;
 		var y = obj.pnts[i][1]-obj.height/2;
 		var z = obj.pnts[i][2]-obj.depth/2;
-		obj.pp[i] = [b[9]  + b[0]*x + b[1]*y + b[2]*z,
-					 b[10] + b[3]*x + b[4]*y + b[5]*z,
-					 b[11] + b[6]*x + b[7]*y + b[8]*z];
+		if(obj.pnts[i][3]==0)	// TODO: check later when mglInPlot will be ready
+			obj.pp[i] = [b[9]  + b[0]*x + b[1]*y + b[2]*z,
+						b[10] + b[3]*x + b[4]*y + b[5]*z,
+						b[11] + b[6]*x + b[7]*y + b[8]*z];
+		else
+			obj.pp[i] = [b[9]+b[12]*x,b[10]+b[13]*y,b[11]+b[14]*z];
 	}
 	if(obj.pf)	for(var i=0;i<obj.npnts;i++)	// perspective
 	{	// NOTE: it is not supported for coordinate determining now
 		var d = (1-obj.pf)/(1-obj.pf*obj.pp[i][2]/obj.depth);
-		obj.pp[i][0] = d*obj.pp[i][0] + (1-d)/2*obj.width;
-		obj.pp[i][1] = d*obj.pp[i][1] + (1-d)/2*obj.height;
+		if(obj.pnts[i][3]==0)	// TODO: check later when mglInPlot will be ready
+		{
+			obj.pp[i][0] = d*obj.pp[i][0] + (1-d)/2*obj.width;
+			obj.pp[i][1] = d*obj.pp[i][1] + (1-d)/2*obj.height;
+		}
 	}
 	// fill z-coordinates for primitives
 	if(!obj.fast)
@@ -476,30 +488,39 @@ var mgl_draw_mark = function(ctx,x,y,st,size,d)
 // This function for internal use only!!!
 var mgl_fill_glyph = function(ctx, x,y, f,g,b)
 {
-	var xx,yy,j,xs,ys;
+	var xx,yy,j;
+	var np=0;	ctx.beginPath();
 	for(j=0;j<g[0];j++)
 	{
-		xx = x+f*g[2][6*j];	yy = y+f*g[2][6*j+1]; ctx.beginPath();
-		ctx.moveTo(b[4]+b[0]*xx+b[1]*yy, b[5]+b[2]*xx+b[3]*yy)
-		xx = x+f*g[2][6*j+2]; yy = y+f*g[2][6*j+3];
-		ctx.lineTo(b[4]+b[0]*xx+b[1]*yy, b[5]+b[2]*xx+b[3]*yy)
-		xx = x+f*g[2][6*j+4]; yy = y+f*g[2][6*j+5];
-		ctx.lineTo(b[4]+b[0]*xx+b[1]*yy, b[5]+b[2]*xx+b[3]*yy)
-		ctx.closePath();	ctx.fill();
+		xx = g[1][2*j]; yy = g[1][2*j+1];
+		if(xx==16383 && yy==16383)
+		{
+			ctx.closePath();	np = 1;
+		}
+		else if(np)
+		{
+			xx = x+f*xx;	yy = y+f*yy;	np = 0;
+			ctx.moveTo(b[4]+b[0]*xx+b[1]*yy, b[5]+b[2]*xx+b[3]*yy);
+		}
+		else
+		{
+			xx = x+f*xx;	yy = y+f*yy;
+			ctx.lineTo(b[4]+b[0]*xx+b[1]*yy, b[5]+b[2]*xx+b[3]*yy);
+		}
 	}
+	ctx.closePath();	ctx.fill('evenodd');
 }
 // This function for internal use only!!!
 var mgl_wire_glyph = function(ctx, x,y, f,g,b)
 {
-	var xx,yy,j,xs,ys;
-	var np=1;	ctx.beginPath();
-	for(j=0;j<g[1];j++)
+	var xx,yy,j;
+	var np=0;	ctx.beginPath();
+	for(j=0;j<g[0];j++)
 	{
-		xx = g[3][2*j]; yy = g[3][2*j+1];
+		xx = g[1][2*j]; yy = g[1][2*j+1];
 		if(xx==16383 && yy==16383)
 		{
-			ctx.closePath();	ctx.stroke();
-			ctx.beginPath();	np = 1;
+			ctx.closePath();	np = 1;
 		}
 		else if(np)
 		{
