@@ -180,15 +180,15 @@ void DatPanel::save()
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Export to PNG"), tr("Enter color scheme for picture"), QLineEdit::Normal, MGL_DEF_SCH, &ok);
-		if(ok)	var->Export(fn.toAscii().constData(), s.toAscii().constData());
+		if(ok)	var->Export(fn.toStdString().c_str(), s.toStdString().c_str());
 	}
 	else if(ext=="h5" || ext=="hdf")
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Save to HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromStdWString(var->s), &ok);
-		if(ok)	var->SaveHDF(fn.toAscii().constData(), s.toAscii().constData());
+		if(ok)	var->SaveHDF(fn.toStdString().c_str(), s.toStdString().c_str());
 	}
-	else 	var->Save(fn.toAscii().constData());
+	else 	var->Save(fn.toStdString().c_str());
 }
 //-----------------------------------------------------------------------------
 void DatPanel::load()
@@ -201,15 +201,15 @@ void DatPanel::load()
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Import PNG"), tr("Enter color scheme for picture"), QLineEdit::Normal, MGL_DEF_SCH, &ok);
-		if(ok)	var->Import(fn.toAscii().constData(), s.toAscii().constData());
+		if(ok)	var->Import(fn.toStdString().c_str(), s.toStdString().c_str());
 	}
 	else if(ext=="h5" || ext=="hdf")
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Read from HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromStdWString(var->s), &ok);
-		if(ok)	var->ReadHDF(fn.toAscii().constData(), s.toAscii().constData());
+		if(ok)	var->ReadHDF(fn.toStdString().c_str(), s.toStdString().c_str());
 	}
-	else 	var->Read(fn.toAscii().constData());
+	else 	var->Read(fn.toStdString().c_str());
 	refresh();
 }
 //-----------------------------------------------------------------------------
@@ -379,7 +379,7 @@ void DatPanel::hist()
 	bool res = d->exec();
 	if(res && !v1->text().isEmpty() && !v2->text().isEmpty() && !id->text().isEmpty())
 	{
-		mglVar *vv = parser.AddVar(id->text().toAscii().constData());
+		mglVar *vv = parser.AddVar(id->text().toStdString().c_str());
 		if(!vv)	return;
 		vv->Set(var->Hist(nm->value(), v1->text().toDouble(), v2->text().toDouble()));
 		updateDataItems();
@@ -437,12 +437,12 @@ bool DatPanel::sizesDialog(const QString &cap, const QString &lab, const QString
 //-----------------------------------------------------------------------------
 #include "xpm/plot.xpm"
 #include "xpm/size.xpm"
-#include "xpm/smth.xpm"
+//#include "xpm/smth.xpm"
 #include "xpm/crop.xpm"
 #include "xpm/squize.xpm"
-#include "xpm/sum.xpm"
-#include "xpm/func.xpm"
-#include "xpm/swap.xpm"
+//#include "xpm/sum.xpm"
+//#include "xpm/func.xpm"
+//#include "xpm/swap.xpm"
 #include "xpm/hist.xpm"
 #include "xpm/oper_dir.xpm"
 #include "xpm/oper_of.xpm"
@@ -514,7 +514,7 @@ void DatPanel::newdat()
 	if(!mgl.isEmpty())
 	{
 		mglGraph gr;
-		parser.Execute(&gr,mgl.toAscii().constData());
+		parser.Execute(&gr,mgl.toStdString().c_str());
 		opers += mgl+"\n";
 		updateDataItems();
 	}
@@ -522,7 +522,6 @@ void DatPanel::newdat()
 //-----------------------------------------------------------------------------
 void DatPanel::oper()
 {
-	QLabel *l;
 	QLineEdit *f1;
 	QPushButton *b;
 	QDialog *d = new QDialog(this);
@@ -586,7 +585,7 @@ void DatPanel::oper()
 	if(!mgl.isEmpty())
 	{
 		mglGraph gr;
-		parser.Execute(&gr,mgl.toAscii().constData());
+		parser.Execute(&gr,mgl.toStdString().c_str());
 		opers += mgl+"\n";
 		updateDataItems();
 	}
@@ -679,7 +678,7 @@ void DatPanel::toolTop(QBoxLayout *l)
 void DatPanel::toolLeft(QBoxLayout *l)
 {
 	QAction *a;
-	QMenu *o, *oo;
+	QMenu *o;
 	QToolButton *bb;
 
 	// size menu
@@ -725,13 +724,13 @@ void DatPanel::toolLeft(QBoxLayout *l)
 	a->setToolTip(tr("Find histogram of data."));
 	a->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_H);	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
-	
+
 /*	a = new QAction(QPixmap(":/xpm/view-refresh.png"), tr("Refresh"), this);
 	connect(a, SIGNAL(triggered()), this, SLOT(refresh()));
 	a->setToolTip(tr("Refresh data values."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);*/
-	
+
 /*	a = new QAction(tr("Rearrange"), this);	// TODO: move in generalized dialog
 	connect(a, SIGNAL(triggered()), this, SLOT(rearrange()));
 	a->setToolTip(tr("Rearrange data sizes without changing data values."));
