@@ -57,7 +57,7 @@ PlotPanel::PlotPanel(QWidget *parent) : QWidget(parent)
 	popup = new QMenu(this);
 	mgl = new QMathGL(this);
 	draw = new mglDrawScript(parser.Self());
-	mgl->getGraph()->set(MGL_SHOW_POS);	mgl->setDraw(draw);
+	mgl_set_flag(mgl->getGraph(),1,MGL_SHOW_POS);	mgl->setDraw(draw);
 	connect(mgl,SIGNAL(askStyle(int)),this,SLOT(setStyle(int)));
 
 	QBoxLayout *v,*h,*m;
@@ -92,7 +92,7 @@ void PlotPanel::execute()
 	raisePanel(this);
 	emit clearWarn();
 	QTime t;	t.start();
-	mgl->getGraph()->FaceNum=0;
+	mgl_set_facenum(mgl->getGraph(),0);
 	draw->text=textMGL->toPlainText();
 	draw->line=curPos;
 	mgl->update();
@@ -275,7 +275,7 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	a = new QAction(QPixmap(":/xpm/alpha.png"), tr("Alpha"), this);
 	a->setShortcut(Qt::CTRL+Qt::Key_T);	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setAlpha(bool)));
-	connect(mgl, SIGNAL(alphaChanged(bool)), a, SLOT(setOn(bool)));
+	connect(mgl, SIGNAL(alphaChanged(bool)), a, SLOT(setChecked(bool)));
 	a->setToolTip(tr("Switch on/off transparency for the graphics (Ctrl+T)."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
@@ -283,7 +283,7 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	a = new QAction(QPixmap(":/xpm/weather-clear.png"), tr("Light"), this);
 	a->setShortcut(Qt::CTRL+Qt::Key_L);	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setLight(bool)));
-	connect(mgl, SIGNAL(lightChanged(bool)), a, SLOT(setOn(bool)));
+	connect(mgl, SIGNAL(lightChanged(bool)), a, SLOT(setChecked(bool)));
 	a->setToolTip(tr("Switch on/off lightning for the graphics (Ctrl+L)."));
 	o->addAction(a);
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
@@ -298,14 +298,14 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	a = new QAction(QPixmap(":/xpm/object-rotate-right.png"), tr("Rotate by mouse"), this);
 	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setRotate(bool)));
-	connect(mgl, SIGNAL(rotateChanged(bool)), a, SLOT(setOn(bool)));
+	connect(mgl, SIGNAL(rotateChanged(bool)), a, SLOT(setChecked(bool)));
 	a->setToolTip(tr("Switch on/off mouse handling of the graphics\n(rotation, shifting, zooming and perspective)."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 
 /*	a = new QAction(QPixmap(":/xpm/zoom-fit-best.png"), tr("Zoom by mouse"), this);
 	a->setCheckable(true);
 	connect(a, SIGNAL(toggled(bool)), mgl, SLOT(setZoom(bool)));
-	connect(mgl, SIGNAL(zoomChanged(bool)), a, SLOT(setOn(bool)));
+	connect(mgl, SIGNAL(zoomChanged(bool)), a, SLOT(setChecked(bool)));
 	a->setToolTip(tr("Switch on/off mouse zoom of selected region."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);*/
 
@@ -365,37 +365,37 @@ void PlotPanel::toolTop(QBoxLayout *l)
 	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 	a->setToolTip(tr("Add curve which properties can be changed later by mouse."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
-	
+
 	a = new QAction(QPixmap(mark_s_xpm), tr("Add rect"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(addRect()));
 	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 	a->setToolTip(tr("Add rectangle which properties can be changed later by mouse."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
-	
+
 	a = new QAction(QPixmap(mark_d_xpm), tr("Add rhombus"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(addRhomb()));
 	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 	a->setToolTip(tr("Add rhombus which properties can be changed later by mouse."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
-	
+
 	a = new QAction(QPixmap(mark_o_xpm), tr("Add ellipse"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(addEllipse()));
 	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 	a->setToolTip(tr("Add ellipse which properties can be changed later by mouse."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
-	
+
 	a = new QAction(QPixmap(mark_a_xpm), tr("Add mark"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(addMark()));
 	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 	a->setToolTip(tr("Add marker which properties can be changed later by mouse."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
-	
+
 	a = new QAction(QPixmap(text_xpm), tr("Add text"), this);
 	connect(a, SIGNAL(triggered()), mgl, SLOT(addText()));
 	connect(mgl, SIGNAL(usePrimChanged(bool)), a, SLOT(setVisible(bool)));
 	a->setToolTip(tr("Add text which properties can be changed later by mouse."));
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);	oo->addAction(a);
-	
+
 	o->addMenu(oo);	l->addStretch(1);
 
 	tet = new QSpinBox(this);	tet->setWrapping(true);
