@@ -104,7 +104,7 @@ StyleDialog::StyleDialog(QWidget *parent) : QDialog(parent)
 
 	nline = new QSlider(p);		g->addWidget(nline, 2, 2);
 	nline->setRange(1, 9);		nline->setValue(5);
-	nline->setTickPosition(QSlider::TicksAbove);
+	nline->setTickPosition(QSlider::TicksBothSides);
 	nline->setTickInterval(1);	nline->setPageStep(2);
 	nline->setOrientation(Qt::Horizontal);
 	
@@ -134,7 +134,7 @@ StyleDialog::StyleDialog(QWidget *parent) : QDialog(parent)
 		fillColors(cc[i]);
 		nn[i] = new QSlider(p);		g->addWidget(nn[i], i+1, 1);
 		nn[i]->setRange(1, 9);		nn[i]->setValue(5);
-		nn[i]->setTickPosition(QSlider::TicksAbove);
+		nn[i]->setTickPosition(QSlider::TicksBothSides);
 		nn[i]->setTickInterval(1);	nn[i]->setPageStep(2);
 		nn[i]->setOrientation(Qt::Horizontal);
 		connect(cc[i],SIGNAL(activated(int)), this, SLOT(updatePic()));
@@ -183,7 +183,7 @@ StyleDialog::StyleDialog(QWidget *parent) : QDialog(parent)
 	connect(rbC,SIGNAL(toggled(bool)), this, SLOT(updatePic()));
 	connect(rbR,SIGNAL(toggled(bool)), this, SLOT(updatePic()));
 	tab->addTab(p, tr("Font style"));
-	connect(tab,SIGNAL(currentChanged(QWidget *)), this, SLOT(updatePic()));
+	connect(tab,SIGNAL(currentChanged(int)), this, SLOT(updatePic()));
 
 	// dialog itself
 	v = new QVBoxLayout(this);	v->addWidget(tab);
@@ -328,11 +328,11 @@ void StyleDialog::updatePic()
 		if(i>0)
 		{
 			j = nline->value();
-			if(j!=5)	result += "{"+col[i-1]+char('0'+i)+"}";
+			if(j!=5)	result += "{"+col[i-1]+char('0'+j)+"}";
 			else		result += col[i-1];
 		}
 		i = width->value();		if(i>1)	result += char('0'+i);
-		gr.Plot(x,y,result.toAscii().constData());
+		gr.Plot(x,y,result.toStdString().c_str());
 		break;
 	case 1: // color sceheme
 		for(j=0;j<7;j++)
@@ -350,7 +350,7 @@ void StyleDialog::updatePic()
 		if(i==2)	result += 'T';
 		i = axial->currentIndex();
 		if(i>0)	result = result+':'+char('x'+i-1);
-		gr.Surf(a,result.toAscii().constData());
+		gr.Surf(a,result.toStdString().c_str());
 		break;
 	case 2: // text style
 		i = cfont->currentIndex();
@@ -364,7 +364,7 @@ void StyleDialog::updatePic()
 		if(rbL->isChecked())	result += 'L';
 		if(rbC->isChecked())	result += 'C';
 		if(rbR->isChecked())	result += 'R';
-		gr.Puts(mglPoint(0,-0.5),"Font test",result.toAscii().constData(),-10);
+		gr.Puts(mglPoint(0,-0.5),"Font test",result.toStdString().c_str(),-10);
 		break;
 	}
 	result = "'" + result + "'";
