@@ -38,6 +38,7 @@
 #include <QTextCodec>
 #include <QTranslator>
 #include <QVariant>
+#include <QMimeData>
 //-----------------------------------------------------------------------------
 #if !defined(WIN32) && !defined(__APPLE__)
 #include <X11/Xlib.h>
@@ -63,7 +64,7 @@ bool mglCompleter = true;
 bool loadInNewWnd = false;
 QString pathHelp;
 extern mglParse parser;
-extern QColor mglColorScheme[9];
+extern QColor mglColorScheme[10];
 extern QString defFontFamily;
 extern int defFontSize;
 extern QString pathFont;
@@ -128,13 +129,14 @@ int main(int argc, char **argv)
 	}
 
 	udavLoadDefCommands();
+	parser.AllowSetSize(true);
 	MainWindow *mw = new MainWindow();
 	if(argc>1)
 	{
 		QTextCodec *codec = QTextCodec::codecForLocale();
 		mw->load(codec->toUnicode(argv[1]), true);
 	}
-	mw->show();	parser.AllowSetSize(true);
+	mw->show();
 	a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));
 	if(showHint)	udavShowHint(mw);
 	return a.exec();
@@ -382,7 +384,7 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
 	QTextCodec *codec = QTextCodec::codecForLocale();
 	QString filename = codec->toUnicode(event->mimeData()->data("text/plain"));
-	if ( event->provides("text/plain") )
+	/*if ( event->provides("text/plain") )
 	{
 		QTextCodec *codec = QTextCodec::codecForLocale();
 		QString instring = codec->toUnicode(event->mimeData()->data("text/plain"));
@@ -391,8 +393,9 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 			event->acceptProposedAction();
 			setStatus(instring);
 		}
-	}else
-	if ( event->mimeData()->hasUrls() )
+	}
+	else */
+	if(event->mimeData()->hasUrls())
 	{
 		QList<QUrl> UrlList;
 		QFileInfo finfo;
@@ -400,11 +403,11 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 		if ( UrlList.size() > 0) // if at least one QUrl is present in list
 		{
 			filename = UrlList[0].toLocalFile(); // convert first QUrl to local path
-			finfo.setFile( filename );
+			finfo.setFile(filename);
 			if ( finfo.isFile() )
 			{
-				setStatus(filename);
 				event->acceptProposedAction();
+				setStatus(filename);
 			}
 		}
 	}
@@ -429,9 +432,9 @@ void MainWindow::editPosChanged()
 	for(i=0;i<n;i++)	if(dlm.contains(text[i]))	break;
 	text.truncate(i);
 
-	const char *desc = parser.CmdDesc(text.toAscii().constData());
-	const char *form = parser.CmdFormat(text.toAscii().constData());
-	if(form)	setStatus(QString::fromAscii(desc)+": "+QString::fromAscii(form));
+	const char *desc = parser.CmdDesc(text.toStdString().c_str());
+	const char *form = parser.CmdFormat(text.toStdString().c_str());
+	if(form)	setStatus(QString::fromLocal8Bit(desc)+": "+QString::fromLocal8Bit(form));
 	else	setStatus(tr("Not recognized"));
 }
 //-----------------------------------------------------------------------------
@@ -470,7 +473,7 @@ void MainWindow::warnChanged()
 void MainWindow::about()
 {
 	QString s = tr("UDAV v. 2.")+QString::number(MGL_VER2)+
-				tr("\n(c) Alexey Balakin, 2008-2012\nhttp://mathgl.sf.net/");
+				tr("\n(c) Alexey Balakin, 2007-2014\nhttp://mathgl.sf.net/");
 	QMessageBox::about(this, tr("UDAV - about"), s);
 }
 //-----------------------------------------------------------------------------
@@ -567,7 +570,7 @@ void MainWindow::setStatus(const QString &txt)
 void MainWindow::setCurrentFile(const QString &fileName)
 {
 	filename = fileName;
-	graph->mgl->getGraph()->PlotId = fileName.toAscii().constData();
+	mgl_set_plotid(graph->mgl->getGraph(), fileName.toStdString().c_str());
 	edit->setModified(false);
 	if(filename.isEmpty())
 		setWindowTitle(tr("untitled - UDAV"));
