@@ -1,6 +1,6 @@
 /***************************************************************************
  * mglcgi.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -52,7 +52,7 @@ void mgl_get_value(const char *buf, const char *name, char *val)
 	}
 }
 //-----------------------------------------------------------------------------
-int main(int argc, char *argv[])
+int main()
 {
 	mglGraph gr;
 	mglParse p(true);
@@ -63,12 +63,13 @@ int main(int argc, char *argv[])
 
 	char *str, *buf;
 	const char *method = getenv("REQUEST_METHOD");
+	bool alloc=false;
 	if(method && strcmp(method,"GET"))
 	{
 		long len=atol(getenv("CONTENT_LENGTH"));
 		buf = new char[len+1];
 		fread(buf,len,1,stdin);
-		buf[len]=0;
+		buf[len]=0;	alloc=true;
 	}
 	else		buf = getenv("QUERY_STRING");
 	if(buf==0)	{	printf("There is no query. Exit.\n");	return 0;	}
@@ -85,6 +86,7 @@ int main(int argc, char *argv[])
 	printf("</img></body></html>\n");*/
 
 	printf("Content-Type: image/png\n\n");	gr.WritePNG("-");
+	if(alloc)	delete []buf;
 	return 0;
 }
 //-----------------------------------------------------------------------------
