@@ -1,6 +1,6 @@
 /***************************************************************************
  * qt.cpp is part of Math Graphic Library                              *
- * Copyright (C) 2007-2012 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -94,7 +94,7 @@ QMathGL::QMathGL(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
 	popup = 0;	grBuf = 0;	draw = 0;
 	phi = tet = per = 0;
 	x1 = y1 = ax1 = ay1 = 0;	x2 = y2 = ax2 = ay2 = 1;
-	alpha = light = zoom = rotate = grid = viewYZ = false;
+	alpha = light = zoom = rotate = grid = viewYZ = custZoom = custDraw = false;
 	resize(600, 400);	mgl_set_flag(gr, true, MGL_CLF_ON_UPD);
 	timer = new QTimer(this);
 	enableWheel = enableMouse = true;
@@ -107,6 +107,21 @@ QMathGL::~QMathGL()
 {
 	if(mgl_use_graph(gr,-1)<1)	mgl_delete_graph(gr);
 	if(grBuf)	delete []grBuf;
+	if(draw)	delete draw;
+}
+//-----------------------------------------------------------------------------
+void QMathGL::setDraw(int (*func)(mglBase *gr, void *par), void *par)
+{
+	if(draw)	delete draw;	draw = 0;
+	draw_func = func;	draw_par = par;
+	emit usePrimChanged(draw_func || draw);
+}
+//-----------------------------------------------------------------------------
+void QMathGL::setDraw(mglDraw *dr)
+{
+	if(draw)	delete draw;
+	draw = dr;	draw_func = 0;
+	emit usePrimChanged(draw_func || draw);
 }
 //-----------------------------------------------------------------------------
 double QMathGL::getRatio()	{	return double(mgl_get_width(gr))/mgl_get_height(gr);	}
@@ -196,6 +211,10 @@ void QMathGL::setZoom(bool z)
 	emit zoomChanged(z);	emit rotateChanged(false);	}
 }
 //-----------------------------------------------------------------------------
+void QMathGL::setCustZoom(bool z)	{	custZoom = z;	}
+//-----------------------------------------------------------------------------
+void QMathGL::setCustDraw(bool z)	{	custDraw = z;	}
+//-----------------------------------------------------------------------------
 void QMathGL::shiftDown()
 {	mreal d=(y2-y1)/4;	y1+=d;	y2+=d;	refresh();	}
 //-----------------------------------------------------------------------------
@@ -258,6 +277,7 @@ void QMathGL::update()
 		if(!isHidden())	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 		setlocale(LC_NUMERIC, "C");
 		// use frames for quickly redrawing while adding/changing primitives
+		if(custDraw)	emit customDraw(x1,y1,x2,y2,true);
 		if(mgl_is_frames(gr))	mgl_new_frame(gr);
 		if(draw_func)	draw_func(gr, draw_par);
 		else if(draw)	{	mglGraph g(gr);	draw->Draw(&g);	}
@@ -274,7 +294,9 @@ void QMathGL::update()
 void QMathGL::draw_thr()
 {
 	mgl_clf(gr);
-	mgl_get_frame(gr, mgl_get_num_frame(gr)-1);
+	mglCanvasWnd *g = dynamic_cast<mglCanvasWnd *>(gr);
+	int c = g?g->GetCurFig():mgl_get_num_frame(gr)-1;
+	mgl_get_frame(gr, c);
 	mglParse pr;
 	long i, n=primitives.count('\n');
 	mglGraph gg(gr);
@@ -286,7 +308,7 @@ void QMathGL::draw_thr()
 	{
 		mgl_set_obj_id(gr,i+MGL_MAX_LINES);
 		QString tst = primitives.section('\n',i,i);
-		pr.Parse(&gg,primitives.section('\n',i,i).toAscii().constData(),i+MGL_MAX_LINES);
+		pr.Parse(&gg,primitives.section('\n',i,i).toStdString().c_str(),i+MGL_MAX_LINES);
 	}
 	gg.SetRanges(ox1,ox2);	gg.Pop();	setlocale(LC_NUMERIC, "");
 }
@@ -306,9 +328,13 @@ void QMathGL::refresh()
 			draw_thr();
 /*#endif*/
 		}
-		mgl_zoom(gr,x1,y1,x2,y2);	mgl_perspective(gr,per);
-		if(viewYZ)	mgl_view(gr,0,phi,tet);
-		else 		mgl_view(gr,phi,0,tet);
+		if(custZoom)	emit customZoom(x1,y1,x2,y2,tet,phi,per);
+		else
+		{	mgl_zoom(gr,x1,y1,x2,y2);
+			mgl_perspective(gr,per);
+			if(viewYZ)	mgl_view(gr,0,phi,tet);
+			else 		mgl_view(gr,phi,0,tet);
+		}
 	}
 	mglConvertFromGraph(pic, gr, &grBuf);
 	if(pic.size()!=size())	setSize(pic.width(), pic.height());
@@ -326,7 +352,7 @@ void QMathGL::mousePressEvent(QMouseEvent *ev)
 		emit mouseClick(p.x,p.y,p.z);
 		int id = mgl_get_obj_id(gr,ev->x(),ev->y());
 		if(id<MGL_MAX_LINES)	emit objChanged(id-1);
-		
+
 		p = gr->CalcXYZ(ev->x(), ev->y(), true);
 		if(mgl_isnan(p.x))	mousePos = "";
 		else	mousePos.sprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
@@ -350,7 +376,12 @@ void QMathGL::mouseReleaseEvent(QMouseEvent *ev)
 			if(x1>x2)	{	_x1=x1;	x1=x2;	x2=_x1;	}
 			if(y1>y2)	{	_x1=y1;	y1=y2;	y2=_x1;	}
 			x0 = xe;	y0 = ye;
-			refresh();
+			if(custDraw)
+			{
+				emit customDraw(x1,y1,x2,y2,false);
+				update();
+			}
+			else	refresh();
 		}
 	}
 	if(ev->button()&Qt::RightButton && popup && !rotate)	// popup menu
@@ -564,28 +595,28 @@ void QMathGL::exportGIF(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_gif(gr,setExtension(fname,"png").toAscii().constData(), appName.toAscii().constData());
+	else	mgl_write_gif(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportPNG(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_png(gr,setExtension(fname,"png").toAscii().constData(), appName.toAscii().constData());
+	else	mgl_write_png(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportPNGs(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_png_solid(gr,setExtension(fname,"png").toAscii().constData(), appName.toAscii().constData());
+	else	mgl_write_png_solid(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportJPG(QString fname)
 {
 	if(fname.isEmpty())	fname = mgl_get_plotid(gr);
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
-	else	mgl_write_jpg(gr,setExtension(fname,"jpg").toAscii().constData(), appName.toAscii().constData());
+	else	mgl_write_jpg(gr,setExtension(fname,"jpg").toStdString().c_str(), appName.toStdString().c_str());
 }
 //-----------------------------------------------------------------------------
 void QMathGL::exportBPS(QString fname)
@@ -595,7 +626,7 @@ void QMathGL::exportBPS(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_bps(gr,setExtension(fname,"eps").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_bps(gr,setExtension(fname,"eps").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -607,7 +638,7 @@ void QMathGL::exportEPS(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_eps(gr,setExtension(fname,"eps").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_eps(gr,setExtension(fname,"eps").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -619,7 +650,7 @@ void QMathGL::exportSVG(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_svg(gr,setExtension(fname,"svg").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_svg(gr,setExtension(fname,"svg").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -631,7 +662,7 @@ void QMathGL::exportXYZ(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_xyz(gr,setExtension(fname,"xyz").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_xyz(gr,setExtension(fname,"xyz").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -643,7 +674,7 @@ void QMathGL::exportTEX(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_tex(gr,setExtension(fname,"tex").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_tex(gr,setExtension(fname,"tex").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -655,7 +686,7 @@ void QMathGL::exportOFF(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_off(gr,setExtension(fname,"off").toAscii().constData(), appName.toAscii().constData(),0);
+		mgl_write_off(gr,setExtension(fname,"off").toStdString().c_str(), appName.toStdString().c_str(),0);
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -667,7 +698,7 @@ void QMathGL::exportOBJ(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_obj(gr,setExtension(fname,"obj").toAscii().constData(), appName.toAscii().constData(),1);
+		mgl_write_obj(gr,setExtension(fname,"obj").toStdString().c_str(), appName.toStdString().c_str(),1);
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -679,7 +710,7 @@ void QMathGL::exportSTL(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_stl(gr,setExtension(fname,"stl").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_stl(gr,setExtension(fname,"stl").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -691,7 +722,7 @@ void QMathGL::exportSTL(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_x3d(gr,setExtension(fname,"x3d").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_x3d(gr,setExtension(fname,"x3d").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }*/
@@ -703,7 +734,7 @@ void QMathGL::exportTGA(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_tga(gr,setExtension(fname,"tga").toAscii().constData(), appName.toAscii().constData());
+		mgl_write_tga(gr,setExtension(fname,"tga").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -715,7 +746,7 @@ void QMathGL::exportPRC(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_prc(gr,setExtension(fname,"prc").toAscii().constData(), appName.toAscii().constData(),1);
+		mgl_write_prc(gr,setExtension(fname,"prc").toStdString().c_str(), appName.toStdString().c_str(),1);
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -727,7 +758,7 @@ void QMathGL::exportMGLD(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_export_mgld(gr,setExtension(fname,"mgld").toAscii().constData(), appName.toAscii().constData());
+		mgl_export_mgld(gr,setExtension(fname,"mgld").toStdString().c_str(), appName.toStdString().c_str());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -758,7 +789,7 @@ void QMathGL::copyClickCoor()
 //-----------------------------------------------------------------------------
 void QMathGL::setMGLFont(QString path)
 {	if(path.isEmpty())	mgl_restore_font(gr);
-	else 	mgl_load_font(gr,path.toAscii().constData(),0);	}
+	else 	mgl_load_font(gr,path.toStdString().c_str(),0);	}
 //-----------------------------------------------------------------------------
 void QMathGL::setSize(int w, int h)
 {
@@ -996,25 +1027,25 @@ QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&p
 		a = new QAction(QPixmap(alpha_xpm), TR("&Alpha"), Wnd);
 		a->setShortcut(Qt::ALT+Qt::Key_T);	a->setCheckable(true);
 		Wnd->connect(a, SIGNAL(toggled(bool)), QMGL, SLOT(setAlpha(bool)));
-		Wnd->connect(QMGL, SIGNAL(alphaChanged(bool)), a, SLOT(setOn(bool)));
+		Wnd->connect(QMGL, SIGNAL(alphaChanged(bool)), a, SLOT(setChecked(bool)));
 		a->setToolTip(TR("Switch on/off TRansparency for the graphics (Alt+T)."));
 		o->addAction(a);		bb->addAction(a);
 		a = new QAction(QPixmap(light_xpm), TR("&Light"), Wnd);
 		a->setShortcut(Qt::ALT+Qt::Key_L);	a->setCheckable(true);
 		Wnd->connect(a, SIGNAL(toggled(bool)), QMGL, SLOT(setLight(bool)));
-		Wnd->connect(QMGL, SIGNAL(lightChanged(bool)), a, SLOT(setOn(bool)));
+		Wnd->connect(QMGL, SIGNAL(lightChanged(bool)), a, SLOT(setChecked(bool)));
 		a->setToolTip(TR("Switch on/off lightning for the graphics (Alt+L)."));
 		o->addAction(a);		bb->addAction(a);
 		a = new QAction(QPixmap(rotate_xpm), TR("&Rotate by mouse"), Wnd);
 		a->setCheckable(true);
 		Wnd->connect(a, SIGNAL(toggled(bool)), QMGL, SLOT(setRotate(bool)));
-		Wnd->connect(QMGL, SIGNAL(rotateChanged(bool)), a, SLOT(setOn(bool)));
+		Wnd->connect(QMGL, SIGNAL(rotateChanged(bool)), a, SLOT(setChecked(bool)));
 		a->setToolTip(TR("Switch on/off mouse handling of the graphics\n(rotation, shifting, zooming and perspective)."));
 		bb->addAction(a);
 		a = new QAction(QPixmap(zoom_in_xpm), TR("&Zoom by mouse"), Wnd);
 		a->setCheckable(true);
 		Wnd->connect(a, SIGNAL(toggled(bool)), QMGL, SLOT(setZoom(bool)));
-		Wnd->connect(QMGL, SIGNAL(zoomChanged(bool)), a, SLOT(setOn(bool)));
+		Wnd->connect(QMGL, SIGNAL(zoomChanged(bool)), a, SLOT(setChecked(bool)));
 		a->setToolTip(TR("Switch on/off mouse zoom of selected region."));
 		bb->addAction(a);
 		o->addSeparator();
@@ -1129,7 +1160,7 @@ QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&p
 		Wnd->addToolBar(Qt::LeftToolBarArea, bb);
 		a = new QAction(QPixmap(next_sl_xpm), TR("&Next slide"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(nextSlide()));
-		a->setToolTip(TR("Show next slide (Alt+Right)."));
+		a->setToolTip(TR("Show next slide (Ctrl+.)."));
 		a->setShortcut(Qt::CTRL+Qt::Key_Period);	o->addAction(a);		bb->addAction(a);
 		a = new QAction(QPixmap(show_sl_xpm), TR("&Slideshow"), Wnd);
 		a->setCheckable(true);
@@ -1138,7 +1169,7 @@ QMenu *mglMakeMenu(QMainWindow *Wnd, QMathGL *QMGL, QSpinBox *&tet, QSpinBox *&p
 		a->setShortcut(Qt::CTRL+Qt::Key_F5);	o->addAction(a);		bb->addAction(a);
 		a = new QAction(QPixmap(prev_sl_xpm), TR("&Prev slide"), Wnd);
 		Wnd->connect(a, SIGNAL(triggered()), QMGL, SLOT(prevSlide()));
-		a->setToolTip(TR("Show previous slide (Alt+Left)."));
+		a->setToolTip(TR("Show previous slide (Ctrl+,)."));
 		a->setShortcut(Qt::CTRL+Qt::Key_Comma);	o->addAction(a);		bb->addAction(a);
 	}
 
